@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 **********************************************************************
-*   Copyright (C) 1997-2003, International Business Machines
+*   Copyright (C) 1997-2013, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *
@@ -49,6 +51,11 @@
 #include "unicode/utypes.h"
 #include "unicode/resbund.h"
 
+#include "cmemory.h"
+#include "mutex.h"
+#include "uassert.h"
+#include "umutex.h"
+
 #include "uresimp.h"
 
 U_NAMESPACE_BEGIN
@@ -128,7 +135,7 @@ U_NAMESPACE_BEGIN
  * so forth, until the chain is exhausted or the tag is found.
  *
  * Thread-safety is implemented around caches, both the cache that
- * stores all the resouce data, and the cache that stores flags
+ * stores all the resource data, and the cache that stores flags
  * indicating whether or not a file has been visited.  These caches
  * delete their storage at static cleanup time, when the process
  * quits.
@@ -169,55 +176,40 @@ U_NAMESPACE_BEGIN
 
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(ResourceBundle)
 
-ResourceBundle::ResourceBundle( const UnicodeString&    path,
-                                const Locale&           locale,
-                                UErrorCode&              error)
-                                :UObject(), locName(NULL)
-{
-    constructForLocale(path, locale, error);
-}
-
 ResourceBundle::ResourceBundle(UErrorCode &err)
-                                :UObject(), locName(NULL)
+                                :UObject(), fLocale(nullptr)
 {
-    resource = ures_open(0, Locale::getDefault().getName(), &err);
-}
-
-ResourceBundle::ResourceBundle( const UnicodeString&    path,
-                                UErrorCode&              error)
-                                :UObject(), locName(NULL)
-{
-    constructForLocale(path, Locale::getDefault(), error);
+    fResource = ures_open(nullptr, Locale::getDefault().getName(), &err);
 }
 
 ResourceBundle::ResourceBundle(const ResourceBundle &other)
-                              :UObject(other), locName(NULL)
+                              :UObject(other), fLocale(nullptr)
 {
     UErrorCode status = U_ZERO_ERROR;
 
-    if (other.resource) {
-        resource = ures_copyResb(0, other.resource, &status);
+    if (other.fResource) {
+        fResource = ures_copyResb(nullptr, other.fResource, &status);
     } else {
         /* Copying a bad resource bundle */
-        resource = NULL;
+        fResource = nullptr;
     }
 }
 
 ResourceBundle::ResourceBundle(UResourceBundle *res, UErrorCode& err)
-                               :UObject(), locName(NULL)
+                               :UObject(), fLocale(nullptr)
 {
     if (res) {
-        resource = ures_copyResb(0, res, &err);
+        fResource = ures_copyResb(nullptr, res, &err);
     } else {
         /* Copying a bad resource bundle */
-        resource = NULL;
+        fResource = nullptr;
     }
 }
 
 ResourceBundle::ResourceBundle(const char* path, const Locale& locale, UErrorCode& err) 
-                               :UObject(), locName(NULL)
+                               :UObject(), fLocale(nullptr)
 {
-    resource = ures_open(path, locale.getName(), &err);
+    fResource = ures_open(path, locale.getName(), &err);
 }
 
 
@@ -226,28 +218,30 @@ ResourceBundle& ResourceBundle::operator=(const ResourceBundle& other)
     if(this == &other) {
         return *this;
     }
-    if(resource != 0) {
-        ures_close(resource);
-        resource = NULL;
+    if (fResource != nullptr) {
+        ures_close(fResource);
+        fResource = nullptr;
+    }
+    if (fLocale != nullptr) {
+        delete fLocale;
+        fLocale = nullptr;
     }
     UErrorCode status = U_ZERO_ERROR;
-    if (other.resource) {
-        resource = ures_copyResb(0, other.resource, &status);
+    if (other.fResource) {
+        fResource = ures_copyResb(nullptr, other.fResource, &status);
     } else {
         /* Copying a bad resource bundle */
-        resource = NULL;
+        fResource = nullptr;
     }
     return *this;
 }
 
 ResourceBundle::~ResourceBundle()
 {
-    if(resource != 0) {
-        ures_close(resource);
-    }
-    if(locName != NULL) {
-      delete(locName);
+    if (fResource != nullptr) {
+        ures_close(fResource);
     }
+    delete fLocale;
 }
 
 ResourceBundle *
@@ -255,72 +249,57 @@ ResourceBundle::clone() const {
     return new ResourceBundle(*this);
 }
 
-void 
-ResourceBundle::constructForLocale(const UnicodeString& path,
-                                   const Locale& locale,
-                                   UErrorCode& error)
-{
-    char name[300];
-
-    if(!path.isEmpty()) {
-        path.extract(name, sizeof(name), 0, error);
-        resource = ures_open(name, locale.getName(), &error);
-    } else {
-        resource = ures_open(0, locale.getName(), &error);
-    }
-}
-
 UnicodeString ResourceBundle::getString(UErrorCode& status) const {
     int32_t len = 0;
-    const UChar *r = ures_getString(resource, &len, &status);
-    return UnicodeString(TRUE, r, len);
+    const char16_t *r = ures_getString(fResource, &len, &status);
+    return UnicodeString(true, r, len);
 }
 
 const uint8_t *ResourceBundle::getBinary(int32_t& len, UErrorCode& status) const {
-    return ures_getBinary(resource, &len, &status);
+    return ures_getBinary(fResource, &len, &status);
 }
 
 const int32_t *ResourceBundle::getIntVector(int32_t& len, UErrorCode& status) const {
-    return ures_getIntVector(resource, &len, &status);
+    return ures_getIntVector(fResource, &len, &status);
 }
 
 uint32_t ResourceBundle::getUInt(UErrorCode& status) const {
-    return ures_getUInt(resource, &status);
+    return ures_getUInt(fResource, &status);
 }
 
 int32_t ResourceBundle::getInt(UErrorCode& status) const {
-    return ures_getInt(resource, &status);
+    return ures_getInt(fResource, &status);
 }
 
-const char *ResourceBundle::getName(void) {
-    return ures_getName(resource);
+const char *ResourceBundle::getName() const {
+    return ures_getName(fResource);
 }
 
-const char *ResourceBundle::getKey(void) {
-    return ures_getKey(resource);
+const char *ResourceBundle::getKey() const {
+    return ures_getKey(fResource);
 }
 
-UResType ResourceBundle::getType(void) {
-    return ures_getType(resource);
+UResType ResourceBundle::getType() const {
+    return ures_getType(fResource);
 }
 
-int32_t ResourceBundle::getSize(void) const {
-    return ures_getSize(resource);
+int32_t ResourceBundle::getSize() const {
+    return ures_getSize(fResource);
 }
 
-UBool ResourceBundle::hasNext(void) const {
-    return ures_hasNext(resource);
+UBool ResourceBundle::hasNext() const {
+    return ures_hasNext(fResource);
 }
 
-void ResourceBundle::resetIterator(void) {
-    ures_resetIterator(resource);
+void ResourceBundle::resetIterator() {
+    ures_resetIterator(fResource);
 }
 
 ResourceBundle ResourceBundle::getNext(UErrorCode& status) {
     UResourceBundle r;
 
     ures_initStackObject(&r);
-    ures_getNextResource(resource, &r, &status);
+    ures_getNextResource(fResource, &r, &status);
     ResourceBundle res(&r, status);
     if (U_SUCCESS(status)) {
         ures_close(&r);
@@ -330,21 +309,21 @@ ResourceBundle ResourceBundle::getNext(UErrorCode& status) {
 
 UnicodeString ResourceBundle::getNextString(UErrorCode& status) {
     int32_t len = 0;
-    const UChar* r = ures_getNextString(resource, &len, 0, &status);
-    return UnicodeString(TRUE, r, len);
+    const char16_t* r = ures_getNextString(fResource, &len, nullptr, &status);
+    return UnicodeString(true, r, len);
 }
 
 UnicodeString ResourceBundle::getNextString(const char ** key, UErrorCode& status) {
     int32_t len = 0;
-    const UChar* r = ures_getNextString(resource, &len, key, &status);
-    return UnicodeString(TRUE, r, len);
+    const char16_t* r = ures_getNextString(fResource, &len, key, &status);
+    return UnicodeString(true, r, len);
 }
 
 ResourceBundle ResourceBundle::get(int32_t indexR, UErrorCode& status) const {
     UResourceBundle r;
 
     ures_initStackObject(&r);
-    ures_getByIndex(resource, indexR, &r, &status);
+    ures_getByIndex(fResource, indexR, &r, &status);
     ResourceBundle res(&r, status);
     if (U_SUCCESS(status)) {
         ures_close(&r);
@@ -354,15 +333,15 @@ ResourceBundle ResourceBundle::get(int32_t indexR, UErrorCode& status) const {
 
 UnicodeString ResourceBundle::getStringEx(int32_t indexS, UErrorCode& status) const {
     int32_t len = 0;
-    const UChar* r = ures_getStringByIndex(resource, indexS, &len, &status);
-    return UnicodeString(TRUE, r, len);
+    const char16_t* r = ures_getStringByIndex(fResource, indexS, &len, &status);
+    return UnicodeString(true, r, len);
 }
 
 ResourceBundle ResourceBundle::get(const char* key, UErrorCode& status) const {
     UResourceBundle r;
 
     ures_initStackObject(&r);
-    ures_getByKey(resource, key, &r, &status);
+    ures_getByKey(fResource, key, &r, &status);
     ResourceBundle res(&r, status);
     if (U_SUCCESS(status)) {
         ures_close(&r);
@@ -370,37 +349,49 @@ ResourceBundle ResourceBundle::get(const char* key, UErrorCode& status) const {
     return res;
 }
 
+ResourceBundle ResourceBundle::getWithFallback(const char* key, UErrorCode& status){
+    UResourceBundle r;
+    ures_initStackObject(&r);
+    ures_getByKeyWithFallback(fResource, key, &r, &status);
+    ResourceBundle res(&r, status);
+    if(U_SUCCESS(status)){
+        ures_close(&r);
+    }
+    return res;
+}
 UnicodeString ResourceBundle::getStringEx(const char* key, UErrorCode& status) const {
     int32_t len = 0;
-    const UChar* r = ures_getStringByKey(resource, key, &len, &status);
-    return UnicodeString(TRUE, r, len);
+    const char16_t* r = ures_getStringByKey(fResource, key, &len, &status);
+    return UnicodeString(true, r, len);
 }
 
 const char*
 ResourceBundle::getVersionNumber()  const
 {
-    return ures_getVersionNumber(resource);
+    return ures_getVersionNumberInternal(fResource);
 }
 
 void ResourceBundle::getVersion(UVersionInfo versionInfo) const {
-    ures_getVersion(resource, versionInfo);
+    ures_getVersion(fResource, versionInfo);
 }
 
-const Locale &ResourceBundle::getLocale(void) const
-{
-  if(locName == NULL) {
+const Locale &ResourceBundle::getLocale() const {
+    static UMutex gLocaleLock;
+    Mutex lock(&gLocaleLock);
+    if (fLocale != nullptr) {
+        return *fLocale;
+    }
     UErrorCode status = U_ZERO_ERROR;
-    const char *localeName = ures_getLocale(resource, &status);
-    ResourceBundle *me = (ResourceBundle *)this; // semantically const
-    me->locName = new Locale(localeName);
-  }
-  return *locName;
+    const char *localeName = ures_getLocaleInternal(fResource, &status);
+    ResourceBundle *ncThis = const_cast<ResourceBundle *>(this);
+    ncThis->fLocale = new Locale(localeName);
+    return ncThis->fLocale != nullptr ? *ncThis->fLocale : Locale::getDefault();
 }
 
-const Locale ResourceBundle::getLocale(ULocDataLocaleType type, UErrorCode &status) const
+Locale ResourceBundle::getLocale(ULocDataLocaleType type, UErrorCode &status) const
 {
-  return ures_getLocaleByType(resource, type, &status);
+  return ures_getLocaleByType(fResource, type, &status);
 }
 
-//eof
 U_NAMESPACE_END
+//eof
