@@ -872,11 +872,12 @@ UTS46::processLabel(UnicodeString &dest,
                 buffer[1]=0x6e;
                 buffer[2]=0x2d;
                 buffer[3]=0x2d;
+                UErrorCode punycodeErrorCode=U_ZERO_ERROR;
                 int32_t punycodeLength=u_strToPunycode(label, labelLength,
                                                       buffer+4, punycode.getCapacity()-4,
-                                                      nullptr, &errorCode);
-                if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
-                    errorCode=U_ZERO_ERROR;
+                                                      nullptr, &punycodeErrorCode);
+                if(punycodeErrorCode==U_BUFFER_OVERFLOW_ERROR) {
+                    punycodeErrorCode=U_ZERO_ERROR;
                     punycode.releaseBuffer(4);
                     buffer=punycode.getBuffer(4+punycodeLength);
                     if(buffer==nullptr) {
@@ -885,11 +886,12 @@ UTS46::processLabel(UnicodeString &dest,
                     }
                     punycodeLength=u_strToPunycode(label, labelLength,
                                                   buffer+4, punycode.getCapacity()-4,
-                                                  nullptr, &errorCode);
+                                                  nullptr, &punycodeErrorCode);
                 }
                 punycodeLength+=4;
                 punycode.releaseBuffer(punycodeLength);
-                if(U_FAILURE(errorCode)) {
+                if(U_FAILURE(punycodeErrorCode)) {
+                    errorCode = punycodeErrorCode;
                     return destLabelLength;
                 }
                 if(punycodeLength>63) {
