@@ -1,7 +1,9 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
  *
- *   Copyright (C) 1999-2001, International Business Machines
+ *   Copyright (C) 1999-2016, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *
  *******************************************************************************
@@ -14,9 +16,12 @@
 #include "unicode/utypes.h"
 #include "unicode/uscript.h"
 
+#include "cmemory.h"
 #include "scrptrun.h"
 
-#define ARRAY_SIZE(array) (sizeof array  / sizeof array[0])
+U_NAMESPACE_BEGIN
+
+const char ScriptRun::fgClassID=0;
 
 UChar32 ScriptRun::pairedChars[] = {
     0x0028, 0x0029, // ascii paired punctuation
@@ -38,7 +43,7 @@ UChar32 ScriptRun::pairedChars[] = {
     0x301a, 0x301b
 };
 
-const int32_t ScriptRun::pairedCharCount = ARRAY_SIZE(pairedChars);
+const int32_t ScriptRun::pairedCharCount = UPRV_LENGTHOF(pairedChars);
 const int32_t ScriptRun::pairedCharPower = 1 << highBit(pairedCharCount);
 const int32_t ScriptRun::pairedCharExtra = pairedCharCount - pairedCharPower;
 
@@ -120,14 +125,14 @@ UBool ScriptRun::next()
     scriptCode = USCRIPT_COMMON;
 
     for (scriptStart = scriptEnd; scriptEnd < charLimit; scriptEnd += 1) {
-        UChar   high = charArray[scriptEnd];
+        char16_t   high = charArray[scriptEnd];
         UChar32 ch   = high;
 
         // if the character is a high surrogate and it's not the last one
         // in the text, see if it's followed by a low surrogate
         if (high >= 0xD800 && high <= 0xDBFF && scriptEnd < charLimit - 1)
         {
-            UChar low = charArray[scriptEnd + 1];
+            char16_t low = charArray[scriptEnd + 1];
 
             // if it is followed by a low surrogate,
             // consume it and form the full character
@@ -150,6 +155,7 @@ UBool ScriptRun::next()
             if ((pairIndex & 1) == 0) {
                 parenStack[++parenSP].pairIndex = pairIndex;
                 parenStack[parenSP].scriptCode  = scriptCode;
+                startSP = parenSP;
             } else if (parenSP >= 0) {
                 int32_t pi = pairIndex & ~1;
 
@@ -198,3 +204,4 @@ UBool ScriptRun::next()
     return true;
 }
 
+U_NAMESPACE_END
