@@ -15,7 +15,8 @@
 
 #include <stdio.h>
 
-UChar testChars[] = {
+char16_t testChars[] = {
+            '(', ')', '(', ')', '(', ')', 'x', '(', ')',
             0x0020, 0x0946, 0x0939, 0x093F, 0x0928, 0x094D, 0x0926, 0x0940, 0x0020,
             0x0627, 0x0644, 0x0639, 0x0631, 0x0628, 0x064A, 0x0629, 0x0020,
             0x0420, 0x0443, 0x0441, 0x0441, 0x043A, 0x0438, 0x0439, 0x0020,
@@ -27,9 +28,9 @@ UChar testChars[] = {
 
 int32_t testLength = UPRV_LENGTHOF(testChars);
 
-void main()
+int main()
 {
-    ScriptRun scriptRun(testChars, 0, testLength);
+    icu::ScriptRun scriptRun(testChars, 0, testLength);
 
     while (scriptRun.next()) {
         int32_t     start = scriptRun.getScriptStart();
@@ -38,4 +39,5 @@ void main()
 
         printf("Script '%s' from %d to %d.\n", uscript_getName(code), start, end);
     }
+    return 0;
 }
