@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  **********************************************************************
  *   Copyright (C) 2005-2016, International Business Machines
@@ -44,7 +46,7 @@ int32_t CharsetRecog_2022::match_2022(const uint8_t *text, int32_t textLen, cons
             escN = 0;
             while(escN < escapeSequences_length) {
                 const uint8_t *seq = escapeSequences[escN];
-                int32_t seq_length = (int32_t)uprv_strlen((const char *) seq);
+                int32_t seq_length = static_cast<int32_t>(uprv_strlen((const char*)seq));
 
                 if (textLen-i >= seq_length) {
                     j = 1;
@@ -81,7 +83,7 @@ scanInput:
     }
 
     //
-    // Initial quality is based on relative proportion of recongized vs.
+    // Initial quality is based on relative proportion of recognized vs.
     //   unrecognized escape sequences. 
     //   All good:  quality = 100;
     //   half or less good: quality = 0;
