@@ -59,73 +59,76 @@ static const char gLatn[] =  "latn";
 static const char gSymbols[] = "symbols";
 static const char gNumberElementsLatnSymbols[] = "NumberElements/latn/symbols";
 
-static const UChar INTL_CURRENCY_SYMBOL_STR[] = {0xa4, 0xa4, 0};
+static const char16_t INTL_CURRENCY_SYMBOL_STR[] = {0xa4, 0xa4, 0};
 
 // List of field names to be loaded from the data files.
 // These are parallel with the enum ENumberFormatSymbol in unicode/dcfmtsym.h.
 static const char *gNumberElementKeys[DecimalFormatSymbols::kFormatSymbolCount] = {
     "decimal",
     "group",
-    NULL, /* #11897: the <list> symbol is NOT the pattern separator symbol */
+    nullptr, /* #11897: the <list> symbol is NOT the pattern separator symbol */
     "percentSign",
-    NULL, /* Native zero digit is deprecated from CLDR - get it from the numbering system */
-    NULL, /* Pattern digit character is deprecated from CLDR - use # by default always */
+    nullptr, /* Native zero digit is deprecated from CLDR - get it from the numbering system */
+    nullptr, /* Pattern digit character is deprecated from CLDR - use # by default always */
     "minusSign",
     "plusSign",
-    NULL, /* currency symbol - Wait until we know the currency before loading from CLDR */
-    NULL, /* intl currency symbol - Wait until we know the currency before loading from CLDR */
+    nullptr, /* currency symbol - Wait until we know the currency before loading from CLDR */
+    nullptr, /* intl currency symbol - Wait until we know the currency before loading from CLDR */
     "currencyDecimal",
     "exponential",
     "perMille",
-    NULL, /* Escape padding character - not in CLDR */
+    nullptr, /* Escape padding character - not in CLDR */
     "infinity",
     "nan",
-    NULL, /* Significant digit symbol - not in CLDR */
+    nullptr, /* Significant digit symbol - not in CLDR */
     "currencyGroup",
-    NULL, /* one digit - get it from the numbering system */
-    NULL, /* two digit - get it from the numbering system */
-    NULL, /* three digit - get it from the numbering system */
-    NULL, /* four digit - get it from the numbering system */
-    NULL, /* five digit - get it from the numbering system */
-    NULL, /* six digit - get it from the numbering system */
-    NULL, /* seven digit - get it from the numbering system */
-    NULL, /* eight digit - get it from the numbering system */
-    NULL, /* nine digit - get it from the numbering system */
+    nullptr, /* one digit - get it from the numbering system */
+    nullptr, /* two digit - get it from the numbering system */
+    nullptr, /* three digit - get it from the numbering system */
+    nullptr, /* four digit - get it from the numbering system */
+    nullptr, /* five digit - get it from the numbering system */
+    nullptr, /* six digit - get it from the numbering system */
+    nullptr, /* seven digit - get it from the numbering system */
+    nullptr, /* eight digit - get it from the numbering system */
+    nullptr, /* nine digit - get it from the numbering system */
     "superscriptingExponent", /* Multiplication (x) symbol for exponents */
+    "approximatelySign" /* Approximately sign symbol */
 };
 
 // -------------------------------------
 // Initializes this with the decimal format symbols in the default locale.
 
 DecimalFormatSymbols::DecimalFormatSymbols(UErrorCode& status)
-        : UObject(), locale(), currPattern(NULL) {
-    initialize(locale, status, TRUE);
+        : UObject(), locale() {
+    initialize(locale, status, true);
 }
 
 // -------------------------------------
 // Initializes this with the decimal format symbols in the desired locale.
 
 DecimalFormatSymbols::DecimalFormatSymbols(const Locale& loc, UErrorCode& status)
-        : UObject(), locale(loc), currPattern(NULL) {
+        : UObject(), locale(loc) {
     initialize(locale, status);
 }
 
 DecimalFormatSymbols::DecimalFormatSymbols(const Locale& loc, const NumberingSystem& ns, UErrorCode& status)
-        : UObject(), locale(loc), currPattern(NULL) {
-    initialize(locale, status, FALSE, &ns);
+        : UObject(), locale(loc) {
+    initialize(locale, status, false, &ns);
 }
 
 DecimalFormatSymbols::DecimalFormatSymbols()
-        : UObject(), locale(Locale::getRoot()), currPattern(NULL) {
-    *validLocale = *actualLocale = 0;
+    : UObject(),
+      locale(Locale::getRoot()),
+      actualLocale(Locale::getRoot()),
+      validLocale(Locale::getRoot()) {
     initialize();
 }
 
 DecimalFormatSymbols*
 DecimalFormatSymbols::createWithLastResortData(UErrorCode& status) {
-    if (U_FAILURE(status)) { return NULL; }
+    if (U_FAILURE(status)) { return nullptr; }
     DecimalFormatSymbols* sym = new DecimalFormatSymbols();
-    if (sym == NULL) {
+    if (sym == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
     }
     return sym;
@@ -153,56 +156,57 @@ DecimalFormatSymbols&
 DecimalFormatSymbols::operator=(const DecimalFormatSymbols& rhs)
 {
     if (this != &rhs) {
-        for(int32_t i = 0; i < (int32_t)kFormatSymbolCount; ++i) {
+        for (int32_t i = 0; i < static_cast<int32_t>(kFormatSymbolCount); ++i) {
             // fastCopyFrom is safe, see docs on fSymbols
-            fSymbols[(ENumberFormatSymbol)i].fastCopyFrom(rhs.fSymbols[(ENumberFormatSymbol)i]);
+            fSymbols[static_cast<ENumberFormatSymbol>(i)].fastCopyFrom(rhs.fSymbols[static_cast<ENumberFormatSymbol>(i)]);
         }
-        for(int32_t i = 0; i < (int32_t)UNUM_CURRENCY_SPACING_COUNT; ++i) {
+        for (int32_t i = 0; i < static_cast<int32_t>(UNUM_CURRENCY_SPACING_COUNT); ++i) {
             currencySpcBeforeSym[i].fastCopyFrom(rhs.currencySpcBeforeSym[i]);
             currencySpcAfterSym[i].fastCopyFrom(rhs.currencySpcAfterSym[i]);
         }
         locale = rhs.locale;
-        uprv_strcpy(validLocale, rhs.validLocale);
-        uprv_strcpy(actualLocale, rhs.actualLocale);
+        actualLocale = rhs.actualLocale;
+        validLocale = rhs.validLocale;
         fIsCustomCurrencySymbol = rhs.fIsCustomCurrencySymbol; 
         fIsCustomIntlCurrencySymbol = rhs.fIsCustomIntlCurrencySymbol; 
         fCodePointZero = rhs.fCodePointZero;
         currPattern = rhs.currPattern;
+        uprv_strcpy(nsName, rhs.nsName);
     }
     return *this;
 }
 
 // -------------------------------------
 
-UBool
+bool
 DecimalFormatSymbols::operator==(const DecimalFormatSymbols& that) const
 {
     if (this == &that) {
-        return TRUE;
+        return true;
     }
     if (fIsCustomCurrencySymbol != that.fIsCustomCurrencySymbol) { 
-        return FALSE; 
+        return false;
     } 
     if (fIsCustomIntlCurrencySymbol != that.fIsCustomIntlCurrencySymbol) { 
-        return FALSE; 
+        return false;
     } 
-    for(int32_t i = 0; i < (int32_t)kFormatSymbolCount; ++i) {
-        if(fSymbols[(ENumberFormatSymbol)i] != that.fSymbols[(ENumberFormatSymbol)i]) {
-            return FALSE;
+    for (int32_t i = 0; i < static_cast<int32_t>(kFormatSymbolCount); ++i) {
+        if (fSymbols[static_cast<ENumberFormatSymbol>(i)] != that.fSymbols[static_cast<ENumberFormatSymbol>(i)]) {
+            return false;
         }
     }
-    for(int32_t i = 0; i < (int32_t)UNUM_CURRENCY_SPACING_COUNT; ++i) {
+    for (int32_t i = 0; i < static_cast<int32_t>(UNUM_CURRENCY_SPACING_COUNT); ++i) {
         if(currencySpcBeforeSym[i] != that.currencySpcBeforeSym[i]) {
-            return FALSE;
+            return false;
         }
         if(currencySpcAfterSym[i] != that.currencySpcAfterSym[i]) {
-            return FALSE;
+            return false;
         }
     }
     // No need to check fCodePointZero since it is based on fSymbols
     return locale == that.locale &&
-        uprv_strcmp(validLocale, that.validLocale) == 0 &&
-        uprv_strcmp(actualLocale, that.actualLocale) == 0;
+           actualLocale == that.actualLocale &&
+           validLocale == that.validLocale;
 }
 
 // -------------------------------------
@@ -221,26 +225,26 @@ struct DecFmtSymDataSink : public ResourceSink {
     // Destination for data, modified via setters.
     DecimalFormatSymbols& dfs;
     // Boolean array of whether or not we have seen a particular symbol yet.
-    // Can't simpy check fSymbols because it is pre-populated with defaults.
+    // Can't simply check fSymbols because it is pre-populated with defaults.
     UBool seenSymbol[DecimalFormatSymbols::kFormatSymbolCount];
 
     // Constructor/Destructor
     DecFmtSymDataSink(DecimalFormatSymbols& _dfs) : dfs(_dfs) {
-        uprv_memset(seenSymbol, FALSE, sizeof(seenSymbol));
+        uprv_memset(seenSymbol, false, sizeof(seenSymbol));
     }
     virtual ~DecFmtSymDataSink();
 
     virtual void put(const char *key, ResourceValue &value, UBool /*noFallback*/,
-            UErrorCode &errorCode) {
+            UErrorCode &errorCode) override {
         ResourceTable symbolsTable = value.getTable(errorCode);
         if (U_FAILURE(errorCode)) { return; }
         for (int32_t j = 0; symbolsTable.getKeyAndValue(j, key, value); ++j) {
             for (int32_t i=0; i<DecimalFormatSymbols::kFormatSymbolCount; i++) {
-                if (gNumberElementKeys[i] != NULL && uprv_strcmp(key, gNumberElementKeys[i]) == 0) {
+                if (gNumberElementKeys[i] != nullptr && uprv_strcmp(key, gNumberElementKeys[i]) == 0) {
                     if (!seenSymbol[i]) {
-                        seenSymbol[i] = TRUE;
+                        seenSymbol[i] = true;
                         dfs.setSymbol(
-                            (DecimalFormatSymbols::ENumberFormatSymbol) i,
+                            static_cast<DecimalFormatSymbols::ENumberFormatSymbol>(i),
                             value.getUnicodeString(errorCode));
                         if (U_FAILURE(errorCode)) { return; }
                     }
@@ -254,10 +258,10 @@ struct DecFmtSymDataSink : public ResourceSink {
     UBool seenAll() {
         for (int32_t i=0; i<DecimalFormatSymbols::kFormatSymbolCount; i++) {
             if (!seenSymbol[i]) {
-                return FALSE;
+                return false;
             }
         }
-        return TRUE;
+        return true;
     }
 
     // If monetary decimal or grouping were not explicitly set, then set them to be the
@@ -282,20 +286,20 @@ struct CurrencySpacingSink : public ResourceSink {
     UBool hasAfterCurrency;
 
     CurrencySpacingSink(DecimalFormatSymbols& _dfs)
-        : dfs(_dfs), hasBeforeCurrency(FALSE), hasAfterCurrency(FALSE) {}
+        : dfs(_dfs), hasBeforeCurrency(false), hasAfterCurrency(false) {}
     virtual ~CurrencySpacingSink();
 
     virtual void put(const char *key, ResourceValue &value, UBool /*noFallback*/,
-            UErrorCode &errorCode) {
+            UErrorCode &errorCode) override {
         ResourceTable spacingTypesTable = value.getTable(errorCode);
         for (int32_t i = 0; spacingTypesTable.getKeyAndValue(i, key, value); ++i) {
             UBool beforeCurrency;
             if (uprv_strcmp(key, gBeforeCurrencyTag) == 0) {
-                beforeCurrency = TRUE;
-                hasBeforeCurrency = TRUE;
+                beforeCurrency = true;
+                hasBeforeCurrency = true;
             } else if (uprv_strcmp(key, gAfterCurrencyTag) == 0) {
-                beforeCurrency = FALSE;
-                hasAfterCurrency = TRUE;
+                beforeCurrency = false;
+                hasAfterCurrency = true;
             } else {
                 continue;
             }
@@ -328,11 +332,13 @@ struct CurrencySpacingSink : public ResourceSink {
         // both beforeCurrency and afterCurrency were found in CLDR.
         static const char* defaults[] = { "[:letter:]", "[:digit:]", " " };
         if (!hasBeforeCurrency || !hasAfterCurrency) {
-            for (UBool beforeCurrency = 0; beforeCurrency <= TRUE; beforeCurrency++) {
-                for (int32_t pattern = 0; pattern < UNUM_CURRENCY_SPACING_COUNT; pattern++) {
-                    dfs.setPatternForCurrencySpacing((UCurrencySpacing)pattern,
-                        beforeCurrency, UnicodeString(defaults[pattern], -1, US_INV));
-                }
+            for (int32_t pattern = 0; pattern < UNUM_CURRENCY_SPACING_COUNT; pattern++) {
+                dfs.setPatternForCurrencySpacing(static_cast<UCurrencySpacing>(pattern),
+                    false, UnicodeString(defaults[pattern], -1, US_INV));
+            }
+            for (int32_t pattern = 0; pattern < UNUM_CURRENCY_SPACING_COUNT; pattern++) {
+                dfs.setPatternForCurrencySpacing(static_cast<UCurrencySpacing>(pattern),
+                    true, UnicodeString(defaults[pattern], -1, US_INV));
             }
         }
     }
@@ -349,7 +355,6 @@ DecimalFormatSymbols::initialize(const Locale& loc, UErrorCode& status,
     UBool useLastResortData, const NumberingSystem* ns)
 {
     if (U_FAILURE(status)) { return; }
-    *validLocale = *actualLocale = 0;
 
     // First initialize all the symbols to the fallbacks for anything we can't find
     initialize();
@@ -380,12 +385,13 @@ DecimalFormatSymbols::initialize(const Locale& loc, UErrorCode& status,
     } else {
         nsName = gLatn;
     }
+    uprv_strcpy(this->nsName, nsName);
 
     // Open resource bundles
     const char* locStr = loc.getName();
-    LocalUResourceBundlePointer resource(ures_open(NULL, locStr, &status));
+    LocalUResourceBundlePointer resource(ures_open(nullptr, locStr, &status));
     LocalUResourceBundlePointer numberElementsRes(
-        ures_getByKeyWithFallback(resource.getAlias(), gNumberElements, NULL, &status));
+        ures_getByKeyWithFallback(resource.getAlias(), gNumberElements, nullptr, &status));
 
     if (U_FAILURE(status)) {
         if ( useLastResortData ) {
@@ -397,14 +403,10 @@ DecimalFormatSymbols::initialize(const Locale& loc, UErrorCode& status,
 
     // Set locale IDs
     // TODO: Is there a way to do this without depending on the resource bundle instance?
-    U_LOCALE_BASED(locBased, *this);
-    locBased.setLocaleIDs(
-        ures_getLocaleByType(
-            numberElementsRes.getAlias(),
-            ULOC_VALID_LOCALE, &status),
-        ures_getLocaleByType(
-            numberElementsRes.getAlias(),
-            ULOC_ACTUAL_LOCALE, &status));
+    actualLocale = Locale(
+        ures_getLocaleByType(numberElementsRes.getAlias(), ULOC_ACTUAL_LOCALE, &status));
+    validLocale = Locale(
+        ures_getLocaleByType(numberElementsRes.getAlias(), ULOC_VALID_LOCALE, &status));
 
     // Now load the rest of the data from the data sink.
     // Start with loading this nsName if it is not Latin.
@@ -456,7 +458,7 @@ DecimalFormatSymbols::initialize(const Locale& loc, UErrorCode& status,
 
     // Get the default currency from the currency API.
     UErrorCode internalStatus = U_ZERO_ERROR; // don't propagate failures out
-    UChar curriso[4];
+    char16_t curriso[4];
     UnicodeString tempStr;
     int32_t currisoLength = ucurr_forLocale(locStr, curriso, UPRV_LENGTHOF(curriso), &internalStatus);
     if (U_SUCCESS(internalStatus) && currisoLength == 3) {
@@ -480,43 +482,44 @@ DecimalFormatSymbols::initialize() {
      * cannot initialize a static array with class constructors.
      *  markus 2000may25
      */
-    fSymbols[kDecimalSeparatorSymbol] = (UChar)0x2e;    // '.' decimal separator
+    fSymbols[kDecimalSeparatorSymbol] = static_cast<char16_t>(0x2e); // '.' decimal separator
     fSymbols[kGroupingSeparatorSymbol].remove();        //     group (thousands) separator
-    fSymbols[kPatternSeparatorSymbol] = (UChar)0x3b;    // ';' pattern separator
-    fSymbols[kPercentSymbol] = (UChar)0x25;             // '%' percent sign
-    fSymbols[kZeroDigitSymbol] = (UChar)0x30;           // '0' native 0 digit
-    fSymbols[kOneDigitSymbol] = (UChar)0x31;            // '1' native 1 digit
-    fSymbols[kTwoDigitSymbol] = (UChar)0x32;            // '2' native 2 digit
-    fSymbols[kThreeDigitSymbol] = (UChar)0x33;          // '3' native 3 digit
-    fSymbols[kFourDigitSymbol] = (UChar)0x34;           // '4' native 4 digit
-    fSymbols[kFiveDigitSymbol] = (UChar)0x35;           // '5' native 5 digit
-    fSymbols[kSixDigitSymbol] = (UChar)0x36;            // '6' native 6 digit
-    fSymbols[kSevenDigitSymbol] = (UChar)0x37;          // '7' native 7 digit
-    fSymbols[kEightDigitSymbol] = (UChar)0x38;          // '8' native 8 digit
-    fSymbols[kNineDigitSymbol] = (UChar)0x39;           // '9' native 9 digit
-    fSymbols[kDigitSymbol] = (UChar)0x23;               // '#' pattern digit
-    fSymbols[kPlusSignSymbol] = (UChar)0x002b;          // '+' plus sign
-    fSymbols[kMinusSignSymbol] = (UChar)0x2d;           // '-' minus sign
-    fSymbols[kCurrencySymbol] = (UChar)0xa4;            // 'OX' currency symbol
-    fSymbols[kIntlCurrencySymbol].setTo(TRUE, INTL_CURRENCY_SYMBOL_STR, 2);
-    fSymbols[kMonetarySeparatorSymbol] = (UChar)0x2e;   // '.' monetary decimal separator
-    fSymbols[kExponentialSymbol] = (UChar)0x45;         // 'E' exponential
-    fSymbols[kPerMillSymbol] = (UChar)0x2030;           // '%o' per mill
-    fSymbols[kPadEscapeSymbol] = (UChar)0x2a;           // '*' pad escape symbol
-    fSymbols[kInfinitySymbol] = (UChar)0x221e;          // 'oo' infinite
-    fSymbols[kNaNSymbol] = (UChar)0xfffd;               // SUB NaN
-    fSymbols[kSignificantDigitSymbol] = (UChar)0x0040;  // '@' significant digit
+    fSymbols[kPatternSeparatorSymbol] = static_cast<char16_t>(0x3b); // ';' pattern separator
+    fSymbols[kPercentSymbol] = static_cast<char16_t>(0x25);          // '%' percent sign
+    fSymbols[kZeroDigitSymbol] = static_cast<char16_t>(0x30);        // '0' native 0 digit
+    fSymbols[kOneDigitSymbol] = static_cast<char16_t>(0x31);         // '1' native 1 digit
+    fSymbols[kTwoDigitSymbol] = static_cast<char16_t>(0x32);         // '2' native 2 digit
+    fSymbols[kThreeDigitSymbol] = static_cast<char16_t>(0x33);       // '3' native 3 digit
+    fSymbols[kFourDigitSymbol] = static_cast<char16_t>(0x34);        // '4' native 4 digit
+    fSymbols[kFiveDigitSymbol] = static_cast<char16_t>(0x35);        // '5' native 5 digit
+    fSymbols[kSixDigitSymbol] = static_cast<char16_t>(0x36);         // '6' native 6 digit
+    fSymbols[kSevenDigitSymbol] = static_cast<char16_t>(0x37);       // '7' native 7 digit
+    fSymbols[kEightDigitSymbol] = static_cast<char16_t>(0x38);       // '8' native 8 digit
+    fSymbols[kNineDigitSymbol] = static_cast<char16_t>(0x39);        // '9' native 9 digit
+    fSymbols[kDigitSymbol] = static_cast<char16_t>(0x23);            // '#' pattern digit
+    fSymbols[kPlusSignSymbol] = static_cast<char16_t>(0x002b);       // '+' plus sign
+    fSymbols[kMinusSignSymbol] = static_cast<char16_t>(0x2d);        // '-' minus sign
+    fSymbols[kCurrencySymbol] = static_cast<char16_t>(0xa4);         // 'OX' currency symbol
+    fSymbols[kIntlCurrencySymbol].setTo(true, INTL_CURRENCY_SYMBOL_STR, 2);
+    fSymbols[kMonetarySeparatorSymbol] = static_cast<char16_t>(0x2e);  // '.' monetary decimal separator
+    fSymbols[kExponentialSymbol] = static_cast<char16_t>(0x45);        // 'E' exponential
+    fSymbols[kPerMillSymbol] = static_cast<char16_t>(0x2030);          // '%o' per mill
+    fSymbols[kPadEscapeSymbol] = static_cast<char16_t>(0x2a);          // '*' pad escape symbol
+    fSymbols[kInfinitySymbol] = static_cast<char16_t>(0x221e);         // 'oo' infinite
+    fSymbols[kNaNSymbol] = static_cast<char16_t>(0xfffd);              // SUB NaN
+    fSymbols[kSignificantDigitSymbol] = static_cast<char16_t>(0x0040); // '@' significant digit
     fSymbols[kMonetaryGroupingSeparatorSymbol].remove(); // 
-    fSymbols[kExponentMultiplicationSymbol] = (UChar)0xd7; // 'x' multiplication symbol for exponents
-    fIsCustomCurrencySymbol = FALSE; 
-    fIsCustomIntlCurrencySymbol = FALSE;
+    fSymbols[kExponentMultiplicationSymbol] = static_cast<char16_t>(0xd7); // 'x' multiplication symbol for exponents
+    fSymbols[kApproximatelySignSymbol] = u'~';          // '~' approximately sign
+    fIsCustomCurrencySymbol = false; 
+    fIsCustomIntlCurrencySymbol = false;
     fCodePointZero = 0x30;
     U_ASSERT(fCodePointZero == fSymbols[kZeroDigitSymbol].char32At(0));
     currPattern = nullptr;
-
+    nsName[0] = 0;
 }
 
-void DecimalFormatSymbols::setCurrency(const UChar* currency, UErrorCode& status) {
+void DecimalFormatSymbols::setCurrency(const char16_t* currency, UErrorCode& status) {
     // TODO: If this method is made public:
     // - Adopt ICU4J behavior of not allowing currency to be null.
     // - Also verify that the length of currency is 3.
@@ -539,21 +542,21 @@ void DecimalFormatSymbols::setCurrency(const UChar* currency, UErrorCode& status
     UErrorCode localStatus = U_ZERO_ERROR;
     LocalUResourceBundlePointer rbTop(ures_open(U_ICUDATA_CURR, locale.getName(), &localStatus));
     LocalUResourceBundlePointer rb(
-        ures_getByKeyWithFallback(rbTop.getAlias(), "Currencies", NULL, &localStatus));
+        ures_getByKeyWithFallback(rbTop.getAlias(), "Currencies", nullptr, &localStatus));
     ures_getByKeyWithFallback(rb.getAlias(), cc, rb.getAlias(), &localStatus);
     if(U_SUCCESS(localStatus) && ures_getSize(rb.getAlias())>2) { // the length is 3 if more data is present
         ures_getByIndex(rb.getAlias(), 2, rb.getAlias(), &localStatus);
         int32_t currPatternLen = 0;
         currPattern =
-            ures_getStringByIndex(rb.getAlias(), (int32_t)0, &currPatternLen, &localStatus);
+            ures_getStringByIndex(rb.getAlias(), static_cast<int32_t>(0), &currPatternLen, &localStatus);
         UnicodeString decimalSep =
-            ures_getUnicodeStringByIndex(rb.getAlias(), (int32_t)1, &localStatus);
+            ures_getUnicodeStringByIndex(rb.getAlias(), static_cast<int32_t>(1), &localStatus);
         UnicodeString groupingSep =
-            ures_getUnicodeStringByIndex(rb.getAlias(), (int32_t)2, &localStatus);
+            ures_getUnicodeStringByIndex(rb.getAlias(), static_cast<int32_t>(2), &localStatus);
         if(U_SUCCESS(localStatus)){
             fSymbols[kMonetaryGroupingSeparatorSymbol] = groupingSep;
             fSymbols[kMonetarySeparatorSymbol] = decimalSep;
-            //pattern.setTo(TRUE, currPattern, currPatternLen);
+            //pattern.setTo(true, currPattern, currPatternLen);
         }
     }
     /* else An explicit currency was requested and is unknown or locale data is malformed. */
@@ -562,8 +565,7 @@ void DecimalFormatSymbols::setCurrency(const UChar* currency, UErrorCode& status
 
 Locale
 DecimalFormatSymbols::getLocale(ULocDataLocaleType type, UErrorCode& status) const {
-    U_LOCALE_BASED(locBased, *this);
-    return locBased.getLocale(type, status);
+    return LocaleBased::getLocale(validLocale, actualLocale, type, status);
 }
 
 const UnicodeString&
@@ -574,9 +576,9 @@ DecimalFormatSymbols::getPatternForCurrencySpacing(UCurrencySpacing type,
       return fNoSymbol;  // always empty.
     }
     if (beforeCurrency) {
-      return currencySpcBeforeSym[(int32_t)type];
+      return currencySpcBeforeSym[static_cast<int32_t>(type)];
     } else {
-      return currencySpcAfterSym[(int32_t)type];
+      return currencySpcAfterSym[static_cast<int32_t>(type)];
     }
 }
 
@@ -585,9 +587,9 @@ DecimalFormatSymbols::setPatternForCurrencySpacing(UCurrencySpacing type,
                                                    UBool beforeCurrency,
                                              const UnicodeString& pattern) {
   if (beforeCurrency) {
-    currencySpcBeforeSym[(int32_t)type] = pattern;
+    currencySpcBeforeSym[static_cast<int32_t>(type)] = pattern;
   } else {
-    currencySpcAfterSym[(int32_t)type] =  pattern;
+    currencySpcAfterSym[static_cast<int32_t>(type)] = pattern;
   }
 }
 U_NAMESPACE_END
