@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  ********************************************************************************
- * Copyright (C) 2003-2007, International Business Machines Corporation
+ * Copyright (C) 2003-2013, International Business Machines Corporation
  * and others. All Rights Reserved.
  ******************************************************************************
  *
@@ -26,7 +28,7 @@ U_NAMESPACE_BEGIN
 
 /**
  * <code>IslamicCalendar</code> is a subclass of <code>Calendar</code>
- * that that implements the Islamic civil and religious calendars.  It
+ * that implements the Islamic civil and religious calendars.  It
  * is used as the civil calendar in most of the Arab world and the
  * liturgical calendar of the Islamic faith worldwide.  This calendar
  * is also known as the "Hijri" calendar, since it starts at the time
@@ -51,7 +53,8 @@ U_NAMESPACE_BEGIN
  * every 30 years.  This calendar is easily calculated and thus predictable in
  * advance, so it is used as the civil calendar in a number of Arab countries.
  * This is the default behavior of a newly-created <code>IslamicCalendar</code>
- * object.
+ * object. This calendar variant is implemented in the IslamicCivilCalendar
+ * class.
  * <p>
  * The Islamic <em>religious</em> calendar, however, is based on the <em>observation</em>
  * of the crescent moon.  It is thus affected by the position at which the
@@ -65,14 +68,10 @@ U_NAMESPACE_BEGIN
  * moon's illumination, and other factors, it is possible to determine the start
  * of a lunar month with a fairly high degree of certainty.  However, these
  * calculations are extremely complicated and thus slow, so most algorithms,
- * including the one used here, are only approximations of the true astronical
+ * including the one used here, are only approximations of the true astronomical
  * calculations.  At present, the approximations used in this class are fairly
  * simplistic; they will be improved in later versions of the code.
  * <p>
- * The {@link #setCivil setCivil} method determines
- * which approach is used to determine the start of a month.  By default, the
- * fixed-cycle civil calendar is used.  However, if <code>setCivil(false)</code>
- * is called, an approximation of the true lunar calendar will be used.
  *
  * @see GregorianCalendar
  *
@@ -81,20 +80,11 @@ U_NAMESPACE_BEGIN
  * @author Steven R. Loomis
  * @internal
  */
-class IslamicCalendar : public Calendar {
+class U_I18N_API IslamicCalendar : public Calendar {
  public:
   //-------------------------------------------------------------------------
   // Constants...
   //-------------------------------------------------------------------------
-  /**
-   * Calendar type - civil or religious
-   * @internal 
-   */
-  enum ECivil {
-    ASTRONOMICAL,
-    CIVIL
-  };
-  
   /**
    * Constants for the months
    * @internal
@@ -176,7 +166,6 @@ class IslamicCalendar : public Calendar {
   }; 
 
 
-
   //-------------------------------------------------------------------------
   // Constructors...
   //-------------------------------------------------------------------------
@@ -188,16 +177,15 @@ class IslamicCalendar : public Calendar {
    * @param aLocale  The given locale.
    * @param success  Indicates the status of IslamicCalendar object construction.
    *                 Returns U_ZERO_ERROR if constructed successfully.
-   * @param beCivil  Whether the calendar should be civil (default-TRUE) or religious (FALSE)
    * @internal
    */
-  IslamicCalendar(const Locale& aLocale, UErrorCode &success, ECivil beCivil = CIVIL);
+  IslamicCalendar(const Locale& aLocale, UErrorCode &success);
 
   /**
    * Copy Constructor
    * @internal
    */
-  IslamicCalendar(const IslamicCalendar& other);
+  IslamicCalendar(const IslamicCalendar& other) = default;
 
   /**
    * Destructor.
@@ -205,41 +193,15 @@ class IslamicCalendar : public Calendar {
    */
   virtual ~IslamicCalendar();
 
-  /**
-   * Determines whether this object uses the fixed-cycle Islamic civil calendar
-   * or an approximation of the religious, astronomical calendar.
-   *
-   * @param beCivil   <code>CIVIL</code> to use the civil calendar,
-   *                  <code>ASTRONOMICAL</code> to use the astronomical calendar.
-   * @internal
-   */
-  void setCivil(ECivil beCivil, UErrorCode &status);
-    
-  /**
-   * Returns <code>true</code> if this object is using the fixed-cycle civil
-   * calendar, or <code>false</code> if using the religious, astronomical
-   * calendar.
-   * @internal
-   */
-  UBool isCivil();
-
-
-  // TODO: copy c'tor, etc
-
   // clone
-  virtual Calendar* clone() const;
+  virtual IslamicCalendar* clone() const override;
 
- private:
-  /**
-   * Determine whether a year is a leap year in the Islamic civil calendar
-   */
-  static UBool civilLeapYear(int32_t year);
-    
+ protected:
   /**
    * Return the day # on which the given year starts.  Days are counted
    * from the Hijri epoch, origin 0.
    */
-  int32_t yearStart(int32_t year);
+  virtual int64_t yearStart(int32_t year, UErrorCode& status) const;
 
   /**
    * Return the day # on which the given month starts.  Days are counted
@@ -248,39 +210,8 @@ class IslamicCalendar : public Calendar {
    * @param year  The hijri year
    * @param year  The hijri month, 0-based
    */
-  int32_t monthStart(int32_t year, int32_t month) const;
-    
-  /**
-   * Find the day number on which a particular month of the true/lunar
-   * Islamic calendar starts.
-   *
-   * @param month The month in question, origin 0 from the Hijri epoch
-   *
-   * @return The day number on which the given month starts.
-   */
-  int32_t trueMonthStart(int32_t month) const;
+  virtual int64_t monthStart(int32_t year, int32_t month, UErrorCode& status) const;
 
-  /**
-   * Return the "age" of the moon at the given time; this is the difference
-   * in ecliptic latitude between the moon and the sun.  This method simply
-   * calls CalendarAstronomer.moonAge, converts to degrees, 
-   * and adjusts the resultto be in the range [-180, 180].
-   *
-   * @param time  The time at which the moon's age is desired,
-   *              in millis since 1/1/1970.
-   */
-  static double moonAge(UDate time);
-
-  //-------------------------------------------------------------------------
-  // Internal data....
-  //
-    
-  /**
-   * <code>CIVIL</code> if this object uses the fixed-cycle Islamic civil calendar,
-   * and <code>ASTRONOMICAL</code> if it approximates the true religious calendar using
-   * astronomical calculations for the time of the new moon.
-   */
-  ECivil civil;
 
   //----------------------------------------------------------------------
   // Calendar framework
@@ -289,7 +220,7 @@ class IslamicCalendar : public Calendar {
   /**
    * @internal
    */
-  virtual int32_t handleGetLimit(UCalendarDateFields field, ELimitType limitType) const;
+  virtual int32_t handleGetLimit(UCalendarDateFields field, ELimitType limitType) const override;
   
   /**
    * Return the length (in days) of the given month.
@@ -298,13 +229,13 @@ class IslamicCalendar : public Calendar {
    * @param year  The hijri month, 0-based
    * @internal
    */
-  virtual int32_t handleGetMonthLength(int32_t extendedYear, int32_t month) const;
+  virtual int32_t handleGetMonthLength(int32_t extendedYear, int32_t month, UErrorCode& status) const override;
   
   /**
    * Return the number of days in the given Islamic year
    * @internal
    */
-  virtual int32_t handleGetYearLength(int32_t extendedYear) const;
+  virtual int32_t handleGetYearLength(int32_t extendedYear, UErrorCode& status) const override;
     
   //-------------------------------------------------------------------------
   // Functions for converting from field values to milliseconds....
@@ -314,7 +245,7 @@ class IslamicCalendar : public Calendar {
   /**
    * @internal
    */
-  virtual int32_t handleComputeMonthStart(int32_t eyear, int32_t month, UBool useMonth) const;
+  virtual int64_t handleComputeMonthStart(int32_t eyear, int32_t month, UBool useMonth, UErrorCode& status) const override;
 
   //-------------------------------------------------------------------------
   // Functions for converting from milliseconds to field values
@@ -323,7 +254,7 @@ class IslamicCalendar : public Calendar {
   /**
    * @internal
    */
-  virtual int32_t handleGetExtendedYear();
+  virtual int32_t handleGetExtendedYear(UErrorCode& status) override;
 
   /**
    * Override Calendar to compute several fields specific to the Islamic
@@ -341,7 +272,13 @@ class IslamicCalendar : public Calendar {
    * calendar equivalents for the given Julian day.
    * @internal
    */
-  virtual void handleComputeFields(int32_t julianDay, UErrorCode &status);
+  virtual void handleComputeFields(int32_t julianDay, UErrorCode &status) override;
+
+  /**
+   * Return the epoc.
+   * @internal
+   */
+  virtual int32_t getEpoc() const;
 
   // UObject stuff
  public: 
@@ -350,7 +287,7 @@ class IslamicCalendar : public Calendar {
    *           same class ID. Objects of other classes have different class IDs.
    * @internal
    */
-  virtual UClassID getDynamicClassID(void) const;
+  virtual UClassID getDynamicClassID() const override;
 
   /**
    * Return the class ID for this class. This is useful only for comparing to a return
@@ -363,101 +300,417 @@ class IslamicCalendar : public Calendar {
    * @return   The class ID for all objects of this class.
    * @internal
    */
-  U_I18N_API static UClassID U_EXPORT2 getStaticClassID(void);
+  /*U_I18N_API*/ static UClassID U_EXPORT2 getStaticClassID();
 
   /**
-   * return the calendar type, "buddhist".
+   * return the calendar type, "islamic".
    *
    * @return calendar type
    * @internal
    */
-  virtual const char * getType() const;
+  virtual const char * getType() const override;
+
+  /**
+   * @return      The related Gregorian year; will be obtained by modifying the value
+   *              obtained by get from UCAL_EXTENDED_YEAR field
+   * @internal
+   */
+  virtual int32_t getRelatedYear(UErrorCode &status) const override;
+
+  /**
+   * @param year  The related Gregorian year to set; will be modified as necessary then
+   *              set in UCAL_EXTENDED_YEAR field
+   * @internal
+   */
+  virtual void setRelatedYear(int32_t year) override;
+
+  /**
+   * Returns true if the date is in a leap year.
+   *
+   * @param status        ICU Error Code
+   * @return       True if the date in the fields is in a Temporal proposal
+   *               defined leap year. False otherwise.
+   */
+  virtual bool inTemporalLeapYear(UErrorCode &status) const override;
 
  private:
-  IslamicCalendar(); // default constructor not implemented
+  IslamicCalendar() = delete; // default constructor not implemented
 
   // Default century.
  protected:
 
+  DECLARE_OVERRIDE_SYSTEM_DEFAULT_CENTURY
+};
+
+/*
+ * IslamicCivilCalendar is one of the two main variants of the Islamic calendar.
+ * The <em>civil</em> calendar, which uses a fixed cycle of alternating 29-
+ * and 30-day months, with a leap day added to the last month of 11 out of
+ * every 30 years.  This calendar is easily calculated and thus predictable in
+ * advance, so it is used as the civil calendar in a number of Arab countries.
+ * This calendar is referring as "Islamic calendar, tabular (intercalary years
+ * [2,5,7,10,13,16,18,21,24,26,29]- civil epoch" in CLDR.
+ */
+class U_I18N_API IslamicCivilCalendar : public IslamicCalendar {
+ public:
   /**
-   * (Overrides Calendar) Return true if the current date for this Calendar is in
-   * Daylight Savings Time. Recognizes DST_OFFSET, if it is set.
+   * Constructs an IslamicCivilCalendar based on the current time in the default time zone
+   * with the given locale.
    *
-   * @param status Fill-in parameter which receives the status of this operation.
-   * @return   True if the current date for this Calendar is in Daylight Savings Time,
-   *           false, otherwise.
+   * @param aLocale  The given locale.
+   * @param success  Indicates the status of IslamicCivilCalendar object construction.
+   *                 Returns U_ZERO_ERROR if constructed successfully.
    * @internal
    */
-  virtual UBool inDaylightTime(UErrorCode& status) const;
-
+  IslamicCivilCalendar(const Locale& aLocale, UErrorCode &success);
 
   /**
-   * Returns TRUE because the Islamic Calendar does have a default century
+   * Copy Constructor
    * @internal
    */
-  virtual UBool haveDefaultCentury() const;
+  IslamicCivilCalendar(const IslamicCivilCalendar& other) = default;
 
   /**
-   * Returns the date of the start of the default century
-   * @return start of century - in milliseconds since epoch, 1970
+   * Destructor.
    * @internal
    */
-  virtual UDate defaultCenturyStart() const;
+  virtual ~IslamicCivilCalendar();
+
+  // clone
+  virtual IslamicCivilCalendar* clone() const override;
 
   /**
-   * Returns the year in which the default century begins
+   * @return   The class ID for this object. All objects of a given class have the
+   *           same class ID. Objects of other classes have different class IDs.
    * @internal
    */
-  virtual int32_t defaultCenturyStartYear() const;
-
- private: // default century stuff.
-  /**
-   * The system maintains a static default century start date.  This is initialized
-   * the first time it is used.  Before then, it is set to SYSTEM_DEFAULT_CENTURY to
-   * indicate an uninitialized state.  Once the system default century date and year
-   * are set, they do not change.
-   */
-  static UDate         fgSystemDefaultCenturyStart;
+  virtual UClassID getDynamicClassID() const override;
 
   /**
-   * See documentation for systemDefaultCenturyStart.
+   * Return the class ID for this class. This is useful only for comparing to a return
+   * value from getDynamicClassID(). For example:
+   *
+   *      Base* polymorphic_pointer = createPolymorphicObject();
+   *      if (polymorphic_pointer->getDynamicClassID() ==
+   *          Derived::getStaticClassID()) ...
+   *
+   * @return   The class ID for all objects of this class.
+   * @internal
    */
-  static int32_t          fgSystemDefaultCenturyStartYear;
+  static UClassID U_EXPORT2 getStaticClassID();
 
   /**
-   * Default value that indicates the defaultCenturyStartYear is unitialized
+   * return the calendar type, "islamic-civil".
+   *
+   * @return calendar type
+   * @internal
    */
-  static const int32_t    fgSystemDefaultCenturyYear;
+  virtual const char * getType() const override;
+
+ protected:
+  /**
+   * Return the day # on which the given year starts.  Days are counted
+   * from the Hijri epoch, origin 0.
+   * @internal
+   */
+  virtual int64_t yearStart(int32_t year, UErrorCode& status) const override;
 
   /**
-   * start of default century, as a date
+   * Return the day # on which the given month starts.  Days are counted
+   * from the Hijri epoch, origin 0.
+   *
+   * @param year  The hijri year
+   * @param year  The hijri month, 0-based
+   * @internal
    */
-  static const UDate        fgSystemDefaultCentury;
+  virtual int64_t monthStart(int32_t year, int32_t month, UErrorCode& status) const override;
 
   /**
-   * Returns the beginning date of the 100-year window that dates 
-   * with 2-digit years are considered to fall within.
+   * Return the length (in days) of the given month.
+   *
+   * @param year  The hijri year
+   * @param year  The hijri month, 0-based
+   * @internal
    */
-  UDate         internalGetDefaultCenturyStart(void) const;
+  virtual int32_t handleGetMonthLength(int32_t extendedYear, int32_t month, UErrorCode& status) const override;
 
   /**
-   * Returns the first year of the 100-year window that dates with 
-   * 2-digit years are considered to fall within.
+   * Return the number of days in the given Islamic year
+   * @internal
    */
-  int32_t          internalGetDefaultCenturyStartYear(void) const;
+  virtual int32_t handleGetYearLength(int32_t extendedYear, UErrorCode& status) const override;
 
   /**
-   * Initializes the 100-year window that dates with 2-digit years
-   * are considered to fall within so that its start date is 80 years
-   * before the current time.
+   * Override Calendar to compute several fields specific to the Islamic
+   * calendar system.  These are:
+   *
+   * <ul><li>ERA
+   * <li>YEAR
+   * <li>MONTH
+   * <li>DAY_OF_MONTH
+   * <li>DAY_OF_YEAR
+   * <li>EXTENDED_YEAR</ul>
+   * 
+   * The DAY_OF_WEEK and DOW_LOCAL fields are already set when this
+   * method is called. The getGregorianXxx() methods return Gregorian
+   * calendar equivalents for the given Julian day.
+   * @internal
    */
-  static void  initializeSystemDefaultCentury(void);
+  virtual void handleComputeFields(int32_t julianDay, UErrorCode &status) override;
+};
+
+/*
+ * IslamicTBLACalendar calendar.
+ * This is a subclass of IslamicCivilCalendar. The only differences in the
+ * calendar math is it uses different epoch.
+ * This calendar is referring as "Islamic calendar, tabular (intercalary years
+ * [2,5,7,10,13,16,18,21,24,26,29] - astronomical epoch" in CLDR.
+ */
+class U_I18N_API IslamicTBLACalendar : public IslamicCivilCalendar {
+ public:
+  /**
+   * Constructs an IslamicTBLACalendar based on the current time in the default time zone
+   * with the given locale.
+   *
+   * @param aLocale  The given locale.
+   * @param success  Indicates the status of IslamicTBLACalendar object construction.
+   *                 Returns U_ZERO_ERROR if constructed successfully.
+   * @internal
+   */
+  IslamicTBLACalendar(const Locale& aLocale, UErrorCode &success);
+
+  /**
+   * Copy Constructor
+   * @internal
+   */
+  IslamicTBLACalendar(const IslamicTBLACalendar& other) = default;
+
+  /**
+   * Destructor.
+   * @internal
+   */
+  virtual ~IslamicTBLACalendar();
+
+  /**
+   * @return   The class ID for this object. All objects of a given class have the
+   *           same class ID. Objects of other classes have different class IDs.
+   * @internal
+   */
+  virtual UClassID getDynamicClassID() const override;
+
+  /**
+   * Return the class ID for this class. This is useful only for comparing to a return
+   * value from getDynamicClassID(). For example:
+   *
+   *      Base* polymorphic_pointer = createPolymorphicObject();
+   *      if (polymorphic_pointer->getDynamicClassID() ==
+   *          Derived::getStaticClassID()) ...
+   *
+   * @return   The class ID for all objects of this class.
+   * @internal
+   */
+  static UClassID U_EXPORT2 getStaticClassID();
+
+  /**
+   * return the calendar type, "islamic-tbla".
+   *
+   * @return calendar type
+   * @internal
+   */
+  virtual const char * getType() const override;
+
+  // clone
+  virtual IslamicTBLACalendar* clone() const override;
+
+ protected:
+  /**
+   * Return the epoc.
+   * @internal
+   */
+  virtual int32_t getEpoc() const override;
+};
+
+/*
+ * IslamicUmalquraCalendar
+ * This calendar is referred as "Islamic calendar, Umm al-Qura" in CLDR.
+ */
+class U_I18N_API IslamicUmalquraCalendar : public IslamicCivilCalendar {
+ public:
+  /**
+   * Constructs an IslamicUmalquraCalendar based on the current time in the default time zone
+   * with the given locale.
+   *
+   * @param aLocale  The given locale.
+   * @param success  Indicates the status of IslamicUmalquraCalendar object construction.
+   *                 Returns U_ZERO_ERROR if constructed successfully.
+   * @internal
+   */
+  IslamicUmalquraCalendar(const Locale& aLocale, UErrorCode &success);
+
+  /**
+   * Copy Constructor
+   * @internal
+   */
+  IslamicUmalquraCalendar(const IslamicUmalquraCalendar& other) = default;
+
+  /**
+   * Destructor.
+   * @internal
+   */
+  virtual ~IslamicUmalquraCalendar();
+
+  /**
+   * @return   The class ID for this object. All objects of a given class have the
+   *           same class ID. Objects of other classes have different class IDs.
+   * @internal
+   */
+  virtual UClassID getDynamicClassID() const override;
+
+  /**
+   * Return the class ID for this class. This is useful only for comparing to a return
+   * value from getDynamicClassID(). For example:
+   *
+   *      Base* polymorphic_pointer = createPolymorphicObject();
+   *      if (polymorphic_pointer->getDynamicClassID() ==
+   *          Derived::getStaticClassID()) ...
+   *
+   * @return   The class ID for all objects of this class.
+   * @internal
+   */
+  static UClassID U_EXPORT2 getStaticClassID();
+
+  /**
+   * return the calendar type, "islamic-umalqura".
+   *
+   * @return calendar type
+   * @internal
+   */
+  virtual const char * getType() const override;
+
+  // clone
+  virtual IslamicUmalquraCalendar* clone() const override;
+
+ protected:
+  /**
+   * Return the day # on which the given year starts.  Days are counted
+   * from the Hijri epoch, origin 0.
+   * @internal
+   */
+  virtual int64_t yearStart(int32_t year, UErrorCode& status) const override;
+
+  /**
+   * Return the day # on which the given month starts.  Days are counted
+   * from the Hijri epoch, origin 0.
+   *
+   * @param year  The hijri year
+   * @param year  The hijri month, 0-based
+   * @internal
+   */
+  virtual int64_t monthStart(int32_t year, int32_t month, UErrorCode& status) const override;
+
+  /**
+   * Return the length (in days) of the given month.
+   *
+   * @param year  The hijri year
+   * @param year  The hijri month, 0-based
+   * @internal
+   */
+  virtual int32_t handleGetMonthLength(int32_t extendedYear, int32_t month, UErrorCode& status) const override;
+
+  /**
+   * Return the number of days in the given Islamic year
+   * @internal
+   */
+  virtual int32_t handleGetYearLength(int32_t extendedYear, UErrorCode& status) const override;
+
+  /**
+   * Override Calendar to compute several fields specific to the Islamic
+   * calendar system.  These are:
+   *
+   * <ul><li>ERA
+   * <li>YEAR
+   * <li>MONTH
+   * <li>DAY_OF_MONTH
+   * <li>DAY_OF_YEAR
+   * <li>EXTENDED_YEAR</ul>
+   * 
+   * The DAY_OF_WEEK and DOW_LOCAL fields are already set when this
+   * method is called. The getGregorianXxx() methods return Gregorian
+   * calendar equivalents for the given Julian day.
+   * @internal
+   */
+  virtual void handleComputeFields(int32_t julianDay, UErrorCode &status) override;
+
+ private:
+  virtual int32_t yearLength(int32_t extendedYear, UErrorCode& status) const;
+};
+
+
+/*
+ * IslamicRGSACalendar
+ * Islamic calendar, Saudi Arabia sighting. Since the calendar depends on the
+ * sighting, it is impossible to implement by algorithm ahead of time. It is
+ * currently identical to IslamicCalendar except the getType will return
+ * "islamic-rgsa".
+ */
+class U_I18N_API IslamicRGSACalendar : public IslamicCalendar {
+ public:
+  /**
+   * Constructs an IslamicRGSACalendar based on the current time in the default time zone
+   * with the given locale.
+   *
+   * @param aLocale  The given locale.
+   * @param success  Indicates the status of IslamicRGSACalendar object construction.
+   *                 Returns U_ZERO_ERROR if constructed successfully.
+   * @internal
+   */
+  IslamicRGSACalendar(const Locale& aLocale, UErrorCode &success);
+
+  /**
+   * Copy Constructor
+   * @internal
+   */
+  IslamicRGSACalendar(const IslamicRGSACalendar& other) = default;
+
+  /**
+   * Destructor.
+   * @internal
+   */
+  virtual ~IslamicRGSACalendar();
+
+  /**
+   * @return   The class ID for this object. All objects of a given class have the
+   *           same class ID. Objects of other classes have different class IDs.
+   * @internal
+   */
+  virtual UClassID getDynamicClassID() const override;
+
+  /**
+   * Return the class ID for this class. This is useful only for comparing to a return
+   * value from getDynamicClassID(). For example:
+   *
+   *      Base* polymorphic_pointer = createPolymorphicObject();
+   *      if (polymorphic_pointer->getDynamicClassID() ==
+   *          Derived::getStaticClassID()) ...
+   *
+   * @return   The class ID for all objects of this class.
+   * @internal
+   */
+  static UClassID U_EXPORT2 getStaticClassID();
+
+  /**
+   * return the calendar type, "islamic-rgsa".
+   *
+   * @return calendar type
+   * @internal
+   */
+  virtual const char * getType() const override;
+
+  // clone
+  virtual IslamicRGSACalendar* clone() const override;
 };
 
 U_NAMESPACE_END
 
 #endif
 #endif
-
-
-
