@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 //
 //   Copyright (C) 2002-2015 International Business Machines Corporation
 //   and others. All rights reserved.
@@ -35,7 +37,7 @@ U_NAMESPACE_BEGIN
 #define REGEX_DUMP_DEBUG
 #define REGEX_RUN_DEBUG
 
-//  End of #defines inteded to be directly set.
+//  End of #defines intended to be directly set.
 
 #include <stdio.h>
 #endif
@@ -121,7 +123,7 @@ enum {
                                //     saved input position, FAIL rather than taking
                                //     the JMP
      URX_LA_START      = 37,   // Starting a LookAround expression.
-                               //   Save InputPos and SP in static data.
+                               //   Save InputPos, SP and active region in static data.
                                //   Operand:  Static data offset for the save
      URX_LA_END        = 38,   // Ending a Lookaround expression.
                                //   Restore InputPos and Stack to saved values.
@@ -138,7 +140,7 @@ enum {
      URX_DOLLAR_M      = 42,   // $ in multi-line mode.
      URX_CARET_M       = 43,   // ^ in multi-line mode.
      URX_LB_START      = 44,   // LookBehind Start.
-                               //   Paramater is data location
+                               //   Parameter is data location
      URX_LB_CONT       = 45,   // LookBehind Continue.
                                //   Param 0:  the data location
                                //   Param 1:  The minimum length of the look-behind match
@@ -341,7 +343,7 @@ inline void  Regex8BitSet::add(UChar32 c) {
 }
 
 inline void Regex8BitSet::init(const UnicodeSet *s) {
-    if (s != NULL) {
+    if (s != nullptr) {
         for (int32_t i=0; i<=255; i++) {
             if (s->contains(i)) {
                 this->add(i);
@@ -351,7 +353,9 @@ inline void Regex8BitSet::init(const UnicodeSet *s) {
 }
 
 inline void Regex8BitSet::operator = (const Regex8BitSet &s) {
-   uprv_memcpy(d, s.d, sizeof(d));
+    if (this != &s) {
+        uprv_memcpy(d, s.d, sizeof(d));
+    }     
 }
 
 
@@ -369,41 +373,39 @@ class CaseFoldingUTextIterator: public UMemory {
 
         UBool   inExpansion();    // True if last char returned from next() and the
                                   //  next to be returned both originated from a string
-                                  //  folding of the same code point from the orignal UText.
+                                  //  folding of the same code point from the original UText.
       private:
         UText             &fUText;
-        const  UCaseProps *fcsp;
-        const  UChar      *fFoldChars;
+        const  char16_t   *fFoldChars;
         int32_t            fFoldLength;
         int32_t            fFoldIndex;
 
 };
 
 
-// Case folded UChar * string iterator.
-//  Wraps a UChar  *, provides a case-folded enumeration over its contents.
+// Case folded char16_t * string iterator.
+//  Wraps a char16_t  *, provides a case-folded enumeration over its contents.
 //  Used in implementing case insensitive matching constructs.
 //  Implementation in rematch.cpp
 
 class CaseFoldingUCharIterator: public UMemory {
       public:
-        CaseFoldingUCharIterator(const UChar *chars, int64_t start, int64_t limit);
+        CaseFoldingUCharIterator(const char16_t *chars, int64_t start, int64_t limit);
         ~CaseFoldingUCharIterator();
 
         UChar32 next();           // Next case folded character
 
         UBool   inExpansion();    // True if last char returned from next() and the
                                   //  next to be returned both originated from a string
-                                  //  folding of the same code point from the orignal UText.
+                                  //  folding of the same code point from the original UText.
 
         int64_t  getIndex();      // Return the current input buffer index.
 
       private:
-        const  UChar      *fChars;
+        const  char16_t   *fChars;
         int64_t            fIndex;
         int64_t            fLimit;
-        const  UCaseProps *fcsp;
-        const  UChar      *fFoldChars;
+        const  char16_t   *fFoldChars;
         int32_t            fFoldLength;
         int32_t            fFoldIndex;
 
