@@ -199,7 +199,7 @@ public:
     virtual UnicodeString& toPattern(UnicodeString& result, UErrorCode& status) const;
 
     /**
-     * Get the date pattern for the the date formatter associated with this object.
+     * Get the date pattern for the date formatter associated with this object.
      * @param result Output param to receive the date pattern.
      * @return       A reference to 'result'.
      * @internal ICU 4.2 technology preview
@@ -207,7 +207,7 @@ public:
     virtual UnicodeString& toPatternDate(UnicodeString& result, UErrorCode& status) const;
 
     /**
-     * Get the time pattern for the the time formatter associated with this object.
+     * Get the time pattern for the time formatter associated with this object.
      * @param result Output param to receive the time pattern.
      * @return       A reference to 'result'.
      * @internal ICU 4.2 technology preview
@@ -233,7 +233,7 @@ public:
      * with this date-time formatter.
      * @internal ICU 4.8
      */
-    virtual const DateFormatSymbols* getDateFormatSymbols(void) const;
+    virtual const DateFormatSymbols* getDateFormatSymbols() const;
 
     /**
      * Set a particular UDisplayContext value in the formatter, such as
@@ -273,9 +273,9 @@ private:
      * Get the string at a specific offset.
      * @param day day offset ( -1, 0, 1, etc.. )
      * @param len on output, length of string.
-     * @return the string, or NULL if none at that location.
+     * @return the string, or nullptr if none at that location.
      */
-    const UChar *getStringForDay(int32_t day, int32_t &len, UErrorCode &status) const;
+    const char16_t *getStringForDay(int32_t day, int32_t &len, UErrorCode &status) const;
 
     /**
      * Load the Date string array
@@ -294,7 +294,7 @@ private:
 
     /**
      * initializes fCalendar from parameters.  Returns fCalendar as a convenience.
-     * @param adoptZone  Zone to be adopted, or NULL for TimeZone::createDefault().
+     * @param adoptZone  Zone to be adopted, or nullptr for TimeZone::createDefault().
      * @param locale Locale of the calendar
      * @param status Error code
      * @return the newly constructed fCalendar
@@ -314,7 +314,7 @@ public:
      * @return          The class ID for all objects of this class.
      * @internal ICU 3.8
      */
-    U_I18N_API static UClassID U_EXPORT2 getStaticClassID(void);
+    U_I18N_API static UClassID U_EXPORT2 getStaticClassID();
 
     /**
      * Returns a unique class ID POLYMORPHICALLY. Pure virtual override. This
@@ -327,7 +327,7 @@ public:
      *                  other classes have different class IDs.
      * @internal ICU 3.8
      */
-    virtual UClassID getDynamicClassID(void) const override;
+    virtual UClassID getDynamicClassID() const override;
 };
 
 
