@@ -28,7 +28,7 @@ U_NAMESPACE_BEGIN
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(TitlecaseTransliterator)
 
 TitlecaseTransliterator::TitlecaseTransliterator() :
-    CaseMapTransliterator(UNICODE_STRING("Any-Title", 9), NULL)
+    CaseMapTransliterator(UNICODE_STRING("Any-Title", 9), nullptr)
 {
     // Need to look back 2 characters in the case of "can't"
     setMaximumContextLength(2);
@@ -117,7 +117,7 @@ void TitlecaseTransliterator::handleTransliterate(
     csc.limit = offsets.contextLimit;
 
     UnicodeString tmp;
-    const UChar *s;
+    const char16_t *s;
     int32_t textPos, delta, result;
 
     for(textPos=offsets.start; textPos<offsets.limit;) {
@@ -132,7 +132,7 @@ void TitlecaseTransliterator::handleTransliterate(
             } else {
                 result=ucase_toFullLower(c, utrans_rep_caseContextIterator, &csc, &s, UCASE_LOC_ROOT);
             }
-            doTitle = (UBool)(type==0); // doTitle=isUncased
+            doTitle = static_cast<UBool>(type == 0); // doTitle=isUncased
 
             if(csc.b1 && isIncremental) {
                 // fMap() tried to look beyond the context limit
