@@ -21,10 +21,10 @@
 #include "unicode/strenum.h"
 #include "unicode/vtzone.h"
 
-#include "bytesinkutil.h"
 #include "charstr.h"
 #include "cmemory.h"
 #include "cstring.h"
+#include "fixedstring.h"
 #include "mutex.h"
 #include "uhash.h"
 #include "uassert.h"
@@ -46,20 +46,20 @@ static const char gZoneStrings[]                = "zoneStrings";
 static const char gRegionFormatTag[]            = "regionFormat";
 static const char gFallbackFormatTag[]          = "fallbackFormat";
 
-static const UChar gEmpty[]                     = {0x00};
+static const char16_t gEmpty[]                     = {0x00};
 
-static const UChar gDefRegionPattern[]          = {0x7B, 0x30, 0x7D, 0x00}; // "{0}"
-static const UChar gDefFallbackPattern[]        = {0x7B, 0x31, 0x7D, 0x20, 0x28, 0x7B, 0x30, 0x7D, 0x29, 0x00}; // "{1} ({0})"
+static const char16_t gDefRegionPattern[]          = {0x7B, 0x30, 0x7D, 0x00}; // "{0}"
+static const char16_t gDefFallbackPattern[]        = {0x7B, 0x31, 0x7D, 0x20, 0x28, 0x7B, 0x30, 0x7D, 0x29, 0x00}; // "{1} ({0})"
 
-static const double kDstCheckRange      = (double)184*U_MILLIS_PER_DAY;
+static const double kDstCheckRange = static_cast<double>(184) * U_MILLIS_PER_DAY;
 
 
 
 U_CDECL_BEGIN
 
 typedef struct PartialLocationKey {
-    const UChar* tzID;
-    const UChar* mzID;
+    const char16_t* tzID;
+    const char16_t* mzID;
     UBool isLong;
 } PartialLocationKey;
 
@@ -71,10 +71,10 @@ hashPartialLocationKey(const UHashTok key) {
     // <tzID>&<mzID>#[L|S]
     PartialLocationKey *p = (PartialLocationKey *)key.pointer;
     UnicodeString str(p->tzID);
-    str.append((UChar)0x26)
+    str.append((char16_t)0x26)
         .append(p->mzID, -1)
-        .append((UChar)0x23)
-        .append((UChar)(p->isLong ? 0x4C : 0x53));
+        .append((char16_t)0x23)
+        .append((char16_t)(p->isLong ? 0x4C : 0x53));
     return str.hashCode();
 }
 
@@ -89,7 +89,7 @@ comparePartialLocationKey(const UHashTok key1, const UHashTok key2) {
     if (p1 == p2) {
         return true;
     }
-    if (p1 == NULL || p2 == NULL) {
+    if (p1 == nullptr || p2 == nullptr) {
         return false;
     }
     // We just check identity of tzID/mzID
@@ -109,7 +109,7 @@ deleteGNameInfo(void *obj) {
  */
 typedef struct GNameInfo {
     UTimeZoneGenericNameType    type;
-    const UChar*                tzID;
+    const char16_t*                tzID;
 } ZNameInfo;
 
 /**
@@ -145,14 +145,12 @@ TimeZoneGenericNameMatchInfo::TimeZoneGenericNameMatchInfo(UVector* matches)
 }
 
 TimeZoneGenericNameMatchInfo::~TimeZoneGenericNameMatchInfo() {
-    if (fMatches != NULL) {
-        delete fMatches;
-    }
+    delete fMatches;
 }
 
 int32_t
 TimeZoneGenericNameMatchInfo::size() const {
-    if (fMatches == NULL) {
+    if (fMatches == nullptr) {
         return 0;
     }
     return fMatches->size();
@@ -160,8 +158,8 @@ TimeZoneGenericNameMatchInfo::size() const {
 
 UTimeZoneGenericNameType
 TimeZoneGenericNameMatchInfo::getGenericNameType(int32_t index) const {
-    GMatchInfo *minfo = (GMatchInfo *)fMatches->elementAt(index);
-    if (minfo != NULL) {
+    GMatchInfo* minfo = static_cast<GMatchInfo*>(fMatches->elementAt(index));
+    if (minfo != nullptr) {
         return static_cast<UTimeZoneGenericNameType>(minfo->gnameInfo->type);
     }
     return UTZGNM_UNKNOWN;
@@ -169,8 +167,8 @@ TimeZoneGenericNameMatchInfo::getGenericNameType(int32_t index) const {
 
 int32_t
 TimeZoneGenericNameMatchInfo::getMatchLength(int32_t index) const {
-    ZMatchInfo *minfo = (ZMatchInfo *)fMatches->elementAt(index);
-    if (minfo != NULL) {
+    ZMatchInfo* minfo = static_cast<ZMatchInfo*>(fMatches->elementAt(index));
+    if (minfo != nullptr) {
         return minfo->matchLength;
     }
     return -1;
@@ -178,8 +176,8 @@ TimeZoneGenericNameMatchInfo::getMatchLength(int32_t index) const {
 
 UnicodeString&
 TimeZoneGenericNameMatchInfo::getTimeZoneID(int32_t index, UnicodeString& tzID) const {
-    GMatchInfo *minfo = (GMatchInfo *)fMatches->elementAt(index);
-    if (minfo != NULL && minfo->gnameInfo->tzID != NULL) {
+    GMatchInfo* minfo = static_cast<GMatchInfo*>(fMatches->elementAt(index));
+    if (minfo != nullptr && minfo->gnameInfo->tzID != nullptr) {
         tzID.setTo(true, minfo->gnameInfo->tzID, -1);
     } else {
         tzID.setToBogus();
@@ -205,13 +203,11 @@ private:
 };
 
 GNameSearchHandler::GNameSearchHandler(uint32_t types)
-: fTypes(types), fResults(NULL), fMaxMatchLen(0) {
+: fTypes(types), fResults(nullptr), fMaxMatchLen(0) {
 }
 
 GNameSearchHandler::~GNameSearchHandler() {
-    if (fResults != NULL) {
-        delete fResults;
-    }
+    delete fResults;
 }
 
 UBool
@@ -223,20 +219,20 @@ GNameSearchHandler::handleMatch(int32_t matchLength, const CharacterNode *node,
         int32_t valuesCount = node->countValues();
         for (int32_t i = 0; i < valuesCount; i++) {
             GNameInfo *nameinfo = (ZNameInfo *)node->getValue(i);
-            if (nameinfo == NULL) {
+            if (nameinfo == nullptr) {
                 break;
             }
             if ((nameinfo->type & fTypes) != 0) {
                 // matches a requested type
-                if (fResults == NULL) {
-                    LocalPointer<UVector> lpResults(new UVector(uprv_free, NULL, status), status);
+                if (fResults == nullptr) {
+                    LocalPointer<UVector> lpResults(new UVector(uprv_free, nullptr, status), status);
                     if (U_FAILURE(status)) {
                         return false;
                     }
                     fResults = lpResults.orphan();
                 }
-                GMatchInfo *gmatch = (GMatchInfo *)uprv_malloc(sizeof(GMatchInfo));
-                if (gmatch == NULL) {
+                GMatchInfo* gmatch = static_cast<GMatchInfo*>(uprv_malloc(sizeof(GMatchInfo)));
+                if (gmatch == nullptr) {
                     status = U_MEMORY_ALLOCATION_ERROR;
                     return false;
                 }
@@ -264,7 +260,7 @@ GNameSearchHandler::getMatches(int32_t& maxMatchLen) {
     maxMatchLen = fMaxMatchLen;
 
     // reset
-    fResults = NULL;
+    fResults = nullptr;
     fMaxMatchLen = 0;
     return results;
 }
@@ -299,14 +295,14 @@ private:
     TextTrieMap fGNamesTrie;
     UBool fGNamesTrieFullyLoaded;
 
-    char fTargetRegion[ULOC_COUNTRY_CAPACITY];
+    FixedString fTargetRegion;
 
     void initialize(const Locale& locale, UErrorCode& status);
     void cleanup();
 
     void loadStrings(const UnicodeString& tzCanonicalID);
 
-    const UChar* getGenericLocationName(const UnicodeString& tzCanonicalID);
+    const char16_t* getGenericLocationName(const UnicodeString& tzCanonicalID);
 
     UnicodeString& formatGenericNonLocationName(const TimeZone& tz, UTimeZoneGenericNameType type,
                         UDate date, UnicodeString& name) const;
@@ -315,7 +311,7 @@ private:
                         const UnicodeString& mzID, UBool isLong, const UnicodeString& mzDisplayName,
                         UnicodeString& name) const;
 
-    const UChar* getPartialLocationName(const UnicodeString& tzCanonicalID,
+    const char16_t* getPartialLocationName(const UnicodeString& tzCanonicalID,
                         const UnicodeString& mzID, UBool isLong, const UnicodeString& mzDisplayName);
 
     TimeZoneGenericNameMatchInfo* findLocal(const UnicodeString& text, int32_t start, uint32_t types, UErrorCode& status) const;
@@ -333,13 +329,14 @@ private:
 // ---------------------------------------------------
 TZGNCore::TZGNCore(const Locale& locale, UErrorCode& status)
 : fLocale(locale),
-  fTimeZoneNames(NULL),
-  fLocationNamesMap(NULL),
-  fPartialLocationNamesMap(NULL),
-  fLocaleDisplayNames(NULL),
+  fTimeZoneNames(nullptr),
+  fLocationNamesMap(nullptr),
+  fPartialLocationNamesMap(nullptr),
+  fLocaleDisplayNames(nullptr),
   fStringPool(status),
   fGNamesTrie(true, deleteGNameInfo),
-  fGNamesTrieFullyLoaded(false) {
+  fGNamesTrieFullyLoaded(false),
+  fTargetRegion() {
     initialize(locale, status);
 }
 
@@ -368,12 +365,12 @@ TZGNCore::initialize(const Locale& locale, UErrorCode& status) {
     zoneStrings = ures_getByKeyWithFallback(zoneStrings, gZoneStrings, zoneStrings, &tmpsts);
 
     if (U_SUCCESS(tmpsts)) {
-        const UChar *regionPattern = ures_getStringByKeyWithFallback(zoneStrings, gRegionFormatTag, NULL, &tmpsts);
+        const char16_t *regionPattern = ures_getStringByKeyWithFallback(zoneStrings, gRegionFormatTag, nullptr, &tmpsts);
         if (U_SUCCESS(tmpsts) && u_strlen(regionPattern) > 0) {
             rpat.setTo(regionPattern, -1);
         }
         tmpsts = U_ZERO_ERROR;
-        const UChar *fallbackPattern = ures_getStringByKeyWithFallback(zoneStrings, gFallbackFormatTag, NULL, &tmpsts);
+        const char16_t *fallbackPattern = ures_getStringByKeyWithFallback(zoneStrings, gFallbackFormatTag, nullptr, &tmpsts);
         if (U_SUCCESS(tmpsts) && u_strlen(fallbackPattern) > 0) {
             fpat.setTo(fallbackPattern, -1);
         }
@@ -391,13 +388,13 @@ TZGNCore::initialize(const Locale& locale, UErrorCode& status) {
     fLocaleDisplayNames = LocaleDisplayNames::createInstance(locale);
 
     // hash table for names - no key/value deleters
-    fLocationNamesMap = uhash_open(uhash_hashUChars, uhash_compareUChars, NULL, &status);
+    fLocationNamesMap = uhash_open(uhash_hashUChars, uhash_compareUChars, nullptr, &status);
     if (U_FAILURE(status)) {
         cleanup();
         return;
     }
 
-    fPartialLocationNamesMap = uhash_open(hashPartialLocationKey, comparePartialLocationKey, NULL, &status);
+    fPartialLocationNamesMap = uhash_open(hashPartialLocationKey, comparePartialLocationKey, nullptr, &status);
     if (U_FAILURE(status)) {
         cleanup();
         return;
@@ -409,29 +406,32 @@ TZGNCore::initialize(const Locale& locale, UErrorCode& status) {
     const char* region = fLocale.getCountry();
     int32_t regionLen = static_cast<int32_t>(uprv_strlen(region));
     if (regionLen == 0) {
-        CharString loc;
-        {
-            CharStringByteSink sink(&loc);
-            ulocimp_addLikelySubtags(fLocale.getName(), sink, &status);
+        CharString loc = ulocimp_addLikelySubtags(fLocale.getName(), status);
+        CharString tmp;
+        ulocimp_getSubtags(loc.toStringPiece(), nullptr, nullptr, &tmp, nullptr, nullptr, status);
+        if (U_FAILURE(status)) {
+            cleanup();
+            return;
         }
-
-        regionLen = uloc_getCountry(loc.data(), fTargetRegion, sizeof(fTargetRegion), &status);
-        if (U_SUCCESS(status)) {
-            fTargetRegion[regionLen] = 0;
-        } else {
+        fTargetRegion = tmp.toStringPiece();
+        if (fTargetRegion.isEmpty() != tmp.isEmpty()) {
+            status = U_MEMORY_ALLOCATION_ERROR;
             cleanup();
             return;
         }
-    } else if (regionLen < (int32_t)sizeof(fTargetRegion)) {
-        uprv_strcpy(fTargetRegion, region);
     } else {
-        fTargetRegion[0] = 0;
+        fTargetRegion = {region, static_cast<std::string_view::size_type>(regionLen)};
+        if (fTargetRegion.isEmpty()) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            cleanup();
+            return;
+        }
     }
 
     // preload generic names for the default zone
     TimeZone *tz = TimeZone::createDefault();
-    const UChar *tzID = ZoneMeta::getCanonicalCLDRID(*tz);
-    if (tzID != NULL) {
+    const char16_t *tzID = ZoneMeta::getCanonicalCLDRID(*tz);
+    if (tzID != nullptr) {
         loadStrings(UnicodeString(true, tzID, -1));
     }
     delete tz;
@@ -439,12 +439,8 @@ TZGNCore::initialize(const Locale& locale, UErrorCode& status) {
 
 void
 TZGNCore::cleanup() {
-    if (fLocaleDisplayNames != NULL) {
-        delete fLocaleDisplayNames;
-    }
-    if (fTimeZoneNames != NULL) {
-        delete fTimeZoneNames;
-    }
+    delete fLocaleDisplayNames;
+    delete fTimeZoneNames;
 
     uhash_close(fLocationNamesMap);
     uhash_close(fPartialLocationNamesMap);
@@ -457,8 +453,8 @@ TZGNCore::getDisplayName(const TimeZone& tz, UTimeZoneGenericNameType type, UDat
     switch (type) {
     case UTZGNM_LOCATION:
         {
-            const UChar* tzCanonicalID = ZoneMeta::getCanonicalCLDRID(tz);
-            if (tzCanonicalID != NULL) {
+            const char16_t* tzCanonicalID = ZoneMeta::getCanonicalCLDRID(tz);
+            if (tzCanonicalID != nullptr) {
                 getGenericLocationName(UnicodeString(true, tzCanonicalID, -1), name);
             }
         }
@@ -467,8 +463,8 @@ TZGNCore::getDisplayName(const TimeZone& tz, UTimeZoneGenericNameType type, UDat
     case UTZGNM_SHORT:
         formatGenericNonLocationName(tz, type, date, name);
         if (name.isEmpty()) {
-            const UChar* tzCanonicalID = ZoneMeta::getCanonicalCLDRID(tz);
-            if (tzCanonicalID != NULL) {
+            const char16_t* tzCanonicalID = ZoneMeta::getCanonicalCLDRID(tz);
+            if (tzCanonicalID != nullptr) {
                 getGenericLocationName(UnicodeString(true, tzCanonicalID, -1), name);
             }
         }
@@ -486,7 +482,7 @@ TZGNCore::getGenericLocationName(const UnicodeString& tzCanonicalID, UnicodeStri
         return name;
     }
 
-    const UChar *locname = NULL;
+    const char16_t *locname = nullptr;
     TZGNCore *nonConstThis = const_cast<TZGNCore *>(this);
     umtx_lock(&gLock);
     {
@@ -494,7 +490,7 @@ TZGNCore::getGenericLocationName(const UnicodeString& tzCanonicalID, UnicodeStri
     }
     umtx_unlock(&gLock);
 
-    if (locname == NULL) {
+    if (locname == nullptr) {
         name.setToBogus();
     } else {
         name.setTo(locname, u_strlen(locname));
@@ -506,25 +502,25 @@ TZGNCore::getGenericLocationName(const UnicodeString& tzCanonicalID, UnicodeStri
 /*
  * This method updates the cache and must be called with a lock
  */
-const UChar*
+const char16_t*
 TZGNCore::getGenericLocationName(const UnicodeString& tzCanonicalID) {
     U_ASSERT(!tzCanonicalID.isEmpty());
     if (tzCanonicalID.length() > ZID_KEY_MAX) {
-        return NULL;
+        return nullptr;
     }
 
     UErrorCode status = U_ZERO_ERROR;
-    UChar tzIDKey[ZID_KEY_MAX + 1];
+    char16_t tzIDKey[ZID_KEY_MAX + 1];
     int32_t tzIDKeyLen = tzCanonicalID.extract(tzIDKey, ZID_KEY_MAX + 1, status);
     U_ASSERT(status == U_ZERO_ERROR);   // already checked length above
     tzIDKey[tzIDKeyLen] = 0;
 
-    const UChar *locname = (const UChar *)uhash_get(fLocationNamesMap, tzIDKey);
+    const char16_t* locname = static_cast<const char16_t*>(uhash_get(fLocationNamesMap, tzIDKey));
 
-    if (locname != NULL) {
+    if (locname != nullptr) {
         // gEmpty indicate the name is not available
         if (locname == gEmpty) {
-            return NULL;
+            return nullptr;
         }
         return locname;
     }
@@ -559,26 +555,26 @@ TZGNCore::getGenericLocationName(const UnicodeString& tzCanonicalID) {
             fRegionFormat.format(city, name, status);
         }
         if (U_FAILURE(status)) {
-            return NULL;
+            return nullptr;
         }
     }
 
-    locname = name.isEmpty() ? NULL : fStringPool.get(name, status);
+    locname = name.isEmpty() ? nullptr : fStringPool.get(name, status);
     if (U_SUCCESS(status)) {
         // Cache the result
-        const UChar* cacheID = ZoneMeta::findTimeZoneID(tzCanonicalID);
-        U_ASSERT(cacheID != NULL);
-        if (locname == NULL) {
+        const char16_t* cacheID = ZoneMeta::findTimeZoneID(tzCanonicalID);
+        U_ASSERT(cacheID != nullptr);
+        if (locname == nullptr) {
             // gEmpty to indicate - no location name available
             uhash_put(fLocationNamesMap, (void *)cacheID, (void *)gEmpty, &status);
         } else {
             uhash_put(fLocationNamesMap, (void *)cacheID, (void *)locname, &status);
             if (U_FAILURE(status)) {
-                locname = NULL;
+                locname = nullptr;
             } else {
                 // put the name info into the trie
-                GNameInfo *nameinfo = (ZNameInfo *)uprv_malloc(sizeof(GNameInfo));
-                if (nameinfo != NULL) {
+                GNameInfo* nameinfo = static_cast<ZNameInfo*>(uprv_malloc(sizeof(GNameInfo)));
+                if (nameinfo != nullptr) {
                     nameinfo->type = UTZGNM_LOCATION;
                     nameinfo->tzID = cacheID;
                     fGNamesTrie.put(locname, nameinfo, status);
@@ -595,8 +591,8 @@ TZGNCore::formatGenericNonLocationName(const TimeZone& tz, UTimeZoneGenericNameT
     U_ASSERT(type == UTZGNM_LONG || type == UTZGNM_SHORT);
     name.setToBogus();
 
-    const UChar* uID = ZoneMeta::getCanonicalCLDRID(tz);
-    if (uID == NULL) {
+    const char16_t* uID = ZoneMeta::getCanonicalCLDRID(tz);
+    if (uID == nullptr) {
         return name;
     }
 
@@ -611,14 +607,14 @@ TZGNCore::formatGenericNonLocationName(const TimeZone& tz, UTimeZoneGenericNameT
     }
 
     // Try meta zone
-    UChar mzIDBuf[32];
+    char16_t mzIDBuf[32];
     UnicodeString mzID(mzIDBuf, 0, UPRV_LENGTHOF(mzIDBuf));
     fTimeZoneNames->getMetaZoneID(tzID, date, mzID);
     if (!mzID.isEmpty()) {
         UErrorCode status = U_ZERO_ERROR;
         UBool useStandard = false;
         int32_t raw, sav;
-        UChar tmpNameBuf[ZONE_NAME_U16_MAX];
+        char16_t tmpNameBuf[ZONE_NAME_U16_MAX];
 
         tz.getOffset(date, false, raw, sav, status);
         if (U_FAILURE(status)) {
@@ -630,15 +626,15 @@ TZGNCore::formatGenericNonLocationName(const TimeZone& tz, UTimeZoneGenericNameT
 
             TimeZone *tmptz = tz.clone();
             // Check if the zone actually uses daylight saving time around the time
-            BasicTimeZone *btz = NULL;
-            if (dynamic_cast<OlsonTimeZone *>(tmptz) != NULL
-                || dynamic_cast<SimpleTimeZone *>(tmptz) != NULL
-                || dynamic_cast<RuleBasedTimeZone *>(tmptz) != NULL
-                || dynamic_cast<VTimeZone *>(tmptz) != NULL) {
+            BasicTimeZone *btz = nullptr;
+            if (dynamic_cast<OlsonTimeZone *>(tmptz) != nullptr
+                || dynamic_cast<SimpleTimeZone *>(tmptz) != nullptr
+                || dynamic_cast<RuleBasedTimeZone *>(tmptz) != nullptr
+                || dynamic_cast<VTimeZone *>(tmptz) != nullptr) {
                 btz = (BasicTimeZone*)tmptz;
             }
 
-            if (btz != NULL) {
+            if (btz != nullptr) {
                 TimeZoneTransition before;
                 UBool beforTrs = btz->getPreviousTransition(date, true, before);
                 if (beforTrs
@@ -686,7 +682,7 @@ TZGNCore::formatGenericNonLocationName(const TimeZone& tz, UTimeZoneGenericNameT
                 // for some meta zones in some locales.  This looks like a data bugs.
                 // For now, we check if the standard name is different from its generic
                 // name below.
-                UChar genNameBuf[ZONE_NAME_U16_MAX];
+                char16_t genNameBuf[ZONE_NAME_U16_MAX];
                 UnicodeString mzGenericName(genNameBuf, 0, UPRV_LENGTHOF(genNameBuf));
                 fTimeZoneNames->getMetaZoneDisplayName(mzID, nameType, mzGenericName);
                 if (stdName.caseCompare(mzGenericName, 0) == 0) {
@@ -702,9 +698,9 @@ TZGNCore::formatGenericNonLocationName(const TimeZone& tz, UTimeZoneGenericNameT
                 // Check if we need to use a partial location format.
                 // This check is done by comparing offset with the meta zone's
                 // golden zone at the given date.
-                UChar idBuf[32];
+                char16_t idBuf[32];
                 UnicodeString goldenID(idBuf, 0, UPRV_LENGTHOF(idBuf));
-                fTimeZoneNames->getReferenceZoneID(mzID, fTargetRegion, goldenID);
+                fTimeZoneNames->getReferenceZoneID(mzID, fTargetRegion.data(), goldenID);
                 if (!goldenID.isEmpty() && goldenID != tzID) {
                     TimeZone *goldenZone = TimeZone::createTimeZone(goldenID);
                     int32_t raw1, sav1;
@@ -741,7 +737,7 @@ TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
         return name;
     }
 
-    const UChar *uplname = NULL;
+    const char16_t *uplname = nullptr;
     TZGNCore *nonConstThis = const_cast<TZGNCore *>(this);
     umtx_lock(&gLock);
     {
@@ -749,7 +745,7 @@ TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
     }
     umtx_unlock(&gLock);
 
-    if (uplname == NULL) {
+    if (uplname == nullptr) {
         name.setToBogus();
     } else {
         name.setTo(true, uplname, -1);
@@ -760,7 +756,7 @@ TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
 /*
  * This method updates the cache and must be called with a lock
  */
-const UChar*
+const char16_t*
 TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
                         const UnicodeString& mzID, UBool isLong, const UnicodeString& mzDisplayName) {
     U_ASSERT(!tzCanonicalID.isEmpty());
@@ -771,10 +767,10 @@ TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
     key.tzID = ZoneMeta::findTimeZoneID(tzCanonicalID);
     key.mzID = ZoneMeta::findMetaZoneID(mzID);
     key.isLong = isLong;
-    U_ASSERT(key.tzID != NULL && key.mzID != NULL);
+    U_ASSERT(key.tzID != nullptr && key.mzID != nullptr);
 
-    const UChar* uplname = (const UChar*)uhash_get(fPartialLocationNamesMap, (void *)&key);
-    if (uplname != NULL) {
+    const char16_t* uplname = static_cast<const char16_t*>(uhash_get(fPartialLocationNamesMap, &key));
+    if (uplname != nullptr) {
         return uplname;
     }
 
@@ -810,14 +806,14 @@ TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
     UnicodeString name;
     fFallbackFormat.format(location, mzDisplayName, name, status);
     if (U_FAILURE(status)) {
-        return NULL;
+        return nullptr;
     }
 
     uplname = fStringPool.get(name, status);
     if (U_SUCCESS(status)) {
         // Add the name to cache
-        PartialLocationKey* cacheKey = (PartialLocationKey *)uprv_malloc(sizeof(PartialLocationKey));
-        if (cacheKey != NULL) {
+        PartialLocationKey* cacheKey = static_cast<PartialLocationKey*>(uprv_malloc(sizeof(PartialLocationKey)));
+        if (cacheKey != nullptr) {
             cacheKey->tzID = key.tzID;
             cacheKey->mzID = key.mzID;
             cacheKey->isLong = key.isLong;
@@ -826,8 +822,8 @@ TZGNCore::getPartialLocationName(const UnicodeString& tzCanonicalID,
                 uprv_free(cacheKey);
             } else {
                 // put the name to the local trie as well
-                GNameInfo *nameinfo = (ZNameInfo *)uprv_malloc(sizeof(GNameInfo));
-                if (nameinfo != NULL) {
+                GNameInfo* nameinfo = static_cast<ZNameInfo*>(uprv_malloc(sizeof(GNameInfo)));
+                if (nameinfo != nullptr) {
                     nameinfo->type = isLong ? UTZGNM_LONG : UTZGNM_SHORT;
                     nameinfo->tzID = key.tzID;
                     fGNamesTrie.put(uplname, nameinfo, status);
@@ -859,14 +855,14 @@ TZGNCore::loadStrings(const UnicodeString& tzCanonicalID) {
     };
 
     StringEnumeration *mzIDs = fTimeZoneNames->getAvailableMetaZoneIDs(tzCanonicalID, status);
-    while ((mzID = mzIDs->snext(status)) != NULL) {
+    while ((mzID = mzIDs->snext(status)) != nullptr) {
         if (U_FAILURE(status)) {
             break;
         }
         // if this time zone is not the golden zone of the meta zone,
         // partial location name (such as "PT (Los Angeles)") might be
         // available.
-        fTimeZoneNames->getReferenceZoneID(*mzID, fTargetRegion, goldenID);
+        fTimeZoneNames->getReferenceZoneID(*mzID, fTargetRegion.data(), goldenID);
         if (tzCanonicalID != goldenID) {
             for (int32_t i = 0; genNonLocTypes[i] != UTZNM_UNKNOWN; i++) {
                 fTimeZoneNames->getMetaZoneDisplayName(*mzID, genNonLocTypes[i], mzGenName);
@@ -878,9 +874,7 @@ TZGNCore::loadStrings(const UnicodeString& tzCanonicalID) {
             }
         }
     }
-    if (mzIDs != NULL) {
-        delete mzIDs;
-    }
+    delete mzIDs;
 }
 
 int32_t
@@ -905,7 +899,7 @@ TZGNCore::findBestMatch(const UnicodeString& text, int32_t start, uint32_t types
     // UBool isLongStandard = false;   // workaround - see the comments below
     UBool isStandard = false;       // TODO: Temporary hack (on hack) for short standard name/location name conflict (found in zh_Hant), should be removed after CLDR 21m1 integration
 
-    if (tznamesMatches != NULL) {
+    if (tznamesMatches != nullptr) {
         UnicodeString mzID;
         for (int32_t i = 0; i < tznamesMatches->size(); i++) {
             int32_t len = tznamesMatches->getMatchLengthAt(i);
@@ -914,7 +908,7 @@ TZGNCore::findBestMatch(const UnicodeString& text, int32_t start, uint32_t types
                 if (!tznamesMatches->getTimeZoneIDAt(i, bestMatchTzID)) {
                     // name for a meta zone
                     if (tznamesMatches->getMetaZoneIDAt(i, mzID)) {
-                        fTimeZoneNames->getReferenceZoneID(mzID, fTargetRegion, bestMatchTzID);
+                        fTimeZoneNames->getReferenceZoneID(mzID, fTargetRegion.data(), bestMatchTzID);
                     }
                 }
                 UTimeZoneNameType nameType = tznamesMatches->getNameTypeAt(i);
@@ -979,7 +973,7 @@ TZGNCore::findBestMatch(const UnicodeString& text, int32_t start, uint32_t types
     if (U_FAILURE(status)) {
         return 0;
     }
-    if (localMatches != NULL) {
+    if (localMatches != nullptr) {
         for (int32_t i = 0; i < localMatches->size(); i++) {
             int32_t len = localMatches->getMatchLength(i);
 
@@ -1017,46 +1011,42 @@ TZGNCore::findLocal(const UnicodeString& text, int32_t start, uint32_t types, UE
     umtx_unlock(&gLock);
 
     if (U_FAILURE(status)) {
-        return NULL;
+        return nullptr;
     }
 
-    TimeZoneGenericNameMatchInfo *gmatchInfo = NULL;
+    TimeZoneGenericNameMatchInfo *gmatchInfo = nullptr;
 
     int32_t maxLen = 0;
     UVector *results = handler.getMatches(maxLen);
-    if (results != NULL && ((maxLen == (text.length() - start)) || fGNamesTrieFullyLoaded)) {
+    if (results != nullptr && ((maxLen == (text.length() - start)) || fGNamesTrieFullyLoaded)) {
         // perfect match
         gmatchInfo = new TimeZoneGenericNameMatchInfo(results);
-        if (gmatchInfo == NULL) {
+        if (gmatchInfo == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             delete results;
-            return NULL;
+            return nullptr;
         }
         return gmatchInfo;
     }
 
-    if (results != NULL) {
-        delete results;
-    }
+    delete results;
 
     // All names are not yet loaded into the local trie.
     // Load all available names into the trie. This could be very heavy.
     umtx_lock(&gLock);
     {
         if (!fGNamesTrieFullyLoaded) {
-            StringEnumeration *tzIDs = TimeZone::createTimeZoneIDEnumeration(UCAL_ZONE_TYPE_CANONICAL, NULL, NULL, status);
+            StringEnumeration *tzIDs = TimeZone::createTimeZoneIDEnumeration(UCAL_ZONE_TYPE_CANONICAL, nullptr, nullptr, status);
             if (U_SUCCESS(status)) {
                 const UnicodeString *tzID;
-                while ((tzID = tzIDs->snext(status)) != NULL) {
+                while ((tzID = tzIDs->snext(status)) != nullptr) {
                     if (U_FAILURE(status)) {
                         break;
                     }
                     nonConstThis->loadStrings(*tzID);
                 }
             }
-            if (tzIDs != NULL) {
-                delete tzIDs;
-            }
+            delete tzIDs;
 
             if (U_SUCCESS(status)) {
                 nonConstThis->fGNamesTrieFullyLoaded = true;
@@ -1066,7 +1056,7 @@ TZGNCore::findLocal(const UnicodeString& text, int32_t start, uint32_t types, UE
     umtx_unlock(&gLock);
 
     if (U_FAILURE(status)) {
-        return NULL;
+        return nullptr;
     }
 
     umtx_lock(&gLock);
@@ -1077,12 +1067,12 @@ TZGNCore::findLocal(const UnicodeString& text, int32_t start, uint32_t types, UE
     umtx_unlock(&gLock);
 
     results = handler.getMatches(maxLen);
-    if (results != NULL && maxLen > 0) {
+    if (results != nullptr && maxLen > 0) {
         gmatchInfo = new TimeZoneGenericNameMatchInfo(results);
-        if (gmatchInfo == NULL) {
+        if (gmatchInfo == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             delete results;
-            return NULL;
+            return nullptr;
         }
     }
 
@@ -1105,7 +1095,7 @@ TZGNCore::findTimeZoneNames(const UnicodeString& text, int32_t start, uint32_t t
         return fTimeZoneNames->find(text, start, nameTypes, status);
     }
 
-    return NULL;
+    return nullptr;
 }
 
 typedef struct TZGNCoreRef {
@@ -1116,7 +1106,7 @@ typedef struct TZGNCoreRef {
 
 // TZGNCore object cache handling
 static UMutex gTZGNLock;
-static UHashtable *gTZGNCoreCache = NULL;
+static UHashtable *gTZGNCoreCache = nullptr;
 static UBool gTZGNCoreCacheInitialized = false;
 
 // Access count - incremented every time up to SWEEP_INTERVAL,
@@ -1136,11 +1126,11 @@ U_CDECL_BEGIN
 /**
  * Cleanup callback func
  */
-static UBool U_CALLCONV tzgnCore_cleanup(void)
+static UBool U_CALLCONV tzgnCore_cleanup()
 {
-    if (gTZGNCoreCache != NULL) {
+    if (gTZGNCoreCache != nullptr) {
         uhash_close(gTZGNCoreCache);
-        gTZGNCoreCache = NULL;
+        gTZGNCoreCache = nullptr;
     }
     gTZGNCoreCacheInitialized = false;
     return true;
@@ -1165,10 +1155,10 @@ U_CDECL_END
 static void sweepCache() {
     int32_t pos = UHASH_FIRST;
     const UHashElement* elem;
-    double now = (double)uprv_getUTCtime();
+    double now = static_cast<double>(uprv_getUTCtime());
 
-    while ((elem = uhash_nextElement(gTZGNCoreCache, &pos)) != NULL) {
-        TZGNCoreRef *entry = (TZGNCoreRef *)elem->value.pointer;
+    while ((elem = uhash_nextElement(gTZGNCoreCache, &pos)) != nullptr) {
+        TZGNCoreRef* entry = static_cast<TZGNCoreRef*>(elem->value.pointer);
         if (entry->refCount <= 0 && (now - entry->lastAccess) > CACHE_EXPIRATION) {
             // delete this entry
             uhash_removeElement(gTZGNCoreCache, elem);
@@ -1177,7 +1167,7 @@ static void sweepCache() {
 }
 
 TimeZoneGenericNames::TimeZoneGenericNames()
-: fRef(0) {
+: fRef(nullptr) {
 }
 
 TimeZoneGenericNames::~TimeZoneGenericNames() {
@@ -1193,21 +1183,20 @@ TimeZoneGenericNames::~TimeZoneGenericNames() {
 TimeZoneGenericNames*
 TimeZoneGenericNames::createInstance(const Locale& locale, UErrorCode& status) {
     if (U_FAILURE(status)) {
-        return NULL;
+        return nullptr;
     }
-    TimeZoneGenericNames* instance = new TimeZoneGenericNames();
-    if (instance == NULL) {
-        status = U_MEMORY_ALLOCATION_ERROR;
-        return NULL;
+    LocalPointer<TimeZoneGenericNames> instance(new TimeZoneGenericNames(), status);
+    if (U_FAILURE(status)) {
+        return nullptr;
     }
 
-    TZGNCoreRef *cacheEntry = NULL;
+    TZGNCoreRef *cacheEntry = nullptr;
     {
         Mutex lock(&gTZGNLock);
 
         if (!gTZGNCoreCacheInitialized) {
             // Create empty hashtable
-            gTZGNCoreCache = uhash_open(uhash_hashChars, uhash_compareChars, NULL, &status);
+            gTZGNCoreCache = uhash_open(uhash_hashChars, uhash_compareChars, nullptr, &status);
             if (U_SUCCESS(status)) {
                 uhash_setKeyDeleter(gTZGNCoreCache, uprv_free);
                 uhash_setValueDeleter(gTZGNCoreCache, deleteTZGNCoreRef);
@@ -1216,56 +1205,54 @@ TimeZoneGenericNames::createInstance(const Locale& locale, UErrorCode& status) {
             }
         }
         if (U_FAILURE(status)) {
-            return NULL;
+            return nullptr;
         }
 
         // Check the cache, if not available, create new one and cache
         const char *key = locale.getName();
-        cacheEntry = (TZGNCoreRef *)uhash_get(gTZGNCoreCache, key);
-        if (cacheEntry == NULL) {
-            TZGNCore *tzgnCore = NULL;
-            char *newKey = NULL;
+        cacheEntry = static_cast<TZGNCoreRef*>(uhash_get(gTZGNCoreCache, key));
+        if (cacheEntry == nullptr) {
+            TZGNCore *tzgnCore = nullptr;
+            char *newKey = nullptr;
 
             tzgnCore = new TZGNCore(locale, status);
-            if (tzgnCore == NULL) {
+            if (tzgnCore == nullptr) {
                 status = U_MEMORY_ALLOCATION_ERROR;
             }
             if (U_SUCCESS(status)) {
-                newKey = (char *)uprv_malloc(uprv_strlen(key) + 1);
-                if (newKey == NULL) {
+                newKey = static_cast<char*>(uprv_malloc(uprv_strlen(key) + 1));
+                if (newKey == nullptr) {
                     status = U_MEMORY_ALLOCATION_ERROR;
                 } else {
                     uprv_strcpy(newKey, key);
                 }
             }
             if (U_SUCCESS(status)) {
-                cacheEntry = (TZGNCoreRef *)uprv_malloc(sizeof(TZGNCoreRef));
-                if (cacheEntry == NULL) {
+                cacheEntry = static_cast<TZGNCoreRef*>(uprv_malloc(sizeof(TZGNCoreRef)));
+                if (cacheEntry == nullptr) {
                     status = U_MEMORY_ALLOCATION_ERROR;
                 } else {
                     cacheEntry->obj = tzgnCore;
                     cacheEntry->refCount = 1;
-                    cacheEntry->lastAccess = (double)uprv_getUTCtime();
+                    cacheEntry->lastAccess = static_cast<double>(uprv_getUTCtime());
 
                     uhash_put(gTZGNCoreCache, newKey, cacheEntry, &status);
                 }
             }
             if (U_FAILURE(status)) {
-                if (tzgnCore != NULL) {
-                    delete tzgnCore;
-                }
-                if (newKey != NULL) {
+                delete tzgnCore;
+                if (newKey != nullptr) {
                     uprv_free(newKey);
                 }
-                if (cacheEntry != NULL) {
+                if (cacheEntry != nullptr) {
                     uprv_free(cacheEntry);
                 }
-                cacheEntry = NULL;
+                cacheEntry = nullptr;
             }
         } else {
             // Update the reference count
             cacheEntry->refCount++;
-            cacheEntry->lastAccess = (double)uprv_getUTCtime();
+            cacheEntry->lastAccess = static_cast<double>(uprv_getUTCtime());
         }
         gAccessCount++;
         if (gAccessCount >= SWEEP_INTERVAL) {
@@ -1275,13 +1262,12 @@ TimeZoneGenericNames::createInstance(const Locale& locale, UErrorCode& status) {
         }
     }  // End of mutex locked block
 
-    if (cacheEntry == NULL) {
-        delete instance;
-        return NULL;
+    if (cacheEntry == nullptr) {
+        return nullptr;
     }
 
     instance->fRef = cacheEntry;
-    return instance;
+    return instance.orphan();
 }
 
 bool
