@@ -1,11 +1,13 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-* Copyright (C) 2007-2010, International Business Machines Corporation and
+* Copyright (C) 2007-2012, International Business Machines Corporation and
 * others. All Rights Reserved.
 *******************************************************************************
 */
 
-#include <typeinfo>  // for 'typeid' to work
+#include "utypeinfo.h"  // for 'typeid' to work
 
 #include "unicode/utypes.h"
 
@@ -23,31 +25,27 @@ TimeZoneTransition::TimeZoneTransition(UDate time, const TimeZoneRule& from, con
 }
 
 TimeZoneTransition::TimeZoneTransition()
-: UObject(), fTime(0), fFrom(NULL), fTo(NULL) {
+: UObject(), fTime(0), fFrom(nullptr), fTo(nullptr) {
 }
 
 TimeZoneTransition::TimeZoneTransition(const TimeZoneTransition& source)
-: UObject(), fTime(source.fTime), fFrom(NULL), fTo(NULL) {
-      if (source.fFrom != NULL) {
+: UObject(), fTime(source.fTime), fFrom(nullptr), fTo(nullptr) {
+      if (source.fFrom != nullptr) {
           fFrom = source.fFrom->clone();
       }
 
-      if (source.fTo != NULL) {
+      if (source.fTo != nullptr) {
           fTo = source.fTo->clone();
       }
 }
 
 TimeZoneTransition::~TimeZoneTransition() {
-    if (fFrom != NULL) {
-        delete fFrom;
-    }
-    if (fTo != NULL) {
-        delete fTo;
-    }
+    delete fFrom;
+    delete fTo;
 }
 
 TimeZoneTransition*
-TimeZoneTransition::clone(void) const {
+TimeZoneTransition::clone() const {
     return new TimeZoneTransition(*this);
 }
 
@@ -61,28 +59,28 @@ TimeZoneTransition::operator=(const TimeZoneTransition& right) {
     return *this;
 }
 
-UBool
+bool
 TimeZoneTransition::operator==(const TimeZoneTransition& that) const {
     if (this == &that) {
-        return TRUE;
+        return true;
     }
     if (typeid(*this) != typeid(that)) {
-        return FALSE;
+        return false;
     }
     if (fTime != that.fTime) {
-        return FALSE;
+        return false;
     }
-    if ((fFrom == NULL && that.fFrom == NULL)
-        || (fFrom != NULL && that.fFrom != NULL && *fFrom == *(that.fFrom))) {
-        if ((fTo == NULL && that.fTo == NULL)
-            || (fTo != NULL && that.fTo != NULL && *fTo == *(that.fTo))) {
-            return TRUE;
+    if ((fFrom == nullptr && that.fFrom == nullptr)
+        || (fFrom != nullptr && that.fFrom != nullptr && *fFrom == *(that.fFrom))) {
+        if ((fTo == nullptr && that.fTo == nullptr)
+            || (fTo != nullptr && that.fTo != nullptr && *fTo == *(that.fTo))) {
+            return true;
         }
     }
-    return FALSE;
+    return false;
 }
 
-UBool
+bool
 TimeZoneTransition::operator!=(const TimeZoneTransition& that) const {
     return !operator==(that);
 }
@@ -94,48 +92,40 @@ TimeZoneTransition::setTime(UDate time) {
 
 void
 TimeZoneTransition::setFrom(const TimeZoneRule& from) {
-    if (fFrom != NULL) {
-        delete fFrom;
-    }
+    delete fFrom;
     fFrom = from.clone();
 }
 
 void
 TimeZoneTransition::adoptFrom(TimeZoneRule* from) {
-    if (fFrom != NULL) {
-        delete fFrom;
-    }
+    delete fFrom;
     fFrom = from;
 }
 
 void
 TimeZoneTransition::setTo(const TimeZoneRule& to) {
-    if (fTo != NULL) {
-        delete fTo;
-    }
+    delete fTo;
     fTo = to.clone();
 }
 
 void
 TimeZoneTransition::adoptTo(TimeZoneRule* to) {
-    if (fTo != NULL) {
-        delete fTo;
-    }
+    delete fTo;
     fTo = to;
 }
 
 UDate
-TimeZoneTransition::getTime(void) const {
+TimeZoneTransition::getTime() const {
     return fTime;
 }
 
 const TimeZoneRule*
-TimeZoneTransition::getTo(void) const {
+TimeZoneTransition::getTo() const {
     return fTo;
 }
 
 const TimeZoneRule*
-TimeZoneTransition::getFrom(void) const {
+TimeZoneTransition::getFrom() const {
     return fFrom;
 }
 
