@@ -1,15 +1,13 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-* Copyright (C) 2007-2010, International Business Machines Corporation and
+* Copyright (C) 2007-2014, International Business Machines Corporation and
 * others. All Rights Reserved.
 *******************************************************************************
 *
 
 * File PLURFMT.H
-*
-* Modification History:*
-*   Date        Name        Description
-*
 ********************************************************************************
 */
 
@@ -18,6 +16,8 @@
 
 #include "unicode/utypes.h"
 
+#if U_SHOW_CPLUSPLUS_API
+
 /**
  * \file
  * \brief C++ API: PluralFormat object
@@ -25,19 +25,21 @@
 
 #if !UCONFIG_NO_FORMATTING
 
+#include "unicode/messagepattern.h"
 #include "unicode/numfmt.h"
 #include "unicode/plurrule.h"
 
 U_NAMESPACE_BEGIN
 
 class Hashtable;
+class NFRule;
 
 /**
  * <p>
  * <code>PluralFormat</code> supports the creation of internationalized
  * messages with plural inflection. It is based on <i>plural
  * selection</i>, i.e. the caller specifies messages for each
- * plural case that can appear in the users language and the
+ * plural case that can appear in the user's language and the
  * <code>PluralFormat</code> selects the appropriate message based on
  * the number.
  * </p>
@@ -51,7 +53,7 @@ class Hashtable;
  * each message and selects the message whose interval contains a
  * given number. This can only handle a finite number of
  * intervals. But in some languages, like Polish, one plural case
- * applies to infinitely many intervals (e.g., paucal applies to
+ * applies to infinitely many intervals (e.g., the plural case applies to
  * numbers ending with 2, 3, or 4 except those ending with 12, 13, or
  * 14). Thus <code>ChoiceFormat</code> is not adequate.
  * </p><p>
@@ -62,15 +64,20 @@ class Hashtable;
  *     conditions for a plural case than just a single interval. These plural
  *     rules define both what plural cases exist in a language, and to
  *     which numbers these cases apply.
- * <li>It provides predefined plural rules for many locales. Thus, the programmer
- *     need not worry about the plural cases of a language. On the flip side,
- *     the localizer does not have to specify the plural cases; he can simply
+ * <li>It provides predefined plural rules for many languages. Thus, the programmer
+ *     need not worry about the plural cases of a language and
+ *     does not have to define the plural cases; they can simply
  *     use the predefined keywords. The whole plural formatting of messages can
- *     be done using localized patterns from resource bundles.
+ *     be done using localized patterns from resource bundles. For predefined plural
+ *     rules, see the CLDR <i>Language Plural Rules</i> page at
+ *     https://unicode-org.github.io/cldr-staging/charts/latest/supplemental/language_plural_rules.html
  * </ul>
  * </p>
  * <h4>Usage of <code>PluralFormat</code></h4>
- * <p>
+ * <p>Note: Typically, plural formatting is done via <code>MessageFormat</code>
+ * with a <code>plural</code> argument type,
+ * rather than using a stand-alone <code>PluralFormat</code>.
+ * </p><p>
  * This discussion assumes that you use <code>PluralFormat</code> with
  * a predefined set of plural rules. You can create one using one of
  * the constructors that takes a <code>locale</code> object. To
@@ -83,79 +90,46 @@ class Hashtable;
  * <h5>Patterns and Their Interpretation</h5>
  * <p>
  * The pattern text defines the message output for each plural case of the
- * used locale. The pattern is a sequence of
- * <code><i>caseKeyword</i>{<i>message</i>}</code> clauses, separated by white
- * space characters. Each clause assigns the message <code><i>message</i></code>
- * to the plural case identified by <code><i>caseKeyword</i></code>.
- * </p><p>
- * You always have to define a message text for the default plural case
- * "<code>other</code>" which is contained in every rule set. If the plural
- * rules of the <code>PluralFormat</code> object do not contain a plural case
- * identified by <code><i>caseKeyword</i></code>, U_DEFAULT_KEYWORD_MISSING
- * will be set to status.
- * If you do not specify a message text for a particular plural case, the
- * message text of the plural case "<code>other</code>" gets assigned to this
- * plural case. If you specify more than one message for the same plural case,
- * U_DUPLICATE_KEYWORD will be set to status.
- * <br>
- * Spaces between <code><i>caseKeyword</i></code> and
- * <code><i>message</i></code>  will be ignored; spaces within
- * <code><i>message</i></code> will be preserved.
- * </p><p>
- * The message text for a particular plural case may contain other message
- * format patterns. <code>PluralFormat</code> preserves these so that you
- * can use the strings produced by <code>PluralFormat</code> with other
- * formatters. If you are using <code>PluralFormat</code> inside a
- * <code>MessageFormat</code> pattern, <code>MessageFormat</code> will
- * automatically evaluate the resulting format pattern.<br>
- * Thus, curly braces (<code>{</code>, <code>}</code>) are <i>only</i> allowed
- * in message texts to define a nested format pattern.<br>
- * The pound sign (<code>#</code>) will be interpreted as the number placeholder
- * in the message text, if it is not contained in curly braces (to preserve
- * <code>NumberFormat</code> patterns). <code>PluralFormat</code> will
- * replace each of those pound signs by the number passed to the
- * <code>format()</code> method. It will be formatted using a
- * <code>NumberFormat</code> for the <code>PluralFormat</code>'s locale. If you
- * need special number formatting, you have to explicitly specify a
- * <code>NumberFormat</code> for the <code>PluralFormat</code> to use.
- * </p>
- * Example
+ * specified locale. Syntax:
  * <pre>
- * UErrorCode status = U_ZERO_ERROR;
- * MessageFormat* msgFmt = new MessageFormat(UnicodeString("{0, plural,
- *   one{{0, number, C''est #,##0.0#  fichier}} other {Ce sont # fichiers}} dans la liste."),
- *   Locale("fr"), status);
- * if (U_FAILURE(status)) {
- *     return;
- * }
- * Formattable args1[] = {(int32_t)0};
- * Formattable args2[] = {(int32_t)3};
- * FieldPosition ignore(FieldPosition::DONT_CARE);
- * UnicodeString result;
- * msgFmt->format(args1, 1, result, ignore, status);
- * cout << result << endl;
- * result.remove();
- * msgFmt->format(args2, 1, result, ignore, status);
- * cout << result << endl;
+ * pluralStyle = [offsetValue] (selector '{' message '}')+
+ * offsetValue = "offset:" number
+ * selector = explicitValue | keyword
+ * explicitValue = '=' number  // adjacent, no white space in between
+ * keyword = [^[[:Pattern_Syntax:][:Pattern_White_Space:]]]+
+ * message: see {@link MessageFormat}
  * </pre>
- * Produces the output:<br>
- * <code>C'est 0,0 fichier dans la liste.</code><br>
- * <code>Ce sont 3 fichiers dans la liste."</code>
- * <p>
- * <strong>Note:</strong><br>
- *   Currently <code>PluralFormat</code>
- *   does not make use of quotes like <code>MessageFormat</code>.
- *   If you use plural format strings with <code>MessageFormat</code> and want
- *   to use a quote sign "<code>'</code>", you have to write "<code>''</code>".
- *   <code>MessageFormat</code> unquotes this pattern and  passes the unquoted
- *   pattern to <code>PluralFormat</code>. It's a bit trickier if you use
- *   nested formats that do quoting. In the example above, we wanted to insert
- *   "<code>'</code>" in the number format pattern. Since
- *   <code>NumberFormat</code> supports quotes, we had to insert
- *   "<code>''</code>". But since <code>MessageFormat</code> unquotes the
- *   pattern before it gets passed to <code>PluralFormat</code>, we have to
- *   double these quotes, i.e. write "<code>''''</code>".
+ * Pattern_White_Space between syntax elements is ignored, except
+ * between the {curly braces} and their sub-message,
+ * and between the '=' and the number of an explicitValue.
+ *
+ * </p><p>
+ * There are 6 predefined casekeyword in CLDR/ICU - 'zero', 'one', 'two', 'few', 'many' and
+ * 'other'. You always have to define a message text for the default plural case
+ * <code>other</code> which is contained in every rule set.
+ * If you do not specify a message text for a particular plural case, the
+ * message text of the plural case <code>other</code> gets assigned to this
+ * plural case.
+ * </p><p>
+ * When formatting, the input number is first matched against the explicitValue clauses.
+ * If there is no exact-number match, then a keyword is selected by calling
+ * the <code>PluralRules</code> with the input number <em>minus the offset</em>.
+ * (The offset defaults to 0 if it is omitted from the pattern string.)
+ * If there is no clause with that keyword, then the "other" clauses is returned.
+ * </p><p>
+ * An unquoted pound sign (<code>#</code>) in the selected sub-message
+ * itself (i.e., outside of arguments nested in the sub-message)
+ * is replaced by the input number minus the offset.
+ * The number-minus-offset value is formatted using a
+ * <code>NumberFormat</code> for the <code>PluralFormat</code>'s locale. If you
+ * need special number formatting, you have to use a <code>MessageFormat</code>
+ * and explicitly specify a <code>NumberFormat</code> argument.
+ * <strong>Note:</strong> That argument is formatting without subtracting the offset!
+ * If you need a custom format and have a non-zero offset, then you need to pass the
+ * number-minus-offset value as a separate parameter.
  * </p>
+ * For a usage example, see the {@link MessageFormat} class documentation.
+ *
  * <h4>Defining Custom Plural Rules</h4>
  * <p>If you need to use <code>PluralFormat</code> with custom rules, you can
  * create a <code>PluralRules</code> object and pass it to
@@ -171,21 +145,21 @@ class Hashtable;
  * @stable ICU 4.0
  */
 
-class U_I18N_API PluralFormat : public Format {
+class U_I18N_API_CLASS PluralFormat : public Format {
 public:
 
     /**
-     * Creates a new <code>PluralFormat</code> for the default locale.
+     * Creates a new cardinal-number <code>PluralFormat</code> for the default locale.
      * This locale will be used to get the set of plural rules and for standard
      * number formatting.
      * @param status  output param set to success/failure code on exit, which
      *                must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(UErrorCode& status);
+    U_I18N_API PluralFormat(UErrorCode& status);
 
     /**
-     * Creates a new <code>PluralFormat</code> for a given locale.
+     * Creates a new cardinal-number <code>PluralFormat</code> for a given locale.
      * @param locale the <code>PluralFormat</code> will be configured with
      *               rules for this locale. This locale will also be used for
      *               standard number formatting.
@@ -193,7 +167,7 @@ public:
      *               must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const Locale& locale, UErrorCode& status);
+    U_I18N_API PluralFormat(const Locale& locale, UErrorCode& status);
 
     /**
      * Creates a new <code>PluralFormat</code> for a given set of rules.
@@ -204,7 +178,7 @@ public:
      *                must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const PluralRules& rules, UErrorCode& status);
+    U_I18N_API PluralFormat(const PluralRules& rules, UErrorCode& status);
 
     /**
      * Creates a new <code>PluralFormat</code> for a given set of rules.
@@ -217,10 +191,22 @@ public:
      *                must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const Locale& locale, const PluralRules& rules, UErrorCode& status);
+    U_I18N_API PluralFormat(const Locale& locale, const PluralRules& rules, UErrorCode& status);
 
     /**
-     * Creates a new <code>PluralFormat</code> for a given pattern string.
+     * Creates a new <code>PluralFormat</code> for the plural type.
+     * The standard number formatting will be done using the given locale.
+     * @param locale  the default number formatting will be done using this
+     *                locale.
+     * @param type    The plural type (e.g., cardinal or ordinal).
+     * @param status  output param set to success/failure code on exit, which
+     *                must not indicate a failure before the function call.
+     * @stable ICU 50
+     */
+    U_I18N_API PluralFormat(const Locale& locale, UPluralType type, UErrorCode& status);
+
+    /**
+     * Creates a new cardinal-number <code>PluralFormat</code> for a given pattern string.
      * The default locale will be used to get the set of plural rules and for
      * standard number formatting.
      * @param  pattern the pattern for this <code>PluralFormat</code>.
@@ -229,10 +215,10 @@ public:
      *                 must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const UnicodeString& pattern, UErrorCode& status);
+    U_I18N_API PluralFormat(const UnicodeString& pattern, UErrorCode& status);
 
     /**
-     * Creates a new <code>PluralFormat</code> for a given pattern string and
+     * Creates a new cardinal-number <code>PluralFormat</code> for a given pattern string and
      * locale.
      * The locale will be used to get the set of plural rules and for
      * standard number formatting.
@@ -245,7 +231,7 @@ public:
      *                 must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const Locale& locale, const UnicodeString& pattern, UErrorCode& status);
+    U_I18N_API PluralFormat(const Locale& locale, const UnicodeString& pattern, UErrorCode& status);
 
     /**
      * Creates a new <code>PluralFormat</code> for a given set of rules, a
@@ -258,9 +244,7 @@ public:
      *                 must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const PluralRules& rules,
-                 const UnicodeString& pattern,
-                 UErrorCode& status);
+    U_I18N_API PluralFormat(const PluralRules& rules, const UnicodeString& pattern, UErrorCode& status);
 
     /**
      * Creates a new <code>PluralFormat</code> for a given set of rules, a
@@ -276,22 +260,40 @@ public:
      *                must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    PluralFormat(const Locale& locale,
-                 const PluralRules& rules,
-                 const UnicodeString& pattern,
-                 UErrorCode& status);
+    U_I18N_API PluralFormat(const Locale& locale,
+                            const PluralRules& rules,
+                            const UnicodeString& pattern,
+                            UErrorCode& status);
+
+    /**
+     * Creates a new <code>PluralFormat</code> for a plural type, a
+     * pattern and a locale.
+     * @param locale  the <code>PluralFormat</code> will be configured with
+     *                rules for this locale. This locale will also be used for
+     *                standard number formatting.
+     * @param type    The plural type (e.g., cardinal or ordinal).
+     * @param pattern the pattern for this <code>PluralFormat</code>.
+     *                errors are returned to status if the pattern is invalid.
+     * @param status  output param set to success/failure code on exit, which
+     *                must not indicate a failure before the function call.
+     * @stable ICU 50
+     */
+    U_I18N_API PluralFormat(const Locale& locale,
+                            UPluralType type,
+                            const UnicodeString& pattern,
+                            UErrorCode& status);
 
     /**
       * copy constructor.
       * @stable ICU 4.0
       */
-    PluralFormat(const PluralFormat& other);
+    U_I18N_API PluralFormat(const PluralFormat& other);
 
     /**
      * Destructor.
      * @stable ICU 4.0
      */
-    virtual ~PluralFormat();
+    U_I18N_API virtual ~PluralFormat();
 
     /**
      * Sets the pattern used by this plural format.
@@ -305,8 +307,7 @@ public:
      *                must not indicate a failure before the function call.
      * @stable ICU 4.0
      */
-    void applyPattern(const UnicodeString& pattern, UErrorCode& status);
-
+    U_I18N_API void applyPattern(const UnicodeString& pattern, UErrorCode& status);
 
     using Format::format;
 
@@ -322,7 +323,7 @@ public:
      * @return        the string containing the formatted plural message.
      * @stable ICU 4.0
      */
-    UnicodeString format(int32_t number, UErrorCode& status) const;
+    U_I18N_API UnicodeString format(int32_t number, UErrorCode& status) const;
 
     /**
      * Formats a plural message for a given number.
@@ -336,7 +337,7 @@ public:
      * @return        the string containing the formatted plural message.
      * @stable ICU 4.0
      */
-    UnicodeString format(double number, UErrorCode& status) const;
+    U_I18N_API UnicodeString format(double number, UErrorCode& status) const;
 
     /**
      * Formats a plural message for a given number.
@@ -354,10 +355,10 @@ public:
      * @return         the string containing the formatted plural message.
      * @stable ICU 4.0
      */
-    UnicodeString& format(int32_t number,
-                          UnicodeString& appendTo,
-                          FieldPosition& pos,
-                          UErrorCode& status) const;
+    U_I18N_API UnicodeString& format(int32_t number,
+                                     UnicodeString& appendTo,
+                                     FieldPosition& pos,
+                                     UErrorCode& status) const;
 
     /**
      * Formats a plural message for a given number.
@@ -375,24 +376,29 @@ public:
      * @return         the string containing the formatted plural message.
      * @stable ICU 4.0
      */
-    UnicodeString& format(double number,
-                          UnicodeString& appendTo,
-                          FieldPosition& pos,
-                          UErrorCode& status) const;
+    U_I18N_API UnicodeString& format(double number,
+                                     UnicodeString& appendTo,
+                                     FieldPosition& pos,
+                                     UErrorCode& status) const;
 
+#ifndef U_HIDE_DEPRECATED_API 
     /**
      * Sets the locale used by this <code>PluraFormat</code> object.
      * Note: Calling this method resets this <code>PluraFormat</code> object,
      *     i.e., a pattern that was applied previously will be removed,
      *     and the NumberFormat is set to the default number format for
      *     the locale.  The resulting format behaves the same as one
-     *     constructed from {@link #PluralFormat(const Locale& locale, UErrorCode& status)}.
+     *     constructed from {@link #PluralFormat(const Locale& locale, UPluralType type, UErrorCode& status)}
+     *     with UPLURAL_TYPE_CARDINAL.
      * @param locale  the <code>locale</code> to use to configure the formatter.
      * @param status  output param set to success/failure code on exit, which
      *                must not indicate a failure before the function call.
-     * @stable ICU 4.0
+     * @deprecated ICU 50 This method clears the pattern and might create
+     *             a different kind of PluralRules instance;
+     *             use one of the constructors to create a new instance instead.
      */
-    void setLocale(const Locale& locale, UErrorCode& status);
+    U_I18N_API void setLocale(const Locale& locale, UErrorCode& status);
+#endif  /* U_HIDE_DEPRECATED_API */
 
     /**
       * Sets the number format used by this formatter.  You only need to
@@ -403,7 +409,7 @@ public:
       *                must not indicate a failure before the function call.
       * @stable ICU 4.0
       */
-    void setNumberFormat(const NumberFormat* format, UErrorCode& status);
+    U_I18N_API void setNumberFormat(const NumberFormat* format, UErrorCode& status);
 
     /**
        * Assignment operator
@@ -411,7 +417,7 @@ public:
        * @param other    the PluralFormat object to copy from.
        * @stable ICU 4.0
        */
-    PluralFormat& operator=(const PluralFormat& other);
+    U_I18N_API PluralFormat& operator=(const PluralFormat& other);
 
     /**
       * Return true if another object is semantically equal to this one.
@@ -420,7 +426,7 @@ public:
       * @return         true if other is semantically equal to this.
       * @stable ICU 4.0
       */
-    virtual UBool operator==(const Format& other) const;
+    U_I18N_API virtual bool operator==(const Format& other) const override;
 
     /**
      * Return true if another object is semantically unequal to this one.
@@ -429,67 +435,69 @@ public:
      * @return         true if other is semantically unequal to this.
      * @stable ICU 4.0
      */
-    virtual UBool operator!=(const Format& other) const;
+    U_I18N_API virtual bool operator!=(const Format& other) const;
 
     /**
      * Clones this Format object polymorphically.  The caller owns the
      * result and should delete it when done.
      * @stable ICU 4.0
      */
-    virtual Format* clone(void) const;
+    U_I18N_API virtual PluralFormat* clone() const override;
 
     /**
-    * Redeclared Format method.
-    *
-    * @param obj       The object to be formatted into a string.
-    * @param appendTo  output parameter to receive result.
-    *                  Result is appended to existing contents.
-    * @param pos       On input: an alignment field, if desired.
-    *                  On output: the offsets of the alignment field.
-    * @param status    output param filled with success/failure status.
-    * @return          Reference to 'appendTo' parameter.
-    * @stable ICU 4.0
-    */
-   UnicodeString& format(const Formattable& obj,
-                         UnicodeString& appendTo,
-                         FieldPosition& pos,
-                         UErrorCode& status) const;
-
-   /**
-    * Returns the pattern from applyPattern() or constructor().
-    *
-    * @param  appendTo  output parameter to receive result.
+     * Formats a plural message for a number taken from a Formattable object.
+     *
+     * @param obj       The object containing a number for which the
+     *                  plural message should be formatted.
+     *                  The object must be of a numeric type.
+     * @param appendTo  output parameter to receive result.
      *                  Result is appended to existing contents.
-    * @return the UnicodeString with inserted pattern.
-    * @stable ICU 4.0
-    */
-   UnicodeString& toPattern(UnicodeString& appendTo);
+     * @param pos       On input: an alignment field, if desired.
+     *                  On output: the offsets of the alignment field.
+     * @param status    output param filled with success/failure status.
+     * @return          Reference to 'appendTo' parameter.
+     * @stable ICU 4.0
+     */
+    U_I18N_API UnicodeString& format(const Formattable& obj,
+                                     UnicodeString& appendTo,
+                                     FieldPosition& pos,
+                                     UErrorCode& status) const override;
 
-   /**
-    * This method is not yet supported by <code>PluralFormat</code>.
-    * <P>
-    * Before calling, set parse_pos.index to the offset you want to start
-    * parsing at in the source. After calling, parse_pos.index is the end of
-    * the text you parsed. If error occurs, index is unchanged.
-    * <P>
-    * When parsing, leading whitespace is discarded (with a successful parse),
-    * while trailing whitespace is left as is.
-    * <P>
-    * See Format::parseObject() for more.
-    *
-    * @param source    The string to be parsed into an object.
-    * @param result    Formattable to be set to the parse result.
-    *                  If parse fails, return contents are undefined.
-    * @param parse_pos The position to start parsing at. Upon return
-    *                  this param is set to the position after the
-    *                  last character successfully parsed. If the
-    *                  source is not parsed successfully, this param
-    *                  will remain unchanged.
-    * @stable ICU 4.0
-    */
-   virtual void parseObject(const UnicodeString& source,
-                            Formattable& result,
-                            ParsePosition& parse_pos) const;
+    /**
+     * Returns the pattern from applyPattern() or constructor().
+     *
+     * @param  appendTo  output parameter to receive result.
+     *                  Result is appended to existing contents.
+     * @return the UnicodeString with inserted pattern.
+     * @stable ICU 4.0
+     */
+    U_I18N_API UnicodeString& toPattern(UnicodeString& appendTo);
+
+    /**
+     * This method is not yet supported by <code>PluralFormat</code>.
+     * <P>
+     * Before calling, set parse_pos.index to the offset you want to start
+     * parsing at in the source. After calling, parse_pos.index is the end of
+     * the text you parsed. If error occurs, index is unchanged.
+     * <P>
+     * When parsing, leading whitespace is discarded (with a successful parse),
+     * while trailing whitespace is left as is.
+     * <P>
+     * See Format::parseObject() for more.
+     *
+     * @param source    The string to be parsed into an object.
+     * @param result    Formattable to be set to the parse result.
+     *                  If parse fails, return contents are undefined.
+     * @param parse_pos The position to start parsing at. Upon return
+     *                  this param is set to the position after the
+     *                  last character successfully parsed. If the
+     *                  source is not parsed successfully, this param
+     *                  will remain unchanged.
+     * @stable ICU 4.0
+     */
+    U_I18N_API virtual void parseObject(const UnicodeString& source,
+                                        Formattable& result,
+                                        ParsePosition& parse_pos) const override;
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for this class.
@@ -497,48 +505,90 @@ public:
      * @stable ICU 4.0
      *
      */
-    static UClassID U_EXPORT2 getStaticClassID(void);
+    U_I18N_API static UClassID getStaticClassID();
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for the actual class.
      *
      * @stable ICU 4.0
      */
-     virtual UClassID getDynamicClassID() const;
+    U_I18N_API virtual UClassID getDynamicClassID() const override;
 
 private:
-    typedef enum fmtToken {
-        none,
-        tLetter,
-        tNumber,
-        tSpace,
-        tNumberSign,
-        tLeftBrace,
-        tRightBrace
-    }fmtToken;
+    class PluralSelector : public UMemory {
+      public:
+        virtual ~PluralSelector();
+        /**
+         * Given a number, returns the appropriate PluralFormat keyword.
+         *
+         * @param context worker object for the selector.
+         * @param number The number to be plural-formatted.
+         * @param ec Error code.
+         * @return The selected PluralFormat keyword.
+         */
+        virtual UnicodeString select(void *context, double number, UErrorCode& ec) const = 0;
+    };
+
+    class PluralSelectorAdapter : public PluralSelector {
+      public:
+        PluralSelectorAdapter() : pluralRules(nullptr) {
+        }
+
+        virtual ~PluralSelectorAdapter();
+
+        virtual UnicodeString select(void *context, double number, UErrorCode& /*ec*/) const override;
+
+        void reset();
+
+        PluralRules* pluralRules;
+    };
 
     Locale  locale;
-    PluralRules* pluralRules;
-    UnicodeString pattern;
-    Hashtable  *fParsedValuesHash;
+    MessagePattern msgPattern;
     NumberFormat*  numberFormat;
-    NumberFormat*  replacedNumberFormat;
+    double offset;
+    PluralSelectorAdapter pluralRulesWrapper;
 
-    PluralFormat();   // default constructor not implemented
-    void init(const PluralRules* rules, const Locale& curlocale, UErrorCode& status);
-    UBool inRange(UChar ch, fmtToken& type);
-    UBool checkSufficientDefinition();
-    void parsingFailure();
-    UnicodeString insertFormattedNumber(double number,
-                                        UnicodeString& message,
-                                        UnicodeString& appendTo,
-                                        FieldPosition& pos) const;
-    void copyHashtable(Hashtable *other, UErrorCode& status);
+    PluralFormat() = delete;   // default constructor not implemented
+    void init(const PluralRules* rules, UPluralType type, UErrorCode& status);
+    /**
+     * Copies dynamically allocated values (pointer fields).
+     * Others are copied using their copy constructors and assignment operators.
+     */
+    void copyObjects(const PluralFormat& other);
+
+    UnicodeString& format(const Formattable& numberObject, double number,
+                          UnicodeString& appendTo,
+                          FieldPosition& pos,
+                          UErrorCode& status) const;
+
+    /**
+     * Finds the PluralFormat sub-message for the given number, or the "other" sub-message.
+     * @param pattern A MessagePattern.
+     * @param partIndex the index of the first PluralFormat argument style part.
+     * @param selector the PluralSelector for mapping the number (minus offset) to a keyword.
+     * @param context worker object for the selector.
+     * @param number a number to be matched to one of the PluralFormat argument's explicit values,
+     *        or mapped via the PluralSelector.
+     * @param ec ICU error code.
+     * @return the sub-message start part index.
+     */
+    static int32_t findSubMessage(
+         const MessagePattern& pattern, int32_t partIndex,
+         const PluralSelector& selector, void *context, double number, UErrorCode& ec);
+
+    void parseType(const UnicodeString& source, const NFRule *rbnfLenientScanner,
+        Formattable& result, FieldPosition& pos) const;
+
+    friend class MessageFormat;
+    friend class NFRule;
 };
 
 U_NAMESPACE_END
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
+#endif /* U_SHOW_CPLUSPLUS_API */
+
 #endif // _PLURFMT
 //eof
