@@ -79,11 +79,11 @@ SPUStringPool::SPUStringPool(UErrorCode &status) : fVec(nullptr), fHash(nullptr)
         return;
     }
     vec->setDeleter(
-        [](void *obj) {delete (SPUString *)obj;});
+        [](void *obj) {delete static_cast<SPUString*>(obj);});
     fVec = vec.orphan();
     fHash = uhash_open(uhash_hashUnicodeString,           // key hash function
                        uhash_compareUnicodeString,        // Key Comparator
-                       NULL,                              // Value Comparator
+                       nullptr,                              // Value Comparator
                        &status);
 }
 
@@ -99,7 +99,7 @@ int32_t SPUStringPool::size() {
 }
 
 SPUString *SPUStringPool::getByIndex(int32_t index) {
-    SPUString *retString = (SPUString *)fVec->elementAt(index);
+    SPUString* retString = static_cast<SPUString*>(fVec->elementAt(index));
     return retString;
 }
 
@@ -153,43 +153,43 @@ SPUString *SPUStringPool::addString(UnicodeString *src, UErrorCode &status) {
 
 ConfusabledataBuilder::ConfusabledataBuilder(SpoofImpl *spImpl, UErrorCode &status) :
     fSpoofImpl(spImpl),
-    fInput(NULL),
-    fTable(NULL),
-    fKeySet(NULL),
-    fKeyVec(NULL),
-    fValueVec(NULL),
-    fStringTable(NULL),
-    stringPool(NULL),
-    fParseLine(NULL),
-    fParseHexNum(NULL),
+    fInput(nullptr),
+    fTable(nullptr),
+    fKeySet(nullptr),
+    fKeyVec(nullptr),
+    fValueVec(nullptr),
+    fStringTable(nullptr),
+    stringPool(nullptr),
+    fParseLine(nullptr),
+    fParseHexNum(nullptr),
     fLineNum(0)
 {
     if (U_FAILURE(status)) {
         return;
     }
 
-    fTable = uhash_open(uhash_hashLong, uhash_compareLong, NULL, &status);
+    fTable = uhash_open(uhash_hashLong, uhash_compareLong, nullptr, &status);
 
     fKeySet = new UnicodeSet();
-    if (fKeySet == NULL) {
+    if (fKeySet == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
 
     fKeyVec = new UVector(status);
-    if (fKeyVec == NULL) {
+    if (fKeyVec == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
 
     fValueVec = new UVector(status);
-    if (fValueVec == NULL) {
+    if (fValueVec == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
 
     stringPool = new SPUStringPool(status);
-    if (stringPool == NULL) {
+    if (stringPool == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
@@ -217,7 +217,7 @@ void ConfusabledataBuilder::buildConfusableData(SpoofImpl * spImpl, const char *
     }
     ConfusabledataBuilder builder(spImpl, status);
     builder.build(confusables, confusablesLen, status);
-    if (U_FAILURE(status) && errorType != NULL) {
+    if (U_FAILURE(status) && errorType != nullptr) {
         *errorType = USPOOF_SINGLE_SCRIPT_CONFUSABLE;
         pe->line = builder.fLineNum;
     }
@@ -227,22 +227,25 @@ void ConfusabledataBuilder::buildConfusableData(SpoofImpl * spImpl, const char *
 void ConfusabledataBuilder::build(const char * confusables, int32_t confusablesLen,
                UErrorCode &status) {
 
-    // Convert the user input data from UTF-8 to UChar (UTF-16)
+    // Convert the user input data from UTF-8 to char16_t (UTF-16)
     int32_t inputLen = 0;
     if (U_FAILURE(status)) {
         return;
     }
-    u_strFromUTF8(NULL, 0, &inputLen, confusables, confusablesLen, &status);
-    if (status != U_BUFFER_OVERFLOW_ERROR) {
+    UErrorCode bufferStatus = U_ZERO_ERROR;
+    u_strFromUTF8(nullptr, 0, &inputLen, confusables, confusablesLen, &bufferStatus);
+    if (bufferStatus != U_BUFFER_OVERFLOW_ERROR) {
+        if (U_FAILURE(bufferStatus)) {
+            status = bufferStatus;
+        }
         return;
     }
-    status = U_ZERO_ERROR;
-    fInput = static_cast<UChar *>(uprv_malloc((inputLen+1) * sizeof(UChar)));
-    if (fInput == NULL) {
+    fInput = static_cast<char16_t *>(uprv_malloc((inputLen+1) * sizeof(char16_t)));
+    if (fInput == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
-    u_strFromUTF8(fInput, inputLen+1, NULL, confusables, confusablesLen, &status);
+    u_strFromUTF8(fInput, inputLen+1, nullptr, confusables, confusablesLen, &status);
 
 
     // Regular Expression to parse a line from Confusables.txt.  The expression will match
@@ -263,12 +266,12 @@ void ConfusabledataBuilder::build(const char * confusables, int32_t confusablesL
         "|^([ \\t]*(?:#.*?)?)$"       // OR match empty lines or lines with only a #comment
         "|^(.*?)$", -1, US_INV);      // OR match any line, which catches illegal lines.
     // TODO: Why are we using the regex C API here? C++ would just take UnicodeString...
-    fParseLine = uregex_open(pattern.getBuffer(), pattern.length(), 0, NULL, &status);
+    fParseLine = uregex_open(pattern.getBuffer(), pattern.length(), 0, nullptr, &status);
 
     // Regular expression for parsing a hex number out of a space-separated list of them.
     //   Capture group 1 gets the number, with spaces removed.
     pattern = UNICODE_STRING_SIMPLE("\\s*([0-9A-F]+)");
-    fParseHexNum = uregex_open(pattern.getBuffer(), pattern.length(), 0, NULL, &status);
+    fParseHexNum = uregex_open(pattern.getBuffer(), pattern.length(), 0, nullptr, &status);
 
     // Zap any Byte Order Mark at the start of input.  Changing it to a space is benign
     //   given the syntax of the input.
@@ -300,7 +303,7 @@ void ConfusabledataBuilder::build(const char * confusables, int32_t confusablesL
         uregex_setText(fParseHexNum, &fInput[mapStringStart], mapStringLength, &status);
 
         UnicodeString  *mapString = new UnicodeString();
-        if (mapString == NULL) {
+        if (mapString == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             return;
         }
@@ -370,7 +373,7 @@ void ConfusabledataBuilder::build(const char * confusables, int32_t confusablesL
         for (UChar32 keyChar=fKeySet->getRangeStart(range);
                 keyChar <= fKeySet->getRangeEnd(range); keyChar++) {
             SPUString *targetMapping = static_cast<SPUString *>(uhash_iget(fTable, keyChar));
-            U_ASSERT(targetMapping != NULL);
+            U_ASSERT(targetMapping != nullptr);
 
             // Set an error code if trying to consume a long string.  Otherwise,
             // codePointAndLengthToKey will abort on a U_ASSERT.
@@ -393,7 +396,6 @@ void ConfusabledataBuilder::build(const char * confusables, int32_t confusablesL
 
     // All of the intermediate allocated data belongs to the ConfusabledataBuilder
     //  object  (this), and is deleted in the destructor.
-    return;
 }
 
 //
@@ -407,7 +409,7 @@ void ConfusabledataBuilder::build(const char * confusables, int32_t confusablesL
 //
 void ConfusabledataBuilder::outputData(UErrorCode &status) {
 
-    U_ASSERT(fSpoofImpl->fSpoofData->fDataOwned == true);
+    U_ASSERT(fSpoofImpl->fSpoofData->fDataOwned);
 
     //  The Key Table
     //     While copying the keys to the runtime array,
@@ -431,7 +433,7 @@ void ConfusabledataBuilder::outputData(UErrorCode &status) {
         previousCodePoint = codePoint;
     }
     SpoofDataHeader *rawData = fSpoofImpl->fSpoofData->fRawData;
-    rawData->fCFUKeys = (int32_t)((char *)keys - (char *)rawData);
+    rawData->fCFUKeys = static_cast<int32_t>(reinterpret_cast<char*>(keys) - reinterpret_cast<char*>(rawData));
     rawData->fCFUKeysSize = numKeys;
     fSpoofImpl->fSpoofData->fCFUKeys = keys;
 
@@ -450,7 +452,7 @@ void ConfusabledataBuilder::outputData(UErrorCode &status) {
         values[i] = static_cast<uint16_t>(value);
     }
     rawData = fSpoofImpl->fSpoofData->fRawData;
-    rawData->fCFUStringIndex = (int32_t)((char *)values - (char *)rawData);
+    rawData->fCFUStringIndex = static_cast<int32_t>(reinterpret_cast<char*>(values) - reinterpret_cast<char*>(rawData));
     rawData->fCFUStringIndexSize = numValues;
     fSpoofImpl->fSpoofData->fCFUValues = values;
 
@@ -459,15 +461,15 @@ void ConfusabledataBuilder::outputData(UErrorCode &status) {
     uint32_t stringsLength = fStringTable->length();
     // Reserve an extra space so the string will be nul-terminated.  This is
     // only a convenience, for when debugging; it is not needed otherwise.
-    UChar *strings =
-        static_cast<UChar *>(fSpoofImpl->fSpoofData->reserveSpace(stringsLength*sizeof(UChar)+2, status));
+    char16_t *strings =
+        static_cast<char16_t *>(fSpoofImpl->fSpoofData->reserveSpace(stringsLength*sizeof(char16_t)+2, status));
     if (U_FAILURE(status)) {
         return;
     }
     fStringTable->extract(strings, stringsLength+1, status);
     rawData = fSpoofImpl->fSpoofData->fRawData;
     U_ASSERT(rawData->fCFUStringTable == 0);
-    rawData->fCFUStringTable = (int32_t)((char *)strings - (char *)rawData);
+    rawData->fCFUStringTable = static_cast<int32_t>(reinterpret_cast<char*>(strings) - reinterpret_cast<char*>(rawData));
     rawData->fCFUStringTableLen = stringsLength;
     fSpoofImpl->fSpoofData->fCFUStrings = strings;
 }
