@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2014, International Business Machines Corporation and
+ * Copyright (c) 1997-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -26,6 +28,7 @@
 #include "unicode/uloc.h"
 #include "unicode/uchar.h"
 #include "unicode/ustring.h"
+#include "unicode/utf16.h"
 #include "unicode/putil.h"
 #include "callcoll.h"
 #include "cmemory.h"
@@ -36,6 +39,7 @@
 #include "cstring.h"
 #include "ucol_imp.h"
 #include "uparse.h"
+#include <stdbool.h>
 #include <stdio.h>
 
 extern uint8_t ucol_uprv_getCaseBits(const UChar *, uint32_t, UErrorCode *);
@@ -61,7 +65,7 @@ void addCollIterTest(TestNode** root)
 
 static const char * LOCALES[] = {"en_AU", "en_BE", "en_CA"};
 
-static void TestBug672() {
+static void TestBug672(void) {
     UErrorCode  status = U_ZERO_ERROR;
     UChar       pattern[20];
     UChar       text[50];
@@ -134,7 +138,7 @@ static void TestBug672() {
 
 /*  Running this test with normalization enabled showed up a bug in the incremental
     normalization code. */
-static void TestBug672Normalize() {
+static void TestBug672Normalize(void) {
     UErrorCode  status = U_ZERO_ERROR;
     UChar       pattern[20];
     UChar       text[50];
@@ -213,7 +217,7 @@ static void TestBug672Normalize() {
  * Test for CollationElementIterator previous and next for the whole set of
  * unicode characters.
  */
-static void TestUnicodeChar()
+static void TestUnicodeChar(void)
 {
     UChar source[0x100];
     UCollator *en_us;
@@ -279,7 +283,7 @@ static void TestUnicodeChar()
  * Test for CollationElementIterator previous and next for the whole set of
  * unicode characters with normalization on.
  */
-static void TestNormalizedUnicodeChar()
+static void TestNormalizedUnicodeChar(void)
 {
     UChar source[0x100];
     UCollator *th_th;
@@ -343,7 +347,7 @@ static void TestNormalizedUnicodeChar()
 /**
 * Test the incremental normalization
 */
-static void TestNormalization()
+static void TestNormalization(void)
 {
           UErrorCode          status = U_ZERO_ERROR;
     const char               *str    =
@@ -414,7 +418,7 @@ static void TestNormalization()
  * @bug 4108758 - Make sure it works with contracting characters
  *
  */
-static void TestPrevious()
+static void TestPrevious(void)
 {
     UCollator *coll=NULL;
     UChar rule[50];
@@ -554,7 +558,7 @@ static void TestPrevious()
 /**
  * Test for getOffset() and setOffset()
  */
-static void TestOffset()
+static void TestOffset(void)
 {
     UErrorCode status= U_ZERO_ERROR;
     UCollator *en_us=NULL;
@@ -699,7 +703,7 @@ static void TestOffset()
 /**
  * Test for setText()
  */
-static void TestSetText()
+static void TestSetText(void)
 {
     int32_t c,i;
     UErrorCode status = U_ZERO_ERROR;
@@ -772,7 +776,7 @@ static void TestSetText()
 /** @bug 4108762
  * Test for getMaxExpansion()
  */
-static void TestMaxExpansion()
+static void TestMaxExpansion(void)
 {
     UErrorCode          status = U_ZERO_ERROR;
     UCollator          *coll   ;/*= ucol_open("en_US", &status);*/
@@ -780,7 +784,7 @@ static void TestMaxExpansion()
     UChar32             unassigned = 0xEFFFD;
     UChar               supplementary[2];
     uint32_t            stringOffset = 0;
-    UBool               isError = FALSE;
+    UBool               isError = false;
     uint32_t            sorder = 0;
     UCollationElements *iter   ;/*= ucol_openElements(coll, &ch, 1, &status);*/
     uint32_t            temporder = 0;
@@ -930,7 +934,7 @@ static void assertEqual(UCollationElements *i1, UCollationElements *i2)
 /**
  * Testing iterators with extremely small buffers
  */
-static void TestSmallBuffer()
+static void TestSmallBuffer(void)
 {
     UErrorCode          status = U_ZERO_ERROR;
     UCollator          *coll;
@@ -1001,9 +1005,9 @@ static void TestSmallBuffer()
 }
 
 /**
-* Testing the discontigous contractions
+* Testing the discontiguous contractions
 */
-static void TestDiscontiguos() {
+static void TestDiscontiguos(void) {
     const char               *rulestr    =
                             "&z < AB < X\\u0300 < ABC < X\\u0300\\u0315";
           UChar               rule[50];
@@ -1073,7 +1077,7 @@ static void TestDiscontiguos() {
         log_verbose("count %d\n", count);
 
         for (;;) {
-            uint32_t  ce;
+            int32_t  ce;
             UChar    *e = u_strchr(s, 0x20);
             if (e == 0) {
                 e = u_strchr(s, 0);
@@ -1085,9 +1089,9 @@ static void TestDiscontiguos() {
                 return;
             }
             while (ce != UCOL_NULLORDER) {
-                if (ce != (uint32_t)ucol_next(iter, &status) ||
+                if (ce != ucol_next(iter, &status) ||
                     U_FAILURE(status)) {
-                    log_err("Discontiguos contraction test mismatch\n");
+                    log_err("Discontiguous contraction test mismatch\n");
                     return;
                 }
                 ce = ucol_next(resultiter, &status);
@@ -1134,7 +1138,7 @@ static const UChar tsceText[] = {   /* Nothing in here should be ignorable */
     0x0020, 0x1E4D,                 /* small letter o with tilde and acute, decomposes */
     0x0020
 };
-enum { kLen_tsceText = sizeof(tsceText)/sizeof(tsceText[0]) };
+enum { kLen_tsceText = UPRV_LENGTHOF(tsceText) };
 
 static const int32_t rootStandardOffsets[] = {
     0,  1,2,
@@ -1150,7 +1154,7 @@ static const int32_t rootStandardOffsets[] = {
     28,
     29
 };
-enum { kLen_rootStandardOffsets = sizeof(rootStandardOffsets)/sizeof(rootStandardOffsets[0]) };
+enum { kLen_rootStandardOffsets = UPRV_LENGTHOF(rootStandardOffsets) };
 
 static const int32_t rootSearchOffsets[] = {
     0,  1,2,
@@ -1166,7 +1170,7 @@ static const int32_t rootSearchOffsets[] = {
     28,
     29
 };
-enum { kLen_rootSearchOffsets = sizeof(rootSearchOffsets)/sizeof(rootSearchOffsets[0]) };
+enum { kLen_rootSearchOffsets = UPRV_LENGTHOF(rootSearchOffsets) };
 
 typedef struct {
     const char *    locale;
