@@ -36,7 +36,7 @@
 #include "string.h"
 
 static UCollator *myCollation;
-const static UChar testSourceCases[][MAX_TOKEN_LEN] = {
+static const UChar testSourceCases[][MAX_TOKEN_LEN] = {
     {0x0073/*'s'*/, 0x0327, 0x0000},
     {0x0076/*'v'*/, 0x00E4, 0x0074/*'t'*/, 0x0000},
     {0x006f/*'o'*/, 0x006c/*'l'*/, 0x0064/*'d'*/, 0x0000},
@@ -50,7 +50,7 @@ const static UChar testSourceCases[][MAX_TOKEN_LEN] = {
     {0x0069/*'i'*/, 0x0064/*'d'*/, 0x0065/*'e'*/, 0x0061/*'a'*/, 0x0000},
 };
 
-const static UChar testTargetCases[][MAX_TOKEN_LEN] = {
+static const UChar testTargetCases[][MAX_TOKEN_LEN] = {
     {0x0075/*'u'*/, 0x0308, 0x0000},
     {0x0076/*'v'*/, 0x0062/*'b'*/, 0x0074/*'t'*/, 0x0000},
     {0x00D6, 0x0061/*'a'*/, 0x0079/*'y'*/, 0x0000},
@@ -64,7 +64,7 @@ const static UChar testTargetCases[][MAX_TOKEN_LEN] = {
     {0x0049/*'I'*/, 0x0064/*'d'*/, 0x0065/*'e'*/, 0x0061/*'a'*/, 0x0000},
 };
 
-const static UCollationResult results[] = {
+static const UCollationResult results[] = {
     UCOL_LESS,
     UCOL_LESS,
     UCOL_LESS,
