@@ -76,10 +76,10 @@ static void setUpDataTable(void)
 {
     int32_t i,j;
     if(dataTable == NULL) {
-        dataTable = (UChar***)calloc(sizeof(UChar**),3);
+        dataTable = (UChar***)calloc(3, sizeof(UChar**));
 
             for (i = 0; i < 3; i++) {
-              dataTable[i] = (UChar**)calloc(sizeof(UChar*),4);
+              dataTable[i] = (UChar**)calloc(4, sizeof(UChar*));
                 for (j = 0; j < 4; j++){
                     dataTable[i][j] = (UChar*) malloc(sizeof(UChar)*(strlen(raw[i][j])+1));
                     u_uastrcpy(dataTable[i][j],raw[i][j]);
