@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2012, International Business Machines Corporation and
+ * Copyright (c) 1997-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ***************************************************************************/
 /*****************************************************************************
@@ -9,9 +11,10 @@
 *
 * Modification History:
 *      Name              Date                  Description            
-* Madhu Katragadda    06/23/2000     Tests for Conveter FallBack API and Functionality
+* Madhu Katragadda    06/23/2000     Tests for Converter FallBack API and Functionality
 ******************************************************************************
 */
+#include <stdbool.h>
 #include <stdio.h>
 #include "unicode/uloc.h"
 #include "unicode/ucnv.h"
@@ -118,7 +121,7 @@ void addTestConverterFallBack(TestNode** root)
 
 static void setNuConvTestName(const char *codepage, const char *direction)
 {
-    sprintf(gNuConvTestName, "[Testing %s %s Unicode, InputBufSiz=%d, OutputBufSiz=%d]",
+    snprintf(gNuConvTestName, sizeof(gNuConvTestName), "[Testing %s %s Unicode, InputBufSiz=%d, OutputBufSiz=%d]",
         codepage,
         direction,
         (int)gInBufferSize,
@@ -144,9 +147,9 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
     char *realBufferEnd;
     const UChar *realSourceEnd;
     const UChar *sourceLimit;
-    UBool checkOffsets = TRUE;
+    UBool checkOffsets = true;
     UBool doFlush;
-    UBool action=FALSE;
+    UBool action=false;
     char *p;
 
 
@@ -163,7 +166,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
     if(U_FAILURE(status))
     {
         log_data_err("Couldn't open converter %s\n",codepage);
-        return TRUE;
+        return true;
     }
 
     log_verbose("Converter opened..\n");
@@ -178,15 +181,15 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
     targ = junkout;
     offs = junokout;
 
-    realBufferSize = (sizeof(junkout)/sizeof(junkout[0]));
+    realBufferSize = UPRV_LENGTHOF(junkout);
     realBufferEnd = junkout + realBufferSize;
     realSourceEnd = source + sourceLen;
 
     if ( gOutBufferSize != realBufferSize )
-        checkOffsets = FALSE;
+        checkOffsets = false;
 
     if( gInBufferSize != NEW_MAX_BUFFER )
-        checkOffsets = FALSE;
+        checkOffsets = false;
 
     do
     {
@@ -198,16 +201,16 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
         if(targ == realBufferEnd)
         {
             log_err("Error, overflowed the real buffer while about to call fromUnicode! targ=%08lx %s", targ, gNuConvTestName);
-            return FALSE;
+            return false;
         }
-        log_verbose("calling fromUnicode @ SOURCE:%08lx to %08lx  TARGET: %08lx to %08lx, flush=%s\n", src,sourceLimit, targ,end, doFlush?"TRUE":"FALSE");
+        log_verbose("calling fromUnicode @ SOURCE:%08lx to %08lx  TARGET: %08lx to %08lx, flush=%s\n", src,sourceLimit, targ,end, doFlush?"true":"false");
 
 
         status = U_ZERO_ERROR;
 
         ucnv_fromUnicode (conv,
-                  (char **)&targ,
-                  (const char *)end,
+                  &targ,
+                  end,
                   &src,
                   sourceLimit,
                   checkOffsets ? offs : NULL,
@@ -219,7 +222,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
     if(U_FAILURE(status))
     {
         log_err("Problem doing toUnicode, errcode %d %s\n", myErrorName(status), gNuConvTestName);
-        return FALSE;
+        return false;
     }
 
     log_verbose("\nConversion done [%d uchars in -> %d chars out]. \nResult :",
@@ -233,8 +236,8 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
         offset_str[0] = 0;
         for(p = junkout;p<targ;p++)
         {
-            sprintf(junk + uprv_strlen(junk), "0x%02x, ", (0xFF) & (unsigned int)*p);
-            sprintf(offset_str + strlen(offset_str), "0x%02x, ", (0xFF) & (unsigned int)junokout[p-junkout]);
+            snprintf(junk + uprv_strlen(junk), sizeof(junk)-uprv_strlen(junk), "0x%02x, ", (0xFF) & (unsigned int)*p);
+            snprintf(offset_str + strlen(offset_str), sizeof(offset_str)-strlen(offset_str), "0x%02x, ", (0xFF) & (unsigned int)junokout[p-junkout]);
         }
 
         log_verbose(junk);
@@ -255,7 +258,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
         log_verbose("Expected %d chars out, got %d %s\n", expectLen, targ-junkout, gNuConvTestName);
         printSeqErr((const unsigned char*)junkout, (int32_t)(targ-junkout));
         printSeqErr((const unsigned char*)expect, expectLen);
-        return FALSE;
+        return false;
     }
 
     if (checkOffsets && (expectOffsets != 0) )
@@ -277,7 +280,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
     if(!memcmp(junkout, expect, expectLen))
     {
         log_verbose("Matches!\n");
-        return TRUE;
+        return true;
     }
     else
     {
@@ -285,7 +288,7 @@ static UBool testConvertFromUnicode(const UChar *source, int sourceLen,  const u
         log_verbose("String does not match. %s\n", gNuConvTestName);
         printSeqErr((const unsigned char*)junkout, expectLen);
         printSeqErr((const unsigned char*)expect, expectLen);
-        return FALSE;
+        return false;
     }
 }
 
@@ -303,7 +306,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     UChar *end;
     int32_t *offs;
     int i;
-    UBool   checkOffsets = TRUE;
+    UBool   checkOffsets = true;
     char junk[9999];
     char offset_str[9999];
     UChar *p;
@@ -327,7 +330,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     if(U_FAILURE(status))
     {
         log_data_err("Couldn't open converter %s\n",gNuConvTestName);
-        return TRUE; /* because it has been logged */
+        return true; /* because it has been logged */
     }
 
     log_verbose("Converter opened..\n");
@@ -336,7 +339,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     targ = junkout;
     offs = junokout;
 
-    realBufferSize = (sizeof(junkout)/sizeof(junkout[0]));
+    realBufferSize = UPRV_LENGTHOF(junkout);
     realBufferEnd = junkout + realBufferSize;
     realSourceEnd = src + sourcelen;
     /*----setting the fallback routine----*/
@@ -347,10 +350,10 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     }
     /*-------------------------------------*/
     if ( gOutBufferSize != realBufferSize )
-      checkOffsets = FALSE;
+      checkOffsets = false;
 
     if( gInBufferSize != NEW_MAX_BUFFER )
-      checkOffsets = FALSE;
+      checkOffsets = false;
 
     do
     {
@@ -360,7 +363,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
         if(targ == realBufferEnd)
         {
             log_err("Error, the end would overflow the real output buffer while about to call toUnicode! tarjey=%08lx %s",targ,gNuConvTestName);
-            return FALSE;
+            return false;
         }
         log_verbose("calling toUnicode @ %08lx to %08lx\n", targ,end);
 
@@ -371,10 +374,10 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
         ucnv_toUnicode (conv,
                 &targ,
                 end,
-                (const char **)&src,
-                (const char *)srcLimit,
+                &src,
+                srcLimit,
                 checkOffsets ? offs : NULL,
-                (UBool)(srcLimit == realSourceEnd), /* flush if we're at the end of hte source data */
+                (UBool)(srcLimit == realSourceEnd), /* flush if we're at the end of the source data */
                 &status);
     } while ( (status == U_BUFFER_OVERFLOW_ERROR) || (srcLimit < realSourceEnd) ); /* while we just need another buffer */
 
@@ -382,7 +385,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     if(U_FAILURE(status))
     {
         log_err("Problem doing toUnicode, errcode %s %s\n", myErrorName(status), gNuConvTestName);
-        return FALSE;
+        return false;
     }
 
     log_verbose("\nConversion done. %d bytes -> %d chars.\nResult :",
@@ -395,8 +398,8 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
 
         for(p = junkout;p<targ;p++)
         {
-            sprintf(junk + strlen(junk), "0x%04x, ", (0xFFFF) & (unsigned int)*p);
-            sprintf(offset_str + strlen(offset_str), "0x%04x, ", (0xFFFF) & (unsigned int)junokout[p-junkout]);
+            snprintf(junk + strlen(junk), sizeof(junk)-strlen(junk), "0x%04x, ", (0xFFFF) & (unsigned int)*p);
+            snprintf(offset_str + strlen(offset_str), sizeof(offset_str)-strlen(offset_str), "0x%04x, ", (0xFFFF) & (unsigned int)junokout[p-junkout]);
         }
 
         log_verbose(junk);
@@ -435,7 +438,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
     if(!memcmp(junkout, expect, expectlen*2))
     {
         log_verbose("Matches!\n");
-        return TRUE;
+        return true;
     }
     else
     {
@@ -444,7 +447,7 @@ static UBool testConvertToUnicode( const uint8_t *source, int sourcelen, const U
         printUSeqErr(junkout, expectlen);
         printf("\n");
         printUSeqErr(expect, expectlen);
-        return FALSE;
+        return false;
     }
 }
 
@@ -542,37 +545,37 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
     gInBufferSize = insize;
     gOutBufferSize = outsize;
 
-    for(i=0; i<sizeof(nativeCodePage)/sizeof(nativeCodePage[0]); i++){
+    for(i=0; i<UPRV_LENGTHOF(nativeCodePage); i++){
         log_verbose("Testing %s\n", nativeCodePage[i]);
-        if(!testConvertFromUnicode(SBCSText, sizeof(SBCSText)/sizeof(SBCSText[0]),
-            expectedNative, sizeof(expectedNative), nativeCodePage[i], TRUE, toNativeOffs ))
+        if(!testConvertFromUnicode(SBCSText, UPRV_LENGTHOF(SBCSText),
+            expectedNative, sizeof(expectedNative), nativeCodePage[i], true, toNativeOffs ))
             log_err("u-> %s(SBCS) with FallBack did not match.\n", nativeCodePage[i]);
 
         if(!testConvertToUnicode(expectedNative, sizeof(expectedNative), 
-            retrievedSBCSText, sizeof(retrievedSBCSText)/sizeof(retrievedSBCSText[0]), nativeCodePage[i], TRUE, fromNativeoffs ))
+            retrievedSBCSText, UPRV_LENGTHOF(retrievedSBCSText), nativeCodePage[i], true, fromNativeoffs ))
             log_err("%s->u(SBCS) with Fallback did not match.\n", nativeCodePage[i]);
     }
     
     /*DBCS*/
-    if(!testConvertFromUnicode(DBCSText, sizeof(DBCSText)/sizeof(DBCSText[0]),
-        expectedIBM1363_DBCS, sizeof(expectedIBM1363_DBCS), "ibm-1363", TRUE, toIBM1363Offs_DBCS ))
+    if(!testConvertFromUnicode(DBCSText, UPRV_LENGTHOF(DBCSText),
+        expectedIBM1363_DBCS, sizeof(expectedIBM1363_DBCS), "ibm-1363", true, toIBM1363Offs_DBCS ))
        log_err("u-> ibm-1363(DBCS portion) with FallBack did not match.\n");
 
     if(!testConvertToUnicode(expectedIBM1363_DBCS, sizeof(expectedIBM1363_DBCS), 
-        retrievedDBCSText, sizeof(retrievedDBCSText)/sizeof(retrievedDBCSText[0]),"ibm-1363", TRUE, fromIBM1363offs_DBCS ))
+        retrievedDBCSText, UPRV_LENGTHOF(retrievedDBCSText),"ibm-1363", true, fromIBM1363offs_DBCS ))
         log_err("ibm-1363->u(DBCS portion) with Fallback did not match.\n");
 
   
     /*MBCS*/
-    if(!testConvertFromUnicode(MBCSText, sizeof(MBCSText)/sizeof(MBCSText[0]),
-        expectedIBM950, sizeof(expectedIBM950), "ibm-950", TRUE, toIBM950Offs ))
+    if(!testConvertFromUnicode(MBCSText, UPRV_LENGTHOF(MBCSText),
+        expectedIBM950, sizeof(expectedIBM950), "ibm-950", true, toIBM950Offs ))
        log_err("u-> ibm-950(MBCS) with FallBack did not match.\n");
 
     if(!testConvertToUnicode(expectedIBM950, sizeof(expectedIBM950), 
-        retrievedMBCSText, sizeof(retrievedMBCSText)/sizeof(retrievedMBCSText[0]),"ibm-950", TRUE, fromIBM950offs ))
+        retrievedMBCSText, UPRV_LENGTHOF(retrievedMBCSText),"ibm-950", true, fromIBM950offs ))
         log_err("ibm-950->u(MBCS) with Fallback did not match.\n");
     
-   /*commented untill data table is available*/
+   /*commented until data table is available*/
     log_verbose("toUnicode fallback with fallback data for MBCS\n");
     {
         const uint8_t IBM950input[] =   {  
@@ -584,10 +587,10 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         UChar expectedFallbackFalse[]= { 0x5165, 0x5165, 0x516b, 0x516b, 0x9ef9, 0x9ef9};
 
         if(!testConvertToUnicode(IBM950input, sizeof(IBM950input), 
-                expectedUnicodeText, sizeof(expectedUnicodeText)/sizeof(expectedUnicodeText[0]),"ibm-950", TRUE, fromIBM950inputOffs ))
+                expectedUnicodeText, UPRV_LENGTHOF(expectedUnicodeText),"ibm-950", true, fromIBM950inputOffs ))
             log_err("ibm-950->u(MBCS) with Fallback did not match.\n");
         if(!testConvertToUnicode(IBM950input, sizeof(IBM950input), 
-                expectedFallbackFalse, sizeof(expectedFallbackFalse)/sizeof(expectedFallbackFalse[0]),"ibm-950", FALSE, fromIBM950inputOffs ))
+                expectedFallbackFalse, UPRV_LENGTHOF(expectedFallbackFalse),"ibm-950", false, fromIBM950inputOffs ))
             log_err("ibm-950->u(MBCS) with Fallback  did not match.\n");
 
     }
@@ -603,11 +606,11 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         UChar expectedFallbackFalse[]= { 0x5C6E, 0x5C6E, 0x81FC, 0x81FC, 0x8278, 0x8278};
 
         if(!testConvertToUnicode(euc_tw_input, sizeof(euc_tw_input), 
-                expectedUnicodeText, sizeof(expectedUnicodeText)/sizeof(expectedUnicodeText[0]),"euc-tw", TRUE, from_euc_tw_offs ))
+                expectedUnicodeText, UPRV_LENGTHOF(expectedUnicodeText),"euc-tw", true, from_euc_tw_offs ))
             log_err("from euc-tw->u with Fallback did not match.\n");
 
         if(!testConvertToUnicode(euc_tw_input, sizeof(euc_tw_input), 
-                expectedFallbackFalse, sizeof(expectedFallbackFalse)/sizeof(expectedFallbackFalse[0]),"euc-tw", FALSE, from_euc_tw_offs ))
+                expectedFallbackFalse, UPRV_LENGTHOF(expectedFallbackFalse),"euc-tw", false, from_euc_tw_offs ))
             log_err("from euc-tw->u with Fallback false did not match.\n");
 
 
@@ -627,19 +630,19 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         int32_t to_euc_tw_offs []  =   {  0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 5, 5, 6, 6, 
             6, 6, 7, 7, 7, 7, 8, 8, 9, 9, 10, 10, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12};
 
-        if(!testConvertFromUnicode(inputText, sizeof(inputText)/sizeof(inputText[0]),
-                expected_euc_tw, sizeof(expected_euc_tw), "euc-tw", TRUE, to_euc_tw_offs ))
+        if(!testConvertFromUnicode(inputText, UPRV_LENGTHOF(inputText),
+                expected_euc_tw, sizeof(expected_euc_tw), "euc-tw", true, to_euc_tw_offs ))
             log_err("u-> euc-tw with FallBack did not match.\n");
 
     }
 
     /*MBCS 1363*/
-    if(!testConvertFromUnicode(MBCSText1363, sizeof(MBCSText1363)/sizeof(MBCSText1363[0]),
-        expectedIBM1363, sizeof(expectedIBM1363), "ibm-1363", TRUE, toIBM1363Offs ))
+    if(!testConvertFromUnicode(MBCSText1363, UPRV_LENGTHOF(MBCSText1363),
+        expectedIBM1363, sizeof(expectedIBM1363), "ibm-1363", true, toIBM1363Offs ))
        log_err("u-> ibm-1363(MBCS) with FallBack did not match.\n");
 
     if(!testConvertToUnicode(expectedIBM1363, sizeof(expectedIBM1363), 
-        retrievedMBCSText1363, sizeof(retrievedMBCSText1363)/sizeof(retrievedMBCSText1363[0]),"ibm-1363", TRUE, fromIBM1363offs ))
+        retrievedMBCSText1363, UPRV_LENGTHOF(retrievedMBCSText1363),"ibm-1363", true, fromIBM1363offs ))
         log_err("ibm-1363->u(MBCS) with Fallback did not match.\n");
 
 
@@ -657,13 +660,13 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         int32_t fromtest1Offs[]       = { 0, 1, 2, 3, 3, 4,5};
 
         /*from Unicode*/
-        if(!testConvertFromUnicode(unicodeInput, sizeof(unicodeInput)/sizeof(unicodeInput[0]),
-                expectedtest1, sizeof(expectedtest1), "@test1", TRUE, totest1Offs ))
+        if(!testConvertFromUnicode(unicodeInput, UPRV_LENGTHOF(unicodeInput),
+                expectedtest1, sizeof(expectedtest1), "@test1", true, totest1Offs ))
             log_err("u-> test1(MBCS conversion with single-byte) did not match.\n");
         
         /*to Unicode*/
         if(!testConvertToUnicode(test1input, sizeof(test1input),
-               expectedUnicode, sizeof(expectedUnicode)/sizeof(expectedUnicode[0]), "@test1", TRUE, fromtest1Offs ))
+               expectedUnicode, UPRV_LENGTHOF(expectedUnicode), "@test1", true, fromtest1Offs ))
             log_err("test1(MBCS conversion with single-byte) -> u  did not match.\n");
 
     }
@@ -684,13 +687,13 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         int32_t fromtest3Offs[]       = { 0, 1, 2, 3, 6, 6, 7, 7, 10, 13, 13, 16, 17};  
 
         /*from Unicode*/
-        if(!testConvertFromUnicode(unicodeInput, sizeof(unicodeInput)/sizeof(unicodeInput[0]),
-                expectedtest3, sizeof(expectedtest3), "@test3", TRUE, totest3Offs ))
+        if(!testConvertFromUnicode(unicodeInput, UPRV_LENGTHOF(unicodeInput),
+                expectedtest3, sizeof(expectedtest3), "@test3", true, totest3Offs ))
             log_err("u-> test3(MBCS conversion with three-byte) did not match.\n");
         
         /*to Unicode*/
         if(!testConvertToUnicode(test3input, sizeof(test3input),
-               expectedUnicode, sizeof(expectedUnicode)/sizeof(expectedUnicode[0]), "@test3", TRUE, fromtest3Offs ))
+               expectedUnicode, UPRV_LENGTHOF(expectedUnicode), "@test3", true, fromtest3Offs ))
             log_err("test3(MBCS conversion with three-byte) -> u  did not match.\n"); 
 
     }
@@ -719,13 +722,13 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
                 { 0, 1, 2, 3, 7, 7, 8, 8, 9, 9, 13, 17, 17, 21, 22,};
 
         /*from Unicode*/
-        if(!testConvertFromUnicode(unicodeInput, sizeof(unicodeInput)/sizeof(unicodeInput[0]),
-                expectedtest4, sizeof(expectedtest4), "@test4", TRUE, totest4Offs ))
+        if(!testConvertFromUnicode(unicodeInput, UPRV_LENGTHOF(unicodeInput),
+                expectedtest4, sizeof(expectedtest4), "@test4", true, totest4Offs ))
             log_err("u-> test4(MBCS conversion with four-byte) did not match.\n");
         
         /*to Unicode*/
         if(!testConvertToUnicode(test4input, sizeof(test4input),
-               expectedUnicode, sizeof(expectedUnicode)/sizeof(expectedUnicode[0]), "@test4", TRUE, fromtest4Offs ))
+               expectedUnicode, UPRV_LENGTHOF(expectedUnicode), "@test4", true, fromtest4Offs ))
             log_err("test4(MBCS conversion with four-byte) -> u  did not match.\n"); 
 
     }
@@ -738,12 +741,12 @@ static void TestConvertFallBackWithBufferSizes(int32_t outsize, int32_t insize )
         const UChar expectedUnicode[] = {0x203e,         0x2014,     0xff5c,    0x004c,    0x5f5e,         0x223c };
         int32_t fromtest1Offs[]       = {1,              3,          5,         8,         10,             12 };
         /*from Unicode*/
-        if(!testConvertFromUnicode(unicodeInput, sizeof(unicodeInput)/sizeof(unicodeInput[0]),
-                expectedtest1, sizeof(expectedtest1), "ibm-1371", TRUE, totest1Offs ))
+        if(!testConvertFromUnicode(unicodeInput, UPRV_LENGTHOF(unicodeInput),
+                expectedtest1, sizeof(expectedtest1), "ibm-1371", true, totest1Offs ))
             log_err("u-> ibm-1371(MBCS conversion with single-byte) did not match.,\n");
         /*to Unicode*/
         if(!testConvertToUnicode(test1input, sizeof(test1input),
-               expectedUnicode, sizeof(expectedUnicode)/sizeof(expectedUnicode[0]), "ibm-1371", TRUE, fromtest1Offs ))
+               expectedUnicode, UPRV_LENGTHOF(expectedUnicode), "ibm-1371", true, fromtest1Offs ))
             log_err("ibm-1371(MBCS conversion with single-byte) -> u  did not match.,\n");
     }
 
