@@ -53,19 +53,15 @@ static UnicodeString str(const char *input)
 
 
 CanonicalIteratorTest::CanonicalIteratorTest() :
-nameTrans(NULL), hexTrans(NULL)
+nameTrans(nullptr), hexTrans(nullptr)
 {
 }
 
 CanonicalIteratorTest::~CanonicalIteratorTest()
 {
 #if !UCONFIG_NO_TRANSLITERATION
-  if(nameTrans != NULL) {
     delete(nameTrans);
-  }
-  if(hexTrans != NULL) {
     delete(hexTrans);
-  }
 #endif
 }
 
@@ -97,7 +93,7 @@ void CanonicalIteratorTest::TestExhaustive() {
         s = i;
         characterTest(s, i, it);
 
-        s += (UChar32)0x0345; //"\\u0345";
+        s += static_cast<UChar32>(0x0345); //"\\u0345";
         characterTest(s, i, it);
     }
 }
@@ -201,7 +197,7 @@ void CanonicalIteratorTest::characterTest(UnicodeString &s, UChar32 ch, Canonica
     }
     
     if (!gotSource || !gotDecomp || !gotComp) {
-        errln("FAIL CanonicalIterator: " + s + (int)ch);
+        errln("FAIL CanonicalIterator: " + s + static_cast<int>(ch));
     }
 }
 
@@ -224,14 +220,14 @@ UnicodeString CanonicalIteratorTest::getReadable(const UnicodeString &s) {
     // set up for readable display
 #if !UCONFIG_NO_TRANSLITERATION
     if(verbose) {
-      if (nameTrans == NULL)
+      if (nameTrans == nullptr)
           nameTrans = Transliterator::createInstance("[^\\ -\\u007F] name", UTRANS_FORWARD, status);
       UnicodeString sName = s;
       nameTrans->transliterate(sName);
       result += sName;
       result += ";";
     }
-    if (hexTrans == NULL)
+    if (hexTrans == nullptr)
         hexTrans = Transliterator::createInstance("[^\\ -\\u007F] hex", UTRANS_FORWARD, status);
 #endif
     UnicodeString sHex = s;
@@ -248,8 +244,8 @@ UnicodeString CanonicalIteratorTest::getReadable(const UnicodeString &s) {
 
 U_CFUNC int U_CALLCONV
 compareUnicodeStrings(const void *s1, const void *s2) {
-  UnicodeString **st1 = (UnicodeString **)s1;
-  UnicodeString **st2 = (UnicodeString **)s2;
+  UnicodeString **st1 = static_cast<UnicodeString **>(const_cast<void*>(s1));
+  UnicodeString **st2 = static_cast<UnicodeString **>(const_cast<void*>(s2));
 
   return (*st1)->compare(**st2);
 }
@@ -263,14 +259,14 @@ UnicodeString CanonicalIteratorTest::collectionToString(Hashtable *col) {
     UnicodeString **resArray = new UnicodeString*[col->count()];
     int32_t i = 0;
 
-    const UHashElement *ne = NULL;
+    const UHashElement *ne = nullptr;
     int32_t el = UHASH_FIRST;
     //Iterator it = basic.iterator();
     ne = col->nextElement(el);
     //while (it.hasNext()) 
-    while (ne != NULL) {
+    while (ne != nullptr) {
       //String item = (String) it.next();
-      UnicodeString *item = (UnicodeString *)(ne->value.pointer);
+      UnicodeString *item = static_cast<UnicodeString *>(ne->value.pointer);
       resArray[i++] = item;
       ne = col->nextElement(el);
     }
