@@ -46,10 +46,10 @@ public:
     virtual void getText(UnicodeString& result) override {
         text.extract(0,text.length(),result);
     }
-    static UClassID getStaticClassID(void){ 
+    static UClassID getStaticClassID(){ 
         return (UClassID)(&fgClassID); 
     }
-    virtual UClassID getDynamicClassID(void) const override {
+    virtual UClassID getDynamicClassID() const override {
         return getStaticClassID(); 
     }
 
@@ -57,26 +57,26 @@ public:
         return true;
     }
 
-    virtual SCharacterIterator* clone(void) const override {
-        return NULL;
+    virtual SCharacterIterator* clone() const override {
+        return nullptr;
     }
-    virtual int32_t hashCode(void) const override {
+    virtual int32_t hashCode() const override {
         return DONE;
     }
-    virtual UChar nextPostInc(void) override { return text.charAt(pos++);}
-    virtual UChar32 next32PostInc(void) override {return text.char32At(pos++);}
+    virtual char16_t nextPostInc() override { return text.charAt(pos++);}
+    virtual UChar32 next32PostInc() override {return text.char32At(pos++);}
     virtual UBool hasNext() override { return true;}
-    virtual UChar first() override {return DONE;}
+    virtual char16_t first() override {return DONE;}
     virtual UChar32 first32() override {return DONE;}
-    virtual UChar last() override {return DONE;}
+    virtual char16_t last() override {return DONE;}
     virtual UChar32 last32() override {return DONE;}
-    virtual UChar setIndex(int32_t /*pos*/) override {return DONE;}
+    virtual char16_t setIndex(int32_t /*pos*/) override {return DONE;}
     virtual UChar32 setIndex32(int32_t /*pos*/) override {return DONE;}
-    virtual UChar current() const override {return DONE;}
+    virtual char16_t current() const override {return DONE;}
     virtual UChar32 current32() const override {return DONE;}
-    virtual UChar next() override {return DONE;}
+    virtual char16_t next() override {return DONE;}
     virtual UChar32 next32() override {return DONE;}
-    virtual UChar previous() override {return DONE;}
+    virtual char16_t previous() override {return DONE;}
     virtual UChar32 previous32() override {return DONE;}
     virtual int32_t move(int32_t delta,CharacterIterator::EOrigin origin) override {
         switch(origin) {
@@ -101,6 +101,11 @@ public:
 
         return pos;
     }
+
+#ifdef move32
+    // One of the system headers right now is sometimes defining a conflicting macro we don't use
+#undef move32
+#endif
     virtual int32_t move32(int32_t delta, CharacterIterator::EOrigin origin) override {
         switch(origin) {
         case kStart:
@@ -241,7 +246,7 @@ void CharIterTest::TestConstructionAndEquality() {
     if (*test1 != *test2 || *test1 == *test5)
         errln("setIndex() failed");
 
-    *((StringCharacterIterator*)test1) = *((StringCharacterIterator*)test3);
+    *(test1) = *(dynamic_cast<StringCharacterIterator*>(test3));
     if (*test1 != *test3 || *test1 == *test5)
         errln("operator= failed");
 
@@ -378,7 +383,7 @@ void CharIterTest::TestConstructionAndEqualityUChariter() {
 void CharIterTest::TestIteration() {
     UnicodeString text("Now is the time for all good men to come to the aid of their country.");
 
-    UChar c;
+    char16_t c;
     int32_t i;
     {
         StringCharacterIterator   iter(text, 5);
@@ -388,7 +393,7 @@ void CharIterTest::TestIteration() {
         if (iterText != text)
           errln("iter.getText() failed");
 
-        if (iter.current() != text[(int32_t)5])
+        if (iter.current() != text[static_cast<int32_t>(5)])
             errln("Iterator didn't start out in the right place.");
 
         c = iter.first();
@@ -402,7 +407,7 @@ void CharIterTest::TestIteration() {
             if (c == CharacterIterator::DONE && i != text.length())
                 errln("Iterator reached end prematurely");
             else if (c != text[i])
-                errln((UnicodeString)"Character mismatch at position " + i +
+                errln(UnicodeString("Character mismatch at position ") + i +
                                     ", iterator has " + UCharToUnicodeString(c) +
                                     ", string has " + UCharToUnicodeString(text[i]));
 
@@ -431,7 +436,7 @@ void CharIterTest::TestIteration() {
             if (c == CharacterIterator::DONE && i >= 0)
                 errln("Iterator reached end prematurely");
             else if (c != text[i])
-                errln((UnicodeString)"Character mismatch at position " + i +
+                errln(UnicodeString("Character mismatch at position ") + i +
                                     ", iterator has " + UCharToUnicodeString(c) +
                                     ", string has " + UCharToUnicodeString(text[i]));
 
@@ -457,10 +462,10 @@ void CharIterTest::TestIteration() {
         i = 0;
         c=iter.firstPostInc();
         if(c != text[i])
-            errln((UnicodeString)"firstPostInc failed.  Expected->" +  
+            errln(UnicodeString("firstPostInc failed.  Expected->") +
                          UCharToUnicodeString(text[i]) + " Got->" + UCharToUnicodeString(c));
         if(iter.getIndex() != i+1)
-            errln((UnicodeString)"getIndex() after firstPostInc() failed");
+            errln(UnicodeString("getIndex() after firstPostInc() failed"));
 
         iter.setToStart();
         i=0;
@@ -473,9 +478,9 @@ void CharIterTest::TestIteration() {
                 c = iter.nextPostInc();
 
             if(c != text[i])
-                errln((UnicodeString)"Character mismatch at position " + i +
-                                    (UnicodeString)", iterator has " + UCharToUnicodeString(c) +
-                                    (UnicodeString)", string has " + UCharToUnicodeString(text[i]));
+                errln(UnicodeString("Character mismatch at position ") + i +
+                      UnicodeString(", iterator has ") + UCharToUnicodeString(c) +
+                      UnicodeString(", string has ") + UCharToUnicodeString(text[i]));
 
             i++;
             if(iter.getIndex() != i)
@@ -493,7 +498,7 @@ void CharIterTest::TestIteration() {
         if (iter.startIndex() != 5 || iter.endIndex() != 15)
             errln("creation of a restricted-range iterator failed");
 
-        if (iter.getIndex() != 10 || iter.current() != text[(int32_t)10])
+        if (iter.getIndex() != 10 || iter.current() != text[static_cast<int32_t>(10)])
             errln("starting the iterator in the middle didn't work");
 
         c = iter.first();
@@ -504,7 +509,7 @@ void CharIterTest::TestIteration() {
             if (c == CharacterIterator::DONE && i != 15)
                 errln("Iterator reached end prematurely");
             else if (c != text[i])
-                errln((UnicodeString)"Character mismatch at position " + i +
+                errln(UnicodeString("Character mismatch at position ") + i +
                                     ", iterator has " + UCharToUnicodeString(c) +
                                     ", string has " + UCharToUnicodeString(text[i]));
 
@@ -529,7 +534,7 @@ void CharIterTest::TestIteration() {
             if (c == CharacterIterator::DONE && i >= 5)
                 errln("Iterator reached end prematurely");
             else if (c != text[i])
-                errln((UnicodeString)"Character mismatch at position " + i +
+                errln(UnicodeString("Character mismatch at position ") + i +
                                     ", iterator has " + UCharToUnicodeString(c) +
                                     ", string has " + UCharToUnicodeString(text[i]));
 
@@ -550,7 +555,7 @@ void CharIterTest::TestIteration() {
 
 //Tests for new API for utf-16 support 
 void CharIterTest::TestIterationUChar32() {
-    UChar textChars[]={ 0x0061, 0x0062, 0xd841, 0xdc02, 0x20ac, 0xd7ff, 0xd842, 0xdc06, 0xd801, 0xdc00, 0x0061, 0x0000};
+    char16_t textChars[]={ 0x0061, 0x0062, 0xd841, 0xdc02, 0x20ac, 0xd7ff, 0xd842, 0xdc06, 0xd801, 0xdc00, 0x0061, 0x0000};
     UnicodeString text(textChars);
     UChar32 c;
     int32_t i;
@@ -562,7 +567,7 @@ void CharIterTest::TestIterationUChar32() {
         if (iterText != text)
           errln("iter.getText() failed");
 
-        if (iter.current32() != text[(int32_t)1])
+        if (iter.current32() != text[static_cast<int32_t>(1)])
             errln("Iterator didn't start out in the right place.");
 
         c=iter.setToStart();
@@ -636,9 +641,9 @@ void CharIterTest::TestIterationUChar32() {
         logln("Testing backward iteration...");
         do {
             if (c == CharacterIterator::DONE && i >= 0)
-                errln((UnicodeString)"Iterator reached start prematurely for i=" + i);
+                errln(UnicodeString("Iterator reached start prematurely for i=") + i);
             else if(iter.hasPrevious() == false && i>0)
-                errln((UnicodeString)"Iterator reached start prematurely for i=" + i);
+                errln(UnicodeString("Iterator reached start prematurely for i=") + i);
             else if (c != text.char32At(i))
                 errln("Character mismatch at position %d, iterator has %X, string has %X", i, c, text.char32At(i));
 
@@ -669,7 +674,7 @@ void CharIterTest::TestIterationUChar32() {
         if(c != text.char32At(i))
             errln("first32PostInc failed.  Expected->%X Got->%X", text.char32At(i), c);
         if(iter.getIndex() != U16_LENGTH(c) + i)
-            errln((UnicodeString)"getIndex() after first32PostInc() failed");
+            errln(UnicodeString("getIndex() after first32PostInc() failed"));
 
         iter.setToStart();
         i=0;
@@ -815,14 +820,14 @@ void CharIterTest::TestUCharIterator(UCharIterator *iter, CharacterIterator &ci,
 
         case '2':
             h=h2=false;
-            c=(UChar32)iter->move(iter, 2, UITER_CURRENT);
-            c2=(UChar32)ci.move(2, CharacterIterator::kCurrent);
+            c = static_cast<UChar32>(iter->move(iter, 2, UITER_CURRENT));
+            c2 = static_cast<UChar32>(ci.move(2, CharacterIterator::kCurrent));
             break;
 
         case '8':
             h=h2=false;
-            c=(UChar32)iter->move(iter, -2, UITER_CURRENT);
-            c2=(UChar32)ci.move(-2, CharacterIterator::kCurrent);
+            c = static_cast<UChar32>(iter->move(iter, -2, UITER_CURRENT));
+            c2 = static_cast<UChar32>(ci.move(-2, CharacterIterator::kCurrent));
             break;
 
         case 0:
@@ -834,7 +839,7 @@ void CharIterTest::TestUCharIterator(UCharIterator *iter, CharacterIterator &ci,
 
         // compare results
         if(c2==0xffff) {
-            c2=(UChar32)-1;
+            c2 = static_cast<UChar32>(-1);
         }
         if(c!=c2 || h!=h2 || ci.getIndex()!=iter->getIndex(iter, UITER_CURRENT)) {
             errln("error: UCharIterator(%s) misbehaving at \"%s\"[%d]='%c'", which, moves, m, moves[m]);
@@ -913,47 +918,47 @@ void CharIterTest::TestUCharIterator() {
     }
 
 
-    if(cIter.getIndex(&cIter, (enum UCharIteratorOrigin)-1) != -1)
+    if (cIter.getIndex(&cIter, static_cast<enum UCharIteratorOrigin>(-1)) != -1)
     {
         errln("error: UCharIterator(char iter).getIndex did not return error value");
     }
 
-    if(cIter.move(&cIter, 0, (enum UCharIteratorOrigin)-1) != -1)
+    if (cIter.move(&cIter, 0, static_cast<enum UCharIteratorOrigin>(-1)) != -1)
     {
         errln("error: UCharIterator(char iter).move did not return error value");
     }
 
 
-    if(rIter.getIndex(&rIter, (enum UCharIteratorOrigin)-1) != -1)
+    if (rIter.getIndex(&rIter, static_cast<enum UCharIteratorOrigin>(-1)) != -1)
     {
         errln("error: UCharIterator(repl iter).getIndex did not return error value");
     }
 
-    if(rIter.move(&rIter, 0, (enum UCharIteratorOrigin)-1) != -1)
+    if (rIter.move(&rIter, 0, static_cast<enum UCharIteratorOrigin>(-1)) != -1)
     {
         errln("error: UCharIterator(repl iter).move did not return error value");
     }
 
 
-    if(sIter.getIndex(&sIter, (enum UCharIteratorOrigin)-1) != -1)
+    if (sIter.getIndex(&sIter, static_cast<enum UCharIteratorOrigin>(-1)) != -1)
     {
         errln("error: UCharIterator(string iter).getIndex did not return error value");
     }
 
-    if(sIter.move(&sIter, 0, (enum UCharIteratorOrigin)-1) != -1)
+    if (sIter.move(&sIter, 0, static_cast<enum UCharIteratorOrigin>(-1)) != -1)
     {
         errln("error: UCharIterator(string iter).move did not return error value");
     }
 
     /* Testing function coverage on bad input */
     UErrorCode status = U_ZERO_ERROR;
-    uiter_setString(&sIter, NULL, 1);
+    uiter_setString(&sIter, nullptr, 1);
     uiter_setState(&sIter, 1, &status);
     if (status != U_UNSUPPORTED_ERROR) {
         errln("error: uiter_setState returned %s instead of U_UNSUPPORTED_ERROR", u_errorName(status));
     }
     status = U_ZERO_ERROR;
-    uiter_setState(NULL, 1, &status);
+    uiter_setState(nullptr, 1, &status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
         errln("error: uiter_setState returned %s instead of U_ILLEGAL_ARGUMENT_ERROR", u_errorName(status));
     }
@@ -976,7 +981,7 @@ public:
     }
 
     // useful stuff, mostly dummy but testing coverage and subclassability
-    virtual UChar nextPostInc() override {
+    virtual char16_t nextPostInc() override {
         if(pos<UPRV_LENGTHOF(s)) {
             return s[pos++];
         } else {
@@ -998,7 +1003,7 @@ public:
         return pos<UPRV_LENGTHOF(s);
     }
 
-    virtual UChar first() override {
+    virtual char16_t first() override {
         pos=0;
         return s[0];
     }
@@ -1011,7 +1016,7 @@ public:
         return c;
     }
 
-    virtual UChar setIndex(int32_t position) override {
+    virtual char16_t setIndex(int32_t position) override {
         if(0<=position && position<=UPRV_LENGTHOF(s)) {
             pos=position;
             if(pos<UPRV_LENGTHOF(s)) {
@@ -1033,7 +1038,7 @@ public:
         return DONE;
     }
 
-    virtual UChar current() const override {
+    virtual char16_t current() const override {
         if(pos<UPRV_LENGTHOF(s)) {
             return s[pos];
         } else {
@@ -1051,7 +1056,7 @@ public:
         }
     }
 
-    virtual UChar next() override {
+    virtual char16_t next() override {
         if(pos<UPRV_LENGTHOF(s) && ++pos<UPRV_LENGTHOF(s)) {
             return s[pos];
         } else {
@@ -1093,10 +1098,10 @@ public:
     }
 
     virtual CharacterIterator *clone() const override {
-        return NULL;
+        return nullptr;
     }
 
-    virtual UChar last() override {
+    virtual char16_t last() override {
         return 0;
     }
 
@@ -1104,7 +1109,7 @@ public:
         return 0;
     }
 
-    virtual UChar previous() override {
+    virtual char16_t previous() override {
         return 0;
     }
 
@@ -1131,7 +1136,7 @@ public:
 
 private:
     // dummy string data
-    UChar s[4];
+    char16_t s[4];
 
     static const char fgClassID;
 };
@@ -1152,10 +1157,10 @@ public:
     }
 
 private:
-    static const UChar u[3];
+    static const char16_t u[3];
 };
 
-const UChar SubUCharCharIter::u[3]={ 0x61, 0x62, 0x63 };
+const char16_t SubUCharCharIter::u[3]={ 0x61, 0x62, 0x63 };
 
 void CharIterTest::TestCharIteratorSubClasses() {
     SubCharIter *p;
