@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  ******************************************************************************
- *   Copyright (C) 1996-2014, International Business Machines
+ *   Copyright (C) 1996-2016, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  ******************************************************************************
  */
@@ -20,6 +22,7 @@
 
 #include "unicode/regex.h"        // TODO: make conditional on regexp being built.
 
+#include "unicode/testlog.h"
 #include "unicode/uniset.h"
 #include "unicode/uset.h"
 #include "unicode/usetiter.h"
@@ -32,13 +35,12 @@
 
 #include "colldata.h"
 
-#define ARRAY_SIZE(array) (sizeof(array)/sizeof(array[0]))
-#define NEW_ARRAY(type, count) (type *) uprv_malloc((count) * sizeof(type))
+#define NEW_ARRAY(type, count) (type *) uprv_malloc((size_t)(count) * sizeof(type))
 #define DELETE_ARRAY(array) uprv_free((void *) (array))
-#define ARRAY_COPY(dst, src, count) uprv_memcpy((void *) (dst), (void *) (src), (count) * sizeof (src)[0])
+#define ARRAY_COPY(dst, src, count) uprv_memcpy((void *) (dst), (void *) (src), (size_t)(count) * sizeof (src)[0])
 
 CEList::CEList(UCollator *coll, const UnicodeString &string, UErrorCode &status)
-    : ces(NULL), listMax(CELIST_BUFFER_SIZE), listSize(0)
+    : ces(nullptr), listMax(CELIST_BUFFER_SIZE), listSize(0)
 {
     UCollationElements *elems = ucol_openElements(coll, string.getBuffer(), string.length(), &status);
     UCollationStrength strength = ucol_getStrength(coll);
@@ -58,12 +60,10 @@ CEList::CEList(UCollator *coll, const UnicodeString &string, UErrorCode &status)
     {
     default:
         strengthMask |= UCOL_TERTIARYORDERMASK;
-        /* fall through */
-
+        U_FALLTHROUGH;
     case UCOL_SECONDARY:
         strengthMask |= UCOL_SECONDARYORDERMASK;
-        /* fall through */
-
+        U_FALLTHROUGH;
     case UCOL_PRIMARY:
         strengthMask |= UCOL_PRIMARYORDERMASK;
     }
@@ -75,7 +75,7 @@ CEList::CEList(UCollator *coll, const UnicodeString &string, UErrorCode &status)
 
         order &= strengthMask;
 
-        if (toShift && variableTop > (uint32_t)order && (order & UCOL_PRIMARYORDERMASK) != 0) {
+        if (toShift && variableTop > static_cast<uint32_t>(order) && (order & UCOL_PRIMARYORDERMASK) != 0) {
             if (strength >= UCOL_QUATERNARY) {
                 order &= UCOL_PRIMARYORDERMASK;
             } else {
@@ -114,7 +114,7 @@ void CEList::add(uint32_t ce, UErrorCode &status)
         int32_t newMax = listMax + CELIST_BUFFER_SIZE;
         uint32_t *newCEs = NEW_ARRAY(uint32_t, newMax);
 
-        if (newCEs == NULL) {
+        if (newCEs == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             return;
         }
@@ -138,7 +138,7 @@ uint32_t CEList::get(int32_t index) const
         return ces[index];
     }
 
-    return (uint32_t)UCOL_NULLORDER;
+    return static_cast<uint32_t>(UCOL_NULLORDER);
 }
 
 uint32_t &CEList::operator[](int32_t index) const
@@ -148,17 +148,17 @@ uint32_t &CEList::operator[](int32_t index) const
 
 UBool CEList::matchesAt(int32_t offset, const CEList *other) const
 {
-    if (other == NULL || listSize - offset < other->size()) {
-        return FALSE;
+    if (other == nullptr || listSize - offset < other->size()) {
+        return false;
     }
 
     for (int32_t i = offset, j = 0; j < other->size(); i += 1, j += 1) {
         if (ces[i] != (*other)[j]) {
-            return FALSE;
+            return false;
         }
     }
 
-    return TRUE;
+    return true;
 }
 
 int32_t CEList::size() const
@@ -167,7 +167,7 @@ int32_t CEList::size() const
 }
 
 StringList::StringList(UErrorCode &status)
-    : strings(NULL), listMax(STRING_LIST_BUFFER_SIZE), listSize(0)
+    : strings(nullptr), listMax(STRING_LIST_BUFFER_SIZE), listSize(0)
 {
     if (U_FAILURE(status)) {
         return;
@@ -175,7 +175,7 @@ StringList::StringList(UErrorCode &status)
 
     strings = new UnicodeString [listMax];
 
-    if (strings == NULL) {
+    if (strings == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return;
     }
@@ -194,7 +194,7 @@ void StringList::add(const UnicodeString *string, UErrorCode &status)
     if (listSize >= listMax) {
         int32_t newMax = listMax + STRING_LIST_BUFFER_SIZE;
         UnicodeString *newStrings = new UnicodeString[newMax];
-        if (newStrings == NULL) {
+        if (newStrings == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             return;
         }
@@ -212,7 +212,7 @@ void StringList::add(const UnicodeString *string, UErrorCode &status)
     strings[listSize++].append(*string);
 }
 
-void StringList::add(const UChar *chars, int32_t count, UErrorCode &status)
+void StringList::add(const char16_t *chars, int32_t count, UErrorCode &status)
 {
     const UnicodeString string(chars, count);
 
@@ -225,7 +225,7 @@ const UnicodeString *StringList::get(int32_t index) const
         return &strings[index];
     }
 
-    return NULL;
+    return nullptr;
 }
 
 int32_t StringList::size() const
@@ -238,7 +238,7 @@ U_CDECL_BEGIN
 static void U_CALLCONV
 deleteStringList(void *obj)
 {
-    StringList *strings = (StringList *) obj;
+    StringList *strings = static_cast<StringList *>(obj);
 
     delete strings;
 }
@@ -259,7 +259,7 @@ private:
 };
 
 CEToStringsMap::CEToStringsMap(UErrorCode &status)
-    : map(NULL)
+    : map(nullptr)
 {
     if (U_FAILURE(status)) {
         return;
@@ -285,10 +285,10 @@ void CEToStringsMap::put(uint32_t ce, UnicodeString *string, UErrorCode &status)
 {
     StringList *strings = getStringList(ce);
 
-    if (strings == NULL) {
+    if (strings == nullptr) {
         strings = new StringList(status);
 
-        if (strings == NULL || U_FAILURE(status)) {
+        if (strings == nullptr || U_FAILURE(status)) {
             status = U_MEMORY_ALLOCATION_ERROR;
             return;
         }
@@ -301,7 +301,7 @@ void CEToStringsMap::put(uint32_t ce, UnicodeString *string, UErrorCode &status)
 
 StringList *CEToStringsMap::getStringList(uint32_t ce) const
 {
-    return (StringList *) uhash_iget(map, ce);
+    return static_cast<StringList *>(uhash_iget(map, ce));
 }
 
 void CEToStringsMap::putStringList(uint32_t ce, StringList *stringList, UErrorCode &status)
@@ -312,7 +312,7 @@ void CEToStringsMap::putStringList(uint32_t ce, StringList *stringList, UErrorCo
 #define CLONE_COLLATOR
 
 CollData::CollData(UCollator *collator, UErrorCode &status)
-    : coll(NULL), ceToCharsStartingWith(NULL)
+    : coll(nullptr), ceToCharsStartingWith(nullptr)
 {
     // [:c:] == [[:cn:][:cc:][:co:][:cf:][:cs:]]
     // i.e. other, control, private use, format, surrogate
@@ -321,7 +321,7 @@ CollData::CollData(UCollator *collator, UErrorCode &status)
     USet *charsToTest = uset_openPattern(test_pattern, 20, &status);
 
     // Han ext. A, Han, Jamo, Hangul, Han Ext. B
-    // i.e. all the characers we handle implicitly
+    // i.e. all the characters we handle implicitly
     U_STRING_DECL(remove_pattern, "[[\\u3400-\\u9FFF][\\u1100-\\u11F9][\\uAC00-\\uD7AF][\\U00020000-\\U0002A6DF]]", 70);
     U_STRING_INIT(remove_pattern, "[[\\u3400-\\u9FFF][\\u1100-\\u11F9][\\uAC00-\\uD7AF][\\U00020000-\\U0002A6DF]]", 70);
     USet *charsToRemove = uset_openPattern(remove_pattern, 70, &status);
@@ -341,7 +341,7 @@ CollData::CollData(UCollator *collator, UErrorCode &status)
     }
 
 #ifdef CLONE_COLLATOR
-    coll = ucol_safeClone(collator, NULL, NULL, &status);
+    coll = ucol_safeClone(collator, nullptr, nullptr, &status);
 
     if (U_FAILURE(status)) {
         goto bail;
@@ -350,7 +350,7 @@ CollData::CollData(UCollator *collator, UErrorCode &status)
     coll = collator;
 #endif
 
-    ucol_getContractionsAndExpansions(coll, contractions, expansions, FALSE, &status);
+    ucol_getContractionsAndExpansions(coll, contractions, expansions, false, &status);
 
     uset_addAll(charsToTest, contractions);
     uset_addAll(charsToTest, expansions);
@@ -359,7 +359,7 @@ CollData::CollData(UCollator *collator, UErrorCode &status)
     itemCount = uset_getItemCount(charsToTest);
     for(int32_t item = 0; item < itemCount; item += 1) {
         UChar32 start = 0, end = 0;
-        UChar buffer[16];
+        char16_t buffer[16];
         int32_t len = uset_getItem(charsToTest, item, &start, &end,
                                    buffer, 16, &status);
 
@@ -367,7 +367,7 @@ CollData::CollData(UCollator *collator, UErrorCode &status)
             for (UChar32 ch = start; ch <= end; ch += 1) {
                 UnicodeString *st = new UnicodeString(ch);
 
-                if (st == NULL) {
+                if (st == nullptr) {
                     status = U_MEMORY_ALLOCATION_ERROR;
                     break;
                 }
@@ -382,7 +382,7 @@ CollData::CollData(UCollator *collator, UErrorCode &status)
         } else if (len > 0) {
             UnicodeString *st = new UnicodeString(buffer, len);
 
-            if (st == NULL) {
+            if (st == nullptr) {
                 status = U_MEMORY_ALLOCATION_ERROR;
                 break;
             }
@@ -427,8 +427,8 @@ bail:
     // and there are also more Jamo T at U+D7CB..U+D7FB.
     // Maybe use [:HST=T:] and look for the end of the last range?
     // Maybe use script boundary mappings instead of this code??
-    UChar  jamoRanges[] = {Hangul::JAMO_L_BASE, Hangul::JAMO_V_BASE, Hangul::JAMO_T_BASE + 1, 0x11FF};
-     UnicodeString jamoString(FALSE, jamoRanges, ARRAY_SIZE(jamoRanges));
+    char16_t  jamoRanges[] = {Hangul::JAMO_L_BASE, Hangul::JAMO_V_BASE, Hangul::JAMO_T_BASE + 1, 0x11FF};
+     UnicodeString jamoString(false, jamoRanges, UPRV_LENGTHOF(jamoRanges));
      CEList hanList(coll, hanString, status);
      CEList jamoList(coll, jamoString, status);
      int32_t j = 0;
@@ -451,7 +451,7 @@ bail:
      maxHan = 0;
 
      for(int32_t h = 0; h < hanList.size(); h += 2) {
-         uint32_t han = (uint32_t) hanList[h];
+         uint32_t han = hanList[h];
 
          if (han < minHan) {
              minHan = han;
@@ -491,7 +491,7 @@ const CEList *CollData::getCEList(const UnicodeString *string) const
 
     if (U_FAILURE(status)) {
         delete list;
-        list = NULL;
+        list = nullptr;
     }
 
     return list;
@@ -516,7 +516,7 @@ int32_t CollData::minLengthInChars(const CEList *ceList, int32_t offset, int32_t
     int32_t shortestLength = INT32_MAX;
     const StringList *strings = ceToCharsStartingWith->getStringList(ce);
 
-    if (strings != NULL) {
+    if (strings != nullptr) {
         int32_t stringCount = strings->size();
 
         for (int32_t s = 0; s < stringCount; s += 1) {
@@ -526,11 +526,11 @@ int32_t CollData::minLengthInChars(const CEList *ceList, int32_t offset, int32_t
 
             if (U_FAILURE(status)) {
                 delete ceList2;
-                ceList2 = NULL;
+                ceList2 = nullptr;
             }
 
             if (ceList->matchesAt(offset, ceList2)) {
-                U_ASSERT(ceList2 != NULL);
+                U_ASSERT(ceList2 != nullptr);
                 int32_t clength = ceList2->size();
                 int32_t slength = string->length();
                 int32_t roffset = offset + clength;
