@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 2007, International Business Machines Corporation and
@@ -29,7 +31,7 @@ class CalendarFieldsSet;
 
 class DataDrivenCalendarTest : public IntlTest {
 	void runIndexedTest(int32_t index, UBool exec, const char* &name,
-			char* par = NULL);
+			char* par = nullptr) override;
 public:
 	DataDrivenCalendarTest();
 	virtual ~DataDrivenCalendarTest();
