@@ -47,12 +47,14 @@ void IntlTestDateTimePatternGeneratorAPI::runIndexedTest( int32_t index, UBool e
         TESTCASE(11, test_jConsistencyOddLocales);
         TESTCASE(12, testBestPattern);
         TESTCASE(13, testDateTimePatterns);
-        TESTCASE(14, testRegionOverride);
+        TESTCASE(14, testISO8601);
+        TESTCASE(15, testRegionOverride);
+        TESTCASE(16, testAlphabeticSubstitution23114);
         default: name = ""; break;
     }
 }
 
-#define MAX_LOCALE   12
+#define MAX_LOCALE   11
 
 /**
  * Test various generic API methods of DateTimePatternGenerator for API coverage.
@@ -92,7 +94,6 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         {"zh", "TW", "", "calendar=roc"},       // 8
         {"ru", "", "", ""},                     // 9
         {"zh", "", "", "calendar=chinese"},     // 10
-        {"ja", "JP", "TRADITIONAL", ""},        // 11
      };
 
     // For Weds, Jan 13, 1999, 23:58:59
@@ -164,9 +165,9 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("1.1999"),                              // 00: yM (fixed expected result per ticket:6626:)
         UnicodeString("tammi 1999"),                          // 01: yMMM
         UnicodeString("13.1.1999"),                           // 02: yMd
-        UnicodeString("13. tammik. 1999"),                    // 03: yMMMd
+        UnicodeString("13.1.1999"),                           // 03: yMMMd
         UnicodeString("13.1."),                               // 04: Md
-        UnicodeString("13. tammik."),                         // 05: MMMd
+        UnicodeString("13.1."),                               // 05: MMMd
         UnicodeString("13. tammikuuta"),                      // 06: MMMMd
         UnicodeString("1. nelj. 1999"),                       // 07: yQQQ
         UnicodeString(u"11.58\u202Fip.", -1),                // 08: hhmm
@@ -174,7 +175,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("23.58"),                               // 10: jjmm
         UnicodeString("58.59"),                               // 11: mmss
         UnicodeString("tammikuu 1999"),                       // 12: yyyyMMMM
-        UnicodeString("ke 13. tammik."),                      // 13: MMMEd -> EEE d. MMM
+        UnicodeString("ke 13.1."),                            // 13: MMMEd -> EEE d. MMM
         UnicodeString("ke 13."),                              // 14: Ed    -> ccc d.
         UnicodeString("23.58.59,123"),                        // 15: jmmssSSS -> "H.mm.ss,SSS"
         UnicodeString("23.58"),                               // 16: JJmm
@@ -190,7 +191,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("13 ene"),                              // 05: MMMd  -> "d 'de' MMM"
         UnicodeString("13 de enero"),                         // 06: MMMMd -> "d 'de' MMMM"
         UnicodeString("T1 1999"),                             // 07: yQQQ  -> "QQQ y"
-        UnicodeString(u"11:58\u202Fp.\u00A0m.", -1),          // 08: hhmm  -> "hh:mm a"
+        UnicodeString(u"11:58\u202Fp.\u202Fm.", -1),          // 08: hhmm  -> "hh:mm a"
         UnicodeString("23:58"),                               // 09: HHmm  -> "HH:mm"
         UnicodeString("23:58"),                               // 10: jjmm  -> "HH:mm"
         UnicodeString("58:59"),                               // 11: mmss  -> "mm:ss"
@@ -327,28 +328,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString("23:58"),                                                 // 16: JJmm
     };
 
-    UnicodeString patternResults_ja_jp_traditional[] = {
-        // ja_JP_TRADITIONAL                 // 11 ja_JP_TRADITIONAL
-        u"AD1999/1",                         // 00: yM
-        u"西暦1999年1月",                     // 01: yMMM
-        u"1999年1月13日",                     // 02: yMd
-        u"西暦1999年1月13日",                  // 03: yMMMd
-        u"1/13",                             // 04: Md
-        u"1月13日",                           // 05: MMMd
-        u"1月13日",                           // 06: MMMMd
-        u"西暦1999/Q1",                       // 07: yQQQ
-        u"午後11:58",                         // 08: hhmm
-        u"23:58",                            // 09: HHmm
-        u"23:58",                            // 10: jjmm
-        u"58:59",                            // 11: mmss
-        u"西暦1999年1月",                     // 12: yyyyMMMM
-        u"1月13日(水)",                       // 13: MMMEd
-        u"13日(水)",                          // 14: Ed
-        u"23:58:59.123",                     // 15: jmmssSSS
-        u"23:58",                            // 16: JJmm
-    };
-
-    UnicodeString* patternResults[] = {
+    UnicodeString* patternResults[MAX_LOCALE] = {
         patternResults_en_US, // 0
         patternResults_en_US_japanese, // 1
         patternResults_de_DE, // 2
@@ -360,7 +340,6 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         patternResults_zh_TW_roc, // 8
         patternResults_ru, // 9
         patternResults_zh_chinese, // 10
-        patternResults_ja_jp_traditional, // 11
     };
 
     UnicodeString patternTests2[] = {
@@ -406,7 +385,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString(u"Thu, Oct 14, 1999, 6:58:59\u202FAM", -1),
         UnicodeString(u"6:58\u202FAM", -1),
         UnicodeString(u"6:58\u202FAM", -1),
-        UnicodeString(u"6:58\u202FAM GMT", -1),
+        UnicodeString(u"6:58\u202FAM GMT+00:00", -1),
         UnicodeString(""),
     };
 
@@ -716,7 +695,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
             resultDate.remove();
             resultDate = sdf.format(testDate, resultDate);
             if ( resultDate != patternResults[localeIndex][resultIndex] ) {
-                auto* calendar = sdf.getCalendar();
+                const auto* calendar = sdf.getCalendar();
                 errln(UnicodeString("\nERROR: Test various skeletons[") + (dataIndex-1) + UnicodeString("], localeIndex ") + localeIndex +
                       u". Got: \"" + resultDate +
                       u"\" with calendar " + calendar->getType() +
@@ -787,7 +766,7 @@ void IntlTestDateTimePatternGeneratorAPI::testAPI(/*char *par*/)
         UnicodeString randomSkeleton;
         int32_t len = rand() % 20;
         for (int32_t j=0; j<len; ++j ) {
-            while ((newChar = (char16_t)(rand()%0x7f))>=(char16_t)0x20) {
+            while ((newChar = static_cast<char16_t>(rand() % 0x7f)) >= static_cast<char16_t>(0x20)) {
                 randomSkeleton += newChar;
             }
         }
@@ -1180,7 +1159,8 @@ void IntlTestDateTimePatternGeneratorAPI::testC() {
             {"en",     "CCCCCCm", "hh:mm\\u202Faaaaa"},
             {"en-BN",  "Cm",      "h:mm\\u202Fb"},
             {"gu-IN",  "Cm",      "h:mm B"},
-            {"und-IN", "Cm",      "h:mm B"}
+            {"und-IN", "Cm",      "h:mm B"},
+            {"fi",     "Cm",      "H.mm"}
     };
 
     UErrorCode status = U_ZERO_ERROR;
@@ -1329,7 +1309,13 @@ void IntlTestDateTimePatternGeneratorAPI::testGetFieldDisplayNames() {
     }
 }
 
-static const char16_t timeCycleChars[] = { (char16_t)0x0048, (char16_t)0x0068, (char16_t)0x004B, (char16_t)0x006B, (char16_t)0 };
+static const char16_t timeCycleChars[] = {
+    static_cast<char16_t>(0x0048),
+    static_cast<char16_t>(0x0068),
+    static_cast<char16_t>(0x004B),
+    static_cast<char16_t>(0x006B),
+    static_cast<char16_t>(0)
+};
 
 void IntlTestDateTimePatternGeneratorAPI::testJjMapping() {
     UErrorCode status = U_ZERO_ERROR;
@@ -1384,17 +1370,18 @@ void IntlTestDateTimePatternGeneratorAPI::testJjMapping() {
             continue;
         }
         // Now check that shortPattern and jPattern use the same hour cycle
-        if ((uprv_strncmp(localeID, "csw", 3) == 0 || uprv_strncmp(localeID, "kxv_", 4) == 0) && logKnownIssue("CLDR-17199", "Need timeFormats with h for csw, kxv_Xxxx")) {
-            continue;
-        }
         UnicodeString jPatSkeleton = DateTimePatternGenerator::staticGetSkeleton(jPattern, status);
         UnicodeString shortPatSkeleton = DateTimePatternGenerator::staticGetSkeleton(shortPattern, status);
         if (U_FAILURE(status)) {
             errln("FAIL: DateTimePatternGenerator::staticGetSkeleton locale %s: %s", localeID, u_errorName(status));
             continue;
         }
+        if (uprv_strcmp(localeID, "ku_Latn_IQ")==0) {
+            logKnownIssue("CLDR-19048", "ku_Latn_IQ needs either 'h' in Grego std time patterns or timeData update");
+            continue;
+        }
         const char16_t* charPtr = timeCycleChars;
-        for (; *charPtr != (char16_t)0; charPtr++) {
+        for (; *charPtr != static_cast<char16_t>(0); charPtr++) {
              if (jPatSkeleton.indexOf(*charPtr) >= 0) {
                  if (shortPatSkeleton.indexOf(*charPtr) < 0) {
                      char jcBuf[2], spBuf[32], jpBuf[32];
@@ -1424,7 +1411,7 @@ void IntlTestDateTimePatternGeneratorAPI::test20640_HourCyclArsEnNH() {
     } cases[] = {
         // ars is interesting because it does not have a region, but it aliases
         // to ar_SA, which has a region.
-        {"ars", u"h a", u"h:mm a", UDAT_HOUR_CYCLE_12},
+        {"ars", u"h\u202Fa", u"h:mm a", UDAT_HOUR_CYCLE_12},
         // en_NH is interesting because NH is a deprecated region code;
         // formerly New Hebrides, now Vanuatu => VU => h.
         {"en_NH", u"h\u202Fa", u"h:mm\u202Fa", UDAT_HOUR_CYCLE_12},
@@ -1434,7 +1421,7 @@ void IntlTestDateTimePatternGeneratorAPI::test20640_HourCyclArsEnNH() {
         {"ja_TRADITIONAL", u"H時", u"H:mm", UDAT_HOUR_CYCLE_23},
     };
 
-    for (auto& cas : cases) {
+    for (const auto& cas : cases) {
         status.setScope(cas.localeName);
 
         Locale loc(cas.localeName);
@@ -1600,7 +1587,7 @@ void IntlTestDateTimePatternGeneratorAPI::testBestPattern() {
         // ICU-21428: Bad patterns for nonstandard calendars
         { "en_GB",                   "yMd", u"dd/MM/y"          },
         { "en_GB@calendar=coptic",   "yMd", u"dd/MM/y GGGGG"    },
-        { "en_GB@calendar=japanese", "yMd", u"dd/MM/y GGGGG"    },
+        { "en_GB@calendar=japanese", "yMd", u"d/M/y GGGGG"    },
         { "en_GB@calendar=buddhist", "yMd", u"dd/MM/y GGGGG"    },
         // ICU-22757: Not inheriting availableFormats patterns from root
         { "sv_SE",                   "yMd", u"y-MM-dd"          },
@@ -1716,7 +1703,7 @@ void IntlTestDateTimePatternGeneratorAPI::testDateTimePatterns() {
         // Test normal getting and setting
         for (int32_t patStyle = 0; patStyle < kNumDateTimePatterns; patStyle++) {
             status = U_ZERO_ERROR;
-            const UnicodeString& dtFormat1 = dtpg->getDateTimeFormat((UDateFormatStyle)patStyle, status);
+            const UnicodeString& dtFormat1 = dtpg->getDateTimeFormat(static_cast<UDateFormatStyle>(patStyle), status);
             if (U_FAILURE(status)) {
                 errln("FAIL: getDateTimeFormat for en before mod, style %d, get %s", patStyle, u_errorName(status));
             } else  if (dtFormat1 != enDTPatterns[patStyle]) {
@@ -1726,11 +1713,11 @@ void IntlTestDateTimePatternGeneratorAPI::testDateTimePatterns() {
                         patStyle, bExpect, bGet);
             }
             status = U_ZERO_ERROR;
-            dtpg->setDateTimeFormat((UDateFormatStyle)patStyle, modDTPatterns[patStyle], status);
+            dtpg->setDateTimeFormat(static_cast<UDateFormatStyle>(patStyle), modDTPatterns[patStyle], status);
             if (U_FAILURE(status)) {
                 errln("FAIL: setDateTimeFormat for en, style %d, get %s", patStyle, u_errorName(status));
             } else {
-                const UnicodeString& dtFormat2 = dtpg->getDateTimeFormat((UDateFormatStyle)patStyle, status);
+                const UnicodeString& dtFormat2 = dtpg->getDateTimeFormat(static_cast<UDateFormatStyle>(patStyle), status);
                 if (U_FAILURE(status)) {
                     errln("FAIL: getDateTimeFormat for en after  mod, style %d, get %s", patStyle, u_errorName(status));
                 } else if (dtFormat2 != modDTPatterns[patStyle]) {
@@ -1754,7 +1741,7 @@ void IntlTestDateTimePatternGeneratorAPI::testDateTimePatterns() {
         modDTPatterns[UDAT_SHORT].extract(0, modDTPatterns[UDAT_SHORT].length(), bExpect, 64);
         for (int32_t patStyle = 0; patStyle < kNumDateTimePatterns; patStyle++) {
             status = U_ZERO_ERROR;
-            const UnicodeString& dtFormat4 = dtpg->getDateTimeFormat((UDateFormatStyle)patStyle, status);
+            const UnicodeString& dtFormat4 = dtpg->getDateTimeFormat(static_cast<UDateFormatStyle>(patStyle), status);
             if (U_FAILURE(status)) {
                 errln("FAIL: getDateTimeFormat for en after second mod, style %d, get %s", patStyle, u_errorName(status));
             } else if (dtFormat4 != modDTPatterns[UDAT_SHORT]) {
@@ -1766,6 +1753,31 @@ void IntlTestDateTimePatternGeneratorAPI::testDateTimePatterns() {
     }
 }
 
+// Test for ICU-21839: Make sure ISO8601 patterns/symbols are inherited from Gregorian
+void IntlTestDateTimePatternGeneratorAPI::testISO8601() {
+    const char* localeIDs[] = {
+        "de-AT-u-ca-iso8601",
+        "de-CH-u-ca-iso8601",
+    };
+    const UChar* skeleton = u"jms";
+
+    for (int32_t i = 0; i < UPRV_LENGTHOF(localeIDs); i++) {
+        UErrorCode err = U_ZERO_ERROR;
+        LocalPointer<DateTimePatternGenerator> dtpg(DateTimePatternGenerator::createInstance(Locale::forLanguageTag(localeIDs[i], err), err));
+        UnicodeString pattern = dtpg->getBestPattern(skeleton, err);
+        if (pattern.indexOf(UnicodeString(u"├")) >= 0 || pattern.indexOf(UnicodeString(u"Minute")) >= 0) {
+            errln(UnicodeString("ERROR: locale ") + localeIDs[i] + UnicodeString(", skeleton ") + skeleton + UnicodeString(", bad pattern: ") + pattern);
+        }
+
+        LocalPointer<DateFormat> df(DateFormat::createTimeInstance(DateFormat::MEDIUM, Locale::forLanguageTag(localeIDs[i], err)));
+        UnicodeString format;
+        df->format(ucal_getNow(), format, err);
+        if (format.indexOf(UnicodeString(u"├")) >= 0 || format.indexOf(UnicodeString(u"Minute")) >= 0) {
+            errln(UnicodeString("ERROR: locale ") + localeIDs[i] + UnicodeString(", MEDIUM, bad format: ") + format);
+        }
+    }
+}
+
 void IntlTestDateTimePatternGeneratorAPI::testRegionOverride() {
     const struct TestCase {
         const char* locale;
@@ -1812,4 +1824,28 @@ void IntlTestDateTimePatternGeneratorAPI::doDTPatternTest(DateTimePatternGenerat
     }
 }
 
+void IntlTestDateTimePatternGeneratorAPI::testAlphabeticSubstitution23114() {
+    IcuTestErrorCode status(*this, "testAlphabeticSubstitution23114");
+
+    LocalPointer<DateTimePatternGenerator> dtpg(
+        DateTimePatternGenerator::createEmptyInstance(status), status);
+    status.assertSuccess();
+
+    // Set up the DTPG with English data from CLDR 47
+    UnicodeString conflictingPattern;
+    dtpg->addPatternWithSkeleton(u"y G", u"Gy", true, conflictingPattern, status);
+    dtpg->addPatternWithSkeleton(u"M/d/y G", u"GyMd", true, conflictingPattern, status);
+    dtpg->addPatternWithSkeleton(u"MMM y G", u"GyMMM", true, conflictingPattern, status);
+    dtpg->addPatternWithSkeleton(u"MMM d, y G", u"GyMMMd", true, conflictingPattern, status);
+    dtpg->addPatternWithSkeleton(u"EEE, MMM d, y G", u"GyMMMEd", true, conflictingPattern, status);
+    status.assertSuccess();
+
+    // Test the behavior of selecting GyMEd. In ICU 77, this selected the GyMMMEd skeleton,
+    // and replaced the alphabetic month with a numeric month, which is wrong. In ICU 78,
+    // we still select GyMMMEd, but we don't change it to a numeric month.
+    UnicodeString bestPattern = dtpg->getBestPattern(u"GyMEd", status);
+    status.assertSuccess();
+    assertEquals("Should not substitute numeric for alpha", u"EEE, MMM d, y G", bestPattern);
+}
+
 #endif /* #if !UCONFIG_NO_FORMATTING */
