@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -11,24 +13,29 @@
 *                         TestSecondary
 ***********************************************************************/
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/coll.h"
 #include "unicode/tblcoll.h"
 #include "unicode/unistr.h"
 #include "unicode/sortkey.h"
+#include "cmemory.h"
 #include "frcoll.h"
 
 #include "sfwdchit.h"
 
 CollationFrenchTest::CollationFrenchTest()
-: myCollation(0)
+: myCollation(nullptr)
 {
     UErrorCode status = U_ZERO_ERROR;
-    myCollation = Collator::createInstance(Locale::getFrance(), status);
+    myCollation = Collator::createInstance(Locale::getCanadaFrench(), status);
     if(!myCollation || U_FAILURE(status)) {
-        errln(__FILE__ "failed to create! err " + UnicodeString(u_errorName(status)));
+        errcheckln(status, __FILE__ "failed to create! err " + UnicodeString(u_errorName(status)));
         /* if it wasn't already: */
         delete myCollation;
-        myCollation = NULL;
+        myCollation = nullptr;
     }
 }
 
@@ -37,7 +44,7 @@ CollationFrenchTest::~CollationFrenchTest()
     delete myCollation;
 }
 
-const UChar CollationFrenchTest::testSourceCases[][CollationFrenchTest::MAX_TOKEN_LEN] =
+const char16_t CollationFrenchTest::testSourceCases[][CollationFrenchTest::MAX_TOKEN_LEN] =
 {
     {0x0061/*'a'*/, 0x0062/*'b'*/, 0x0063/*'c'*/, 0x0000},
     {0x0043/*'C'*/, 0x004f/*'O'*/, 0x0054/*'T'*/, 0x0045/*'E'*/, 0x0000},
@@ -53,7 +60,7 @@ const UChar CollationFrenchTest::testSourceCases[][CollationFrenchTest::MAX_TOKE
     {0x0101, 0x0000}
 };
 
-const UChar CollationFrenchTest::testTargetCases[][CollationFrenchTest::MAX_TOKEN_LEN] =
+const char16_t CollationFrenchTest::testTargetCases[][CollationFrenchTest::MAX_TOKEN_LEN] =
 {
     {0x0041/*'A'*/, 0x0042/*'B'*/, 0x0043/*'C'*/, 0x0000},
     {0x0063/*'c'*/, 0x00f4, 0x0074/*'t'*/, 0x0065/*'e'*/, 0x0000},
@@ -87,7 +94,7 @@ const Collator::EComparisonResult CollationFrenchTest::results[] =
 
 // 0x0300 is grave, 0x0301 is acute
 // the order of elements in this array must be different than the order in CollationEnglishTest
-const UChar CollationFrenchTest::testAcute[][CollationFrenchTest::MAX_TOKEN_LEN] =
+const char16_t CollationFrenchTest::testAcute[][CollationFrenchTest::MAX_TOKEN_LEN] =
 {
 /*00*/    {0x0065/*'e'*/, 0x0065/*'e'*/,  0x0000},
 /*01*/    {0x0065/*'e'*/, 0x0301, 0x0065/*'e'*/,  0x0000},
@@ -116,7 +123,7 @@ const UChar CollationFrenchTest::testAcute[][CollationFrenchTest::MAX_TOKEN_LEN]
 /*18*/    {0x0065/*'e'*/, 0x0301, 0x0300, 0x0065/*'e'*/, 0x0301, 0x0300, 0x0000}
 };
 
-const UChar CollationFrenchTest::testBugs[][CollationFrenchTest::MAX_TOKEN_LEN] =
+const char16_t CollationFrenchTest::testBugs[][CollationFrenchTest::MAX_TOKEN_LEN] =
 {
     {0x0061/*'a'*/, 0x000},
     {0x0041/*'A'*/, 0x000},
@@ -130,23 +137,6 @@ const UChar CollationFrenchTest::testBugs[][CollationFrenchTest::MAX_TOKEN_LEN]
     {0x0078/*'x'*/, 0x000}
 };
 
-void CollationFrenchTest::doTest( UnicodeString source, UnicodeString target, Collator::EComparisonResult result)
-{
-    Collator::EComparisonResult compareResult = myCollation->compare(source, target);
-    CollationKey sortKey1, sortKey2;
-    UErrorCode key1status = U_ZERO_ERROR, key2status = U_ZERO_ERROR; //nos
-    myCollation->getCollationKey(source, /*nos*/ sortKey1, key1status );
-    myCollation->getCollationKey(target, /*nos*/ sortKey2, key2status );
-    if (U_FAILURE(key1status) || U_FAILURE(key2status))
-    {
-        errln("SortKey generation Failed.\n");
-        return;
-    }
-
-    Collator::EComparisonResult keyResult = sortKey1.compareTo(sortKey2);
-    reportCResult( source, target, sortKey1, sortKey2, compareResult, keyResult, compareResult, result );
-}
-
 void CollationFrenchTest::TestTertiary(/* char* par */)
 {
     int32_t i = 0;
@@ -161,7 +151,7 @@ void CollationFrenchTest::TestTertiary(/* char* par */)
     {
         for (i = 0; i < 12 ; i++)
         {
-            doTest(testSourceCases[i], testTargetCases[i], results[i]);
+            doTest(myCollation, testSourceCases[i], testTargetCases[i], results[i]);
         }
     }
 }
@@ -179,7 +169,7 @@ void CollationFrenchTest::TestSecondary(/* char* par */)
         errln("Error setting attribute in French collator");
     else
     {
-        const int32_t testAcuteSize = (int32_t)(sizeof(testAcute) / sizeof(testAcute[0]));
+        const int32_t testAcuteSize = UPRV_LENGTHOF(testAcute);
         for (i = 0; i < testAcuteSize; i++)
         {
             for (j = 0; j < testAcuteSize; j++)
@@ -190,7 +180,7 @@ void CollationFrenchTest::TestSecondary(/* char* par */)
                     expected = Collator::EQUAL;
                 else // (i >  j)
                     expected = Collator::GREATER;
-                doTest(testAcute[i], testAcute[j], expected );
+                doTest(myCollation, testAcute[i], testAcute[j], expected );
             }
         }
     }
@@ -204,7 +194,7 @@ void CollationFrenchTest::TestExtra(/* char* par */)
     {
         for (j = i + 1; j < 10; j += 1)
         {
-            doTest(testBugs[i], testBugs[j], Collator::LESS);
+            doTest(myCollation, testBugs[i], testBugs[j], Collator::LESS);
         }
     }
 }
@@ -214,8 +204,8 @@ void CollationFrenchTest::runIndexedTest( int32_t index, UBool exec, const char*
     if (exec) logln("TestSuite CollationFrenchTest: ");
 
     if((!myCollation) && exec) {
-        errln(__FILE__ " cannot test - failed to create collator.");
-        name = "";
+        dataerrln(__FILE__ " cannot test - failed to create collator.");
+        name = "some test";
         return;
     }
 
@@ -227,3 +217,4 @@ void CollationFrenchTest::runIndexedTest( int32_t index, UBool exec, const char*
     }
 }
 
+#endif /* #if !UCONFIG_NO_COLLATION */
