@@ -89,7 +89,7 @@ class Integer : public UObject {
     virtual bool operator==(const UObject& other) const
     {
         return typeid(*this) == typeid(other) &&
-            _val == ((Integer&)other)._val;
+            _val == (dynamic_cast<Integer&>(const_cast<UObject&>(other)))._val;
     }
 
     public:
@@ -114,20 +114,20 @@ const char Integer::fgClassID = '\0';
 class TestIntegerService : public ICUService {
     public:
     ICUServiceKey* createKey(const UnicodeString* id, UErrorCode& status) const override {
-        return LocaleKey::createWithCanonicalFallback(id, NULL, status); // no fallback locale
+        return LocaleKey::createWithCanonicalFallback(id, nullptr, status); // no fallback locale
     }
 
     virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& status) override
     {
         Integer* i;
-        if (U_SUCCESS(status) && obj && (i = dynamic_cast<Integer*>(obj)) != NULL) {
+        if (U_SUCCESS(status) && obj && (i = dynamic_cast<Integer*>(obj)) != nullptr) {
             return new SimpleFactory(i, id, visible);
         }
-        return NULL;
+        return nullptr;
     }
 
     virtual UObject* cloneInstance(UObject* instance) const override {
-        return instance ? new Integer(*(Integer*)instance) : NULL;
+        return instance ? new Integer(*dynamic_cast<Integer*>(instance)) : nullptr;
     }
 };
 
@@ -157,21 +157,21 @@ char* /*par*/)
 UnicodeString append(UnicodeString& result, const UObject* obj) 
 {
     char buffer[128];
-    if (obj == NULL) {
-        result.append("NULL");
+    if (obj == nullptr) {
+        result.append("nullptr");
     } else {
         const UnicodeString* s;
         const Locale* loc;
         const Integer* i;
-        if ((s = dynamic_cast<const UnicodeString*>(obj)) != NULL) {
+        if ((s = dynamic_cast<const UnicodeString*>(obj)) != nullptr) {
             result.append(*s);
-        } else if ((loc = dynamic_cast<const Locale*>(obj)) != NULL) {
+        } else if ((loc = dynamic_cast<const Locale*>(obj)) != nullptr) {
             result.append(loc->getName());
-        } else if ((i = dynamic_cast<const Integer*>(obj)) != NULL) {
-            sprintf(buffer, "%d", (int)i->_val);
+        } else if ((i = dynamic_cast<const Integer*>(obj)) != nullptr) {
+            snprintf(buffer, sizeof(buffer), "%d", static_cast<int>(i->_val));
             result.append(buffer);
         } else {
-            sprintf(buffer, "%p", (const void*)obj);
+            snprintf(buffer, sizeof(buffer), "%p", (const void*)obj);
             result.append(buffer);
         }
     }
@@ -203,9 +203,9 @@ ICUServiceTest::confirmBoolean(const UnicodeString& message, UBool val)
 void
 ICUServiceTest::confirmEqual(const UnicodeString& message, const UObject* lhs, const UObject* rhs) 
 {
-    UBool equ = (lhs == NULL)
-        ? (rhs == NULL)
-        : (rhs != NULL && lhs->operator==(*rhs));
+    UBool equ = (lhs == nullptr)
+        ? (rhs == nullptr)
+        : (rhs != nullptr && lhs->operator==(*rhs));
 
     UnicodeString temp;
     lrmsg(temp, message, lhs, rhs);
@@ -220,9 +220,9 @@ ICUServiceTest::confirmEqual(const UnicodeString& message, const UObject* lhs, c
 void
 ICUServiceTest::confirmEqual(const UnicodeString& message, const Integer* lhs, const Integer* rhs) 
 {
-    UBool equ = (lhs == NULL)
-        ? (rhs == NULL)
-        : (rhs != NULL && lhs->operator==(*rhs));
+    UBool equ = (lhs == nullptr)
+        ? (rhs == nullptr)
+        : (rhs != nullptr && lhs->operator==(*rhs));
 
     UnicodeString temp;
     lrmsg(temp, message, lhs, rhs);
@@ -237,9 +237,9 @@ ICUServiceTest::confirmEqual(const UnicodeString& message, const Integer* lhs, c
 void
 ICUServiceTest::confirmEqual(const UnicodeString& message, const UnicodeString* lhs, const UnicodeString* rhs) 
 {
-    UBool equ = (lhs == NULL)
-        ? (rhs == NULL)
-        : (rhs != NULL && lhs->operator==(*rhs));
+    UBool equ = (lhs == nullptr)
+        ? (rhs == nullptr)
+        : (rhs != nullptr && lhs->operator==(*rhs));
 
     UnicodeString temp;
     lrmsg(temp, message, lhs, rhs);
@@ -254,9 +254,9 @@ ICUServiceTest::confirmEqual(const UnicodeString& message, const UnicodeString*
 void
 ICUServiceTest::confirmEqual(const UnicodeString& message, const Locale* lhs, const Locale* rhs) 
 {
-    UBool equ = (lhs == NULL)
-        ? (rhs == NULL)
-        : (rhs != NULL && lhs->operator==(*rhs));
+    UBool equ = (lhs == nullptr)
+        ? (rhs == nullptr)
+        : (rhs != nullptr && lhs->operator==(*rhs));
 
     UnicodeString temp;
     lrmsg(temp, message, lhs, rhs);
@@ -315,11 +315,11 @@ void
 ICUServiceTest::msgstr(const UnicodeString& message, UObject* obj, UBool err)
 {
     if (obj) {
-    UnicodeString* str = (UnicodeString*)obj;
+    UnicodeString* str = dynamic_cast<UnicodeString*>(obj);
         logln(message + *str);
         delete str;
     } else if (err) {
-        errln("Error " + message + "string is NULL");
+        errln("Error " + message + "string is nullptr");
     }
 }
 
@@ -337,7 +337,7 @@ ICUServiceTest::testAPI_One()
     service.registerInstance(singleton0, "en_US", status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_FOO", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_FOO", status));
         confirmEqual("1) en_US_FOO -> en_US", result, singleton0);
         delete result;
     }
@@ -349,7 +349,7 @@ ICUServiceTest::testAPI_One()
     service.registerInstance(singleton1, "en_US_FOO", status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_FOO", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_FOO", status));
         confirmEqual("2) en_US_FOO -> en_US_FOO", result, singleton1);
         delete result;
     }
@@ -357,7 +357,7 @@ ICUServiceTest::testAPI_One()
     // search for an object that falls back to the first registered locale
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_BAR", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_BAR", status));
         confirmEqual("3) en_US_BAR -> en_US", result, singleton0);
         delete result;
     }
@@ -379,7 +379,7 @@ ICUServiceTest::testAPI_One()
     // search for en_US should still find en_US object
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_BAR", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_BAR", status));
         confirmEqual("6) en_US_BAR -> en_US", result, singleton0);
         delete result;
     }
@@ -394,7 +394,7 @@ ICUServiceTest::testAPI_One()
     // should get data from that new factory
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_BAR", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_BAR", status));
         confirmEqual("10) en_US_BAR -> (3)", result, singleton3);
         delete result;
     }
@@ -411,7 +411,7 @@ ICUServiceTest::testAPI_One()
     // should get original data again after remove factory
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_BAR", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_BAR", status));
         confirmEqual("12) en_US_BAR -> (3)", result, singleton0);
         delete result;
     }
@@ -419,8 +419,8 @@ ICUServiceTest::testAPI_One()
     // shouldn't find unregistered ids
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("foo", status);
-        confirmIdentical("13) foo -> null", result, NULL);
+        Integer* result = dynamic_cast<Integer*>(service.get("foo", status));
+        confirmIdentical("13) foo -> null", result, nullptr);
         delete result;
     }
 
@@ -428,7 +428,7 @@ ICUServiceTest::testAPI_One()
     {
         UnicodeString resultID;
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("EN_us_fOo", &resultID, status);
+        Integer* result = dynamic_cast<Integer*>(service.get("EN_us_fOo", &resultID, status));
         confirmEqual("14a) find-non-canonical", result, singleton1);
         confirmStringsEqual("14b) find non-canonical", resultID, "en_US_FOO");
         delete result;
@@ -440,7 +440,7 @@ ICUServiceTest::testAPI_One()
     {
         UnicodeString resultID;
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("En_Ca_DuDe", &resultID, status);
+        Integer* result = dynamic_cast<Integer*>(service.get("En_Ca_DuDe", &resultID, status));
         confirmEqual("15a) find-non-canonical", result, singleton4);
         confirmStringsEqual("15b) register non-canonical", resultID, "en_CA_DUDE");
         delete result;
@@ -453,7 +453,7 @@ ICUServiceTest::testAPI_One()
     service.registerInstance(singleton5, "en_US_BAR", false, status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        Integer* result = (Integer*)service.get("en_US_BAR", status);
+        Integer* result = dynamic_cast<Integer*>(service.get("en_US_BAR", status));
         confirmEqual("17) get invisible", result, singleton5);
         delete result;
     }
@@ -484,34 +484,34 @@ public:
                 // have to implement cloneInstance.  Otherwise we could just tell the service
                 // what the object type is when we create it, and the default implementation
                 // could handle everything for us.  Phooey.
-        if (obj && dynamic_cast<UnicodeString*>(obj) != NULL) {
+        if (obj && dynamic_cast<UnicodeString*>(obj) != nullptr) {
                         return ICUService::createSimpleFactory(obj, id, visible, status);
         }
-        return NULL;
+        return nullptr;
     }
 
     virtual UObject* cloneInstance(UObject* instance) const override {
-        return instance ? new UnicodeString(*(UnicodeString*)instance) : NULL;
+        return instance ? new UnicodeString(*dynamic_cast<UnicodeString*>(instance)) : nullptr;
     }
 };
 
 class TestStringService : public ICUService {
     public:
     ICUServiceKey* createKey(const UnicodeString* id, UErrorCode& status) const override {
-        return LocaleKey::createWithCanonicalFallback(id, NULL, status); // no fallback locale
+        return LocaleKey::createWithCanonicalFallback(id, nullptr, status); // no fallback locale
     }
 
   virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& /* status */) override
     {
         UnicodeString* s;
-        if (obj && (s = dynamic_cast<UnicodeString*>(obj)) != NULL) {
+        if (obj && (s = dynamic_cast<UnicodeString*>(obj)) != nullptr) {
             return new SimpleFactory(s, id, visible);
         }
-        return NULL;
+        return nullptr;
     }
 
     virtual UObject* cloneInstance(UObject* instance) const override {
-        return instance ? new UnicodeString(*(UnicodeString*)instance) : NULL;
+        return instance ? new UnicodeString(*dynamic_cast<UnicodeString*>(instance)) : nullptr;
     }
 };
 
@@ -567,7 +567,7 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
 
     UObject* create(const ICUServiceKey& key, const ICUService* /* service */, UErrorCode& status) const override {
         if (U_FAILURE(status)) {
-        return NULL;
+        return nullptr;
         }
         UnicodeString temp;
         key.currentID(temp);
@@ -578,13 +578,13 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
         } else {
         status = _status;
     }
-        return NULL;
+        return nullptr;
     }
 
     void updateVisibleIDs(Hashtable& result, UErrorCode& status) const override {
         if (U_SUCCESS(_status)) {
             for (int32_t i = 0; i < _ids.size(); ++i) {
-                result.put(*(UnicodeString*)_ids[i], (void*)this, status);
+                result.put(*static_cast<UnicodeString*>(_ids[i]), (void*)this, status);
             }
         }
     }
@@ -593,7 +593,7 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
         if (U_SUCCESS(_status) && _ids.contains((void*)&id)) {
             char buffer[128];
             UErrorCode status = U_ZERO_ERROR;
-            int32_t len = id.extract(buffer, sizeof(buffer), NULL, status);
+            int32_t len = id.extract(buffer, sizeof(buffer), nullptr, status);
             if (U_SUCCESS(status)) {
                 if (len == sizeof(buffer)) {
                     --len;
@@ -633,7 +633,7 @@ ICUServiceTest::testAPI_Two()
     {
         UErrorCode status = U_ZERO_ERROR;
         const UnicodeString en_US = "en_US";
-        UnicodeString* result = (UnicodeString*)service.get(en_US, status);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(service.get(en_US, status));
         confirmEqual("21) locale", result, &en_US);
         delete result;
     }
@@ -643,7 +643,7 @@ ICUServiceTest::testAPI_Two()
         UErrorCode status = U_ZERO_ERROR;
         const UnicodeString en_US_BAR = "en_US_BAR";
         UnicodeString resultID;
-        UnicodeString* result = (UnicodeString*)service.get("EN_us_bar", &resultID, status);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(service.get("EN_us_bar", &resultID, status));
         confirmEqual("22) locale", &resultID, &en_US_BAR);
         delete result;
     }
@@ -653,7 +653,7 @@ ICUServiceTest::testAPI_Two()
     service.registerInstance(singleton0, "en_US_BAR", status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* result = (UnicodeString*)service.get("en_US_BAR", status);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(service.get("en_US_BAR", status));
         confirmEqual("23) override super", result, singleton0);
         delete result;
     }
@@ -662,8 +662,8 @@ ICUServiceTest::testAPI_Two()
     service.reset();
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* result = (UnicodeString*)service.get("en_US", status);
-        confirmIdentical("24) empty", result, NULL);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(service.get("en_US", status));
+        confirmIdentical("24) empty", result, nullptr);
     }
 
     // create a custom multiple key factory
@@ -686,8 +686,8 @@ ICUServiceTest::testAPI_Two()
         UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         for (int i = 0; i < ids.size(); ++i) {
-            const UnicodeString* id = (const UnicodeString*)ids[i];
-            UnicodeString* result = (UnicodeString*)service.get(*id, status);
+            const UnicodeString* id = static_cast<const UnicodeString*>(const_cast<const void*>(ids[i]));
+            UnicodeString* result = dynamic_cast<UnicodeString*>(service.get(*id, status));
             if (result) {
                 logln("  " + *id + " --> " + *result);
                 delete result;
@@ -705,7 +705,7 @@ ICUServiceTest::testAPI_Two()
         UVector names(status);
         service.getDisplayNames(names, status);
         for (int i = 0; i < names.size(); ++i) {
-            const StringPair* pair = (const StringPair*)names[i];
+            const StringPair* pair = static_cast<const StringPair*>(names[i]);
             logln("  " + pair->displayName + " --> " + pair->id);
         }
         confirmIdentical("26) display names", names.size(), 4);
@@ -745,7 +745,7 @@ ICUServiceTest::testAPI_Two()
         UVector names(status);
         service.getDisplayNames(names, Locale("es"), status);
         for (int i = 0; i < names.size(); ++i) {
-            const StringPair* pair = (const StringPair*)names[i];
+            const StringPair* pair = static_cast<const StringPair*>(names[i]);
             logln("  " + pair->displayName + " --> " + pair->id);
         }
         confirmIdentical("29) display names", names.size(), 7);
@@ -757,8 +757,8 @@ ICUServiceTest::testAPI_Two()
         UErrorCode status = U_ZERO_ERROR;
         UnicodeString actualID;
         UnicodeString id = "en_us_surfer_gal";
-        UnicodeString* gal = (UnicodeString*)service.get(id, &actualID, status);
-        if (gal != NULL) {
+        UnicodeString* gal = dynamic_cast<UnicodeString*>(service.get(id, &actualID, status));
+        if (gal != nullptr) {
             UnicodeString displayName;
             logln("actual id: " + actualID);
             service.getDisplayName(actualID, displayName, Locale::getEnglish());
@@ -780,8 +780,8 @@ ICUServiceTest::testAPI_Two()
         UErrorCode status = U_ZERO_ERROR;
         UnicodeString actualID;
         UnicodeString id = "en_US_SURFER_BOZO";
-        UnicodeString* bozo = (UnicodeString*)service.get(id, &actualID, status);
-        if (bozo != NULL) {
+        UnicodeString* bozo = dynamic_cast<UnicodeString*>(service.get(id, &actualID, status));
+        if (bozo != nullptr) {
             UnicodeString displayName;
             service.getDisplayName(actualID, displayName, Locale::getEnglish());
             logln("found actual: " + *bozo + " with display name: " + displayName);
@@ -807,7 +807,7 @@ ICUServiceTest::testAPI_Two()
         UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         for (int i = 0; i < ids.size(); ++i) {
-            const UnicodeString* id = (const UnicodeString*)ids[i];
+            const UnicodeString* id = static_cast<const UnicodeString*>(const_cast<const void*>(ids[i]));
             msgstr(*id + "? ", service.get(*id, status));
         }
 
@@ -826,16 +826,16 @@ class CalifornioLanguageFactory : public ICUResourceBundleFactory
     static const char* valley; // = californio ## "_VALLEY";
     static const char* surfer; // = californio ## "_SURFER";
     static const char* geek; // = californio ## "_GEEK";
-    static Hashtable* supportedIDs; // = NULL;
+    static Hashtable* supportedIDs; // = nullptr;
 
-    static void cleanup(void) {
+    static void cleanup() {
       delete supportedIDs;
-      supportedIDs = NULL;
+      supportedIDs = nullptr;
     }
 
     const Hashtable* getSupportedIDs(UErrorCode& status) const override
     {
-        if (supportedIDs == NULL) {
+        if (supportedIDs == nullptr) {
             Hashtable* table = new Hashtable();
             table->put(UnicodeString(californio), (void*)table, status);
             table->put(UnicodeString(valley), (void*)table, status);
@@ -887,7 +887,7 @@ const char* CalifornioLanguageFactory::californio = "en_US_CA";
 const char* CalifornioLanguageFactory::valley = "en_US_CA_VALLEY";
 const char* CalifornioLanguageFactory::surfer = "en_US_CA_SURFER";
 const char* CalifornioLanguageFactory::geek = "en_US_CA_GEEK";
-Hashtable* CalifornioLanguageFactory::supportedIDs = NULL;
+Hashtable* CalifornioLanguageFactory::supportedIDs = nullptr;
 
 void
 ICUServiceTest::testRBF()
@@ -904,7 +904,7 @@ ICUServiceTest::testRBF()
         service.getVisibleIDs(ids, status);
         logln("all visible ids:");
         for (int i = 0; i < ids.size(); ++i) {
-            const UnicodeString* id = (const UnicodeString*)ids[i];
+            const UnicodeString* id = static_cast<const UnicodeString*>(const_cast<const void*>(ids[i]));
             logln(*id);
         }
     }
@@ -917,7 +917,7 @@ ICUServiceTest::testRBF()
         service.getDisplayNames(names, Locale::getGermany(), status);
         logln("service display names for de_DE");
         for (int i = 0; i < names.size(); ++i) {
-            const StringPair* pair = (const StringPair*)names[i];
+            const StringPair* pair = static_cast<const StringPair*>(names[i]);
             logln("  " + pair->displayName + " --> " + pair->id);
         }
     }
@@ -942,7 +942,7 @@ ICUServiceTest::testRBF()
                 UVector names(status);
                 service.getDisplayNames(names, idNames[i], status);
                 for (int i = 0; i < names.size(); ++i) {
-                    const StringPair* pair = (const StringPair*)names[i];
+                    const StringPair* pair = static_cast<const StringPair*>(names[i]);
                     logln("  " + pair->displayName + " --> " + pair->id);
                 }
             }
@@ -986,7 +986,7 @@ ICUServiceTest::testNotification()
         logln("registering bar... ");
         ls.registerInstance(new UnicodeString("Bar"), "en_BAR", status);
         logln("getting foo...");
-        UnicodeString* result = (UnicodeString*)ls.get("en_FOO", status);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(ls.get("en_FOO", status));
         logln(*result);
         delete result;
 
@@ -1031,7 +1031,7 @@ public void serviceChanged(ICUService s) {
 class TestStringLocaleService : public ICULocaleService {
     public:
     virtual UObject* cloneInstance(UObject* instance) const override {
-        return instance ? new UnicodeString(*(UnicodeString*)instance) : NULL;
+        return instance ? new UnicodeString(*dynamic_cast<UnicodeString*>(instance)) : nullptr;
     }
 };
 
@@ -1048,26 +1048,26 @@ void ICUServiceTest::testLocale() {
     service.registerInstance(root, "", status);
     service.registerInstance(german, "de", status);
     service.registerInstance(germany, Locale::getGermany(), status);
-    service.registerInstance(japanese, (UnicodeString)"ja", true, status);
+    service.registerInstance(japanese, UnicodeString("ja"), true, status);
     service.registerInstance(japan, Locale::getJapan(), status);
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", status));
         confirmEqual("test de_US", german, target);
         delete target;
     }
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", LocaleKey::KIND_ANY, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", LocaleKey::KIND_ANY, status));
         confirmEqual("test de_US 2", german, target);
         delete target;
     }
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", 1234, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", 1234, status));
         confirmEqual("test de_US 3", german, target);
         delete target;
     }
@@ -1075,7 +1075,7 @@ void ICUServiceTest::testLocale() {
     {
         UErrorCode status = U_ZERO_ERROR;
         Locale actualReturn;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", &actualReturn, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", &actualReturn, status));
         confirmEqual("test de_US 5", german, target);
         confirmEqual("test de_US 6", &actualReturn, &Locale::getGerman());
         delete target;
@@ -1084,7 +1084,7 @@ void ICUServiceTest::testLocale() {
     {
         UErrorCode status = U_ZERO_ERROR;
         Locale actualReturn;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", LocaleKey::KIND_ANY, &actualReturn, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", LocaleKey::KIND_ANY, &actualReturn, status));
         confirmEqual("test de_US 7", &actualReturn, &Locale::getGerman());
         delete target;
     }
@@ -1092,7 +1092,7 @@ void ICUServiceTest::testLocale() {
     {
         UErrorCode status = U_ZERO_ERROR;
         Locale actualReturn;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", 1234, &actualReturn, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", 1234, &actualReturn, status));
         confirmEqual("test de_US 8", german, target);
         confirmEqual("test de_US 9", &actualReturn, &Locale::getGerman());
         delete target;
@@ -1106,21 +1106,21 @@ void ICUServiceTest::testLocale() {
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", 1, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", 1, status));
         confirmEqual("test de_US kind 1", one, target);
         delete target;
     }
         
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", 2, status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", 2, status));
         confirmEqual("test de_US kind 2", two, target);
         delete target;
     }
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("de_US", status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("de_US", status));
         confirmEqual("test de_US kind 3", german, target);
         delete target;
     }
@@ -1130,7 +1130,7 @@ void ICUServiceTest::testLocale() {
         UnicodeString english = "en";
         Locale localeResult;
         UnicodeString result;
-        LocaleKey* lkey = LocaleKey::createWithCanonicalFallback(&english, NULL, 1234, status);
+        LocaleKey* lkey = LocaleKey::createWithCanonicalFallback(&english, nullptr, 1234, status);
         logln("lkey prefix: " + lkey->prefix(result));
         result.remove();
         logln("lkey descriptor: " + lkey->currentDescriptor(result));
@@ -1150,7 +1150,7 @@ void ICUServiceTest::testLocale() {
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("za_PPP", status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("za_PPP", status));
         confirmEqual("test zappp", root, target);
         delete target;
     }
@@ -1159,7 +1159,7 @@ void ICUServiceTest::testLocale() {
     Locale::setDefault(Locale::getJapanese(), status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("za_PPP", status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("za_PPP", status));
         confirmEqual("test with ja locale", japanese, target);
         delete target;
     }
@@ -1170,7 +1170,7 @@ void ICUServiceTest::testLocale() {
         service.getVisibleIDs(ids, status);
         logln("all visible ids:");
         for (int i = 0; i < ids.size(); ++i) {
-            const UnicodeString* id = (const UnicodeString*)ids[i];
+            const UnicodeString* id = static_cast<const UnicodeString*>(const_cast<const void*>(ids[i]));
             logln(*id);
         }
     }
@@ -1182,14 +1182,14 @@ void ICUServiceTest::testLocale() {
         service.getVisibleIDs(ids, status);
         logln("all visible ids:");
         for (int i = 0; i < ids.size(); ++i) {
-            const UnicodeString* id = (const UnicodeString*)ids[i];
+            const UnicodeString* id = static_cast<const UnicodeString*>(const_cast<const void*>(ids[i]));
             logln(*id);
         }
     }
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* target = (UnicodeString*)service.get("za_PPP", status);
+        UnicodeString* target = dynamic_cast<UnicodeString*>(service.get("za_PPP", status));
         confirmEqual("test with en locale", root, target);
         delete target;
     }
@@ -1202,7 +1202,7 @@ void ICUServiceTest::testLocale() {
             logln("locales: ");
             {
                 const char* p;
-                while ((p = locales->next(NULL, status))) {
+                while ((p = locales->next(nullptr, status))) {
                     logln(p);
                 }
             }
@@ -1217,7 +1217,7 @@ void ICUServiceTest::testLocale() {
 class WrapFactory : public ICUServiceFactory {
     public:
     static const UnicodeString& getGreetingID() {
-      if (greetingID == NULL) {
+      if (greetingID == nullptr) {
     greetingID = new UnicodeString("greeting");
       }
       return *greetingID;
@@ -1225,14 +1225,14 @@ class WrapFactory : public ICUServiceFactory {
 
   static void cleanup() {
     delete greetingID;
-    greetingID = NULL;
+    greetingID = nullptr;
   }
 
     UObject* create(const ICUServiceKey& key, const ICUService* service, UErrorCode& status) const override {
         if (U_SUCCESS(status)) {
             UnicodeString temp;
             if (key.currentID(temp).compare(getGreetingID()) == 0) {
-                UnicodeString* previous = (UnicodeString*)service->getKey((ICUServiceKey&)key, NULL, this, status);
+                UnicodeString* previous = dynamic_cast<UnicodeString*>(service->getKey(const_cast<ICUServiceKey&>(key), nullptr, this, status));
                 if (previous) {
                     previous->insert(0, "A different greeting: \"");
                     previous->append("\"");
@@ -1240,7 +1240,7 @@ class WrapFactory : public ICUServiceFactory {
                 }
             }
         }
-        return NULL;
+        return nullptr;
     }
 
     void updateVisibleIDs(Hashtable& result, UErrorCode& status) const override {
@@ -1272,7 +1272,7 @@ class WrapFactory : public ICUServiceFactory {
     static UnicodeString* greetingID;
 };
 
-UnicodeString* WrapFactory::greetingID = NULL;
+UnicodeString* WrapFactory::greetingID = nullptr;
 const char WrapFactory::fgClassID = '\0';
 
 void 
@@ -1286,7 +1286,7 @@ ICUServiceTest::testWrapFactory()
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* result = (UnicodeString*)service.get(greetingID, status);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(service.get(greetingID, status));
         if (result) {
             logln("test one: " + *result);
             delete result;
@@ -1296,7 +1296,7 @@ ICUServiceTest::testWrapFactory()
     service.registerFactory(new WrapFactory(), status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        UnicodeString* result = (UnicodeString*)service.get(greetingID, status);
+        UnicodeString* result = dynamic_cast<UnicodeString*>(service.get(greetingID, status));
         UnicodeString target = "A different greeting: \"Hello There\"";
         confirmEqual("wrap test: ", result, &target);
         delete result;
@@ -1342,7 +1342,7 @@ void ICUServiceTest::testCoverage()
                 service.registerFactory(sf,     status);
 
                 {
-                        UnicodeString* result   = (UnicodeString*)service.get("object", status);
+                        UnicodeString* result   = dynamic_cast<UnicodeString*>(service.get("object", status));
                         if (result) {
                                 logln("object is: "     + *result);
                                 delete result;
@@ -1361,7 +1361,7 @@ void ICUServiceTest::testCoverage()
           TestStringSimpleKeyService service;
           service.registerInstance(howdy, "Greetings", status);
           {
-                  UnicodeString* result = (UnicodeString*)service.get("Greetings",      status);
+                  UnicodeString* result = dynamic_cast<UnicodeString*>(service.get("Greetings",      status));
                   if (result) {
                           logln("object is: "   + *result);
                           delete result;
@@ -1416,7 +1416,7 @@ void ICUServiceTest::testCoverage()
     LKFSubclass lkf(true); // empty
     Hashtable table;
 
-    UObject *obj = lkf.create(*key, NULL, status);
+    UObject *obj = lkf.create(*key, nullptr, status);
     logln("obj: " + UnicodeString(obj ? "obj" : "null"));
     logln(lkf.getDisplayName("en_US", Locale::getDefault(), result));
     lkf.updateVisibleIDs(table, status);
@@ -1426,7 +1426,7 @@ void ICUServiceTest::testCoverage()
     }
 
     LKFSubclass invisibleLKF(false);
-    obj = lkf.create(*key, NULL, status);
+    obj = lkf.create(*key, nullptr, status);
     logln("obj: " + UnicodeString(obj ? "obj" : "null"));
     logln(invisibleLKF.getDisplayName("en_US", Locale::getDefault(), result.remove()));
     invisibleLKF.updateVisibleIDs(table, status);
@@ -1459,8 +1459,8 @@ void ICUServiceTest::testCoverage()
         // ResourceBundleFactory
     key = LocaleKey::createWithCanonicalFallback(&primary, &fallback, status);
         ICUResourceBundleFactory rbf;
-        UObject* icurb = rbf.create(*key, NULL, status);
-        if (icurb != NULL) {
+        UObject* icurb = rbf.create(*key, nullptr, status);
+        if (icurb != nullptr) {
                 logln("got resource bundle for key");
                 delete icurb;
         }
