@@ -13,21 +13,22 @@
 #ifndef _INTLTEST
 #define _INTLTEST
 
-// The following includes utypes.h, uobject.h and unistr.h
-#include "unicode/fmtable.h"
-#include "unicode/testlog.h"
-#include "unicode/uniset.h"
-
 #include <vector>
 #include <string>
+#include <string_view>
+
+#include "unicode/utypes.h"
+#include "unicode/testlog.h"
+
+#if U_SHOW_CPLUSPLUS_API
+#include "unicode/fmtable.h"
+#include "unicode/uniset.h"
+#include "unicode/unistr.h"
+#endif
 
 U_NAMESPACE_USE
 
-#if U_PLATFORM == U_PF_OS390
-// avoid collision with math.h/log()
-// this must be after including utypes.h so that U_PLATFORM is actually defined
-#pragma map(IntlTest::log( const UnicodeString &message ),"logos390")
-#endif
+#if U_SHOW_CPLUSPLUS_API
 
 //-----------------------------------------------------------------------------
 //convenience classes to ease porting code that uses the Java
@@ -35,9 +36,8 @@ U_NAMESPACE_USE
 UnicodeString UCharToUnicodeString(char16_t c);
 UnicodeString Int64ToUnicodeString(int64_t num);
 UnicodeString DoubleToUnicodeString(double num);
-//UnicodeString operator+(const UnicodeString& left, int64_t num); // Some compilers don't allow this because of the long type.
-UnicodeString operator+(const UnicodeString& left, long num);
-UnicodeString operator+(const UnicodeString& left, unsigned long num);
+UnicodeString operator+(const UnicodeString& left, int64_t num);
+UnicodeString operator+(const UnicodeString& left, uint64_t num);
 UnicodeString operator+(const UnicodeString& left, double num);
 UnicodeString operator+(const UnicodeString& left, char num);
 UnicodeString operator+(const UnicodeString& left, short num);
@@ -52,6 +52,8 @@ UnicodeString toString(int32_t n);
 #endif
 UnicodeString toString(UBool b);
 
+#endif  // U_SHOW_CPLUSPLUS_API
+
 //-----------------------------------------------------------------------------
 
 // Use the TESTCASE macro in subclasses of IntlTest.  Define the
@@ -159,9 +161,9 @@ public:
     virtual void setCaller( IntlTest* callingTest ); // for internal use only
     virtual void setPath( char* path ); // for internal use only
 
-    virtual void log( const UnicodeString &message );
+    virtual void log(std::u16string_view message);
 
-    virtual void logln( const UnicodeString &message ) override;
+    virtual void logln(std::u16string_view message) override;
 
     virtual void logln();
 
@@ -173,7 +175,7 @@ public:
      * @param message optional message string
      * @return true if test should be skipped
      */
-    UBool logKnownIssue( const char *ticket, const UnicodeString &message );
+    UBool logKnownIssue( const char *ticket, std::u16string_view message);
     /**
      * Logs that an issue is known. Can be called multiple times.
      * Usually used this way:
@@ -197,23 +199,23 @@ public:
     UBool skipLSTMTest();
 #endif /* #if !UCONFIG_NO_BREAK_ITERATION */
 
-    virtual void info( const UnicodeString &message );
+    virtual void info(std::u16string_view message);
 
-    virtual void infoln( const UnicodeString &message );
+    virtual void infoln(std::u16string_view message);
 
     virtual void infoln();
 
     virtual void err();
 
-    virtual void err( const UnicodeString &message );
+    virtual void err(std::u16string_view message);
 
-    virtual void errln( const UnicodeString &message ) override;
+    virtual void errln(std::u16string_view message) override;
 
-    virtual void dataerr( const UnicodeString &message );
+    virtual void dataerr(std::u16string_view message);
 
-    virtual void dataerrln( const UnicodeString &message ) override;
+    virtual void dataerrln(std::u16string_view message) override;
 
-    void errcheckln(UErrorCode status, const UnicodeString &message );
+    void errcheckln(UErrorCode status, std::u16string_view message);
 
     // convenience functions: sprintf() + errln() etc.
     void log(const char *fmt, ...);
@@ -294,13 +296,99 @@ public:
      * @return true on success, false on failure.
      */
     UBool assertSuccess(const char* message, UErrorCode ec, UBool possibleDataError=false, const char *file=nullptr, int line=0);
-    UBool assertEquals(const char* message, const UnicodeString& expected,
-                       const UnicodeString& actual, UBool possibleDataError=false);
+    UBool assertEquals(const char* message, std::u16string_view expected,
+                       std::u16string_view actual, UBool possibleDataError=false);
     UBool assertEquals(const char* message, const char* expected, const char* actual);
-    UBool assertEquals(const char* message, UBool expected, UBool actual);
-    UBool assertEquals(const char* message, int32_t expected, int32_t actual);
-    UBool assertEquals(const char* message, int64_t expected, int64_t actual);
+
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<(std::is_same_v<T, UBool> || std::is_same_v<T, bool>) &&
+                                          (std::is_same_v<U, UBool> || std::is_same_v<U, bool>)>>
+    bool assertEquals(const Message &message, T expected, U actual) {
+        return assertBooleanEquals(extractToAssertBuf(message), expected, actual);
+    }
+
+    // Since UChar32 is int32_t, we do not know whether we are looking at code points or some other
+    // number.  Show both the decimal and hexadecimal representations on failure.
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<std::is_same_v<T, int32_t> && std::is_same_v<U, int32_t>>,
+              typename = void, typename = void>
+    bool assertEquals(const Message &message, T expected, U actual) {
+        return assertSigned32Equals(extractToAssertBuf(message), expected, actual);
+    }
+
+    // This overload applies when at least one of the sides of the comparison is char32_t or char16_t.
+    // In this case, we can be pretty sure that we are comparing code points or UTF-16 (or UTF-32) code
+    // units, so we do not show the decimal value, only the hex value in U+ notation, and we show the
+    // literal character as well.
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<std::is_same_v<T, char32_t> || std::is_same_v<U, char32_t> ||
+                                          std::is_same_v<T, char16_t> || std::is_same_v<U, char16_t>>,
+              typename = void>
+    bool assertEquals(const Message& message, T expected, U actual) {
+        return assertCodePointEquals(extractToAssertBuf(message),
+                                     static_cast<char32_t>(expected),
+                                     static_cast<char32_t>(actual));
+    }
+
+    // Otherwise, for enumeration or integral types, log only the decimal value.
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<(std::is_integral_v<T> || std::is_enum_v<T>) &&
+                                          (std::is_integral_v<U> || std::is_enum_v<U>) &&
+                                          !(std::is_same_v<T, int32_t> && std::is_same_v<U, int32_t>) &&
+                                          !((std::is_same_v<T, UBool> || std::is_same_v<T, bool>) &&
+                                            (std::is_same_v<U, UBool> || std::is_same_v<U, bool>)) &&
+                                          !(std::is_same_v<T, char32_t> || std::is_same_v<U, char32_t> ||
+                                            std::is_same_v<T, char16_t> || std::is_same_v<U, char16_t>)>,
+              typename = void, typename = void, typename = void>
+    bool assertEquals(const Message &message, T expected, U actual) {
+        return assertSigned64Equals(extractToAssertBuf(message), expected, actual);
+    }
+
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<(std::is_same_v<T, UBool> || std::is_same_v<T, bool>) &&
+                                          (std::is_same_v<U, UBool> || std::is_same_v<U, bool>)>>
+    bool assertNotEquals(const Message &message, T expected, U actual) {
+        return assertBooleanNotEquals(extractToAssertBuf(message), expected, actual);
+    }
+
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<std::is_same_v<T, int32_t> && std::is_same_v<U, int32_t>>,
+              typename = void, typename = void>
+    bool assertNotEquals(const Message &message, T expected, U actual) {
+        return assertSigned32NotEquals(extractToAssertBuf(message), expected, actual);
+    }
+
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<std::is_same_v<T, char32_t> || std::is_same_v<U, char32_t> ||
+                                          std::is_same_v<T, char16_t> || std::is_same_v<U, char16_t>>,
+              typename = void>
+    bool assertNotEquals(const Message &message, T expected, U actual) {
+        return assertCodePointNotEquals(extractToAssertBuf(message),
+                                        static_cast<char32_t>(expected),
+                                        static_cast<char32_t>(actual));
+    }
+
+    template <typename Message, typename T, typename U,
+              typename = std::enable_if_t<(std::is_integral_v<T> || std::is_enum_v<T>) &&
+                                          (std::is_integral_v<U> || std::is_enum_v<U>) &&
+                                          !(std::is_same_v<T, int32_t> && std::is_same_v<U, int32_t>) &&
+                                          !((std::is_same_v<T, UBool> || std::is_same_v<T, bool>) &&
+                                            (std::is_same_v<U, UBool> || std::is_same_v<U, bool>)) &&
+                                          !(std::is_same_v<T, char32_t> || std::is_same_v<U, char32_t> ||
+                                            std::is_same_v<T, char16_t> || std::is_same_v<U, char16_t>)>,
+              typename = void, typename = void, typename = void>
+    bool assertNotEquals(const Message &message, T expected, U actual) {
+        return assertSigned64NotEquals(extractToAssertBuf(message), expected, actual);
+    }
+
     UBool assertEquals(const char* message, double expected, double actual);
+
+    // for disambiguation
+    UBool assertEquals(const char* message, const char* expected,
+                       std::u16string_view actual, UBool possibleDataError=false);
+    UBool assertEquals(const char* message, std::u16string_view expected,
+                       const char* actual, UBool possibleDataError=false);
+
     /**
      * Asserts that two doubles are equal to within a positive delta. Returns
      * false if they are not.
@@ -316,27 +404,33 @@ public:
      */
     UBool assertEqualsNear(const char* message, double expected, double actual, double delta);
     UBool assertEquals(const char* message, UErrorCode expected, UErrorCode actual);
+#if U_SHOW_CPLUSPLUS_API
     UBool assertEquals(const char* message, const UnicodeSet& expected, const UnicodeSet& actual);
+#endif
     UBool assertEquals(const char* message,
         const std::vector<std::string>& expected, const std::vector<std::string>& actual);
 
+#if U_SHOW_CPLUSPLUS_API
 #if !UCONFIG_NO_FORMATTING
-    UBool assertEquals(const char* message, const Formattable& expected,
-                       const Formattable& actual, UBool possibleDataError=false);
-    UBool assertEquals(const UnicodeString& message, const Formattable& expected,
-                       const Formattable& actual);
+    UBool assertEqualFormattables(const char* message, const Formattable& expected,
+                                  const Formattable& actual, UBool possibleDataError=false);
+    UBool assertEqualFormattables(std::u16string_view message, const Formattable& expected,
+                                  const Formattable& actual);
 #endif
-    UBool assertNotEquals(const char* message, int32_t expectedNot, int32_t actual);
-    UBool assertTrue(const UnicodeString& message, UBool condition, UBool quiet=false, UBool possibleDataError=false);
-    UBool assertFalse(const UnicodeString& message, UBool condition, UBool quiet=false, UBool possibleDataError=false);
-    UBool assertSuccess(const UnicodeString& message, UErrorCode ec);
-    UBool assertEquals(const UnicodeString& message, const UnicodeString& expected,
-                       const UnicodeString& actual, UBool possibleDataError=false);
-    UBool assertEquals(const UnicodeString& message, const char* expected, const char* actual);
-    UBool assertEquals(const UnicodeString& message, UBool expected, UBool actual);
-    UBool assertEquals(const UnicodeString& message, int32_t expected, int32_t actual);
-    UBool assertEquals(const UnicodeString& message, int64_t expected, int64_t actual);
-    UBool assertEquals(const UnicodeString& message, double expected, double actual);
+#endif
+    UBool assertTrue(std::u16string_view message, UBool condition, UBool quiet=false, UBool possibleDataError=false);
+    UBool assertFalse(std::u16string_view message, UBool condition, UBool quiet=false, UBool possibleDataError=false);
+    UBool assertSuccess(std::u16string_view message, UErrorCode ec);
+    UBool assertEquals(std::u16string_view message, std::u16string_view expected,
+                       std::u16string_view actual, UBool possibleDataError=false);
+    UBool assertEquals(std::u16string_view message, const char* expected, const char* actual);
+    UBool assertEquals(std::u16string_view message, double expected, double actual);
+
+    // for disambiguation
+    UBool assertEquals(std::u16string_view message, const char* expected,
+                       std::u16string_view actual, UBool possibleDataError=false);
+
+
     /**
      * Asserts that two doubles are equal to within a positive delta. Returns
      * false if they are not.
@@ -350,12 +444,13 @@ public:
      * @param delta - the maximum delta between expected and actual for which
      * both numbers are still considered equal.
      */
-    UBool assertEqualsNear(const UnicodeString& message, double expected, double actual, double delta);
-    UBool assertEquals(const UnicodeString& message, UErrorCode expected, UErrorCode actual);
-    UBool assertEquals(const UnicodeString& message, const UnicodeSet& expected, const UnicodeSet& actual);
-    UBool assertEquals(const UnicodeString& message,
+    UBool assertEqualsNear(std::u16string_view message, double expected, double actual, double delta);
+    UBool assertEquals(std::u16string_view message, UErrorCode expected, UErrorCode actual);
+#if U_SHOW_CPLUSPLUS_API
+    UBool assertEquals(std::u16string_view message, const UnicodeSet& expected, const UnicodeSet& actual);
+#endif
+    UBool assertEquals(std::u16string_view message,
         const std::vector<std::string>& expected, const std::vector<std::string>& actual);
-    UBool assertNotEquals(const UnicodeString& message, int32_t expectedNot, int32_t actual);
 
     virtual void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = nullptr ); // override !
 
@@ -378,6 +473,24 @@ public:
     int32_t     threadCount;
 
 private:
+    // This takes int8_t (aka UBool), not bool, so that in case we actually mean to compare int8_t
+    // integers, the test result is at least correct. The error message will still be useless (unless one
+    // of the values is 0, it will say that we expected true and got true instead).
+    bool assertBooleanEquals(const char *message, int8_t expected, int8_t actual);
+    bool assertSigned32Equals(const char *message, int32_t expected, int32_t actual);
+    bool assertSigned64Equals(const char *message, int64_t expected, int64_t actual);
+    bool assertCodePointEquals(const char *message, char32_t expected, char32_t actual);
+
+    bool assertBooleanNotEquals(const char *message, int8_t expected, int8_t actual);
+    bool assertSigned32NotEquals(const char *message, int32_t expected, int32_t actual);
+    bool assertSigned64NotEquals(const char *message, int64_t expected, int64_t actual);
+    bool assertCodePointNotEquals(const char *message, char32_t expected, char32_t actual);
+
+    static const char* extractToAssertBuf(const char* message) {
+      return message;
+    }
+    static const char* extractToAssertBuf(std::u16string_view message);
+
     UBool       LL_linestart;
     int32_t     LL_indentlevel;
 
@@ -397,8 +510,9 @@ private:
 
 protected:
 
-    virtual void LL_message( UnicodeString message, UBool newline );
+    virtual void LL_message(std::u16string_view message, UBool newline);
 
+#if U_SHOW_CPLUSPLUS_API
     // used for collation result reporting, defined here for convenience
 
     static UnicodeString &prettify(const UnicodeString &source, UnicodeString &target);
@@ -407,8 +521,9 @@ protected:
     static UnicodeString &appendHex(uint32_t number, int32_t digits, UnicodeString &target);
     static UnicodeString toHex(uint32_t number, int32_t digits=-1);
     static inline UnicodeString toHex(int32_t number, int32_t digits=-1) {
-        return toHex((uint32_t)number, digits);
+        return toHex(static_cast<uint32_t>(number), digits);
     }
+#endif
 
 public:
     static void setICU_DATA();       // Set up ICU_DATA if necessary.
@@ -420,6 +535,8 @@ public:
     static const char* loadTestData(UErrorCode& err);
     virtual const char* getTestDataPath(UErrorCode& err) override;
     static const char* getSourceTestData(UErrorCode& err);
+    // Gets the path for the top-level testdata/ directory
+    static const char* getSharedTestData(UErrorCode& err);
     static char *getUnidataPath(char path[]);
     char16_t *ReadAndConvertFile(const char *fileName, int &ulen, const char *encoding, UErrorCode &status);
 
@@ -431,17 +548,24 @@ public:
 
 };
 
-void it_log( UnicodeString message );
-void it_logln( UnicodeString message );
+void it_log(std::u16string_view message);
+void it_logln(std::u16string_view message);
 void it_logln();
-void it_info( UnicodeString message );
-void it_infoln( UnicodeString message );
+void it_info(std::u16string_view message);
+void it_infoln(std::u16string_view message);
 void it_infoln();
 void it_err();
-void it_err( UnicodeString message );
-void it_errln( UnicodeString message );
-void it_dataerr( UnicodeString message );
-void it_dataerrln( UnicodeString message );
+void it_err(std::u16string_view message);
+void it_errln(std::u16string_view message);
+void it_dataerr(std::u16string_view message);
+void it_dataerrln(std::u16string_view message);
+
+void it_logln(const char* message);
+void it_err(const char* message);
+void it_errln(const char* message);
+void it_dataerrln(const char* message);
+
+#if U_SHOW_CPLUSPLUS_API
 
 /**
  * This is a variant of cintltst/ccolltst.c:CharsToUChars().
@@ -453,4 +577,6 @@ extern UnicodeString CharsToUnicodeString(const char* chars);
 /* alias for CharsToUnicodeString */
 extern UnicodeString ctou(const char* chars);
 
+#endif
+
 #endif // _INTLTEST
