@@ -91,6 +91,7 @@ public:
    void TestGetUnicodeKeywordValueStdString();
 
    /* Test setting keyword values */
+   void TestSetKeywordValueImp();
    void TestSetKeywordValue();
    void TestSetKeywordValueStringPiece();
    void TestSetUnicodeKeywordValueStringPiece();
@@ -164,6 +165,7 @@ public:
     void TestLongLocaleSetKeywordAssign();
     void TestLongLocaleSetKeywordMoveAssign();
     void TestSierraLeoneCurrency21997();
+    void TestPayload();
 
 private:
     void _checklocs(const char* label,
