@@ -54,6 +54,7 @@ MessageFormatRegressionTest::runIndexedTest( int32_t index, UBool exec, const ch
     TESTCASE_AUTO(TestChoicePatternQuote);
     TESTCASE_AUTO(Test4112104);
     TESTCASE_AUTO(TestICU12584);
+    TESTCASE_AUTO(TestICU22798);
     TESTCASE_AUTO(TestAPI);
     TESTCASE_AUTO_END;
 }
@@ -114,15 +115,15 @@ void MessageFormatRegressionTest::Test4074764() {
         failure(status, "messageFormatter->applyPattern");
 
         // {sfb} how much does this apply in C++?
-        // do we want to verify that the Formattable* array is not NULL,
+        // do we want to verify that the Formattable* array is not nullptr,
         // or is that the user's responsibility?
         // additionally, what should be the item count?
         // for bug testing purposes, assume that something was set to
-        // NULL by mistake, and that the length should be non-zero
+        // nullptr by mistake, and that the length should be non-zero
 
-        //tempBuffer = messageFormatter->format(NULL, 1, tempBuffer, FieldPosition(FieldPosition::DONT_CARE), status);
+        //tempBuffer = messageFormatter->format(nullptr, 1, tempBuffer, FieldPosition(FieldPosition::DONT_CARE), status);
         tempBuffer.remove();
-        tempBuffer = messageFormatter->format(NULL, 0, tempBuffer, pos, status);
+        tempBuffer = messageFormatter->format(nullptr, 0, tempBuffer, pos, status);
 
         if( tempBuffer != "Message without param" || failure(status, "messageFormat->format"))
             errln("MessageFormat with no param test failed.");
@@ -194,7 +195,7 @@ void MessageFormatRegressionTest::Test4031438()
         failure(status, "messageFormat->applyPattern");
         //Object[] params = {new Integer(7)};
         Formattable params []= {
-            Formattable((int32_t)7)
+            Formattable(static_cast<int32_t>(7))
         };
         UnicodeString tempBuffer;
         FieldPosition pos(FieldPosition::DONT_CARE);
@@ -205,10 +206,10 @@ void MessageFormatRegressionTest::Test4031438()
         ParsePosition pp(0);
         int32_t count = 0;
         Formattable *objs = messageFormatter->parse(tempBuffer, pp, count);
-        //if(objs[7/*params.length*/] != NULL)
+        //if(objs[7/*params.length*/] != nullptr)
         //    errln("Parse failed with more than expected arguments");
 
-        NumberFormat *fmt = 0;
+        NumberFormat* fmt = nullptr;
         UnicodeString temp, temp1;
 
         for (int i = 0; i < count; i++) {
@@ -243,7 +244,7 @@ void MessageFormatRegressionTest::Test4031438()
                 }
             }
 
-            //if (objs[i] != NULL && objs[i].getString(temp1) != params[i].getString(temp2)) {
+            //if (objs[i] != nullptr && objs[i].getString(temp1) != params[i].getString(temp2)) {
             if (temp != temp1) {
                 errln("Parse failed on object " + objs[i].getString(temp1) + " at index : " + i);
             }
@@ -344,7 +345,7 @@ void MessageFormatRegressionTest::Test4104976()
     failure(status, "new ChoiceFormat");
     //try {
         log("Compares to null is always false, returned : ");
-        logln(cf == NULL ? "true" : "false");
+        logln(cf == nullptr ? "true" : "false");
     /*} catch (Exception foo) {
         errln("ChoiceFormat.equals(null) throws exception.");
     }*/
@@ -368,7 +369,7 @@ void MessageFormatRegressionTest::Test4106659()
     UnicodeString formats [] = {
         "one", "two"
     };
-    ChoiceFormat *cf = NULL;
+    ChoiceFormat *cf = nullptr;
     //try {
     //    cf = new ChoiceFormat(limits, formats, 3);
     //} catch (Exception foo) {
@@ -401,7 +402,7 @@ void MessageFormatRegressionTest::Test4106660()
     FieldPosition pos(FieldPosition::DONT_CARE);
     str = cf->format(d, str, pos);
     if (str != "Two")
-        errln( (UnicodeString) "format(" + d + ") = " + str);
+        errln(UnicodeString("format(") + d + ") = " + str);
 
     delete cf;
 }
@@ -476,9 +477,9 @@ void MessageFormatRegressionTest::Test4114743()
 void MessageFormatRegressionTest::Test4116444()
 {
     UnicodeString patterns [] = {
-        (UnicodeString)"",
-        (UnicodeString)"one",
-        (UnicodeString) "{0,date,short}"
+        UnicodeString(""),
+        UnicodeString("one"),
+        UnicodeString("{0,date,short}")
     };
 
     UErrorCode status = U_ZERO_ERROR;
@@ -496,7 +497,7 @@ void MessageFormatRegressionTest::Test4116444()
         Formattable *array = mf->parse(UnicodeString(""), pp, count);
             logln("pattern: \"" + pattern + "\"");
             log(" parsedObjects: ");
-            if (array != NULL) {
+            if (array != nullptr) {
                 log("{");
                 for (int j = 0; j < count; j++) {
                     //if (array[j] != null)
@@ -533,9 +534,9 @@ void MessageFormatRegressionTest::Test4114739()
     MessageFormat *mf = new MessageFormat("<{0}>", status);
     failure(status, "new MessageFormat");
 
-    Formattable *objs1 = NULL;
+    Formattable *objs1 = nullptr;
     //Formattable objs2 [] = {};
-    //Formattable *objs3 [] = {NULL};
+    //Formattable *objs3 [] = {nullptr};
     //try {
     UnicodeString pat;
     UnicodeString res;
@@ -606,7 +607,7 @@ void MessageFormatRegressionTest::Test4106661()
     logln("Format with -1.0 : " + fmt->format(Formattable(-1.0), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
-    logln("Format with 0 : " + fmt->format(Formattable((int32_t)0), str, bogus, status));
+    logln("Format with 0 : " + fmt->format(Formattable(static_cast<int32_t>(0)), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
     logln("Format with 0.9 : " + fmt->format(Formattable(0.9), str, bogus, status));
@@ -618,7 +619,7 @@ void MessageFormatRegressionTest::Test4106661()
     logln("Format with 1.5 : " + fmt->format(Formattable(1.5), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
-    logln("Format with 2 : " + fmt->format(Formattable((int32_t)2), str, bogus, status));
+    logln("Format with 2 : " + fmt->format(Formattable(static_cast<int32_t>(2)), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
     logln("Format with 2.1 : " + fmt->format(Formattable(2.1), str, bogus, status));
@@ -640,17 +641,17 @@ void MessageFormatRegressionTest::Test4094906()
 {
     UErrorCode status = U_ZERO_ERROR;
     UnicodeString pattern("-");
-    pattern += (UChar) 0x221E;
+    pattern += static_cast<char16_t>(0x221E);
     pattern += "<are negative|0<are no or fraction|1#is one|1<is 1+|";
-    pattern += (UChar) 0x221E;
+    pattern += static_cast<char16_t>(0x221E);
     pattern += "<are many.";
 
     ChoiceFormat *fmt = new ChoiceFormat(pattern, status);
     failure(status, "new ChoiceFormat");
     UnicodeString pat;
     if (fmt->toPattern(pat) != pattern) {
-        errln( (UnicodeString) "Formatter Pattern : " + pat);
-        errln( (UnicodeString) "Expected Pattern  : " + pattern);
+        errln(UnicodeString("Formatter Pattern : ") + pat);
+        errln(UnicodeString("Expected Pattern  : ") + pattern);
     }
     FieldPosition bogus(FieldPosition::DONT_CARE);
     UnicodeString str;
@@ -665,7 +666,7 @@ void MessageFormatRegressionTest::Test4094906()
     logln("Format with -1.0 : " + fmt->format(Formattable(-1.0), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
-    logln("Format with 0 : " + fmt->format(Formattable((int32_t)0), str, bogus, status));
+    logln("Format with 0 : " + fmt->format(Formattable(static_cast<int32_t>(0)), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
     logln("Format with 0.9 : " + fmt->format(Formattable(0.9), str, bogus, status));
@@ -677,7 +678,7 @@ void MessageFormatRegressionTest::Test4094906()
     logln("Format with 1.5 : " + fmt->format(Formattable(1.5), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
-    logln("Format with 2 : " + fmt->format(Formattable((int32_t)2), str, bogus, status));
+    logln("Format with 2 : " + fmt->format(Formattable(static_cast<int32_t>(2)), str, bogus, status));
     failure(status, "fmt->format");
     str.remove();
     logln("Format with 2.1 : " + fmt->format(Formattable(2.1), str, bogus, status));
@@ -702,7 +703,7 @@ void MessageFormatRegressionTest::Test4118592()
     failure(status, "new messageFormat");
     UnicodeString pattern("{0,choice,1#YES|2#NO}");
     UnicodeString prefix("");
-    Formattable *objs = 0;
+    Formattable* objs = nullptr;
 
     for (int i = 0; i < 5; i++) {
         UnicodeString formatted;
@@ -717,14 +718,14 @@ void MessageFormatRegressionTest::Test4118592()
         UnicodeString pat;
         logln(UnicodeString("") + i + ". pattern :\"" + mf->toPattern(pat) + "\"");
         log(" \"" + formatted + "\" parsed as ");
-        if (objs == NULL)
+        if (objs == nullptr)
             logln("  null");
         else {
             UnicodeString temp;
             if(objs[0].getType() == Formattable::kString)
-                logln((UnicodeString)"  " + objs[0].getString(temp));
+                logln(UnicodeString("  ") + objs[0].getString(temp));
             else
-                logln((UnicodeString)"  " + (objs[0].getType() == Formattable::kLong ? objs[0].getLong() : objs[0].getDouble()));
+                logln(UnicodeString("  ") + (objs[0].getType() == Formattable::kLong ? objs[0].getLong() : objs[0].getDouble()));
             delete[] objs;
 
         }
@@ -771,7 +772,7 @@ void MessageFormatRegressionTest::Test4118594()
     pp.setIndex(0);
     Formattable *newobjs = mf->parse(result, pp, count1);
     // newobjs now equals {new Double(3.1)}
-    if (newobjs == NULL) {
+    if (newobjs == nullptr) {
         dataerrln("Error calling MessageFormat::parse");
     } else {
         if (newobjs[0].getDouble() != 3.1)
@@ -797,17 +798,17 @@ void MessageFormatRegressionTest::Test4105380()
     failure(status, "new MessageFormat");
     double filelimits [] = {0,1,2};
     UnicodeString filepart [] = {
-        (UnicodeString)"no files",
-            (UnicodeString)"one file",
-            (UnicodeString)"{0,number} files"
+        UnicodeString("no files"),
+        UnicodeString("one file"),
+        UnicodeString("{0,number} files")
     };
     ChoiceFormat *fileform = new ChoiceFormat(filelimits, filepart, 3);
     form1->setFormat(1, *fileform);
     form2->setFormat(0, *fileform);
     //Object[] testArgs = {new Long(12373), "MyDisk"};
     Formattable testArgs [] = {
-        Formattable((int32_t)12373),
-            Formattable((UnicodeString)"MyDisk")
+        Formattable(static_cast<int32_t>(12373)),
+        Formattable(UnicodeString("MyDisk"))
     };
 
     FieldPosition bogus(FieldPosition::DONT_CARE);
@@ -832,9 +833,9 @@ void MessageFormatRegressionTest::Test4120552()
     MessageFormat *mf = new MessageFormat("pattern", status);
     failure(status, "new MessageFormat");
     UnicodeString texts[] = {
-        (UnicodeString)"pattern",
-            (UnicodeString)"pat",
-            (UnicodeString)"1234"
+        UnicodeString("pattern"),
+        UnicodeString("pat"),
+        UnicodeString("1234")
     };
     UnicodeString pat;
     logln("pattern: \"" + mf->toPattern(pat) + "\"");
@@ -844,7 +845,7 @@ void MessageFormatRegressionTest::Test4120552()
         int32_t count = 0;
         Formattable *objs = mf->parse(texts[i], pp, count);
         log("  text for parsing: \"" + texts[i] + "\"");
-        if (objs == NULL) {
+        if (objs == nullptr) {
             logln("  (incorrectly formatted string)");
             if (pp.getErrorIndex() == -1)
                 errln(UnicodeString("Incorrect error index: ") + pp.getErrorIndex());
@@ -886,19 +887,19 @@ void MessageFormatRegressionTest::Test4142938()
         UnicodeString out;
         //out = mf->format(new Object[]{new Integer(i)});
         Formattable objs [] = {
-            Formattable((int32_t)i)
+            Formattable(static_cast<int32_t>(i))
         };
         FieldPosition pos(FieldPosition::DONT_CARE);
         out = mf->format(objs, 1, out, pos, status);
         if (!failure(status, "mf->format", true)) {
             if (SUFFIX[i] == "") {
                 if (out != PREFIX[i])
-                    errln((UnicodeString)"" + i + ": Got \"" + out + "\"; Want \"" + PREFIX[i] + "\"");
+                    errln(UnicodeString("") + i + ": Got \"" + out + "\"; Want \"" + PREFIX[i] + "\"");
             }
             else {
                 if (!out.startsWith(PREFIX[i]) ||
                     !out.endsWith(SUFFIX[i]))
-                    errln((UnicodeString)"" + i + ": Got \"" + out + "\"; Want \"" + PREFIX[i] + "\"...\"" +
+                    errln(UnicodeString("") + i + ": Got \"" + out + "\"; Want \"" + PREFIX[i] + "\"...\"" +
                           SUFFIX[i] + "\"");
             }
         }
@@ -941,7 +942,7 @@ void MessageFormatRegressionTest::TestChoicePatternQuote()
             for (int j=0; j<=1; ++j) {
                 UnicodeString out;
                 FieldPosition pos(FieldPosition::DONT_CARE);
-                out = cf->format((double)j, out, pos);
+                out = cf->format(static_cast<double>(j), out, pos);
                 if (out != DATA[i+1+j])
                     errln("Fail: Pattern \"" + DATA[i] + "\" x "+j+" -> " +
                           out + "; want \"" + DATA[i+1+j] + "\"");
@@ -977,7 +978,7 @@ void MessageFormatRegressionTest::Test4112104()
     failure(status, "new MessageFormat");
     //try {
         // This should NOT throw an exception
-        if (format == NULL) {
+        if (format == nullptr) {
             // It also should return false
             errln("MessageFormat.equals(null) returns false");
         }
@@ -1013,6 +1014,23 @@ void MessageFormatRegressionTest::TestICU12584() {
     inner_msg.getFormats(count);
     assertEquals("Inner placeholder match", 3, count);
 }
+void MessageFormatRegressionTest::TestICU22798() {
+    // Test deep nested choice will not cause stack overflow but return error
+    // instead.
+    UErrorCode status = U_ZERO_ERROR;
+    UnicodeString pattern;
+    constexpr static int testNestedLevel = 30000;
+    for (int i = 0; i < testNestedLevel; i++) {
+      pattern += u"A{0,choice,0#";
+    }
+    pattern += u"text";
+    for (int i = 0; i < testNestedLevel; i++) {
+      pattern += u"}a";
+    }
+    MessageFormat msg(pattern, status);
+    assertEquals("Deep nested choice should cause error but not crash",
+                 U_INDEX_OUTOFBOUNDS_ERROR, status);
+}
 
 void MessageFormatRegressionTest::TestAPI() {
     UErrorCode status = U_ZERO_ERROR;
@@ -1025,7 +1043,7 @@ void MessageFormatRegressionTest::TestAPI() {
     failure(status, "adoptFormat");
 
     // Test getFormat
-    format->setFormat((int32_t)0,*fmt);
+    format->setFormat(static_cast<int32_t>(0), *fmt);
     format->getFormat("some_other_name",status);  // Must at least pass a valid identifier.
     failure(status, "getFormat");
     delete format;
