@@ -36,7 +36,7 @@
 #include "unicode/putil.h"
 #include "idnaconf.h"
 
-static const UChar unicodeIn[][41] ={
+static const char16_t unicodeIn[][41] ={
     { 
         0x0644, 0x064A, 0x0647, 0x0645, 0x0627, 0x0628, 0x062A, 0x0643, 0x0644,
         0x0645, 0x0648, 0x0634, 0x0639, 0x0631, 0x0628, 0x064A, 0x061F, 0x0000
@@ -240,7 +240,7 @@ typedef struct ErrorCases ErrorCases;
 
 static const struct ErrorCases{
 
-    UChar unicode[100];
+    char16_t unicode[100];
     const char *ascii;
     UErrorCode expected;
     UBool useSTD3ASCIIRules;
@@ -405,7 +405,7 @@ static const struct ErrorCases{
     },
     { 
       {0},
-      NULL,
+      nullptr,
       U_ILLEGAL_ARGUMENT_ERROR,
       true, true, false
     }
@@ -416,21 +416,21 @@ static const struct ErrorCases{
 
 #define MAX_DEST_SIZE 300
 
-void TestIDNA::debug(const UChar* src, int32_t srcLength, int32_t options){
+void TestIDNA::debug(const char16_t* src, int32_t srcLength, int32_t options){
     UParseError parseError;
     UErrorCode transStatus = U_ZERO_ERROR;
     UErrorCode prepStatus  = U_ZERO_ERROR;
     NamePrepTransform* trans = NamePrepTransform::createInstance(parseError,transStatus);
     int32_t prepOptions = (((options & UIDNA_ALLOW_UNASSIGNED) != 0) ? USPREP_ALLOW_UNASSIGNED: 0);
     LocalUStringPrepProfilePointer prep(usprep_openByType(USPREP_RFC3491_NAMEPREP,&prepStatus));
-    UChar *transOut=NULL, *prepOut=NULL;
+    char16_t *transOut=nullptr, *prepOut=nullptr;
     int32_t transOutLength=0, prepOutLength=0;
     
     
     transOutLength  = trans->process(src,srcLength,transOut, 0, prepOptions>0, &parseError, transStatus);
     if( transStatus == U_BUFFER_OVERFLOW_ERROR){
         transStatus = U_ZERO_ERROR;
-        transOut    = (UChar*) malloc(U_SIZEOF_UCHAR * transOutLength);
+        transOut = static_cast<char16_t*>(malloc(U_SIZEOF_UCHAR * transOutLength));
         transOutLength = trans->process(src,srcLength,transOut, transOutLength, prepOptions>0, &parseError, transStatus);
     }
 
@@ -438,7 +438,7 @@ void TestIDNA::debug(const UChar* src, int32_t srcLength, int32_t options){
 
     if( prepStatus == U_BUFFER_OVERFLOW_ERROR){
         prepStatus = U_ZERO_ERROR;
-        prepOut    = (UChar*) malloc(U_SIZEOF_UCHAR * prepOutLength);
+        prepOut = static_cast<char16_t*>(malloc(U_SIZEOF_UCHAR * prepOutLength));
         prepOutLength  = usprep_prepare(prep.getAlias(), src, srcLength, prepOut, prepOutLength, prepOptions, &parseError, &prepStatus);
     }
 
@@ -451,28 +451,28 @@ void TestIDNA::debug(const UChar* src, int32_t srcLength, int32_t options){
     delete trans;
 }
 
-void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* testName, 
+void TestIDNA::testAPI(const char16_t* src, const char16_t* expected, const char* testName,
             UBool useSTD3ASCIIRules,UErrorCode expectedStatus,
             UBool doCompare, UBool testUnassigned, TestFunc func, UBool testSTD3ASCIIRules){
 
     UErrorCode status = U_ZERO_ERROR;
-    UChar destStack[MAX_DEST_SIZE];
+    char16_t destStack[MAX_DEST_SIZE];
     int32_t destLen = 0;
-    UChar* dest = NULL;
-    int32_t expectedLen = (expected != NULL) ? u_strlen(expected) : 0;
+    char16_t* dest = nullptr;
+    int32_t expectedLen = (expected != nullptr) ? u_strlen(expected) : 0;
     int32_t options = (useSTD3ASCIIRules == true) ? UIDNA_USE_STD3_RULES : UIDNA_DEFAULT;
     UParseError parseError;
     int32_t tSrcLen = 0; 
-    UChar* tSrc = NULL; 
+    char16_t* tSrc = nullptr;
 
-    if(src != NULL){
+    if(src != nullptr){
         tSrcLen = u_strlen(src);
-        tSrc  =(UChar*) malloc( U_SIZEOF_UCHAR * tSrcLen );
+        tSrc = static_cast<char16_t*>(malloc(U_SIZEOF_UCHAR * tSrcLen));
         memcpy(tSrc,src,tSrcLen * U_SIZEOF_UCHAR);
     }
 
     // test null-terminated source and return value of number of UChars required
-    destLen = func(src,-1,NULL,0,options, &parseError , &status);
+    destLen = func(src,-1,nullptr,0,options, &parseError , &status);
     if (status == U_FILE_ACCESS_ERROR) {
         dataerrln("U_FILE_ACCESS_ERROR. Skipping the remainder of this test.");
         return;
@@ -506,7 +506,7 @@ void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* test
     } 
     if(testUnassigned ){
         status = U_ZERO_ERROR;
-        destLen = func(src,-1,NULL,0,options | UIDNA_ALLOW_UNASSIGNED, &parseError, &status);
+        destLen = func(src,-1,nullptr,0,options | UIDNA_ALLOW_UNASSIGNED, &parseError, &status);
         if(status == U_BUFFER_OVERFLOW_ERROR){
             status = U_ZERO_ERROR; // reset error code
             if(destLen+1 < MAX_DEST_SIZE){
@@ -541,7 +541,7 @@ void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* test
     status = U_ZERO_ERROR;
 
     // test source with lengthand return value of number of UChars required
-    destLen = func(tSrc, tSrcLen, NULL,0,options, &parseError, &status);
+    destLen = func(tSrc, tSrcLen, nullptr,0,options, &parseError, &status);
     if(status == U_BUFFER_OVERFLOW_ERROR){
         status = U_ZERO_ERROR; // reset error code
         if(destLen+1 < MAX_DEST_SIZE){
@@ -567,7 +567,7 @@ void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* test
     if(testUnassigned){
         status = U_ZERO_ERROR;
 
-        destLen = func(tSrc,tSrcLen,NULL,0,options | UIDNA_ALLOW_UNASSIGNED, &parseError, &status);
+        destLen = func(tSrc,tSrcLen,nullptr,0,options | UIDNA_ALLOW_UNASSIGNED, &parseError, &status);
 
         if(status == U_BUFFER_OVERFLOW_ERROR){
             status = U_ZERO_ERROR; // reset error code
@@ -595,7 +595,7 @@ void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* test
 
     status = U_ZERO_ERROR;
     if(testSTD3ASCIIRules==true){
-        destLen = func(src,-1,NULL,0,options | UIDNA_USE_STD3_RULES, &parseError, &status);
+        destLen = func(src,-1,nullptr,0,options | UIDNA_USE_STD3_RULES, &parseError, &status);
         if(status == U_BUFFER_OVERFLOW_ERROR){
             status = U_ZERO_ERROR; // reset error code
             if(destLen+1 < MAX_DEST_SIZE){
@@ -623,7 +623,7 @@ void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* test
 
         status = U_ZERO_ERROR;
 
-        destLen = func(tSrc,tSrcLen,NULL,0,options | UIDNA_USE_STD3_RULES, &parseError, &status);
+        destLen = func(tSrc,tSrcLen,nullptr,0,options | UIDNA_USE_STD3_RULES, &parseError, &status);
 
         if(status == U_BUFFER_OVERFLOW_ERROR){
             status = U_ZERO_ERROR; // reset error code
@@ -651,8 +651,8 @@ void TestIDNA::testAPI(const UChar* src, const UChar* expected, const char* test
     free(tSrc);
 }
 
-void TestIDNA::testCompare(const UChar* s1, int32_t s1Len,
-                        const UChar* s2, int32_t s2Len,
+void TestIDNA::testCompare(const char16_t* s1, int32_t s1Len,
+                        const char16_t* s2, int32_t s2Len,
                         const char* testName, CompareFunc func,
                         UBool isEqual){
 
@@ -700,10 +700,10 @@ void TestIDNA::testCompare(const UChar* s1, int32_t s1Len,
 void TestIDNA::testToASCII(const char* testName, TestFunc func){
 
     int32_t i;
-    UChar buf[MAX_DEST_SIZE];
+    char16_t buf[MAX_DEST_SIZE];
 
     for(i=0;i< UPRV_LENGTHOF(unicodeIn); i++){
-        u_charsToUChars(asciiIn[i],buf, (int32_t)(strlen(asciiIn[i])+1));
+        u_charsToUChars(asciiIn[i], buf, static_cast<int32_t>(strlen(asciiIn[i]) + 1));
         testAPI(unicodeIn[i], buf,testName, false,U_ZERO_ERROR, true, true, func);
         
     }
@@ -712,10 +712,10 @@ void TestIDNA::testToASCII(const char* testName, TestFunc func){
 void TestIDNA::testToUnicode(const char* testName, TestFunc func){
 
     int32_t i;
-    UChar buf[MAX_DEST_SIZE];
+    char16_t buf[MAX_DEST_SIZE];
     
     for(i=0;i< UPRV_LENGTHOF(asciiIn); i++){
-        u_charsToUChars(asciiIn[i],buf, (int32_t)(strlen(asciiIn[i])+1));
+        u_charsToUChars(asciiIn[i], buf, static_cast<int32_t>(strlen(asciiIn[i]) + 1));
         testAPI(buf,unicodeIn[i],testName,false,U_ZERO_ERROR, true, true, func);
     }
 }
@@ -723,13 +723,13 @@ void TestIDNA::testToUnicode(const char* testName, TestFunc func){
 
 void TestIDNA::testIDNToUnicode(const char* testName, TestFunc func){
     int32_t i;
-    UChar buf[MAX_DEST_SIZE];
-    UChar expected[MAX_DEST_SIZE];
+    char16_t buf[MAX_DEST_SIZE];
+    char16_t expected[MAX_DEST_SIZE];
     UErrorCode status = U_ZERO_ERROR;
     int32_t bufLen = 0;
     UParseError parseError;
     for(i=0;i< UPRV_LENGTHOF(domainNames); i++){
-        bufLen = (int32_t)strlen(domainNames[i]);
+        bufLen = static_cast<int32_t>(strlen(domainNames[i]));
         bufLen = u_unescape(domainNames[i],buf, bufLen+1);
         func(buf,bufLen,expected,MAX_DEST_SIZE, UIDNA_ALLOW_UNASSIGNED, &parseError,&status);
         if(U_FAILURE(status)){
@@ -749,13 +749,13 @@ void TestIDNA::testIDNToUnicode(const char* testName, TestFunc func){
 
 void TestIDNA::testIDNToASCII(const char* testName, TestFunc func){
     int32_t i;
-    UChar buf[MAX_DEST_SIZE];
-    UChar expected[MAX_DEST_SIZE];
+    char16_t buf[MAX_DEST_SIZE];
+    char16_t expected[MAX_DEST_SIZE];
     UErrorCode status = U_ZERO_ERROR;
     int32_t bufLen = 0;
     UParseError parseError; 
     for(i=0;i< UPRV_LENGTHOF(domainNames); i++){
-        bufLen = (int32_t)strlen(domainNames[i]);
+        bufLen = static_cast<int32_t>(strlen(domainNames[i]));
         bufLen = u_unescape(domainNames[i],buf, bufLen+1);
         func(buf,bufLen,expected,MAX_DEST_SIZE, UIDNA_ALLOW_UNASSIGNED, &parseError,&status);
         if(U_FAILURE(status)){
@@ -777,31 +777,31 @@ void TestIDNA::testCompare(const char* testName, CompareFunc func){
     int32_t i;
 
 
-    UChar www[] = {0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
-    UChar com[] = {0x002E, 0x0043, 0x004F, 0x004D, 0x0000};
-    UChar buf[MAX_DEST_SIZE]={0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
+    char16_t www[] = {0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
+    char16_t com[] = {0x002E, 0x0043, 0x004F, 0x004D, 0x0000};
+    char16_t buf[MAX_DEST_SIZE]={0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
 
     UnicodeString source(www), uni0(www),uni1(www), ascii0(www), ascii1(www);
 
     uni0.append(unicodeIn[0]);
     uni0.append(com);
-    uni0.append((UChar)0x0000);
+    uni0.append(static_cast<char16_t>(0x0000));
 
     uni1.append(unicodeIn[1]);
     uni1.append(com);
-    uni1.append((UChar)0x0000);
+    uni1.append(static_cast<char16_t>(0x0000));
 
     ascii0.append(asciiIn[0]);
     ascii0.append(com);
-    ascii0.append((UChar)0x0000);
+    ascii0.append(static_cast<char16_t>(0x0000));
 
     ascii1.append(asciiIn[1]);
     ascii1.append(com);
-    ascii1.append((UChar)0x0000);
+    ascii1.append(static_cast<char16_t>(0x0000));
 
     for(i=0;i< UPRV_LENGTHOF(unicodeIn); i++){
 
-        u_charsToUChars(asciiIn[i],buf+4, (int32_t)(strlen(asciiIn[i])+1));
+        u_charsToUChars(asciiIn[i], buf + 4, static_cast<int32_t>(strlen(asciiIn[i]) + 1));
         u_strcat(buf,com);
 
         // for every entry in unicodeIn array
@@ -809,9 +809,9 @@ void TestIDNA::testCompare(const char* testName, CompareFunc func){
         source.truncate(4);
         source.append(unicodeIn[i]);
         source.append(com);
-        source.append((UChar)0x0000);
+        source.append(static_cast<char16_t>(0x0000));
         // a) compare it with itself
-        const UChar* src = source.getBuffer();
+        const char16_t* src = source.getBuffer();
         int32_t srcLen = u_strlen(src); //subtract null
 
         testCompare(src,srcLen,src,srcLen,testName, func, true);
@@ -838,8 +838,8 @@ void TestIDNA::testCompare(const char* testName, CompareFunc func){
 #if 0
 
 static int32_t
-getNextSeperator(UChar *src,int32_t srcLength,
-                 UChar **limit){
+getNextSeperator(char16_t *src,int32_t srcLength,
+                 char16_t **limit){
     if(srcLength == -1){
         int32_t i;
         for(i=0 ; ;i++){
@@ -875,10 +875,10 @@ getNextSeperator(UChar *src,int32_t srcLength,
 
 void printPunycodeOutput(){
 
-    UChar dest[MAX_DEST_SIZE];
+    char16_t dest[MAX_DEST_SIZE];
     int32_t destCapacity=MAX_DEST_SIZE;
-    UChar* start;
-    UChar* limit;
+    char16_t* start;
+    char16_t* limit;
     int32_t labelLen=0;
     UBool caseFlags[MAX_DEST_SIZE];
     
@@ -905,14 +905,14 @@ void printPunycodeOutput(){
 
 void TestIDNA::testErrorCases(const char* IDNToASCIIName, TestFunc IDNToASCII,
                               const char* IDNToUnicodeName, TestFunc IDNToUnicode){
-    UChar buf[MAX_DEST_SIZE];
+    char16_t buf[MAX_DEST_SIZE];
     int32_t bufLen=0;
 
     for(int32_t i=0;i< UPRV_LENGTHOF(errorCases);i++){
         ErrorCases errorCase = errorCases[i];
-        UChar* src =NULL;
-        if(errorCase.ascii != NULL){
-            bufLen =  (int32_t)strlen(errorCase.ascii);
+        char16_t* src =nullptr;
+        if(errorCase.ascii != nullptr){
+            bufLen = static_cast<int32_t>(strlen(errorCase.ascii));
             u_charsToUChars(errorCase.ascii,buf, bufLen+1);
         }else{
             bufLen = 1 ;
@@ -932,7 +932,7 @@ void TestIDNA::testErrorCases(const char* IDNToASCIIName, TestFunc IDNToASCII,
                 errorCase.expected, false,true, IDNToASCII);
         }
         if(errorCase.testToUnicode ==true){
-            testAPI((src==NULL)? NULL : buf,src,
+            testAPI((src==nullptr)? nullptr : buf,src,
                     IDNToUnicodeName, errorCase.useSTD3ASCIIRules,
                     errorCase.expected, true, true, IDNToUnicode);   
         }
@@ -945,15 +945,15 @@ void TestIDNA::testConformance(const char* toASCIIName, TestFunc toASCII,
                                const char* IDNToASCIIName, TestFunc IDNToASCII,
                                const char* IDNToUnicodeName, TestFunc IDNToUnicode,
                                const char* toUnicodeName, TestFunc toUnicode){
-    UChar src[MAX_DEST_SIZE];
+    char16_t src[MAX_DEST_SIZE];
     int32_t srcLen=0;
-    UChar expected[MAX_DEST_SIZE];
+    char16_t expected[MAX_DEST_SIZE];
     int32_t expectedLen = 0;
     for(int32_t i=0;i< UPRV_LENGTHOF(conformanceTestCases);i++){
         const char* utf8Chars1 = conformanceTestCases[i].in;
         int32_t utf8Chars1Len = (int32_t)strlen(utf8Chars1);
         const char* utf8Chars2 = conformanceTestCases[i].out;
-        int32_t utf8Chars2Len = (utf8Chars2 == NULL) ? 0 : (int32_t)strlen(utf8Chars2);
+        int32_t utf8Chars2Len = (utf8Chars2 == nullptr) ? 0 : (int32_t)strlen(utf8Chars2);
 
         UErrorCode status = U_ZERO_ERROR;
         u_strFromUTF8(src,MAX_DEST_SIZE,&srcLen,utf8Chars1,utf8Chars1Len,&status);
@@ -961,7 +961,7 @@ void TestIDNA::testConformance(const char* toASCIIName, TestFunc toASCII,
             errln(UnicodeString("Conversion of UTF8 source in conformanceTestCases[") + i +UnicodeString( "].in ( ")+prettify(utf8Chars1) +UnicodeString(" ) failed. Error: ")+ UnicodeString(u_errorName(status)));
             continue;
         }
-        if(utf8Chars2 != NULL){
+        if(utf8Chars2 != nullptr){
             u_strFromUTF8(expected,MAX_DEST_SIZE,&expectedLen,utf8Chars2,utf8Chars2Len, &status);
             if(U_FAILURE(status)){
                 errln(UnicodeString("Conversion of UTF8 source in conformanceTestCases[") + i +UnicodeString( "].in ( ")+prettify(utf8Chars1) +UnicodeString(" ) failed. Error: ")+ UnicodeString(u_errorName(status)));
@@ -998,11 +998,11 @@ void TestIDNA::testConformance(const char* toASCIIName, TestFunc toASCII,
 */
 // test and ascertain
 // func(func(func(src))) == func(src)
-void TestIDNA::testChaining(const UChar* src,int32_t numIterations,const char* testName,
+void TestIDNA::testChaining(const char16_t* src,int32_t numIterations,const char* testName,
                   UBool useSTD3ASCIIRules, UBool caseInsensitive, TestFunc func){
-    UChar even[MAX_DEST_SIZE];
-    UChar odd[MAX_DEST_SIZE];
-    UChar expected[MAX_DEST_SIZE];
+    char16_t even[MAX_DEST_SIZE];
+    char16_t odd[MAX_DEST_SIZE];
+    char16_t expected[MAX_DEST_SIZE];
     int32_t i=0,evenLen=0,oddLen=0,expectedLen=0;
     UErrorCode status = U_ZERO_ERROR;
     int32_t srcLen = u_strlen(src);
@@ -1158,10 +1158,10 @@ void TestIDNA::testChaining(const UChar* src,int32_t numIterations,const char* t
 void TestIDNA::testChaining(const char* toASCIIName, TestFunc toASCII,
                   const char* toUnicodeName, TestFunc toUnicode){
     int32_t i;
-    UChar buf[MAX_DEST_SIZE];
+    char16_t buf[MAX_DEST_SIZE];
     
     for(i=0;i< UPRV_LENGTHOF(asciiIn); i++){
-        u_charsToUChars(asciiIn[i],buf, (int32_t)(strlen(asciiIn[i])+1));
+        u_charsToUChars(asciiIn[i], buf, static_cast<int32_t>(strlen(asciiIn[i]) + 1));
         testChaining(buf,5,toUnicodeName, false, false, toUnicode);
     }
     for(i=0;i< UPRV_LENGTHOF(unicodeIn); i++){
@@ -1176,31 +1176,31 @@ void TestIDNA::testRootLabelSeparator(const char* testName, CompareFunc func,
     int32_t i;
 
 
-    UChar www[] = {0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
-    UChar com[] = {0x002E, 0x0043, 0x004F, 0x004D, 0x002E, /* root label separator */0x0000};
-    UChar buf[MAX_DEST_SIZE]={0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
+    char16_t www[] = {0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
+    char16_t com[] = {0x002E, 0x0043, 0x004F, 0x004D, 0x002E, /* root label separator */0x0000};
+    char16_t buf[MAX_DEST_SIZE]={0x0057, 0x0057, 0x0057, 0x002E, 0x0000};
 
     UnicodeString source(www), uni0(www),uni1(www), ascii0(www), ascii1(www);
 
     uni0.append(unicodeIn[0]);
     uni0.append(com);
-    uni0.append((UChar)0x0000);
+    uni0.append(static_cast<char16_t>(0x0000));
 
     uni1.append(unicodeIn[1]);
     uni1.append(com);
-    uni1.append((UChar)0x0000);
+    uni1.append(static_cast<char16_t>(0x0000));
 
     ascii0.append(asciiIn[0]);
     ascii0.append(com);
-    ascii0.append((UChar)0x0000);
+    ascii0.append(static_cast<char16_t>(0x0000));
 
     ascii1.append(asciiIn[1]);
     ascii1.append(com);
-    ascii1.append((UChar)0x0000);
+    ascii1.append(static_cast<char16_t>(0x0000));
 
     for(i=0;i< UPRV_LENGTHOF(unicodeIn); i++){
 
-        u_charsToUChars(asciiIn[i],buf+4, (int32_t)(strlen(asciiIn[i])+1));
+        u_charsToUChars(asciiIn[i], buf + 4, static_cast<int32_t>(strlen(asciiIn[i]) + 1));
         u_strcat(buf,com);
 
         // for every entry in unicodeIn array
@@ -1208,9 +1208,9 @@ void TestIDNA::testRootLabelSeparator(const char* testName, CompareFunc func,
         source.truncate(4);
         source.append(unicodeIn[i]);
         source.append(com);
-        source.append((UChar)0x0000);
+        source.append(static_cast<char16_t>(0x0000));
         
-        const UChar* src = source.getBuffer();
+        const char16_t* src = source.getBuffer();
         int32_t srcLen = u_strlen(src); //subtract null
         
         // b) compare it with asciiIn equivalent
@@ -1248,7 +1248,7 @@ extern IntlTest *createUTS46Test();
 
 void TestIDNA::runIndexedTest( int32_t index, UBool exec, const char* &name, char* par)
 {
-    if (exec) logln((UnicodeString)"TestSuite IDNA API ");
+    if (exec) logln(UnicodeString("TestSuite IDNA API "));
     switch (index) {
 
         case 0: name = "TestToASCII"; if (exec) TestToASCII(); break;
@@ -1335,13 +1335,13 @@ randul()
     static UBool initialized = false;
     if (!initialized)
     {
-        srand((unsigned)time(NULL));
+        srand(static_cast<unsigned>(time(nullptr)));
         initialized = true;
     }
     // Assume rand has at least 12 bits of precision
     uint32_t l = 0;
     for (uint32_t i=0; i<sizeof(l); ++i)
-        ((char*)&l)[i] = (char)((rand() & 0x0FF0) >> 4);
+        reinterpret_cast<char*>(&l)[i] = static_cast<char>((rand() & 0x0FF0) >> 4);
     return l;
 }
 
@@ -1351,7 +1351,7 @@ randul()
  */
 static int32_t rand_uni()
 {
-   int32_t retVal = (int32_t)(randul()& 0x3FFFF);
+   int32_t retVal = static_cast<int32_t>(randul() & 0x3FFFF);
    if(retVal >= 0x30000){
        retVal+=0xB0000;
    }
@@ -1359,7 +1359,7 @@ static int32_t rand_uni()
 }
 
 static int32_t randi(int32_t n){
-    return (int32_t) (randul() % (n+1));
+    return static_cast<int32_t>(randul() % (n + 1));
 }
 
 void getTestSource(UnicodeString& fillIn) {
@@ -1370,7 +1370,7 @@ void getTestSource(UnicodeString& fillIn) {
         if(codepoint == 0x0000){
             continue;
         }
-        fillIn.append((UChar32)codepoint);
+        fillIn.append(static_cast<UChar32>(codepoint));
         i++;
     }
        
@@ -1381,8 +1381,8 @@ UnicodeString TestIDNA::testCompareReferenceImpl(UnicodeString& src,
                                         TestFunc uIDNA, const char* uIDNAName,
                                         int32_t options){
   
-    const UChar* srcUChars = src.getBuffer();
-    UChar exp[MAX_DEST_SIZE]={0};
+    const char16_t* srcUChars = src.getBuffer();
+    char16_t exp[MAX_DEST_SIZE]={0};
     int32_t expCap = MAX_DEST_SIZE, expLen=0;
     UErrorCode expStatus = U_ZERO_ERROR;
     UParseError parseError;
@@ -1394,7 +1394,7 @@ UnicodeString TestIDNA::testCompareReferenceImpl(UnicodeString& src,
     expLen = refIDNA(srcUChars, src.length()-1, exp, expCap,
                       options, &parseError, &expStatus);
 
-    UChar got[MAX_DEST_SIZE]={0};
+    char16_t got[MAX_DEST_SIZE]={0};
     int32_t gotCap = MAX_DEST_SIZE, gotLen=0;
     UErrorCode gotStatus = U_ZERO_ERROR;
 
@@ -1408,7 +1408,7 @@ UnicodeString TestIDNA::testCompareReferenceImpl(UnicodeString& src,
                + " Got: " + UnicodeString(u_errorName(gotStatus))
                + " for Source: "+ prettify(srcUChars) 
                + " Options: " + options);
-        return UnicodeString("");
+        return {""};
     }
     
     // now we know that both implementations yielded same error
@@ -1430,12 +1430,12 @@ UnicodeString TestIDNA::testCompareReferenceImpl(UnicodeString& src,
             + " with "+ UnicodeString(uIDNAName) 
             +" for input: " + prettify(srcUChars));
     }
-    return UnicodeString("");
+    return {""};
 }
 
-void TestIDNA::testCompareReferenceImpl(const UChar* src, int32_t srcLen){
+void TestIDNA::testCompareReferenceImpl(const char16_t* src, int32_t srcLen){
     UnicodeString label(src,srcLen);
-    label.append((UChar)0x0000);
+    label.append(static_cast<char16_t>(0x0000));
 
     //test idnaref_toASCII and idnare
     UnicodeString asciiLabel = testCompareReferenceImpl(label, 
@@ -1456,7 +1456,7 @@ void TestIDNA::testCompareReferenceImpl(const UChar* src, int32_t srcLen){
                               UIDNA_USE_STD3_RULES | UIDNA_ALLOW_UNASSIGNED);
 
     if(asciiLabel.length()!=0){
-        asciiLabel.append((UChar)0x0000);
+        asciiLabel.append(static_cast<char16_t>(0x0000));
 
         // test toUnciode
         testCompareReferenceImpl(asciiLabel, 
@@ -1497,8 +1497,8 @@ void TestIDNA::TestIDNAMonkeyTest(){
     for(i=0; i<loopCount; i++){
         source.truncate(0);
         getTestSource(source);
-        source.append((UChar)0x0000);
-        const UChar* src = source.getBuffer();
+        source.append(static_cast<char16_t>(0x0000));
+        const char16_t* src = source.getBuffer();
         testCompareReferenceImpl(src,source.length()-1);
         testCompareReferenceImpl(src,source.length()-1);
     }
@@ -1508,8 +1508,8 @@ void TestIDNA::TestIDNAMonkeyTest(){
         source.truncate(0);
         source.append( UnicodeString(failures[i], -1, US_INV) );
         source = source.unescape();
-        source.append((UChar)0x0000);
-        const UChar *src = source.getBuffer();
+        source.append(static_cast<char16_t>(0x0000));
+        const char16_t *src = source.getBuffer();
         testCompareReferenceImpl(src,source.length()-1);
         //debug(source.getBuffer(),source.length(),UIDNA_ALLOW_UNASSIGNED);
     }
@@ -1525,7 +1525,7 @@ void TestIDNA::TestIDNAMonkeyTest(){
         UnicodeString expected("\\u043f\\u043e\\u0447\\u0435\\u043c\\u0443\\u0436\\u0435\\u043e\\u043d\\u0438\\u043d\\u0435\\u0433\\u043e\\u0432\\u043e\\u0440\\u044f\\u0442\\u043f\\u043e\\u0440\\u0443\\u0441\\u0441\\u043a\\u0438\\u0000", -1, US_INV);
         expected = expected.unescape();
         UnicodeString ascii("xn--b1abfaaepdrnnbgefbadotcwatmq2g4l");
-        ascii.append((UChar)0x0000);
+        ascii.append(static_cast<char16_t>(0x0000));
         testAPI(source.getBuffer(),ascii.getBuffer(), "uidna_toASCII", false, U_ZERO_ERROR, true, true, uidna_toASCII);
         
         testAPI(source.getBuffer(),ascii.getBuffer(), "idnaref_toASCII", false, U_ZERO_ERROR, true, true, idnaref_toASCII);
@@ -1537,7 +1537,7 @@ void TestIDNA::TestIDNAMonkeyTest(){
 
 void TestIDNA::TestCompareReferenceImpl(){
 
-    UChar src [2] = {0,0};
+    char16_t src [2] = {0,0};
     int32_t srcLen = 0;
 
     // data even OK?
@@ -1563,7 +1563,7 @@ void TestIDNA::TestCompareReferenceImpl(){
             src[1] = U16_TRAIL(i);
             srcLen =2;
         } else {
-            src[0] = (UChar)i;
+            src[0] = static_cast<char16_t>(i);
             src[1] = 0;
             srcLen = 1;
         }
@@ -1600,16 +1600,16 @@ void TestIDNA::TestDataFile(){
 
 TestIDNA::~TestIDNA(){
     delete gPrep;
-    gPrep = NULL;
+    gPrep = nullptr;
 }
 
 NamePrepTransform* TestIDNA::getInstance(UErrorCode& status){
-    if(gPrep == NULL){
+    if(gPrep == nullptr){
         UParseError parseError;
         gPrep = NamePrepTransform::createInstance(parseError, status);
-        if(gPrep == NULL){
+        if(gPrep == nullptr){
            //status = U_MEMORY_ALLOCATION_ERROR;
-           return NULL;
+           return nullptr;
         }
     }
     return gPrep;
