@@ -81,7 +81,7 @@ TestMessageFormat::runIndexedTest(int32_t index, UBool exec,
 void TestMessageFormat::testBug3()
 {
     double myNumber = -123456;
-    DecimalFormat *form = 0;
+    DecimalFormat* form = nullptr;
     Locale locale[] = {
         Locale("ar", "", ""),
         Locale("be", "", ""),
@@ -170,7 +170,7 @@ void TestMessageFormat::testBug1()
     ChoiceFormat *cf = new ChoiceFormat(limit, formats, 3);
     FieldPosition status(FieldPosition::DONT_CARE);
     UnicodeString toAppendTo;
-    cf->format((int32_t)1, toAppendTo, status);
+    cf->format(static_cast<int32_t>(1), toAppendTo, status);
     if (toAppendTo != "1.0<=Arg<2.0") {
         errln("ChoiceFormat cmp in testBug1");
     }
@@ -254,8 +254,8 @@ operator<<( IntlTest&           stream,
 void TestMessageFormat::PatternTest() 
 {
     Formattable testArgs[] = {
-        Formattable(double(1)), Formattable(double(3456)),
-            Formattable("Disk"), Formattable(UDate((int32_t)1000000000L), Formattable::kIsDate)
+        Formattable(static_cast<double>(1)), Formattable(static_cast<double>(3456)),
+            Formattable("Disk"), Formattable(static_cast<UDate>(static_cast<int32_t>(1000000000L)), Formattable::kIsDate)
     };
     UnicodeString testCases[] = {
        "Quotes '', '{', 'a' {0} '{0}'",
@@ -305,7 +305,7 @@ void TestMessageFormat::PatternTest()
         form.adoptInstead(new MessageFormat(testCases[i], Locale::getUS(), success));
         if (U_FAILURE(success)) {
             dataerrln("MessageFormat creation failed.#1 - %s", u_errorName(success));
-            logln(((UnicodeString)"MessageFormat for ") + testCases[i] + " creation failed.\n");
+            logln(UnicodeString("MessageFormat for ") + testCases[i] + " creation failed.\n");
             continue;
         }
         // ICU 4.8 returns the original pattern (testCases),
@@ -317,9 +317,9 @@ void TestMessageFormat::PatternTest()
             // (Too much trouble...)
             errln(UnicodeString("TestMessageFormat::PatternTest failed test #2, i = ") + i);
             //form->toPattern(buffer);
-            errln(((UnicodeString)" Orig: ") + testCases[i]);
-            errln(((UnicodeString)" Exp:  ") + testCases[i]);
-            errln(((UnicodeString)" Got:  ") + buffer);
+            errln(UnicodeString(" Orig: ") + testCases[i]);
+            errln(UnicodeString(" Exp:  ") + testCases[i]);
+            errln(UnicodeString(" Got:  ") + buffer);
         }
 
         //it_out << "Pat out: " << form->toPattern(buffer));
@@ -374,7 +374,7 @@ void TestMessageFormat::PatternTest()
 
 void TestMessageFormat::sample() 
 {
-    MessageFormat *form = 0;
+    MessageFormat* form = nullptr;
     UnicodeString buffer1, buffer2;
     UErrorCode success = U_ZERO_ERROR;
     form = new MessageFormat("There are {0} files on {1}", success);
@@ -398,8 +398,8 @@ void TestMessageFormat::testStaticFormat()
 {
     UErrorCode err = U_ZERO_ERROR;
     Formattable arguments[] = {
-        (int32_t)7,
-        Formattable(UDate(8.71068e+011), Formattable::kIsDate),
+        static_cast<int32_t>(7),
+        Formattable(static_cast<UDate>(8.71068e+011), Formattable::kIsDate),
         "a disturbance in the Force"
         };
 
@@ -413,7 +413,7 @@ void TestMessageFormat::testStaticFormat()
 
     if (U_FAILURE(err)) {
         dataerrln("TestMessageFormat::testStaticFormat #1 - %s", u_errorName(err));
-        logln(UnicodeString("TestMessageFormat::testStaticFormat failed test #1 with error code ")+(int32_t)err);
+        logln(UnicodeString("TestMessageFormat::testStaticFormat failed test #1 with error code ") + static_cast<int32_t>(err));
         return;
     }
 
@@ -434,8 +434,8 @@ void TestMessageFormat::TestTurkishCasing()
     Locale::setDefault( Locale("tr"), err );
 
     Formattable arguments[] = {
-        (int32_t)7,
-        Formattable(UDate(8.71068e+011), Formattable::kIsDate),
+        static_cast<int32_t>(7),
+        Formattable(static_cast<UDate>(8.71068e+011), Formattable::kIsDate),
         "a disturbance in the Force"
         };
 
@@ -468,9 +468,9 @@ void TestMessageFormat::testSimpleFormat(/* char* par */)
 
     UErrorCode err = U_ZERO_ERROR;
 
-    Formattable testArgs1[] = {(int32_t)0, "MyDisk"};
-    Formattable testArgs2[] = {(int32_t)1, "MyDisk"};
-    Formattable testArgs3[] = {(int32_t)12, "MyDisk"};
+    Formattable testArgs1[] = {static_cast<int32_t>(0), "MyDisk"};
+    Formattable testArgs2[] = {static_cast<int32_t>(1), "MyDisk"};
+    Formattable testArgs3[] = {static_cast<int32_t>(12), "MyDisk"};
    
     MessageFormat* form = new MessageFormat(
         "The disk \"{1}\" contains {0} file(s).", err);
@@ -515,7 +515,7 @@ void TestMessageFormat::testMsgFormatChoice(/* char* par */)
 
     FieldPosition ignore(FieldPosition::DONT_CARE);
     UnicodeString string;
-    Formattable testArgs1[] = {(int32_t)0, "MyDisk"};    
+    Formattable testArgs1[] = {static_cast<int32_t>(0), "MyDisk"};
     form->format(testArgs1, 2, string, ignore, err);
     if (string != "The disk \"MyDisk\" contains no files.") {
         errln("TestMessageFormat::testMsgFormatChoice failed on test #1");
@@ -523,7 +523,7 @@ void TestMessageFormat::testMsgFormatChoice(/* char* par */)
  
     ignore.setField(FieldPosition::DONT_CARE);
     string.remove();
-    Formattable testArgs2[] = {(int32_t)1, "MyDisk"};    
+    Formattable testArgs2[] = {static_cast<int32_t>(1), "MyDisk"};
     form->format(testArgs2, 2, string, ignore, err);
     if (string != "The disk \"MyDisk\" contains one file.") {
         errln("TestMessageFormat::testMsgFormatChoice failed on test #2");
@@ -531,7 +531,7 @@ void TestMessageFormat::testMsgFormatChoice(/* char* par */)
 
     ignore.setField(FieldPosition::DONT_CARE);
     string.remove();
-    Formattable testArgs3[] = {(int32_t)1273, "MyDisk"};    
+    Formattable testArgs3[] = {static_cast<int32_t>(1273), "MyDisk"};
     form->format(testArgs3, 2, string, ignore, err);
     if (string != "The disk \"MyDisk\" contains 1,273 files.") {
         dataerrln("TestMessageFormat::testMsgFormatChoice failed on test #3 - %s", u_errorName(err));
@@ -555,10 +555,10 @@ void TestMessageFormat::testMsgFormatPlural(/* char* par */)
     MessageFormat* mfNum = new MessageFormat(t1, Locale("fr"), err);
     if (U_FAILURE(err)) {
         dataerrln("TestMessageFormat::testMsgFormatPlural #1 - argumentIndex - %s", u_errorName(err));
-        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #1 with error code ")+(int32_t)err);
+        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #1 with error code ") + static_cast<int32_t>(err));
         return;
     }
-    Formattable testArgs1((int32_t)0);
+    Formattable testArgs1(static_cast<int32_t>(0));
     FieldPosition ignore(FieldPosition::DONT_CARE);
     UnicodeString numResult1;
     mfNum->format(&testArgs1, 1, numResult1, ignore, err);
@@ -569,7 +569,7 @@ void TestMessageFormat::testMsgFormatPlural(/* char* par */)
     mfAlpha->format(argName, &testArgs1, 1, argNameResult, err);
     if (U_FAILURE(err)) {
         dataerrln("TestMessageFormat::testMsgFormatPlural #1 - argumentName - %s", u_errorName(err));
-        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #1 with error code ")+(int32_t)err);
+        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #1 with error code ") + static_cast<int32_t>(err));
         delete mfNum;
         return;
     }
@@ -588,7 +588,7 @@ void TestMessageFormat::testMsgFormatPlural(/* char* par */)
 
     MessageFormat* mfNum2 = new MessageFormat(t3, Locale("uk"), err);
     numResult1.remove();
-    Formattable testArgs2((int32_t)4);
+    Formattable testArgs2(static_cast<int32_t>(4));
     mfNum2->format(&testArgs2, 1, numResult1, ignore, err);
     MessageFormat* mfAlpha2 = new MessageFormat(t4, Locale("uk"), err);
     argNameResult.remove();
@@ -596,7 +596,7 @@ void TestMessageFormat::testMsgFormatPlural(/* char* par */)
 
     if (U_FAILURE(err)) {
         errln("TestMessageFormat::testMsgFormatPlural #2 - argumentName");
-        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #2 with error code ")+(int32_t)err);
+        logln(UnicodeString("TestMessageFormat::testMsgFormatPlural #2 with error code ") + static_cast<int32_t>(err));
         delete mfNum2;
         return;
     }
@@ -617,11 +617,11 @@ void TestMessageFormat::testMsgFormatPlural(/* char* par */)
     MessageFormat* msgFmt = new MessageFormat(t5, Locale("fr"), err);
     if (U_FAILURE(err)) {
         errln("TestMessageFormat::test nested PluralFormat with argumentName");
-        logln(UnicodeString("TestMessageFormat::test nested PluralFormat with error code ")+(int32_t)err);
+        logln(UnicodeString("TestMessageFormat::test nested PluralFormat with error code ") + static_cast<int32_t>(err));
         delete msgFmt;
         return;
     }
-    Formattable testArgs3((int32_t)0);
+    Formattable testArgs3(static_cast<int32_t>(0));
     argNameResult.remove();
     msgFmt->format(&testArgs3, 1, argNameResult, ignore, err);
     if (U_FAILURE(err)) {
@@ -645,7 +645,7 @@ void TestMessageFormat::testApostropheInPluralAndSelect() {
         return;
     }
     UnicodeString expected = UNICODE_STRING_SIMPLE("abc_3#3{3'_def_sel}ect'_xyz");
-    Formattable args[] = { (int32_t)3, UNICODE_STRING_SIMPLE("x") };
+    Formattable args[] = { static_cast<int32_t>(3), UNICODE_STRING_SIMPLE("x") };
     internalFormat(
         &msgFmt, args, 2, expected,
         "MessageFormat with apostrophes in plural/select arguments failed:\n");
@@ -679,7 +679,7 @@ MessageFormat* TestMessageFormat::internalCreate(
     LocalPointer<MessageFormat> msgFmt(new MessageFormat(pattern, locale, status));
     if (U_FAILURE(status)) {
         dataerrln( "%s error while constructing with ErrorCode as %s" ,errMsg, u_errorName(status) );
-        logln(UnicodeString("TestMessageFormat::testMsgFormatSelect #1 with error code ")+(int32_t)status);
+        logln(UnicodeString("TestMessageFormat::testMsgFormatSelect #1 with error code ") + static_cast<int32_t>(status));
         return nullptr;
     }
     return msgFmt.orphan();
@@ -765,9 +765,9 @@ void TestMessageFormat::testMsgFormatSelect(/* char* par */)
     MessageFormat* msgFmt4 = internalCreate(t4.unescape(), Locale("fr"),err,(char*)"From TestMessageFormat::TestSelectFormat create t4");
     if (!U_FAILURE(err)) {
         //Arguments 
-        Formattable testArgs10[] = {"Kirti","female",(int32_t)6};    
+        Formattable testArgs10[] = {"Kirti", "female", static_cast<int32_t>(6)};
         Formattable testArgs11[] = {"Kirti","female",100.100};    
-        Formattable testArgs12[] = {"Kirti","other",(int32_t)6};    
+        Formattable testArgs12[] = {"Kirti", "other", static_cast<int32_t>(6)};
         Formattable* testArgs[] = {testArgs10,testArgs11,testArgs12};
         UnicodeString exp[] = {
             "Kirti est 6 all\\u00E9e \\u00E0 Paris." ,
@@ -788,10 +788,10 @@ void TestMessageFormat::testMsgFormatSelect(/* char* par */)
     // with no data the above should fail but it seems to construct an invalid MessageFormat with no reported error. See #13079
     if (!U_FAILURE(err)) {
         //Arguments 
-        Formattable testArgs10[] = {"Kirti",(int32_t)6,"female"};  
-        Formattable testArgs11[] = {"Kirti",(int32_t)1,"female"};  
-        Formattable testArgs12[] = {"Ash",(int32_t)1,"other"};
-        Formattable testArgs13[] = {"Ash",(int32_t)5,"other"};  
+        Formattable testArgs10[] = {"Kirti", static_cast<int32_t>(6), "female"};
+        Formattable testArgs11[] = {"Kirti", static_cast<int32_t>(1), "female"};
+        Formattable testArgs12[] = {"Ash", static_cast<int32_t>(1), "other"};
+        Formattable testArgs13[] = {"Ash", static_cast<int32_t>(5), "other"};
         Formattable* testArgs[] = {testArgs10,testArgs11,testArgs12,testArgs13};
         UnicodeString exp[] = {
             "Kirti sont all\\u00E9es \\u00E0 Paris." ,
@@ -812,21 +812,21 @@ void TestMessageFormat::testMsgFormatSelect(/* char* par */)
     LocalPointer<MessageFormat> msgFmt6(
             internalCreate(t6, Locale("de"),err,(char*)"From TestMessageFormat::TestSelectFormat create t6"));
     if (!U_FAILURE(err)) {
-        //Arguments 
-        Formattable testArgs10[] = {"Kirti","other",(int32_t)1,"other"}; 
-        Formattable testArgs11[] = {"Kirti","other",(int32_t)6,"other"};
-        Formattable testArgs12[] = {"Kirti","other",(int32_t)1,"female"};
-        Formattable testArgs13[] = {"Kirti","other",(int32_t)3,"female"};
-        Formattable testArgs14[] = {"Kirti","female",(int32_t)1,"female"};
-        Formattable testArgs15[] = {"Kirti","female",(int32_t)5,"female"};
-        Formattable testArgs16[] = {"Kirti","female",(int32_t)1,"other"};
-        Formattable testArgs17[] = {"Kirti","female",(int32_t)5,"other"};
-        Formattable testArgs18[] = {"Kirti","mixed",(int32_t)1,"mixed"};
-        Formattable testArgs19[] = {"Kirti","mixed",(int32_t)1,"other"};
-        Formattable testArgs20[] = {"Kirti","female",(int32_t)1,"mixed"};
-        Formattable testArgs21[] = {"Kirti","mixed",(int32_t)5,"mixed"};
-        Formattable testArgs22[] = {"Kirti","mixed",(int32_t)5,"other"};
-        Formattable testArgs23[] = {"Kirti","female",(int32_t)5,"mixed"};
+        //Arguments
+        Formattable testArgs10[] = {"Kirti", "other", static_cast<int32_t>(1), "other"};
+        Formattable testArgs11[] = {"Kirti", "other", static_cast<int32_t>(6), "other"};
+        Formattable testArgs12[] = {"Kirti", "other", static_cast<int32_t>(1), "female"};
+        Formattable testArgs13[] = {"Kirti", "other", static_cast<int32_t>(3), "female"};
+        Formattable testArgs14[] = {"Kirti", "female", static_cast<int32_t>(1), "female"};
+        Formattable testArgs15[] = {"Kirti", "female", static_cast<int32_t>(5), "female"};
+        Formattable testArgs16[] = {"Kirti", "female", static_cast<int32_t>(1), "other"};
+        Formattable testArgs17[] = {"Kirti", "female", static_cast<int32_t>(5), "other"};
+        Formattable testArgs18[] = {"Kirti", "mixed", static_cast<int32_t>(1), "mixed"};
+        Formattable testArgs19[] = {"Kirti", "mixed", static_cast<int32_t>(1), "other"};
+        Formattable testArgs20[] = {"Kirti", "female", static_cast<int32_t>(1), "mixed"};
+        Formattable testArgs21[] = {"Kirti", "mixed", static_cast<int32_t>(5), "mixed"};
+        Formattable testArgs22[] = {"Kirti", "mixed", static_cast<int32_t>(5), "other"};
+        Formattable testArgs23[] = {"Kirti", "female", static_cast<int32_t>(5), "mixed"};
         Formattable* testArgs[] = {testArgs10,testArgs11,testArgs12,testArgs13,
                                    testArgs14,testArgs15,testArgs16,testArgs17,
                                    testArgs18,testArgs19,testArgs20,testArgs21,
@@ -863,7 +863,7 @@ void TestMessageFormat::testCopyConstructor()
     UErrorCode success = U_ZERO_ERROR;
     MessageFormat *x = new MessageFormat("There are {0} files on {1}", success);
     MessageFormat *z = new MessageFormat("There are {0} files on {1} created", success);
-    MessageFormat *y = 0;
+    MessageFormat* y = nullptr;
     y = new MessageFormat(*x);
     if ( (*x == *y) && 
          (*x != *z) && 
@@ -922,7 +922,7 @@ void TestMessageFormat::testClone()
     UErrorCode success = U_ZERO_ERROR;
     MessageFormat *x = new MessageFormat("There are {0} files on {1}", success);
     MessageFormat *z = new MessageFormat("There are {0} files on {1} created", success);
-    MessageFormat *y = 0;
+    MessageFormat* y = nullptr;
     y = x->clone();
     if ( (*x == *y) && 
          (*x != *z) && 
@@ -983,7 +983,7 @@ void TestMessageFormat::testSetLocale()
     GregorianCalendar cal(err);   
     Formattable arguments[] = {
         456.83,
-        Formattable(UDate(8.71068e+011), Formattable::kIsDate),
+        Formattable(static_cast<UDate>(8.71068e+011), Formattable::kIsDate),
         "deposit"
         };
    
@@ -995,14 +995,14 @@ void TestMessageFormat::testSetLocale()
     // Just use unlocalized currency symbol.
     //UnicodeString compareStrEng = "At <time> on Aug 8, 1997, you made a deposit of $456.83.";
     UnicodeString compareStrEng = "At <time> on Aug 8, 1997, you made a deposit of ";
-    compareStrEng += (char16_t) 0x00a4;
+    compareStrEng += static_cast<char16_t>(0x00a4);
     compareStrEng += "456.83.";
     // {sfb} to get DM, would need Locale::GERMANY, not Locale::GERMAN
     // Just use unlocalized currency symbol.
     //UnicodeString compareStrGer = "At <time> on 08.08.1997, you made a deposit of 456,83 DM.";
     UnicodeString compareStrGer = "At <time> on 08.08.1997, you made a deposit of ";
     compareStrGer += "456,83";
-    compareStrGer += (char16_t) 0x00a0;
+    compareStrGer += static_cast<char16_t>(0x00a0);
     compareStrGer += "XXX.";
 
     MessageFormat msg( formatStr, err);
@@ -1064,14 +1064,14 @@ void TestMessageFormat::testFormat()
     UErrorCode err = U_ZERO_ERROR;
     GregorianCalendar cal(err);   
 
-    const Formattable ftarray[] = 
+    const Formattable ftarray[] =
     {
-        Formattable( UDate(8.71068e+011), Formattable::kIsDate )
+        Formattable(static_cast<UDate>(8.71068e+011), Formattable::kIsDate)
     };
     const int32_t ft_cnt = UPRV_LENGTHOF(ftarray);
     Formattable ft_arr( ftarray, ft_cnt );
 
-    Formattable* fmt = new Formattable(UDate(8.71068e+011), Formattable::kIsDate);
+    Formattable* fmt = new Formattable(static_cast<UDate>(8.71068e+011), Formattable::kIsDate);
    
     UnicodeString result;
 
@@ -1210,8 +1210,8 @@ void TestMessageFormat::testAdopt()
     int32_t count, countCmp;
     const Format** formats = msg.getFormats(count);
     const Format** formatsCmp = msgCmp.getFormats(countCmp);
-    const Format** formatsChg = 0;
-    const Format** formatsAct = 0;
+    const Format** formatsChg = nullptr;
+    const Format** formatsAct = nullptr;
     int32_t countAct;
     const Format* a;
     const Format* b;
@@ -1483,9 +1483,9 @@ void TestMessageFormat::TestUnlimitedArgsAndSubformats() {
     }
 
     const Formattable ARGS[] = {
-        Formattable(UDate(1e13), Formattable::kIsDate),
-        Formattable((int32_t)1303),
-        Formattable((int32_t)1202),
+        Formattable(static_cast<UDate>(1e13), Formattable::kIsDate),
+        Formattable(static_cast<int32_t>(1303)),
+        Formattable(static_cast<int32_t>(1202)),
         Formattable(1303.0/1202 - 1),
         Formattable("Glimmung"),
         Formattable("the printers"),
@@ -1512,7 +1512,7 @@ void TestMessageFormat::TestUnlimitedArgsAndSubformats() {
     if (result == expected) {
         logln(result);
     } else {
-        errln((UnicodeString)"FAIL: Got " + result +
+        errln(UnicodeString("FAIL: Got ") + result +
               ", expected " + expected);
     }
 }
@@ -1551,25 +1551,25 @@ void TestMessageFormat::TestRBNF() {
 
     for (int i = 0; i < formats_count; ++i) {
         MessageFormat* fmt = new MessageFormat(formats[i], locale, ec);
-        logln((UnicodeString)"Testing format pattern: '" + formats[i] + "'");
+        logln(UnicodeString("Testing format pattern: '") + formats[i] + "'");
 
         for (int j = 0; j < values_count; ++j) {
             ec = U_ZERO_ERROR;
             numFmt->parse(values[j], args[0], ec);
             if (U_FAILURE(ec)) {
-                errln((UnicodeString)"Failed to parse test argument " + values[j]);
+                errln(UnicodeString("Failed to parse test argument ") + values[j]);
             } else {
                 FieldPosition fp(FieldPosition::DONT_CARE);
                 UnicodeString result;
                 fmt->format(args, 1, result, fp, ec);
-                logln((UnicodeString)"value: " + toString(args[0]) + " --> " + result + UnicodeString(" ec: ") + u_errorName(ec));
+                logln(UnicodeString("value: ") + toString(args[0]) + " --> " + result + UnicodeString(" ec: ") + u_errorName(ec));
                
                 int32_t count = 0;
                 Formattable* parseResult = fmt->parse(result, count, ec);
                 if (count != 1) {
-                    errln((UnicodeString)"parse returned " + count + " args");
+                    errln(UnicodeString("parse returned ") + count + " args");
                 } else if (parseResult[0] != args[0]) {
-                    errln((UnicodeString)"parsed argument " + toString(parseResult[0]) + " != " + toString(args[0]));
+                    errln(UnicodeString("parsed argument ") + toString(parseResult[0]) + " != " + toString(args[0]));
                 }
                 delete []parseResult;
             }
@@ -1650,7 +1650,7 @@ void TestMessageFormat::TestCompatibleApostrophe() {
         errln("wrong value from  icuMsg.getApostropheMode().");
     }
 
-    Formattable zero0[] = { (int32_t)0 };
+    Formattable zero0[] = {static_cast<int32_t>(0)};
     FieldPosition fieldpos(FieldPosition::DONT_CARE);
     UnicodeString buffer1, buffer2;
     assertEquals("incompatible ICU MessageFormat compatibility-apostrophe behavior",
@@ -1854,7 +1854,7 @@ void TestMessageFormat::TestTrimArgumentName() {
     if (errorCode.errDataIfFailureAndReset("Unable to instantiate MessageFormat")) {
         return;
     }
-    Formattable args[1] = { (int32_t)2 };
+    Formattable args[1] = {static_cast<int32_t>(2)};
     FieldPosition ignore(FieldPosition::DONT_CARE);
     UnicodeString result;
     assertEquals("trim-numbered-arg format() failed", "a  #,#2.0  z",
@@ -1879,7 +1879,7 @@ void TestMessageFormat::TestSelectOrdinal() {
     if (errorCode.errDataIfFailureAndReset("Unable to instantiate MessageFormat")) {
         return;
     }
-    Formattable args[1] = { (int32_t)21 };
+    Formattable args[1] = {static_cast<int32_t>(21)};
     FieldPosition ignore(FieldPosition::DONT_CARE);
     UnicodeString result;
     assertEquals("plural-and-ordinal format(21) failed", "21 files, 21st file",
@@ -1906,13 +1906,13 @@ void TestMessageFormat::TestDecimals() {
     MessageFormat m(
             "{0,plural,one{one meter}other{# meters}}",
             Locale::getEnglish(), errorCode);
-    Formattable args[1] = { (int32_t)1 };
+    Formattable args[1] = {static_cast<int32_t>(1)};
     FieldPosition ignore;
     UnicodeString result;
     assertEquals("simple format(1)", "one meter",
             m.format(args, 1, result, ignore, errorCode), true);
 
-    args[0] = (double)1.5;
+    args[0] = 1.5;
     result.remove();
     assertEquals("simple format(1.5)", "1.5 meters",
             m.format(args, 1, result, ignore, errorCode), true);
@@ -1921,12 +1921,12 @@ void TestMessageFormat::TestDecimals() {
     MessageFormat m0(
             "{0,plural,one{one meter}other{{0} meters}}",
             Locale::getEnglish(), errorCode);
-    args[0] = (int32_t)1;
+    args[0] = static_cast<int32_t>(1);
     result.remove();
     assertEquals("explicit format(1)", "one meter",
             m0.format(args, 1, result, ignore, errorCode), true);
 
-    args[0] = (double)1.5;
+    args[0] = 1.5;
     result.remove();
     assertEquals("explicit format(1.5)", "1.5 meters",
             m0.format(args, 1, result, ignore, errorCode), true);
@@ -1935,17 +1935,17 @@ void TestMessageFormat::TestDecimals() {
     MessageFormat m1(
             "{0,plural,offset:1 one{another meter}other{{0,number,00.#} meters}}",
             Locale::getEnglish(), errorCode);
-    args[0] = (int32_t)1;
+    args[0] = static_cast<int32_t>(1);
     result.remove();
     assertEquals("offset format(1)", "01 meters",
             m1.format(args, 1, result, ignore, errorCode), true);
 
-    args[0] = (int32_t)2;
+    args[0] = static_cast<int32_t>(2);
     result.remove();
     assertEquals("offset format(1)", "another meter",
             m1.format(args, 1, result, ignore, errorCode), true);
 
-    args[0] = (double)2.5;
+    args[0] = 2.5;
     result.remove();
     assertEquals("offset format(1)", "02.5 meters",
             m1.format(args, 1, result, ignore, errorCode), true);
@@ -1954,17 +1954,17 @@ void TestMessageFormat::TestDecimals() {
     MessageFormat m2(
             "{0,plural,offset:1 one{another meter}other{{0,number,0.0} meters}}",
             Locale::getEnglish(), errorCode);
-    args[0] = (int32_t)1;
+    args[0] = static_cast<int32_t>(1);
     result.remove();
     assertEquals("offset-decimals format(1)", "1.0 meters",
             m2.format(args, 1, result, ignore, errorCode), true);
 
-    args[0] = (int32_t)2;
+    args[0] = static_cast<int32_t>(2);
     result.remove();
     assertEquals("offset-decimals format(1)", "2.0 meters",
             m2.format(args, 1, result, ignore, errorCode), true);
 
-    args[0] = (double)2.5;
+    args[0] = 2.5;
     result.remove();
     assertEquals("offset-decimals format(1)", "2.5 meters",
             m2.format(args, 1, result, ignore, errorCode), true);
@@ -2019,7 +2019,7 @@ void TestMessageFormat::TestMessageFormatNumberSkeleton() {
             { u"{0,number,'::'0.00}", "en", 50, u"::50.00" }, // pattern literal
     };
 
-    for (auto& cas : cases) {
+    for (const auto& cas : cases) {
         status.setScope(cas.messagePattern);
         MessageFormat msgf(cas.messagePattern, cas.localeName, status);
         UnicodeString sb;
