@@ -33,6 +33,7 @@
 #include "cmemory.h"
 #include "transrt.h"
 #include "testutil.h"
+#include "uassert.h"
 #include <string.h>
 #include <stdio.h>
 
@@ -49,7 +50,7 @@
                           break
 
 #define EXHAUSTIVE(id,test) case id:                            \
-                              if(quick==FALSE){                 \
+                              if(quick==false){                 \
                                   name = #test;                 \
                                   if (exec){                    \
                                       logln(#test "---");       \
@@ -117,7 +118,7 @@ class Legal {
 public:
     Legal() {}
     virtual ~Legal() {}
-    virtual UBool is(const UnicodeString& /*sourceString*/) const {return TRUE;}
+    virtual UBool is(const UnicodeString& /*sourceString*/) const {return true;}
 };
 
 class LegalJamo : public Legal {
@@ -127,35 +128,35 @@ class LegalJamo : public Legal {
 public:
     LegalJamo() {}
     virtual ~LegalJamo() {}
-    virtual UBool is(const UnicodeString& sourceString) const;
-            int   getType(UChar c) const;
+    virtual UBool is(const UnicodeString& sourceString) const override;
+            int   getType(char16_t c) const;
 };
 
 UBool LegalJamo::is(const UnicodeString& sourceString) const {
     int t;
     UnicodeString decomp;
     UErrorCode ec = U_ZERO_ERROR;
-    Normalizer::decompose(sourceString, FALSE, 0, decomp, ec); 
+    Normalizer::decompose(sourceString, false, 0, decomp, ec); 
     if (U_FAILURE(ec)) {
-        return FALSE;
+        return false;
     }      
     for (int i = 0; i < decomp.length(); ++i) { // don't worry about surrogates             
         switch (getType(decomp.charAt(i))) {
         case 0: t = getType(decomp.charAt(i+1));
-                if (t != 0 && t != 1) { return FALSE; }
+                if (t != 0 && t != 1) { return false; }
                 break;
         case 1: t = getType(decomp.charAt(i-1));
-                if (t != 0 && t != 1) { return FALSE; }
+                if (t != 0 && t != 1) { return false; }
                 break;
         case 2: t = getType(decomp.charAt(i-1));
-                if (t != 1 && t != 2) { return FALSE; }
+                if (t != 1 && t != 2) { return false; }
                 break;
         }
     }              
-    return TRUE;
+    return true;
 }
 
-int LegalJamo::getType(UChar c) const {
+int LegalJamo::getType(char16_t c) const {
     if (0x1100 <= c && c <= 0x1112) 
         return 0;
     else if (0x1161 <= c && c  <= 0x1175) 
@@ -171,55 +172,55 @@ public:
     LegalGreek(UBool _full) { full = _full; }
     virtual ~LegalGreek() {}
 
-    virtual UBool is(const UnicodeString& sourceString) const;
+    virtual UBool is(const UnicodeString& sourceString) const override;
 
-    static UBool isVowel(UChar c);
+    static UBool isVowel(char16_t c);
     
-    static UBool isRho(UChar c);
+    static UBool isRho(char16_t c);
 };
 
 UBool LegalGreek::is(const UnicodeString& sourceString) const { 
     UnicodeString decomp;
     UErrorCode ec = U_ZERO_ERROR;
-    Normalizer::decompose(sourceString, FALSE, 0, decomp, ec);
+    Normalizer::decompose(sourceString, false, 0, decomp, ec);
                 
     // modern is simpler: don't care about anything but a grave
-    if (full == FALSE) {
+    if (full == false) {
         // A special case which is legal but should be
         // excluded from round trip
         // if (sourceString == UnicodeString("\\u039C\\u03C0", "")) {
-        //    return FALSE;
+        //    return false;
         // }       
         for (int32_t i = 0; i < decomp.length(); ++i) {
-            UChar c = decomp.charAt(i);
+            char16_t c = decomp.charAt(i);
             // exclude all the accents
             if (c == 0x0313 || c == 0x0314 || c == 0x0300 || c == 0x0302
                 || c == 0x0342 || c == 0x0345
-                ) return FALSE;
+                ) return false;
         }
-        return TRUE;
+        return true;
     }
 
     // Legal greek has breathing marks IFF there is a vowel or RHO at the start
     // IF it has them, it has exactly one.
     // IF it starts with a RHO, then the breathing mark must come before the second letter.
     // Since there are no surrogates in greek, don't worry about them
-    UBool firstIsVowel = FALSE;
-    UBool firstIsRho = FALSE;
-    UBool noLetterYet = TRUE;
+    UBool firstIsVowel = false;
+    UBool firstIsRho = false;
+    UBool noLetterYet = true;
     int32_t breathingCount = 0;
     int32_t letterCount = 0;
     for (int32_t i = 0; i < decomp.length(); ++i) {
-        UChar c = decomp.charAt(i);
+        char16_t c = decomp.charAt(i);
         if (u_isalpha(c)) {
             ++letterCount;
             if (noLetterYet) {
-                noLetterYet =  FALSE;
+                noLetterYet =  false;
                 firstIsVowel = isVowel(c);
                 firstIsRho = isRho(c);
             }
             if (firstIsRho && letterCount == 2 && breathingCount == 0) {
-                return FALSE;
+                return false;
             }
         }
         if (c == 0x0313 || c == 0x0314) {
@@ -231,7 +232,7 @@ UBool LegalGreek::is(const UnicodeString& sourceString) const {
     return breathingCount == 0;
 }
 
-UBool LegalGreek::isVowel(UChar c) {
+UBool LegalGreek::isVowel(char16_t c) {
     switch (c) {
     case 0x03B1:
     case 0x03B5:
@@ -247,81 +248,55 @@ UBool LegalGreek::isVowel(UChar c) {
     case 0x039F:
     case 0x03A5:
     case 0x03A9:
-        return TRUE;
+        return true;
     }
-    return FALSE;
+    return false;
 }
 
-UBool LegalGreek::isRho(UChar c) {
+UBool LegalGreek::isRho(char16_t c) {
     switch (c) {
     case 0x03C1:
     case 0x03A1:
-        return TRUE;
+        return true;
     }
-    return FALSE;
+    return false;
 }
 
-// AbbreviatedUnicodeSetIterator Interface ---------------------------------------------
-//
-//      Iterate over a UnicodeSet, only returning a sampling of the contained code points.
-//        density is the approximate total number of code points to returned for the entire set.
-//
+namespace {
 
-class AbbreviatedUnicodeSetIterator : public UnicodeSetIterator {
-public :
-
-    AbbreviatedUnicodeSetIterator();
-    virtual ~AbbreviatedUnicodeSetIterator();
-    void reset(UnicodeSet& set, UBool abb = FALSE, int32_t density = 100);
-
-    /**
-     * ICU "poor man's RTTI", returns a UClassID for this class.
-     */
-    static inline UClassID getStaticClassID() { return (UClassID)&fgClassID; }
-
-    /**
-     * ICU "poor man's RTTI", returns a UClassID for the actual class.
-     */
-    virtual inline UClassID getDynamicClassID() const { return getStaticClassID(); }
-
-private :
-    UBool abbreviated;
-    int32_t perRange;           // The maximum number of code points to be returned from each range
-    virtual void loadRange(int32_t range);
-
-    /**
-     * The address of this static class variable serves as this class's ID
-     * for ICU "poor man's RTTI".
-     */
-    static const char fgClassID;
-};
-
-// AbbreviatedUnicodeSetIterator Implementation ---------------------------------------
-
-const char AbbreviatedUnicodeSetIterator::fgClassID=0;
-
-AbbreviatedUnicodeSetIterator::AbbreviatedUnicodeSetIterator() :
-    UnicodeSetIterator(), abbreviated(FALSE) {
-}
-
-AbbreviatedUnicodeSetIterator::~AbbreviatedUnicodeSetIterator() {
-}
-        
-void AbbreviatedUnicodeSetIterator::reset(UnicodeSet& newSet, UBool abb, int32_t density) {
-    UnicodeSetIterator::reset(newSet);
-    abbreviated = abb;
-    perRange = newSet.getRangeCount();
+/**
+ * If abbreviated=true, returns a set which only a sampling of the original code points.
+ * density is the approximate total number of code points to returned for the entire set.
+ */
+const UnicodeSet &abbreviateSet(const UnicodeSet &set, bool abbreviated, int density,
+                                UnicodeSet &copy) {
+    if (!abbreviated) {
+        return set;
+    }
+    int32_t rangeCount = set.getRangeCount();
+    int32_t perRange = rangeCount;
     if (perRange != 0) {
         perRange = density / perRange;
     }
+    const UnicodeSet *p = &set;
+    bool unchanged = true;
+    for (int32_t i = 0; i < rangeCount; ++i) {
+        int32_t start = set.getRangeStart(i);
+        int32_t end = set.getRangeEnd(i);
+        int32_t newEnd = start + perRange;
+        if (end > newEnd) {
+            if (unchanged) {
+                copy = set;
+                p = &copy;
+                unchanged = false;
+            }
+            copy.remove(newEnd + 1, end);
+        }
+    }
+    return *p;
 }
 
-void AbbreviatedUnicodeSetIterator::loadRange(int32_t myRange) {
-    UnicodeSetIterator::loadRange(myRange);
-    if (abbreviated && (endElement > nextElement + perRange)) {
-        endElement = nextElement + perRange;
-    }
-}
+}  // namespace
 
 //--------------------------------------------------------------------
 // RTTest Interface
@@ -427,38 +402,38 @@ void RTTest::setPairLimit(int32_t limit) {
 }
 
 UBool RTTest::isSame(const UnicodeString& a, const UnicodeString& b) {
-    if (a == b) return TRUE;
-    if (a.caseCompare(b, U_FOLD_CASE_DEFAULT)==0 && isCamel(a)) return TRUE;
+    if (a == b) return true;
+    if (a.caseCompare(b, U_FOLD_CASE_DEFAULT)==0 && isCamel(a)) return true;
     UnicodeString aa, bb;
     UErrorCode ec = U_ZERO_ERROR;
-    Normalizer::decompose(a, FALSE, 0, aa, ec);
-    Normalizer::decompose(b, FALSE, 0, bb, ec);
-    if (aa == bb) return TRUE;
-    if (aa.caseCompare(bb, U_FOLD_CASE_DEFAULT)==0 && isCamel(aa)) return TRUE;
-    return FALSE;
+    Normalizer::decompose(a, false, 0, aa, ec);
+    Normalizer::decompose(b, false, 0, bb, ec);
+    if (aa == bb) return true;
+    if (aa.caseCompare(bb, U_FOLD_CASE_DEFAULT)==0 && isCamel(aa)) return true;
+    return false;
 }
 
 UBool RTTest::isCamel(const UnicodeString& a) {
     // see if string is of the form aB; e.g. lower, then upper or title
     UChar32 cp;
-    UBool haveLower = FALSE;
+    UBool haveLower = false;
     for (int32_t i = 0; i < a.length(); i += U16_LENGTH(cp)) {
         cp = a.char32At(i);
         int8_t t = u_charType(cp);
         switch (t) {
         case U_UPPERCASE_LETTER:
-            if (haveLower) return TRUE;
+            if (haveLower) return true;
             break;
         case U_TITLECASE_LETTER:
-            if (haveLower) return TRUE;
+            if (haveLower) return true;
             // fall through, since second letter is lower.
             U_FALLTHROUGH;
         case U_LOWERCASE_LETTER:
-            haveLower = TRUE;
+            haveLower = true;
             break;
         }
     }
-    return FALSE;
+    return false;
 }
 
 void RTTest::test(const UnicodeString& sourceRangeVal,
@@ -519,7 +494,7 @@ void RTTest::test(const UnicodeString& sourceRangeVal,
     this->toTarget.addAll(okAnyway);
 
     this->roundtripExclusionsSet.clear();
-    if (roundtripExclusions != NULL && strlen(roundtripExclusions) > 0) {
+    if (roundtripExclusions != nullptr && strlen(roundtripExclusions) > 0) {
         this->roundtripExclusionsSet.applyPattern(UnicodeString(roundtripExclusions, -1, US_INV), status);
         if (U_FAILURE(status)) {
             parent->errln("FAIL: UnicodeSet::applyPattern(%s)", roundtripExclusions);
@@ -538,12 +513,12 @@ void RTTest::test(const UnicodeString& sourceRangeVal,
 
     if (errorCount > 0) {
         char str[100];
-        int32_t length = transliteratorID.extract(str, 100, NULL, status);
+        int32_t length = transliteratorID.extract(str, 100, nullptr, status);
         str[length] = 0;
         parent->errln("FAIL: %s errors: %d %s", str, errorCount, (errorCount > errorLimit ? " (at least!)" : " ")); // + ", see " + logFileName);
     } else {
         char str[100];
-        int32_t length = transliteratorID.extract(str, 100, NULL, status);
+        int32_t length = transliteratorID.extract(str, 100, nullptr, status);
         str[length] = 0;
         parent->logln("%s ok", str);
     }
@@ -552,13 +527,13 @@ void RTTest::test(const UnicodeString& sourceRangeVal,
 UBool RTTest::checkIrrelevants(Transliterator *t, 
                                const UnicodeString& irrelevants) {
     for (int i = 0; i < irrelevants.length(); ++i) {
-        UChar c = irrelevants.charAt(i);
+        char16_t c = irrelevants.charAt(i);
         UnicodeString srcStr(c);
         UnicodeString targ = srcStr;
         t->transliterate(targ);
-        if (srcStr == targ) return TRUE;
+        if (srcStr == targ) return true;
     }
-    return FALSE;
+    return false;
 }
 
 void RTTest::test2(UBool quickRt, int32_t density) {
@@ -569,34 +544,34 @@ void RTTest::test2(UBool quickRt, int32_t density) {
     TransliteratorPointer sourceToTarget(
         Transliterator::createInstance(transliteratorID, UTRANS_FORWARD, parseError,
                                        status));
-    if ((Transliterator *)sourceToTarget == NULL) {
+    if (sourceToTarget == nullptr) {
         parent->dataerrln("FAIL: createInstance(" + transliteratorID +
-                   ") returned NULL. Error: " + u_errorName(status)
+                   ") returned nullptr. Error: " + u_errorName(status)
                    + "\n\tpreContext : " + prettify(parseError.preContext) 
                    + "\n\tpostContext : " + prettify(parseError.postContext));
         
                 return;
     }
     TransliteratorPointer targetToSource(sourceToTarget->createInverse(status));
-    if ((Transliterator *)targetToSource == NULL) {
+    if (targetToSource == nullptr) {
         parent->errln("FAIL: " + transliteratorID +
-                   ".createInverse() returned NULL. Error:" + u_errorName(status)          
+                   ".createInverse() returned nullptr. Error:" + u_errorName(status)          
                    + "\n\tpreContext : " + prettify(parseError.preContext) 
                    + "\n\tpostContext : " + prettify(parseError.postContext));
         return;
     }
 
-    AbbreviatedUnicodeSetIterator usi;
-    AbbreviatedUnicodeSetIterator usi2;
+    UnicodeSetIterator usi;
+    UnicodeSetIterator usi2;
 
     parent->logln("Checking that at least one irrelevant character is not NFC'ed");
     // string is from NFC_NO in the UCD
     UnicodeString irrelevants = CharsToUnicodeString("\\u2000\\u2001\\u2126\\u212A\\u212B\\u2329"); 
 
-    if (checkIrrelevants(sourceToTarget, irrelevants) == FALSE) {
+    if (checkIrrelevants(sourceToTarget, irrelevants) == false) {
         logFails("Source-Target, irrelevants");
     }
-    if (checkIrrelevants(targetToSource, irrelevants) == FALSE) {
+    if (checkIrrelevants(targetToSource, irrelevants) == false) {
         logFails("Target-Source, irrelevants");
     }
             
@@ -605,7 +580,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
       UnicodeString rules = "";
        
       UParseError parseError;
-      rules = sourceToTarget->toRules(rules, TRUE);
+      rules = sourceToTarget->toRules(rules, true);
       // parent->logln((UnicodeString)"toRules => " + rules);
       TransliteratorPointer sourceToTarget2(Transliterator::createFromRules(
                                                        "s2t2", rules, 
@@ -616,7 +591,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
           return;
       }
 
-      rules = targetToSource->toRules(rules, FALSE);
+      rules = targetToSource->toRules(rules, false);
       TransliteratorPointer targetToSource2(Transliterator::createFromRules(
                                                        "t2s2", rules, 
                                                        UTRANS_FORWARD,
@@ -631,7 +606,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
           if (!usi.next() || usi.isString()) break;
           UChar32 c = usi.getCodepoint();
                     
-          UnicodeString srcStr((UChar32)c);
+          UnicodeString srcStr(c);
           UnicodeString targ = srcStr;
           sourceToTarget->transliterate(targ);
           UnicodeString targ2 = srcStr;
@@ -646,7 +621,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
           if (!usi.next() || usi.isString()) break;
           UChar32 c = usi.getCodepoint();
               
-          UnicodeString srcStr((UChar32)c);
+          UnicodeString srcStr(c);
           UnicodeString targ = srcStr;
           targetToSource->transliterate(targ);
           UnicodeString targ2 = srcStr;
@@ -665,19 +640,19 @@ void RTTest::test2(UBool quickRt, int32_t density) {
         if (!usi.next() || usi.isString()) break;
         UChar32 c = usi.getCodepoint();
                 
-        UnicodeString srcStr((UChar32)c);
+        UnicodeString srcStr(c);
         UnicodeString targ = srcStr;
         sourceToTarget->transliterate(targ);
-        if (toTarget.containsAll(targ) == FALSE
-            || badCharacters.containsSome(targ) == TRUE) {
+        if (toTarget.containsAll(targ) == false
+            || badCharacters.containsSome(targ) == true) {
             UnicodeString targD;
-            Normalizer::decompose(targ, FALSE, 0, targD, status);
+            Normalizer::decompose(targ, false, 0, targD, status);
             if (U_FAILURE(status)) {
                 parent->errln("FAIL: Internal error during decomposition %s\n", u_errorName(status));
                 return;
             }
-            if (toTarget.containsAll(targD) == FALSE || 
-                badCharacters.containsSome(targD) == TRUE) {
+            if (toTarget.containsAll(targD) == false || 
+                badCharacters.containsSome(targD) == true) {
                 logWrongScript("Source-Target", srcStr, targ);
                 failSourceTarg.add(c);
                 continue;
@@ -685,7 +660,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
         }
 
         UnicodeString cs2;
-        Normalizer::decompose(srcStr, FALSE, 0, cs2, status);
+        Normalizer::decompose(srcStr, false, 0, cs2, status);
         if (U_FAILURE(status)) {
             parent->errln("FAIL: Internal error during decomposition %s\n", u_errorName(status));
             return;
@@ -701,39 +676,40 @@ void RTTest::test2(UBool quickRt, int32_t density) {
 
     UnicodeSet sourceRangeMinusFailures(sourceRange);
     sourceRangeMinusFailures.removeAll(failSourceTarg);
-            
-    usi.reset(sourceRangeMinusFailures, quickRt, density);
+
+    UnicodeSet copy, copy2;
+    usi.reset(abbreviateSet(sourceRangeMinusFailures, quickRt, density, copy));
     for (;;) { 
         if (!usi.next() || usi.isString()) break;
         UChar32 c = usi.getCodepoint();
              
-        usi2.reset(sourceRangeMinusFailures, quickRt, density);
+        usi2.reset(abbreviateSet(sourceRangeMinusFailures, quickRt, density, copy2));
         for (;;) {
             if (!usi2.next() || usi2.isString()) break;
             UChar32 d = usi2.getCodepoint();
                     
             UnicodeString srcStr;
-            srcStr += (UChar32)c;
-            srcStr += (UChar32)d;
+            srcStr += c;
+            srcStr += d;
             UnicodeString targ = srcStr;
             sourceToTarget->transliterate(targ);
-            if (toTarget.containsAll(targ) == FALSE || 
-                badCharacters.containsSome(targ) == TRUE)
+            if (toTarget.containsAll(targ) == false || 
+                badCharacters.containsSome(targ) == true)
             {
                 UnicodeString targD;
-                Normalizer::decompose(targ, FALSE, 0, targD, status);
+                Normalizer::decompose(targ, false, 0, targD, status);
                 if (U_FAILURE(status)) {
                     parent->errln("FAIL: Internal error during decomposition %s\n", u_errorName(status));
                     return;
                 }
-                if (toTarget.containsAll(targD) == FALSE ||
-                    badCharacters.containsSome(targD) == TRUE) {
+                if (toTarget.containsAll(targD) == false ||
+                    badCharacters.containsSome(targD) == true) {
                     logWrongScript("Source-Target", srcStr, targ);
                     continue;
                 }
             }
             UnicodeString cs2;
-            Normalizer::decompose(srcStr, FALSE, 0, cs2, status);
+            Normalizer::decompose(srcStr, false, 0, cs2, status);
             if (U_FAILURE(status)) {
                 parent->errln("FAIL: Internal error during decomposition %s\n", u_errorName(status));
                 return;
@@ -758,7 +734,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
         if(usi.isString()){
             srcStr = usi.getString();
         }else{
-            srcStr = (UnicodeString)usi.getCodepoint();
+            srcStr = UnicodeString(usi.getCodepoint());
         }
 
         UChar32 c = srcStr.char32At(0);
@@ -768,35 +744,35 @@ void RTTest::test2(UBool quickRt, int32_t density) {
         reverse = targ;
         sourceToTarget->transliterate(reverse);
 
-        if (toSource.containsAll(targ) == FALSE ||
-            badCharacters.containsSome(targ) == TRUE) {
+        if (toSource.containsAll(targ) == false ||
+            badCharacters.containsSome(targ) == true) {
             UnicodeString targD;
-            Normalizer::decompose(targ, FALSE, 0, targD, status);
+            Normalizer::decompose(targ, false, 0, targD, status);
             if (U_FAILURE(status)) {
                 parent->errln("FAIL: Internal error during decomposition%s\n", u_errorName(status));
                 return;
             }
-            if (toSource.containsAll(targD) == FALSE) {
+            if (toSource.containsAll(targD) == false) {
                 logWrongScript("Target-Source", srcStr, targ);
                 failTargSource.add(c);
                 continue;
             }
-            if (badCharacters.containsSome(targD) == TRUE) {
+            if (badCharacters.containsSome(targD) == true) {
                 logWrongScript("Target-Source*", srcStr, targ);
                 failTargSource.add(c);
                 continue;
             }
         }
-        if (isSame(srcStr, reverse) == FALSE && 
-            roundtripExclusionsSet.contains(c) == FALSE
-            && roundtripExclusionsSet.contains(srcStr)==FALSE) {
+        if (isSame(srcStr, reverse) == false && 
+            roundtripExclusionsSet.contains(c) == false
+            && roundtripExclusionsSet.contains(srcStr)==false) {
             logRoundTripFailure(srcStr,targetToSource->getID(), targ,sourceToTarget->getID(), reverse);
             failRound.add(c);
             continue;
         } 
         
         UnicodeString targ2;
-        Normalizer::decompose(targ, FALSE, 0, targ2, status);
+        Normalizer::decompose(targ, false, 0, targ2, status);
         if (U_FAILURE(status)) {
             parent->errln("FAIL: Internal error during decomposition%s\n", u_errorName(status));
             return;
@@ -815,7 +791,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
     targetRangeMinusFailures.removeAll(failTargSource);
     targetRangeMinusFailures.removeAll(failRound);
 
-    usi.reset(targetRangeMinusFailures, quickRt, density);
+    usi.reset(abbreviateSet(targetRangeMinusFailures, quickRt, density, copy));
     UnicodeString targ2;
     UnicodeString reverse2;
     UnicodeString targD;
@@ -825,11 +801,11 @@ void RTTest::test2(UBool quickRt, int32_t density) {
         if (++count > pairLimit) {
             //throw new TestTruncated("Test truncated at " + pairLimit + " x 64k pairs");
             parent->logln("");
-            parent->logln((UnicodeString)"Test truncated at " + pairLimit + " x 64k pairs");
+            parent->logln(UnicodeString("Test truncated at ") + pairLimit + " x 64k pairs");
             return;
         }
 
-        usi2.reset(targetRangeMinusFailures, quickRt, density);
+        usi2.reset(abbreviateSet(targetRangeMinusFailures, quickRt, density, copy2));
         for (;;) {
             if (!usi2.next() || usi2.isString())
                 break;
@@ -843,34 +819,34 @@ void RTTest::test2(UBool quickRt, int32_t density) {
             reverse = targ;
             sourceToTarget->transliterate(reverse);
 
-            if (toSource.containsAll(targ) == FALSE || 
-                badCharacters.containsSome(targ) == TRUE) 
+            if (toSource.containsAll(targ) == false || 
+                badCharacters.containsSome(targ) == true) 
             {
                 targD.truncate(0);  // empty the variable without construction/destruction
-                Normalizer::decompose(targ, FALSE, 0, targD, status);
+                Normalizer::decompose(targ, false, 0, targD, status);
                 if (U_FAILURE(status)) {
                     parent->errln("FAIL: Internal error during decomposition%s\n", 
                                u_errorName(status));
                     return;
                 }
-                if (toSource.containsAll(targD) == FALSE 
-                    || badCharacters.containsSome(targD) == TRUE)
+                if (toSource.containsAll(targD) == false 
+                    || badCharacters.containsSome(targD) == true)
                 {
                     logWrongScript("Target-Source", srcStr, targ);
                     continue;
                 }
             }
-            if (isSame(srcStr, reverse) == FALSE && 
-                roundtripExclusionsSet.contains(c) == FALSE&&
-                roundtripExclusionsSet.contains(d) == FALSE &&
-                roundtripExclusionsSet.contains(srcStr)== FALSE)
+            if (isSame(srcStr, reverse) == false && 
+                roundtripExclusionsSet.contains(c) == false&&
+                roundtripExclusionsSet.contains(d) == false &&
+                roundtripExclusionsSet.contains(srcStr)== false)
             {
                 logRoundTripFailure(srcStr,targetToSource->getID(), targ, sourceToTarget->getID(),reverse);
                 continue;
             } 
         
             targ2.truncate(0);  // empty the variable without construction/destruction
-            Normalizer::decompose(targ, FALSE, 0, targ2, status);
+            Normalizer::decompose(targ, false, 0, targ2, status);
             if (U_FAILURE(status)) {
                 parent->errln("FAIL: Internal error during decomposition%s\n", u_errorName(status));
                 return;
@@ -888,7 +864,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
 void RTTest::logWrongScript(const UnicodeString& label,
                             const UnicodeString& from,
                             const UnicodeString& to) {
-    parent->errln((UnicodeString)"FAIL " +
+    parent->errln(UnicodeString("FAIL ") +
                label + ": " +
                from + "(" + TestUtility::hex(from) + ") => " +
                to + "(" + TestUtility::hex(to) + ")");
@@ -900,7 +876,7 @@ void RTTest::logNotCanonical(const UnicodeString& label,
                              const UnicodeString& to,
                              const UnicodeString& fromCan,
                              const UnicodeString& toCan) {
-    parent->errln((UnicodeString)"FAIL (can.equiv)" +
+    parent->errln(UnicodeString("FAIL (can.equiv)") +
                label + ": " +
                from + "(" + TestUtility::hex(from) + ") => " +
                to + "(" + TestUtility::hex(to) + ")" +
@@ -912,7 +888,7 @@ void RTTest::logNotCanonical(const UnicodeString& label,
 }
 
 void RTTest::logFails(const UnicodeString& label) {
-    parent->errln((UnicodeString)"<br>FAIL " + label);
+    parent->errln(UnicodeString("<br>FAIL ") + label);
     ++errorCount;
 }
 
@@ -921,7 +897,7 @@ void RTTest::logToRulesFails(const UnicodeString& label,
                              const UnicodeString& to, 
                              const UnicodeString& otherTo)
 {
-    parent->errln((UnicodeString)"FAIL: " +
+    parent->errln(UnicodeString("FAIL: ") +
                label + ": " +
                from + "(" + TestUtility::hex(from) + ") => " +
                to + "(" + TestUtility::hex(to) + ")" +
@@ -938,12 +914,12 @@ void RTTest::logRoundTripFailure(const UnicodeString& from,
                                  const UnicodeString& to,
                                  const UnicodeString& backID,
                                  const UnicodeString& back) {
-    if (legalSource->is(from) == FALSE) return; // skip illegals
+    if (legalSource->is(from) == false) return; // skip illegals
 
-    parent->errln((UnicodeString)"FAIL Roundtrip: " +
+    parent->errln(UnicodeString("FAIL Roundtrip: ") +
                from + "(" + TestUtility::hex(from) + ") => " +
-               to + "(" + TestUtility::hex(to) + ")  "+toID+" => " +
-               back + "(" + TestUtility::hex(back) + ") "+backID+" => ");
+               to + "(" + TestUtility::hex(to) + ")  " + toID + " => " +
+               back + "(" + TestUtility::hex(back) + ") " + backID + " => ");
     ++errorCount;
 }
 
@@ -1018,7 +994,7 @@ void TransliteratorRoundTripTest::TestJamo() {
     t.test(UnicodeString("[a-zA-Z]", ""), 
            UnicodeString("[\\u1100-\\u1112 \\u1161-\\u1175 \\u11A8-\\u11C2]", 
                          ""), 
-           NULL, this, quick, legal);
+           nullptr, this, quick, legal);
     delete legal;
 }
 
@@ -1028,25 +1004,30 @@ void TransliteratorRoundTripTest::TestHangul() {
     if (quick) t.setPairLimit(1000);
     t.test(UnicodeString("[a-zA-Z]", ""), 
            UnicodeString("[\\uAC00-\\uD7A4]", ""), 
-           NULL, this, quick, legal, 1);
+           nullptr, this, quick, legal, 1);
     delete legal;
 }
 
 
-#define ASSERT_SUCCESS(status) {if (U_FAILURE(status)) { \
-     errcheckln(status, "error at file %s, line %d, status = %s", __FILE__, __LINE__, \
-         u_errorName(status)); \
-         return;}}
-    
+#define ASSERT_SUCCESS(status) UPRV_BLOCK_MACRO_BEGIN { \
+    if (U_FAILURE(status)) { \
+        errcheckln(status, "error at file %s, line %d, status = %s", __FILE__, __LINE__, \
+                   u_errorName(status)); \
+        return; \
+    } \
+} UPRV_BLOCK_MACRO_END
+
 
 static void writeStringInU8(FILE *out, const UnicodeString &s) {
     int i;
     for (i=0; i<s.length(); i=s.moveIndex32(i, 1)) {
         UChar32  c = s.char32At(i);
         uint8_t  bufForOneChar[10];
-        UBool    isError = FALSE;
+        UBool    isError = false;
         int32_t  destIdx = 0;
         U8_APPEND(bufForOneChar, destIdx, (int32_t)sizeof(bufForOneChar), c, isError);
+        U_ASSERT(!isError);
+        (void)isError;
         fwrite(bufForOneChar, 1, destIdx, out);
     }
 }
@@ -1057,7 +1038,10 @@ static void writeStringInU8(FILE *out, const UnicodeString &s) {
 void TransliteratorRoundTripTest::TestHan() {
     UErrorCode  status = U_ZERO_ERROR;
     LocalULocaleDataPointer uld(ulocdata_open("zh",&status));
-    LocalUSetPointer USetExemplars(ulocdata_getExemplarSet(uld.getAlias(), uset_openEmpty(), 0, ULOCDATA_ES_STANDARD, &status));
+    LocalUSetPointer USetExemplars(uset_openEmpty());
+    assertTrue("", USetExemplars.isValid(), false, false, __FILE__, __LINE__);
+    if (! USetExemplars.isValid()) return;
+    ulocdata_getExemplarSet(uld.getAlias(), USetExemplars.getAlias(), 0, ULOCDATA_ES_STANDARD, &status);
     ASSERT_SUCCESS(status);
 
     UnicodeString source;
@@ -1066,7 +1050,7 @@ void TransliteratorRoundTripTest::TestHan() {
     for (i=0; ;i++) {
         // Add all of the Chinese exemplar chars to the string "source".
         c = uset_charAt(USetExemplars.getAlias(), i);
-        if (c == (UChar32)-1) {
+        if (c == static_cast<UChar32>(-1)) {
             break;
         }
         source.append(c);
@@ -1134,7 +1118,7 @@ void TransliteratorRoundTripTest::TestGreek() {
     // It is left in its current state as a regression test.
 
     RTTest test("Latin-Greek");
-    LegalGreek *legal = new LegalGreek(TRUE);
+    LegalGreek *legal = new LegalGreek(true);
 
     test.test(UnicodeString("[a-zA-Z]", ""), 
         UnicodeString("[\\u003B\\u00B7[[:Greek:]&[:Letter:]]-["
@@ -1159,7 +1143,7 @@ void TransliteratorRoundTripTest::TestGreekUNGEGN() {
     // It is left in its current state as a regression test.
 
     RTTest test("Latin-Greek/UNGEGN");
-    LegalGreek *legal = new LegalGreek(FALSE);
+    LegalGreek *legal = new LegalGreek(false);
 
     test.test(UnicodeString("[a-zA-Z]", ""), 
         UnicodeString("[\\u003B\\u00B7[[:Greek:]&[:Letter:]]-["
@@ -1181,7 +1165,7 @@ void TransliteratorRoundTripTest::Testel() {
     // It is left in its current state as a regression test.
 
     RTTest test("Latin-el");
-    LegalGreek *legal = new LegalGreek(FALSE);
+    LegalGreek *legal = new LegalGreek(false);
 
     test.test(UnicodeString("[a-zA-Z]", ""), 
         UnicodeString("[\\u003B\\u00B7[[:Greek:]&[:Letter:]]-["
@@ -1215,7 +1199,7 @@ private:
 public:
     LegalHebrew(UErrorCode& error);
     virtual ~LegalHebrew() {}
-    virtual UBool is(const UnicodeString& sourceString) const;
+    virtual UBool is(const UnicodeString& sourceString) const override;
 };
 
 LegalHebrew::LegalHebrew(UErrorCode& error){
@@ -1225,18 +1209,18 @@ LegalHebrew::LegalHebrew(UErrorCode& error){
 }
 UBool LegalHebrew::is(const UnicodeString& sourceString)const{
  
-    if (sourceString.length() == 0) return TRUE;
+    if (sourceString.length() == 0) return true;
     // don't worry about surrogates.
     for (int i = 0; i < sourceString.length(); ++i) {
-        UChar ch = sourceString.charAt(i);
-        UChar next = i+1 == sourceString.length() ? 0x0000 : sourceString.charAt(i);
+        char16_t ch = sourceString.charAt(i);
+        char16_t next = i+1 == sourceString.length() ? 0x0000 : sourceString.charAt(i);
         if (FINAL.contains(ch)) {
-            if (LETTER.contains(next)) return FALSE;
+            if (LETTER.contains(next)) return false;
         } else if (NON_FINAL.contains(ch)) {
-            if (!LETTER.contains(next)) return FALSE;
+            if (!LETTER.contains(next)) return false;
         }
     }
-    return TRUE;
+    return true;
 }
 void TransliteratorRoundTripTest::TestHebrew() {
     logKnownIssue( "cldrbug:1911");
@@ -1260,7 +1244,7 @@ void TransliteratorRoundTripTest::TestCyrillic() {
     Legal *legal = new Legal();
 
     test.test(UnicodeString("[a-zA-Z\\u0110\\u0111\\u02BA\\u02B9]", ""), 
-              UnicodeString("[[\\u0400-\\u045F] & [:Age=3.2:]]", ""), NULL, this, quick, 
+              UnicodeString("[[\\u0400-\\u045F] & [:Age=3.2:]]", ""), nullptr, this, quick, 
               legal);
 
     delete legal;
@@ -1278,29 +1262,29 @@ class LegalIndic :public Legal{
     
 public:
     LegalIndic();
-    virtual UBool is(const UnicodeString& sourceString) const;
-    virtual ~LegalIndic() {};
+    virtual UBool is(const UnicodeString& sourceString) const override;
+    virtual ~LegalIndic() {}
 };
 UBool LegalIndic::is(const UnicodeString& sourceString) const{
     int cp=sourceString.charAt(0);
     
     // A vowel sign cannot be the first char
     if(vowelSignSet.contains(cp)){
-        return FALSE;
+        return false;
     }else if(avagraha.contains(cp)){
-        return FALSE;
+        return false;
     }else if(virama.contains(cp)){
-        return FALSE;
+        return false;
     }else if(nukta.contains(cp)){
-        return FALSE;
+        return false;
     }else if(sanskritStressSigns.contains(cp)){
-        return FALSE;
+        return false;
     }else if(chandrabindu.contains(cp) && 
                 ((sourceString.length()>1) && 
                     vowelSignSet.contains(sourceString.charAt(1)))){
-        return FALSE;
+        return false;
     }
-    return TRUE;
+    return true;
 }
 LegalIndic::LegalIndic(){
         UErrorCode status = U_ZERO_ERROR;
@@ -1345,7 +1329,7 @@ void TransliteratorRoundTripTest::TestDevanagariLatin() {
         UErrorCode status = U_ZERO_ERROR;
         UParseError parseError;
         TransliteratorPointer t1(Transliterator::createInstance("[\\u0964-\\u0965\\u0981-\\u0983\\u0985-\\u098C\\u098F-\\u0990\\u0993-\\u09A8\\u09AA-\\u09B0\\u09B2\\u09B6-\\u09B9\\u09BC\\u09BE-\\u09C4\\u09C7-\\u09C8\\u09CB-\\u09CD\\u09D7\\u09DC-\\u09DD\\u09DF-\\u09E3\\u09E6-\\u09FA];NFD;Bengali-InterIndic;InterIndic-Gujarati;NFC;",UTRANS_FORWARD, parseError, status));
-        if((Transliterator *)t1 != NULL){
+        if(t1 != nullptr){
             TransliteratorPointer t2(t1->createInverse(status));
             if(U_FAILURE(status)){
                 errln("FAIL: could not create the Inverse:-( \n");
@@ -1603,13 +1587,13 @@ static const char * const interIndicArray[] = {
     "[\\u0BF0\\u0BF1\\u0BF2]" /*roundtrip exclusions*/,
 
     "Latin-Telugu",latinForIndic, "[:Telugu:]", 
-    NULL /*roundtrip exclusions*/,
+    nullptr /*roundtrip exclusions*/,
 
     "Latin-Kannada",latinForIndic, "[:Kannada:]", 
-    NULL /*roundtrip exclusions*/,
+    nullptr /*roundtrip exclusions*/,
 
     "Latin-Malayalam",latinForIndic, "[:Malayalam:]", 
-    NULL /*roundtrip exclusions*/  
+    nullptr /*roundtrip exclusions*/  
 };
 
 void TransliteratorRoundTripTest::TestDebug(const char* name,const char fromSet[],
@@ -1621,7 +1605,7 @@ void TransliteratorRoundTripTest::TestDebug(const char* name,const char fromSet[
 }
 
 void TransliteratorRoundTripTest::TestInterIndic() {
-    //TestDebug("Latin-Gurmukhi", latinForIndic, "[:Gurmukhi:]","[\\u0965\\u0a02\\u0a72\\u0a73\\u0a74]",TRUE);
+    //TestDebug("Latin-Gurmukhi", latinForIndic, "[:Gurmukhi:]","[\\u0965\\u0a02\\u0a72\\u0a73\\u0a74]",true);
     int32_t num = UPRV_LENGTHOF(interIndicArray)/INTER_INDIC_ARRAY_WIDTH;
     if(quick){
         logln("Testing only 5 of %i. Skipping rest (use -e for exhaustive)",num);
