@@ -166,11 +166,11 @@ private:
 };
 
 
-void TestArabicShapeThreads::doTailTest(void) {
-    static const UChar src[] = { 0x0020, 0x0633, 0 };
-    static const UChar dst_old[] = { 0xFEB1, 0x200B,0 };
-    static const UChar dst_new[] = { 0xFEB1, 0xFE73,0 };
-    UChar dst[3] = { 0x0000, 0x0000,0 };
+void TestArabicShapeThreads::doTailTest() {
+    static const char16_t src[] = { 0x0020, 0x0633, 0 };
+    static const char16_t dst_old[] = { 0xFEB1, 0x200B,0 };
+    static const char16_t dst_new[] = { 0xFEB1, 0xFE73,0 };
+    char16_t dst[3] = { 0x0000, 0x0000,0 };
     int32_t length;
     UErrorCode status;
 
@@ -207,7 +207,6 @@ void TestArabicShapeThreads::doTailTest(void) {
             return;
         }
     }
-    return;
 }
 	
 
@@ -304,7 +303,7 @@ static void formatErrorMessage(UErrorCode &realStatus, const UnicodeString& patt
     inCountry2.getDisplayCountry(theLocale,countryName2);
 
     Formattable myArgs[] = {
-        Formattable((int32_t)inStatus0),   // inStatus0      {0}
+        Formattable(static_cast<int32_t>(inStatus0)), // inStatus0      {0}
         Formattable(errString1), // statusString1 {1}
         Formattable(countryName2),  // inCountry2 {2}
         Formattable(currency3)// currency3  {3,number,currency}
@@ -337,24 +336,24 @@ class ThreadSafeFormatSharedData {
     UnicodeString  fBBDStr;
 };
 
-const ThreadSafeFormatSharedData *gSharedData = NULL;
+const ThreadSafeFormatSharedData *gSharedData = nullptr;
 
 ThreadSafeFormatSharedData::ThreadSafeFormatSharedData(UErrorCode &status) {
     fFormat.adoptInstead(NumberFormat::createCurrencyInstance(Locale::getUS(), status));
-    static const UChar *kYDD = u"YDD";
-    static const UChar *kBBD = u"BBD";
+    static const char16_t *kYDD = u"YDD";
+    static const char16_t *kBBD = u"BBD";
     fYDDThing.adoptObject(new CurrencyAmount(123.456, kYDD, status));
     fBBDThing.adoptObject(new CurrencyAmount(987.654, kBBD, status));
     if (U_FAILURE(status)) {
         return;
     }
-    fFormat->format(fYDDThing, fYDDStr, NULL, status);
-    fFormat->format(fBBDThing, fBBDStr, NULL, status);
+    fFormat->format(fYDDThing, fYDDStr, nullptr, status);
+    fFormat->format(fBBDThing, fBBDStr, nullptr, status);
     gSharedData = this;
 }
 
 ThreadSafeFormatSharedData::~ThreadSafeFormatSharedData() {
-    gSharedData = NULL;
+    gSharedData = nullptr;
 }
 
 /**
@@ -378,7 +377,7 @@ ThreadSafeFormat::ThreadSafeFormat(UErrorCode &status) {
   fFormat.adoptInstead(NumberFormat::createCurrencyInstance(Locale::getUS(), status));
 }
 
-static const UChar *kUSD = u"USD";
+static const char16_t *kUSD = u"USD";
 
 UBool ThreadSafeFormat::doStuff(int32_t offset, UnicodeString &appendErr, UErrorCode &status) const {
   UBool okay = true;
@@ -401,16 +400,16 @@ UBool ThreadSafeFormat::doStuff(int32_t offset, UnicodeString &appendErr, UError
     okay = false;
   }
   UnicodeString str;
-  const UnicodeString *o=NULL;
+  const UnicodeString *o=nullptr;
   Formattable f;
-  const NumberFormat *nf = NULL; // only operate on it as const.
+  const NumberFormat *nf = nullptr; // only operate on it as const.
   switch(offset%4) {
   case 0:  f = gSharedData->fYDDThing;  o = &gSharedData->fYDDStr;  nf = gSharedData->fFormat.getAlias();  break;
   case 1:  f = gSharedData->fBBDThing;  o = &gSharedData->fBBDStr;  nf = gSharedData->fFormat.getAlias();  break;
   case 2:  f = gSharedData->fYDDThing;  o = &gSharedData->fYDDStr;  nf = fFormat.getAlias();  break;
   case 3:  f = gSharedData->fBBDThing;  o = &gSharedData->fBBDStr;  nf = fFormat.getAlias();  break;
   }
-  nf->format(f, str, NULL, status);
+  nf->format(f, str, nullptr, status);
 
   if(*o != str) {
     appendErr.append(showDifference(*o, str));
@@ -423,7 +422,7 @@ UBool U_CALLCONV isAcceptable(void *, const char *, const char *, const UDataInf
     return true;
 }
 
-//static UMTX debugMutex = NULL;
+//static UMTX debugMutex = nullptr;
 //static UMTX gDebugMutex;
 
 
@@ -439,7 +438,7 @@ public:
         : SimpleThread(),
         fNum(0),
         fTraceInfo(0),
-        fTSF(NULL),
+        fTSF(nullptr),
         fOffset(0)
         // the locale to use
     {
@@ -478,8 +477,8 @@ public:
         int m;
         for (m=0; m<4000; m++) {
             status         = U_ZERO_ERROR;
-            UResourceBundle *res   = NULL;
-            const char *localeName = NULL;
+            UResourceBundle *res   = nullptr;
+            const char *localeName = nullptr;
 
             Locale  loc = Locale::getEnglish();
 
@@ -488,7 +487,7 @@ public:
 
             // ResourceBundle bund = ResourceBundle(0, loc, status);
             //umtx_lock(&gDebugMutex);
-            res = ures_open(NULL, localeName, &status);
+            res = ures_open(nullptr, localeName, &status);
             //umtx_unlock(&gDebugMutex);
 
             //umtx_lock(&gDebugMutex);
@@ -506,7 +505,7 @@ public:
         // Keep this data here to avoid static initialization.
         FormatThreadTestData kNumberFormatTestData[] =
         {
-            FormatThreadTestData((double)5.0, UnicodeString(u"5")),
+            FormatThreadTestData(5.0, UnicodeString(u"5")),
                 FormatThreadTestData( 6.0, UnicodeString(u"6")),
                 FormatThreadTestData( 20.0, UnicodeString(u"20")),
                 FormatThreadTestData( 8.0, UnicodeString(u"8")),
@@ -519,7 +518,7 @@ public:
         // Keep this data here to avoid static initialization.
         FormatThreadTestData kPercentFormatTestData[] =
         {
-            FormatThreadTestData((double)5.0, CharsToUnicodeString("500\\u00a0%")),
+            FormatThreadTestData(5.0, CharsToUnicodeString("500\\u00a0%")),
                 FormatThreadTestData( 1.0, CharsToUnicodeString("100\\u00a0%")),
                 FormatThreadTestData( 0.26, CharsToUnicodeString("26\\u00a0%")),
                 FormatThreadTestData(
@@ -707,7 +706,7 @@ void MultithreadTest::TestThreadedIntl()
 #define kCollatorThreadPatience kCollatorThreadThreads*30
 
 struct Line {
-    UChar buff[25];
+    char16_t buff[25];
     int32_t buflen;
 } ;
 
@@ -726,8 +725,8 @@ private:
     UBool isAtLeastUCA62;
 public:
     CollatorThreadTest()  : SimpleThread(),
-        coll(NULL),
-        lines(NULL),
+        coll(nullptr),
+        lines(nullptr),
         noLines(0),
         isAtLeastUCA62(true)
     {
@@ -741,7 +740,7 @@ public:
     }
     virtual void run() override {
         uint8_t sk1[1024], sk2[1024];
-        uint8_t *oldSk = NULL, *newSk = sk1;
+        uint8_t *oldSk = nullptr, *newSk = sk1;
         int32_t oldLen = 0;
         int32_t prev = 0;
         int32_t i = 0;
@@ -751,8 +750,8 @@ public:
 
             int32_t resLen = coll->getSortKey(lines[i].buff, lines[i].buflen, newSk, 1024);
 
-            if(oldSk != NULL) {
-                int32_t skres = strcmp((char *)oldSk, (char *)newSk);
+            if(oldSk != nullptr) {
+                int32_t skres = strcmp(reinterpret_cast<char*>(oldSk), reinterpret_cast<char*>(newSk));
                 int32_t cmpres = coll->compare(lines[prev].buff, lines[prev].buflen, lines[i].buff, lines[i].buflen);
                 int32_t cmpres2 = coll->compare(lines[i].buff, lines[i].buflen, lines[prev].buff, lines[prev].buflen);
 
@@ -795,7 +794,7 @@ void MultithreadTest::TestCollators()
 {
 
     UErrorCode status = U_ZERO_ERROR;
-    FILE *testFile = NULL;
+    FILE *testFile = nullptr;
     char testDataPath[1024];
     strcpy(testDataPath, IntlTest::getSourceTestData(status));
     if (U_FAILURE(status)) {
@@ -825,17 +824,17 @@ void MultithreadTest::TestCollators()
 
     testFile = fopen(buffer, "rb");
 
-    if(testFile == 0) {
+    if (testFile == nullptr) {
         strcpy(buffer+bufLen, "_SHORT");
         strcat(buffer, ext);
         testFile = fopen(buffer, "rb");
 
-        if(testFile == 0) {
+        if (testFile == nullptr) {
             strcpy(buffer+bufLen, "_STUB");
             strcat(buffer, ext);
             testFile = fopen(buffer, "rb");
 
-            if (testFile == 0) {
+            if (testFile == nullptr) {
                 *(buffer+bufLen) = 0;
                 dataerrln("could not open any of the conformance test files, tried opening base %s", buffer);
                 return;
@@ -849,14 +848,16 @@ void MultithreadTest::TestCollators()
         }
     }
 
-    LocalArray<Line> lines(new Line[200000]);
-    memset(lines.getAlias(), 0, sizeof(Line)*200000);
+    // UCA 16.0 CollationTest_CLDR_SHIFTED_SHORT.txt has over 225000 lines.
+    constexpr int32_t MAX_LINES_IN_COLLATION_TEST_FILE = 500000;
+    LocalArray<Line> lines(new Line[MAX_LINES_IN_COLLATION_TEST_FILE]);
+    memset(lines.getAlias(), 0, sizeof(Line)*MAX_LINES_IN_COLLATION_TEST_FILE);
     int32_t lineNum = 0;
 
-    UChar bufferU[1024];
+    char16_t bufferU[1024];
     uint32_t first = 0;
 
-    while (fgets(buffer, 1024, testFile) != NULL) {
+    while (fgets(buffer, 1024, testFile) != nullptr) {
         if(*buffer == 0 || buffer[0] == '#') {
             // Store empty and comment lines so that errors are reported
             // for the real test file lines.
@@ -970,7 +971,7 @@ public:
 
 };
 
-const UnicodeString *StringThreadTest2::gSharedString = NULL;
+const UnicodeString *StringThreadTest2::gSharedString = nullptr;
 
 // ** The actual test function.
 
@@ -998,7 +999,7 @@ void MultithreadTest::TestString()
     }
 
     delete StringThreadTest2::gSharedString;
-    StringThreadTest2::gSharedString = NULL;
+    StringThreadTest2::gSharedString = nullptr;
 }
 
 
@@ -1010,7 +1011,7 @@ void MultithreadTest::TestString()
 //
 
 #if !UCONFIG_NO_TRANSLITERATION
-Transliterator *gSharedTranslit = NULL;
+Transliterator *gSharedTranslit = nullptr;
 class TxThread: public SimpleThread {
   public:
     TxThread() {}
@@ -1043,7 +1044,7 @@ void MultithreadTest::TestAnyTranslit() {
     for (i=0; i<UPRV_LENGTHOF(threads); i++) {
         threads[i].join();
     }
-    gSharedTranslit = NULL;
+    gSharedTranslit = nullptr;
 #endif  // !UCONFIG_NO_TRANSLITERATION
 }
 
@@ -1072,7 +1073,7 @@ static const int32_t CACHE_LOAD = 3;
 class UCTMultiThreadItem : public SharedObject {
   public:
     char *value;
-    UCTMultiThreadItem(const char *x) : value(NULL) {
+    UCTMultiThreadItem(const char *x) : value(nullptr) {
         value = uprv_strdup(x);
     }
     virtual ~UCTMultiThreadItem() {
@@ -1088,11 +1089,11 @@ static std::condition_variable *gCTConditionVar = nullptr;
 template<> U_EXPORT
 const UCTMultiThreadItem *LocaleCacheKey<UCTMultiThreadItem>::createObject(
         const void *context, UErrorCode &status) const {
-    const UnifiedCache *cacheContext = (const UnifiedCache *) context;
+    const UnifiedCache *cacheContext = static_cast<const UnifiedCache*>(context);
 
     if (uprv_strcmp(fLoc.getLanguage(), fLoc.getName()) != 0) {
-        const UCTMultiThreadItem *result = NULL;
-        if (cacheContext == NULL) {
+        const UCTMultiThreadItem *result = nullptr;
+        if (cacheContext == nullptr) {
             UnifiedCache::getByLocale(fLoc.getLanguage(), result, status);
             return result;
         }
@@ -1120,7 +1121,7 @@ const UCTMultiThreadItem *LocaleCacheKey<UCTMultiThreadItem>::createObject(
 
     const UCTMultiThreadItem *result =
         new UCTMultiThreadItem(fLoc.getLanguage());
-    if (result == NULL) {
+    if (result == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
     } else {
         result->addRef();
@@ -1157,7 +1158,7 @@ class UnifiedCacheThread: public SimpleThread {
 
 void UnifiedCacheThread::exerciseByLocale(const Locale &locale) {
     UErrorCode status = U_ZERO_ERROR;
-    const UCTMultiThreadItem *origItem = NULL;
+    const UCTMultiThreadItem *origItem = nullptr;
     fCache->get(
             LocaleCacheKey<UCTMultiThreadItem>(locale), fCache, origItem, status);
     U_ASSERT(U_SUCCESS(status));
@@ -1166,11 +1167,11 @@ void UnifiedCacheThread::exerciseByLocale(const Locale &locale) {
     // Fetch the same item again many times. We should always get the same
     // pointer since this client is already holding onto it
     for (int32_t i = 0; i < 1000; ++i) {
-        const UCTMultiThreadItem *item = NULL;
+        const UCTMultiThreadItem *item = nullptr;
         fCache->get(
                 LocaleCacheKey<UCTMultiThreadItem>(locale), fCache, item, status);
         IntlTest::gTest->assertTrue(WHERE, item == origItem);
-        if (item != NULL) {
+        if (item != nullptr) {
             item->removeRef();
         }
     }
@@ -1299,8 +1300,8 @@ void MultithreadTest::TestBreakTranslit() {
     }
 
     delete gSharedTransliterator;
-    gTranslitInput = NULL;
-    gTranslitExpected = NULL;
+    gTranslitInput = nullptr;
+    gTranslitExpected = nullptr;
 }
 
 
