@@ -52,6 +52,10 @@ public:
     void TestPropertyNames();
     void TestIDSUnaryOperator();
     void TestIDCompatMath();
+    void TestPropertiesUsingPpucd();
+    void TestIDStatus();
+    void TestIDType();
+    void TestScriptExtensionsCodePoints();
 
 private:
 
