@@ -42,6 +42,7 @@ public:
     void TestNotSTD3();
     void TestInvalidPunycodeDigits();
     void TestACELabelEdgeCases();
+    void TestDefaultNontransitional();
     void TestTooLong();
     void TestSomeCases();
     void IdnaTest();
@@ -88,12 +89,15 @@ void UTS46Test::runIndexedTest(int32_t index, UBool exec, const char *&name, cha
     TESTCASE_AUTO(TestNotSTD3);
     TESTCASE_AUTO(TestInvalidPunycodeDigits);
     TESTCASE_AUTO(TestACELabelEdgeCases);
+    TESTCASE_AUTO(TestDefaultNontransitional);
     TESTCASE_AUTO(TestTooLong);
     TESTCASE_AUTO(TestSomeCases);
     TESTCASE_AUTO(IdnaTest);
     TESTCASE_AUTO_END;
 }
 
+namespace {
+
 const uint32_t severeErrors=
     UIDNA_ERROR_LEADING_COMBINING_MARK|
     UIDNA_ERROR_DISALLOWED|
@@ -101,7 +105,7 @@ const uint32_t severeErrors=
     UIDNA_ERROR_LABEL_HAS_DOT|
     UIDNA_ERROR_INVALID_ACE_LABEL;
 
-static UBool isASCII(const UnicodeString &str) {
+UBool isASCII(const UnicodeString &str) {
     const char16_t *s=str.getBuffer();
     int32_t length=str.length();
     for(int32_t i=0; i<length; ++i) {
@@ -125,6 +129,8 @@ public:
     UBool calledFlush;
 };
 
+}  // namespace
+
 void UTS46Test::TestAPI() {
     UErrorCode errorCode=U_ZERO_ERROR;
     UnicodeString result;
@@ -134,7 +140,7 @@ void UTS46Test::TestAPI() {
     trans->nameToASCII(input, result, info, errorCode);
     if(U_FAILURE(errorCode) || info.hasErrors() || result!=expected) {
         errln("T.nameToASCII(www.example.com) info.errors=%04lx result matches=%d %s",
-              (long)info.getErrors(), result==expected, u_errorName(errorCode));
+              static_cast<long>(info.getErrors()), result == expected, u_errorName(errorCode));
     }
     errorCode=U_USELESS_COLLATOR_ERROR;
     trans->nameToUnicode(input, result, info, errorCode);
@@ -182,7 +188,7 @@ void UTS46Test::TestAPI() {
               u_errorName(errorCode));
     }
 
-    static const char s[]={ 0x61, (char)0xc3, (char)0x9f };
+    static const char s[] = { 0x61, static_cast<char>(0xc3), static_cast<char>(0x9f) };
     sink.Reset();
     errorCode=U_USELESS_COLLATOR_ERROR;
     nontrans->nameToUnicodeUTF8(StringPiece(s, 3), sink, info, errorCode);
@@ -234,7 +240,7 @@ void UTS46Test::TestNotSTD3() {
     ) {
         prettify(result).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
         errln("notSTD3.nameToUnicode(non-LDH ASCII) unexpected errors %04lx string %s",
-              (long)info.getErrors(), buffer);
+              static_cast<long>(info.getErrors()), buffer);
     }
     // A space (BiDi class WS) is not allowed in a BiDi domain name.
     input=UNICODE_STRING_SIMPLE("a z.xn--4db.edu");
@@ -336,6 +342,39 @@ void UTS46Test::TestACELabelEdgeCases() {
         idna->labelToUnicode(u"Xn---", result, info, errorCode);
         assertTrue("empty Xn---", (info.getErrors()&UIDNA_ERROR_PUNYCODE)!=0);
     }
+
+    {
+        // Unicode 15.1 UTS #46:
+        // Added an additional condition in 4.1 Validity Criteria to
+        // disallow labels such as xn--xn---epa., which do not round-trip.
+        // --> Validity Criteria new criterion 4:
+        // If not CheckHyphens, the label must not begin with “xn--”.
+        IDNAInfo info;
+        idna->labelToUnicode("xn--xn---epa", result, info, errorCode);
+        assertTrue("error for xn--xn---epa",
+                (info.getErrors()&UIDNA_ERROR_INVALID_ACE_LABEL)!=0);
+    }
+}
+
+void UTS46Test::TestDefaultNontransitional() {
+    IcuTestErrorCode errorCode(*this, "TestDefaultNontransitional()");
+    // Unicode 15.1 UTS #46 deprecated transitional processing.
+    // ICU 76 changed UIDNA_DEFAULT to set the nontransitional options.
+    LocalPointer<IDNA> forZero(IDNA::createUTS46Instance(0, errorCode));
+    LocalPointer<IDNA> forDefault(IDNA::createUTS46Instance(UIDNA_DEFAULT, errorCode));
+    if(errorCode.isFailure()) {
+        return;
+    }
+    UnicodeString result;
+    IDNAInfo info;
+    forZero->labelToUnicode(u"Fⓤßẞ", result, info, errorCode);
+    assertEquals("forZero.toUnicode(Fⓤßẞ)", u"fussss", result);
+    forZero->labelToASCII(u"Fⓤßẞ", result, info, errorCode);
+    assertEquals("forZero.toASCII(Fⓤßẞ)", u"fussss", result);
+    forDefault->labelToUnicode(u"Fⓤßẞ", result, info, errorCode);
+    assertEquals("forDefault.toUnicode(Fⓤßẞ)", u"fußß", result);
+    forDefault->labelToASCII(u"Fⓤßẞ", result, info, errorCode);
+    assertEquals("forDefault.toASCII(Fⓤßẞ)", u"xn--fu-hiaa", result);
 }
 
 void UTS46Test::TestTooLong() {
@@ -352,6 +391,8 @@ void UTS46Test::TestTooLong() {
     assertEquals("decode: expected an error for too-long input", U_INPUT_TOO_LONG_ERROR, errorCode);
 }
 
+namespace {
+
 struct TestCase {
     // Input string and options string (Nontransitional/Transitional/Both).
     const char *s, *o;
@@ -360,7 +401,7 @@ struct TestCase {
     uint32_t errors;
 };
 
-static const TestCase testCases[]={
+const TestCase testCases[] = {
     { "www.eXample.cOm", "B",  // all ASCII
       "www.example.com", 0 },
     { "B\\u00FCcher.de", "B",  // u-umlaut
@@ -699,6 +740,8 @@ static const TestCase testCases[]={
     //   "", 0 },
 };
 
+}  // namespace
+
 void UTS46Test::TestSomeCases() {
     IcuTestErrorCode errorCode(*this, "TestSomeCases");
     char buffer[400], buffer2[400];
@@ -715,7 +758,7 @@ void UTS46Test::TestSomeCases() {
         nontrans->nameToASCII(input, aN, aNInfo, errorCode);
         nontrans->nameToUnicode(input, uN, uNInfo, errorCode);
         if(errorCode.errIfFailureAndReset("first-level processing [%d/%s] %s",
-                                          (int)i, testCase.o, testCase.s)
+                                          static_cast<int>(i), testCase.o, testCase.s)
         ) {
             continue;
         }
@@ -727,36 +770,36 @@ void UTS46Test::TestSomeCases() {
         if(mode=='B' || mode=='N') {
             if(uNInfo.getErrors()!=uniErrors) {
                 errln("N.nameToUnicode([%d] %s) unexpected errors %04lx",
-                      (int)i, testCase.s, (long)uNInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(uNInfo.getErrors()));
                 continue;
             }
             if(uN!=expected) {
                 prettify(uN).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 errln("N.nameToUnicode([%d] %s) unexpected string %s",
-                      (int)i, testCase.s, buffer);
+                      static_cast<int>(i), testCase.s, buffer);
                 continue;
             }
             if(aNInfo.getErrors()!=testCase.errors) {
                 errln("N.nameToASCII([%d] %s) unexpected errors %04lx",
-                      (int)i, testCase.s, (long)aNInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(aNInfo.getErrors()));
                 continue;
             }
         }
         if(mode=='B' || mode=='T') {
             if(uTInfo.getErrors()!=uniErrors) {
                 errln("T.nameToUnicode([%d] %s) unexpected errors %04lx",
-                      (int)i, testCase.s, (long)uTInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(uTInfo.getErrors()));
                 continue;
             }
             if(uT!=expected) {
                 prettify(uT).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 errln("T.nameToUnicode([%d] %s) unexpected string %s",
-                      (int)i, testCase.s, buffer);
+                      static_cast<int>(i), testCase.s, buffer);
                 continue;
             }
             if(aTInfo.getErrors()!=testCase.errors) {
                 errln("T.nameToASCII([%d] %s) unexpected errors %04lx",
-                      (int)i, testCase.s, (long)aTInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(aTInfo.getErrors()));
                 continue;
             }
         }
@@ -764,24 +807,24 @@ void UTS46Test::TestSomeCases() {
         if((aNInfo.getErrors()&severeErrors)==0 && !isASCII(aN)) {
             prettify(aN).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
             errln("N.nameToASCII([%d] %s) (errors %04lx) result is not ASCII %s",
-                  (int)i, testCase.s, aNInfo.getErrors(), buffer);
+                  static_cast<int>(i), testCase.s, aNInfo.getErrors(), buffer);
             continue;
         }
         if((aTInfo.getErrors()&severeErrors)==0 && !isASCII(aT)) {
             prettify(aT).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
             errln("T.nameToASCII([%d] %s) (errors %04lx) result is not ASCII %s",
-                  (int)i, testCase.s, aTInfo.getErrors(), buffer);
+                  static_cast<int>(i), testCase.s, aTInfo.getErrors(), buffer);
             continue;
         }
         if(verbose) {
             char m= mode=='B' ? mode : 'N';
             prettify(aN).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
             logln("%c.nameToASCII([%d] %s) (errors %04lx) result string: %s",
-                  m, (int)i, testCase.s, aNInfo.getErrors(), buffer);
+                  m, static_cast<int>(i), testCase.s, aNInfo.getErrors(), buffer);
             if(mode!='B') {
                 prettify(aT).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 logln("T.nameToASCII([%d] %s) (errors %04lx) result string: %s",
-                      (int)i, testCase.s, aTInfo.getErrors(), buffer);
+                      static_cast<int>(i), testCase.s, aTInfo.getErrors(), buffer);
             }
         }
         // second-level processing
@@ -792,7 +835,7 @@ void UTS46Test::TestSomeCases() {
         nontrans->nameToUnicode(aN, aNuN, aNuNInfo, errorCode);
         nontrans->nameToASCII(uN, uNaN, uNaNInfo, errorCode);
         if(errorCode.errIfFailureAndReset("second-level processing [%d/%s] %s",
-                                          (int)i, testCase.o, testCase.s)
+                                          static_cast<int>(i), testCase.o, testCase.s)
         ) {
             continue;
         }
@@ -801,7 +844,7 @@ void UTS46Test::TestSomeCases() {
             prettify(uNaN).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
             errln("N.nameToASCII([%d] %s)!=N.nameToUnicode().N.nameToASCII() "
                   "(errors %04lx) %s vs. %s",
-                  (int)i, testCase.s, aNInfo.getErrors(), buffer, buffer2);
+                  static_cast<int>(i), testCase.s, aNInfo.getErrors(), buffer, buffer2);
             continue;
         }
         if(aT!=uTaN) {
@@ -809,7 +852,7 @@ void UTS46Test::TestSomeCases() {
             prettify(uTaN).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
             errln("T.nameToASCII([%d] %s)!=T.nameToUnicode().N.nameToASCII() "
                   "(errors %04lx) %s vs. %s",
-                  (int)i, testCase.s, aNInfo.getErrors(), buffer, buffer2);
+                  static_cast<int>(i), testCase.s, aNInfo.getErrors(), buffer, buffer2);
             continue;
         }
         if(uN!=aNuN) {
@@ -817,7 +860,7 @@ void UTS46Test::TestSomeCases() {
             prettify(aNuN).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
             errln("N.nameToUnicode([%d] %s)!=N.nameToASCII().N.nameToUnicode() "
                   "(errors %04lx) %s vs. %s",
-                  (int)i, testCase.s, uNInfo.getErrors(), buffer, buffer2);
+                  static_cast<int>(i), testCase.s, uNInfo.getErrors(), buffer, buffer2);
             continue;
         }
         if(uT!=aTuN) {
@@ -825,7 +868,7 @@ void UTS46Test::TestSomeCases() {
             prettify(aTuN).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
             errln("T.nameToUnicode([%d] %s)!=T.nameToASCII().N.nameToUnicode() "
                   "(errors %04lx) %s vs. %s",
-                  (int)i, testCase.s, uNInfo.getErrors(), buffer, buffer2);
+                  static_cast<int>(i), testCase.s, uNInfo.getErrors(), buffer, buffer2);
             continue;
         }
         // labelToUnicode
@@ -836,71 +879,71 @@ void UTS46Test::TestSomeCases() {
         nontrans->labelToASCII(input, aNL, aNLInfo, errorCode);
         nontrans->labelToUnicode(input, uNL, uNLInfo, errorCode);
         if(errorCode.errIfFailureAndReset("labelToXYZ processing [%d/%s] %s",
-                                          (int)i, testCase.o, testCase.s)
+                                          static_cast<int>(i), testCase.o, testCase.s)
         ) {
             continue;
         }
-        if(aN.indexOf((char16_t)0x2e)<0) {
+        if (aN.indexOf(static_cast<char16_t>(0x2e)) < 0) {
             if(aN!=aNL || aNInfo.getErrors()!=aNLInfo.getErrors()) {
                 prettify(aN).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 prettify(aNL).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
                 errln("N.nameToASCII([%d] %s)!=N.labelToASCII() "
                       "(errors %04lx vs %04lx) %s vs. %s",
-                      (int)i, testCase.s, aNInfo.getErrors(), aNLInfo.getErrors(), buffer, buffer2);
+                      static_cast<int>(i), testCase.s, aNInfo.getErrors(), aNLInfo.getErrors(), buffer, buffer2);
                 continue;
             }
         } else {
             if((aNLInfo.getErrors()&UIDNA_ERROR_LABEL_HAS_DOT)==0) {
                 errln("N.labelToASCII([%d] %s) errors %04lx missing UIDNA_ERROR_LABEL_HAS_DOT",
-                      (int)i, testCase.s, (long)aNLInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(aNLInfo.getErrors()));
                 continue;
             }
         }
-        if(aT.indexOf((char16_t)0x2e)<0) {
+        if (aT.indexOf(static_cast<char16_t>(0x2e)) < 0) {
             if(aT!=aTL || aTInfo.getErrors()!=aTLInfo.getErrors()) {
                 prettify(aT).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 prettify(aTL).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
                 errln("T.nameToASCII([%d] %s)!=T.labelToASCII() "
                       "(errors %04lx vs %04lx) %s vs. %s",
-                      (int)i, testCase.s, aTInfo.getErrors(), aTLInfo.getErrors(), buffer, buffer2);
+                      static_cast<int>(i), testCase.s, aTInfo.getErrors(), aTLInfo.getErrors(), buffer, buffer2);
                 continue;
             }
         } else {
             if((aTLInfo.getErrors()&UIDNA_ERROR_LABEL_HAS_DOT)==0) {
                 errln("T.labelToASCII([%d] %s) errors %04lx missing UIDNA_ERROR_LABEL_HAS_DOT",
-                      (int)i, testCase.s, (long)aTLInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(aTLInfo.getErrors()));
                 continue;
             }
         }
-        if(uN.indexOf((char16_t)0x2e)<0) {
+        if (uN.indexOf(static_cast<char16_t>(0x2e)) < 0) {
             if(uN!=uNL || uNInfo.getErrors()!=uNLInfo.getErrors()) {
                 prettify(uN).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 prettify(uNL).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
                 errln("N.nameToUnicode([%d] %s)!=N.labelToUnicode() "
                       "(errors %04lx vs %04lx) %s vs. %s",
-                      (int)i, testCase.s, uNInfo.getErrors(), uNLInfo.getErrors(), buffer, buffer2);
+                      static_cast<int>(i), testCase.s, uNInfo.getErrors(), uNLInfo.getErrors(), buffer, buffer2);
                 continue;
             }
         } else {
             if((uNLInfo.getErrors()&UIDNA_ERROR_LABEL_HAS_DOT)==0) {
                 errln("N.labelToUnicode([%d] %s) errors %04lx missing UIDNA_ERROR_LABEL_HAS_DOT",
-                      (int)i, testCase.s, (long)uNLInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(uNLInfo.getErrors()));
                 continue;
             }
         }
-        if(uT.indexOf((char16_t)0x2e)<0) {
+        if (uT.indexOf(static_cast<char16_t>(0x2e)) < 0) {
             if(uT!=uTL || uTInfo.getErrors()!=uTLInfo.getErrors()) {
                 prettify(uT).extract(0, 0x7fffffff, buffer, UPRV_LENGTHOF(buffer));
                 prettify(uTL).extract(0, 0x7fffffff, buffer2, UPRV_LENGTHOF(buffer2));
                 errln("T.nameToUnicode([%d] %s)!=T.labelToUnicode() "
                       "(errors %04lx vs %04lx) %s vs. %s",
-                      (int)i, testCase.s, uTInfo.getErrors(), uTLInfo.getErrors(), buffer, buffer2);
+                      static_cast<int>(i), testCase.s, uTInfo.getErrors(), uTLInfo.getErrors(), buffer, buffer2);
                 continue;
             }
         } else {
             if((uTLInfo.getErrors()&UIDNA_ERROR_LABEL_HAS_DOT)==0) {
                 errln("T.labelToUnicode([%d] %s) errors %04lx missing UIDNA_ERROR_LABEL_HAS_DOT",
-                      (int)i, testCase.s, (long)uTLInfo.getErrors());
+                      static_cast<int>(i), testCase.s, static_cast<long>(uTLInfo.getErrors()));
                 continue;
             }
         }
@@ -915,7 +958,7 @@ void UTS46Test::TestSomeCases() {
                 uNLInfo.isTransitionalDifferent() ||
                 uTLInfo.isTransitionalDifferent()
             ) {
-                errln("B.process([%d] %s) isTransitionalDifferent()", (int)i, testCase.s);
+                errln("B.process([%d] %s) isTransitionalDifferent()", static_cast<int>(i), testCase.s);
                 continue;
             }
             if( aN!=aT || uN!=uT || aNL!=aTL || uNL!=uTL ||
@@ -923,7 +966,7 @@ void UTS46Test::TestSomeCases() {
                 aNLInfo.getErrors()!=aTLInfo.getErrors() || uNLInfo.getErrors()!=uTLInfo.getErrors()
             ) {
                 errln("N.process([%d] %s) vs. T.process() different errors or result strings",
-                      (int)i, testCase.s);
+                      static_cast<int>(i), testCase.s);
                 continue;
             }
         } else {
@@ -937,12 +980,12 @@ void UTS46Test::TestSomeCases() {
                 !uTLInfo.isTransitionalDifferent()
             ) {
                 errln("%s.process([%d] %s) !isTransitionalDifferent()",
-                      testCase.o, (int)i, testCase.s);
+                      testCase.o, static_cast<int>(i), testCase.s);
                 continue;
             }
             if(aN==aT || uN==uT || aNL==aTL || uNL==uTL) {
                 errln("N.process([%d] %s) vs. T.process() same result strings",
-                      (int)i, testCase.s);
+                      static_cast<int>(i), testCase.s);
                 continue;
             }
         }
@@ -956,7 +999,7 @@ void UTS46Test::TestSomeCases() {
         nontrans->nameToASCII_UTF8(input8, aN8Sink, aN8Info, errorCode);
         nontrans->nameToUnicodeUTF8(input8, uN8Sink, uN8Info, errorCode);
         if(errorCode.errIfFailureAndReset("UTF-8 processing [%d/%s] %s",
-                                          (int)i, testCase.o, testCase.s)
+                                          static_cast<int>(i), testCase.o, testCase.s)
         ) {
             continue;
         }
@@ -968,21 +1011,21 @@ void UTS46Test::TestSomeCases() {
             uN8Info.getErrors()!=uNInfo.getErrors()
         ) {
             errln("N.xyzUTF8([%d] %s) vs. UTF-16 processing different errors %04lx vs. %04lx",
-                  (int)i, testCase.s,
-                  (long)aN8Info.getErrors(), (long)aNInfo.getErrors());
+                  static_cast<int>(i), testCase.s,
+                  static_cast<long>(aN8Info.getErrors()), static_cast<long>(aNInfo.getErrors()));
             continue;
         }
         if( aT8Info.getErrors()!=aTInfo.getErrors() ||
             uT8Info.getErrors()!=uTInfo.getErrors()
         ) {
             errln("T.xyzUTF8([%d] %s) vs. UTF-16 processing different errors %04lx vs. %04lx",
-                  (int)i, testCase.s,
-                  (long)aT8Info.getErrors(), (long)aTInfo.getErrors());
+                  static_cast<int>(i), testCase.s,
+                  static_cast<long>(aT8Info.getErrors()), static_cast<long>(aTInfo.getErrors()));
             continue;
         }
         if(aT16!=aT || uT16!=uT || aN16!=aN || uN16!=uN) {
             errln("%s.xyzUTF8([%d] %s) vs. UTF-16 processing different string results",
-                  testCase.o, (int)i, testCase.s, (long)aTInfo.getErrors());
+                  testCase.o, static_cast<int>(i), testCase.s, static_cast<long>(aTInfo.getErrors()));
             continue;
         }
         if( aT8Info.isTransitionalDifferent()!=aTInfo.isTransitionalDifferent() ||
@@ -991,7 +1034,7 @@ void UTS46Test::TestSomeCases() {
             uN8Info.isTransitionalDifferent()!=uNInfo.isTransitionalDifferent()
         ) {
             errln("%s.xyzUTF8([%d] %s) vs. UTF-16 processing different isTransitionalDifferent()",
-                  testCase.o, (int)i, testCase.s);
+                  testCase.o, static_cast<int>(i), testCase.s);
             continue;
         }
     }
@@ -1008,16 +1051,22 @@ idnaTestLineFn(void *context,
     reinterpret_cast<UTS46Test *>(context)->idnaTestOneLine(fields, *pErrorCode);
 }
 
-UnicodeString s16FromField(char *(&field)[2]) {
-    int32_t length = (int32_t)(field[1] - field[0]);
-    return UnicodeString::fromUTF8(StringPiece(field[0], length)).trim().unescape();
+UnicodeString s16FromField(char *(&field)[2], const UnicodeString &sameAs) {
+    int32_t length = static_cast<int32_t>(field[1] - field[0]);
+    UnicodeString s = UnicodeString::fromUTF8(StringPiece(field[0], length)).trim().unescape();
+    if (s.isEmpty()) {
+        s = sameAs;  // blank means same as another string
+    } else if (s == u"\"\"") {
+        s.remove();  // explicit empty string (new in Unicode 16)
+    }
+    return s;
 }
 
 std::string statusFromField(char *(&field)[2]) {
     const char *start = u_skipWhitespace(field[0]);
     std::string status;
     if (start != field[1]) {
-        int32_t length = (int32_t)(field[1] - start);
+        int32_t length = static_cast<int32_t>(field[1] - start);
         while (length > 0 && (start[length - 1] == u' ' || start[length - 1] == u'\t')) {
             --length;
         }
@@ -1041,10 +1090,24 @@ void UTS46Test::checkIdnaTestResult(const char *line, const char *type,
         if (strcmp(status, reinterpret_cast<const char*>(u8"[]")) != 0) {
             expectedHasErrors = true;
         }
+        // ICU workaround:
+        // We do effectively VerifyDnsLength (we always check for lengths), except,
+        // based on past bug reports, we do not do the following in UTS #46 ToASCII:
+        // When VerifyDnsLength is true, the empty root label is disallowed.
+        // Ignore the expected error if it is the only one.
+        // TODO: ICU-22882 - Report the empty root label separately from empty non-root labels.
+        if (strncmp(type, "toASCII", 7) == 0 &&  // startsWith
+                strcmp(status, "[A4_2]") == 0 && !info.hasErrors()) {
+            if (result.endsWith(UnicodeString::readOnlyAlias(u".")) &&
+                    // !contains
+                    result.indexOf(UnicodeString::readOnlyAlias(u"..")) < 0) {
+                expectedHasErrors = false;
+            }
+        }
     }
     if (expectedHasErrors != info.hasErrors()) {
         errln("%s  expected errors %s %d != %d = actual has errors: %04lx\n    %s",
-              type, status, expectedHasErrors, info.hasErrors(), (long)info.getErrors(), line);
+              type, status, expectedHasErrors, info.hasErrors(), static_cast<long>(info.getErrors()), line);
     }
     if (!expectedHasErrors && expected != result) {
         errln("%s  expected != actual\n    %s", type, line);
@@ -1056,16 +1119,15 @@ void UTS46Test::checkIdnaTestResult(const char *line, const char *type,
 void UTS46Test::idnaTestOneLine(char *fields[][2], UErrorCode &errorCode) {
     // IdnaTestV2.txt (since Unicode 11)
     // Column 1: source
-    // The source string to be tested
-    UnicodeString source = s16FromField(fields[0]);
+    // The source string to be tested.
+    // "" means the empty string.
+    UnicodeString source = s16FromField(fields[0], UnicodeString());
 
     // Column 2: toUnicode
     // The result of applying toUnicode to the source, with Transitional_Processing=false.
     // A blank value means the same as the source value.
-    UnicodeString toUnicode = s16FromField(fields[1]);
-    if (toUnicode.isEmpty()) {
-        toUnicode = source;
-    }
+    // "" means the empty string.
+    UnicodeString toUnicode = s16FromField(fields[1], source);
 
     // Column 3: toUnicodeStatus
     // A set of status codes, each corresponding to a particular test.
@@ -1075,10 +1137,8 @@ void UTS46Test::idnaTestOneLine(char *fields[][2], UErrorCode &errorCode) {
     // Column 4: toAsciiN
     // The result of applying toASCII to the source, with Transitional_Processing=false.
     // A blank value means the same as the toUnicode value.
-    UnicodeString toAsciiN = s16FromField(fields[3]);
-    if (toAsciiN.isEmpty()) {
-        toAsciiN = toUnicode;
-    }
+    // "" means the empty string.
+    UnicodeString toAsciiN = s16FromField(fields[3], toUnicode);
 
     // Column 5: toAsciiNStatus
     // A set of status codes, each corresponding to a particular test.
@@ -1091,10 +1151,8 @@ void UTS46Test::idnaTestOneLine(char *fields[][2], UErrorCode &errorCode) {
     // Column 6: toAsciiT
     // The result of applying toASCII to the source, with Transitional_Processing=true.
     // A blank value means the same as the toAsciiN value.
-    UnicodeString toAsciiT = s16FromField(fields[5]);
-    if (toAsciiT.isEmpty()) {
-        toAsciiT = toAsciiN;
-    }
+    // "" means the empty string.
+    UnicodeString toAsciiT = s16FromField(fields[5], toAsciiN);
 
     // Column 7: toAsciiTStatus
     // A set of status codes, each corresponding to a particular test.
@@ -1118,10 +1176,13 @@ void UTS46Test::idnaTestOneLine(char *fields[][2], UErrorCode &errorCode) {
                         toAsciiTStatus.c_str(), aTInfo);
 }
 
+namespace {
+
 // TODO: de-duplicate
 U_DEFINE_LOCAL_OPEN_POINTER(LocalStdioFilePointer, FILE, fclose);
 
-// http://www.unicode.org/Public/idna/latest/IdnaTest.txt
+}  // namespace
+
 void UTS46Test::IdnaTest() {
     IcuTestErrorCode errorCode(*this, "IdnaTest");
     const char *sourceTestDataPath = getSourceTestData(errorCode);
@@ -1142,7 +1203,7 @@ void UTS46Test::IdnaTest() {
     // Comments are indicated with hash marks.
     char *fields[kNumFields][2];
     u_parseDelimitedFile(path.data(), ';', fields, kNumFields, idnaTestLineFn, this, errorCode);
-    if (errorCode.errIfFailureAndReset("error parsing IdnaTest.txt")) {
+    if (errorCode.errIfFailureAndReset("error parsing IdnaTestV2.txt")) {
         return;
     }
 }
