@@ -1,10 +1,12 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 2005-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /************************************************************************
-*   Tests for the UText and UTextIterator text abstraction classses
+*   Tests for the UText and UTextIterator text abstraction classes
 *
 ************************************************************************/
 
@@ -27,7 +29,7 @@ public:
     UTextTest();
     virtual ~UTextTest();
 
-    void runIndexedTest(int32_t index, UBool exec, const char* &name, char* par=NULL);
+    void runIndexedTest(int32_t index, UBool exec, const char* &name, char* par=nullptr) override;
     void TextTest();
     void ErrorTest();
     void FreezeTest();
@@ -36,6 +38,8 @@ public:
     void Ticket10562();
     void Ticket10983();
     void Ticket12130();
+    void Ticket13344();
+    void AccessChangesChunkSize();
 
 private:
     struct m {                              // Map between native indices & code points.
