@@ -82,7 +82,7 @@ UVectorTest_compareInt32(UElement key1, UElement key2) {
 }
 
 U_CDECL_BEGIN
-static int8_t U_CALLCONV
+static UBool U_CALLCONV
 UVectorTest_compareCstrings(const UElement key1, const UElement key2) {
     return !strcmp((const char *)key1.pointer, (const char *)key2.pointer);
 }
@@ -109,10 +109,10 @@ void UVectorTest::UVector_API() {
 
     status = U_ZERO_ERROR;
     a = new UVector(status);
-    a->sortedInsert((int32_t)10, UVectorTest_compareInt32, status);
-    a->sortedInsert((int32_t)20, UVectorTest_compareInt32, status);
-    a->sortedInsert((int32_t)30, UVectorTest_compareInt32, status);
-    a->sortedInsert((int32_t)15, UVectorTest_compareInt32, status);
+    a->sortedInsert(static_cast<int32_t>(10), UVectorTest_compareInt32, status);
+    a->sortedInsert(static_cast<int32_t>(20), UVectorTest_compareInt32, status);
+    a->sortedInsert(static_cast<int32_t>(30), UVectorTest_compareInt32, status);
+    a->sortedInsert(static_cast<int32_t>(15), UVectorTest_compareInt32, status);
     TEST_CHECK_STATUS(status);
     TEST_ASSERT(a->elementAti(0) == 10);
     TEST_ASSERT(a->elementAti(1) == 15);
