@@ -59,8 +59,8 @@ static UDataInfo dataInfo={
     0,
 
     { 0x4e, 0x72, 0x6d, 0x32 }, /* dataFormat="Nrm2" */
-    { 4, 0, 0, 0 },             /* formatVersion */
-    { 11, 0, 0, 0 }             /* dataVersion (Unicode version) */
+    { 5, 0, 0, 0 },             /* formatVersion */
+    { 16, 0, 0, 0 }             /* dataVersion (Unicode version) */
 };
 
 U_NAMESPACE_BEGIN
@@ -76,7 +76,7 @@ public:
         if(rangeIndex<UPRV_LENGTHOF(ranges)) {
             return ranges+rangeIndex++;
         } else {
-            return NULL;
+            return nullptr;
         }
     }
 private:
@@ -123,7 +123,7 @@ Normalizer2DataBuilder::setUnicodeVersion(const char *v) {
 }
 
 Norm *Normalizer2DataBuilder::checkNormForMapping(Norm *p, UChar32 c) {
-    if(p!=NULL) {
+    if(p!=nullptr) {
         if(p->mappingType!=Norm::NONE) {
             if( overrideHandling==OVERRIDE_NONE ||
                 (overrideHandling==OVERRIDE_PREVIOUS && p->mappingPhase==phase)
@@ -131,11 +131,11 @@ Norm *Normalizer2DataBuilder::checkNormForMapping(Norm *p, UChar32 c) {
                 fprintf(stderr,
                         "error in gennorm2 phase %d: "
                         "not permitted to override mapping for U+%04lX from phase %d\n",
-                        (int)phase, (long)c, (int)p->mappingPhase);
+                        static_cast<int>(phase), static_cast<long>(c), static_cast<int>(p->mappingPhase));
                 exit(U_INVALID_FORMAT_ERROR);
             }
             delete p->mapping;
-            p->mapping=NULL;
+            p->mapping=nullptr;
         }
         p->mappingPhase=phase;
     }
@@ -154,7 +154,7 @@ void Normalizer2DataBuilder::setCC(UChar32 c, uint8_t cc) {
 
 static UBool isWellFormed(const UnicodeString &s) {
     UErrorCode errorCode=U_ZERO_ERROR;
-    u_strToUTF8(NULL, 0, NULL, toUCharPtr(s.getBuffer()), s.length(), &errorCode);
+    u_strToUTF8(nullptr, 0, nullptr, toUCharPtr(s.getBuffer()), s.length(), &errorCode);
     return U_SUCCESS(errorCode) || errorCode==U_BUFFER_OVERFLOW_ERROR;
 }
 
@@ -163,7 +163,7 @@ void Normalizer2DataBuilder::setOneWayMapping(UChar32 c, const UnicodeString &m)
         fprintf(stderr,
                 "error in gennorm2 phase %d: "
                 "illegal one-way mapping from U+%04lX to malformed string\n",
-                (int)phase, (long)c);
+                static_cast<int>(phase), static_cast<long>(c));
         exit(U_INVALID_FORMAT_ERROR);
     }
     Norm *p=checkNormForMapping(norms.createNorm(c), c);
@@ -178,14 +178,14 @@ void Normalizer2DataBuilder::setRoundTripMapping(UChar32 c, const UnicodeString
         fprintf(stderr,
                 "error in gennorm2 phase %d: "
                 "illegal round-trip mapping from surrogate code point U+%04lX\n",
-                (int)phase, (long)c);
+                static_cast<int>(phase), static_cast<long>(c));
         exit(U_INVALID_FORMAT_ERROR);
     }
     if(!isWellFormed(m)) {
         fprintf(stderr,
                 "error in gennorm2 phase %d: "
                 "illegal round-trip mapping from U+%04lX to malformed string\n",
-                (int)phase, (long)c);
+                static_cast<int>(phase), static_cast<long>(c));
         exit(U_INVALID_FORMAT_ERROR);
     }
     int32_t numCP=u_countChar32(toUCharPtr(m.getBuffer()), m.length());
@@ -193,7 +193,7 @@ void Normalizer2DataBuilder::setRoundTripMapping(UChar32 c, const UnicodeString
         fprintf(stderr,
                 "error in gennorm2 phase %d: "
                 "illegal round-trip mapping from U+%04lX to %d!=2 code points\n",
-                (int)phase, (long)c, (int)numCP);
+                static_cast<int>(phase), static_cast<long>(c), static_cast<int>(numCP));
         exit(U_INVALID_FORMAT_ERROR);
     }
     Norm *p=checkNormForMapping(norms.createNorm(c), c);
@@ -254,7 +254,7 @@ UBool Normalizer2DataBuilder::mappingHasCompBoundaryAfter(const BuilderReorderin
     // characters following this mapping are possible.
     const Norm *starterNorm=norms.getNorm(starter);
     if(i==lastStarterIndex &&
-            (starterNorm==nullptr || starterNorm->compositions==nullptr)) {
+            (starterNorm==nullptr || !starterNorm->combinesFwd())) {
         return true;  // The last starter does not combine forward.
     }
     uint8_t prevCC=0;
@@ -270,14 +270,14 @@ UBool Normalizer2DataBuilder::mappingHasCompBoundaryAfter(const BuilderReorderin
             // The starter combines with c into a composite replacement starter.
             starterNorm=norms.getNorm(starter);
             if(i>=lastStarterIndex &&
-                    (starterNorm==nullptr || starterNorm->compositions==nullptr)) {
+                    (starterNorm==nullptr || !starterNorm->combinesFwd())) {
                 return true;  // The composite does not combine further.
             }
             // Keep prevCC because we "removed" the combining mark.
         } else if(cc==0) {
             starterNorm=norms.getNorm(c);
             if(i==lastStarterIndex &&
-                    (starterNorm==nullptr || starterNorm->compositions==nullptr)) {
+                    (starterNorm==nullptr || !starterNorm->combinesFwd())) {
                 return true;  // The new starter does not combine forward.
             }
             prevCC=0;
@@ -354,19 +354,38 @@ void Normalizer2DataBuilder::postProcess(Norm &norm) {
 
         norm.hasCompBoundaryBefore=
             !buffer.isEmpty() && norm.leadCC==0 && !norms.combinesBack(buffer.charAt(0));
+        // No comp-boundary-after when norm.combinesBack:
+        // MaybeNo character whose first mapping character may combine-back,
+        // in which case we would not recompose to this character,
+        // and may need more context.
         norm.hasCompBoundaryAfter=
-            norm.compositions==nullptr && mappingHasCompBoundaryAfter(buffer, norm.mappingType);
+            !norm.combinesBack && !norm.combinesFwd() &&
+            mappingHasCompBoundaryAfter(buffer, norm.mappingType);
 
         if(norm.combinesBack) {
-            norm.error="combines-back and decomposes, not possible in Unicode normalization";
+            if(norm.mappingType!=Norm::ROUND_TRIP) {
+                // One-way mappings don't get NFC_QC=Maybe, and
+                // should not have gotten combinesBack set.
+                norm.error="combines-back and has a one-way mapping, "
+                           "not possible in Unicode normalization";
+            } else if(norm.combinesFwd()) {
+                // Earlier code checked ccc=0.
+                norm.type=Norm::MAYBE_NO_COMBINES_FWD;
+            } else if(norm.cc==0) {
+                norm.type=Norm::MAYBE_NO_MAPPING_ONLY;
+            } else {
+                norm.error="combines-back and decomposes with ccc!=0, "
+                           "not possible in Unicode normalization";
+                // ... because we don't reorder again after composition.
+            }
         } else if(norm.mappingType==Norm::ROUND_TRIP) {
-            if(norm.compositions!=NULL) {
+            if(norm.combinesFwd()) {
                 norm.type=Norm::YES_NO_COMBINES_FWD;
             } else {
                 norm.type=Norm::YES_NO_MAPPING_ONLY;
             }
         } else {  // one-way mapping
-            if(norm.compositions!=NULL) {
+            if(norm.combinesFwd()) {
                 norm.error="combines-forward and has a one-way mapping, "
                            "not possible in Unicode normalization";
             } else if(buffer.isEmpty()) {
@@ -386,16 +405,16 @@ void Normalizer2DataBuilder::postProcess(Norm &norm) {
         norm.hasCompBoundaryBefore=
             norm.cc==0 && !norm.combinesBack;
         norm.hasCompBoundaryAfter=
-            norm.cc==0 && !norm.combinesBack && norm.compositions==nullptr;
+            norm.cc==0 && !norm.combinesBack && !norm.combinesFwd();
 
         if(norm.combinesBack) {
-            if(norm.compositions!=nullptr) {
+            if(norm.combinesFwd()) {
                 // Earlier code checked ccc=0.
                 norm.type=Norm::MAYBE_YES_COMBINES_FWD;
             } else {
                 norm.type=Norm::MAYBE_YES_SIMPLE;  // any ccc
             }
-        } else if(norm.compositions!=nullptr) {
+        } else if(norm.combinesFwd()) {
             // Earlier code checked ccc=0.
             norm.type=Norm::YES_YES_COMBINES_FWD;
         } else if(norm.cc!=0) {
@@ -410,7 +429,7 @@ class Norm16Writer : public Norms::Enumerator {
 public:
     Norm16Writer(UMutableCPTrie *trie, Norms &n, Normalizer2DataBuilder &b) :
             Norms::Enumerator(n), builder(b), norm16Trie(trie) {}
-    void rangeHandler(UChar32 start, UChar32 end, Norm &norm) U_OVERRIDE {
+    void rangeHandler(UChar32 start, UChar32 end, Norm &norm) override {
         builder.writeNorm16(norm16Trie, start, end, norm);
     }
     Normalizer2DataBuilder &builder;
@@ -419,7 +438,7 @@ public:
 
 void Normalizer2DataBuilder::setSmallFCD(UChar32 c) {
     UChar32 lead= c<=0xffff ? c : U16_LEAD(c);
-    smallFCD[lead>>8]|=(uint8_t)1<<((lead>>5)&7);
+    smallFCD[lead >> 8] |= static_cast<uint8_t>(1) << ((lead >> 5) & 7);
 }
 
 void Normalizer2DataBuilder::writeNorm16(UMutableCPTrie *norm16Trie, UChar32 start, UChar32 end, Norm &norm) {
@@ -469,6 +488,12 @@ void Normalizer2DataBuilder::writeNorm16(UMutableCPTrie *norm16Trie, UChar32 sta
             norm16=getMinNoNoDelta()+offset;
             break;
         }
+    case Norm::MAYBE_NO_MAPPING_ONLY:
+        norm16=indexes[Normalizer2Impl::IX_MIN_MAYBE_NO]+norm.offset*2;
+        break;
+    case Norm::MAYBE_NO_COMBINES_FWD:
+        norm16=indexes[Normalizer2Impl::IX_MIN_MAYBE_NO_COMBINES_FWD]+norm.offset*2;
+        break;
     case Norm::MAYBE_YES_COMBINES_FWD:
         norm16=indexes[Normalizer2Impl::IX_MIN_MAYBE_YES]+norm.offset*2;
         break;
@@ -487,7 +512,7 @@ void Normalizer2DataBuilder::writeNorm16(UMutableCPTrie *norm16Trie, UChar32 sta
         norm16|=Normalizer2Impl::HAS_COMP_BOUNDARY_AFTER;
     }
     IcuToolErrorCode errorCode("gennorm2/writeNorm16()");
-    umutablecptrie_setRange(norm16Trie, start, end, (uint32_t)norm16, errorCode);
+    umutablecptrie_setRange(norm16Trie, start, end, static_cast<uint32_t>(norm16), errorCode);
 
     // Set the minimum code points for real data lookups in the quick check loops.
     UBool isDecompNo=
@@ -509,13 +534,13 @@ void Normalizer2DataBuilder::setHangulData(UMutableCPTrie *norm16Trie) {
     HangulIterator hi;
     const HangulIterator::Range *range;
     // Check that none of the Hangul/Jamo code points have data.
-    while((range=hi.nextRange())!=NULL) {
+    while((range=hi.nextRange())!=nullptr) {
         for(UChar32 c=range->start; c<=range->end; ++c) {
             if(umutablecptrie_get(norm16Trie, c)>Normalizer2Impl::INERT) {
                 fprintf(stderr,
                         "gennorm2 error: "
                         "illegal mapping/composition/ccc data for Hangul or Jamo U+%04lX\n",
-                        (long)c);
+                        static_cast<long>(c));
                 exit(U_INVALID_FORMAT_ERROR);
             }
         }
@@ -593,19 +618,28 @@ LocalUCPTriePointer Normalizer2DataBuilder::processData() {
     extraData.append(extra.noNoMappingsEmpty);
     indexes[Normalizer2Impl::IX_LIMIT_NO_NO]=extraData.length()*2;
 
-    // Pad the maybeYesCompositions length to a multiple of 4,
+    int32_t maybeDataLength=
+        extra.maybeNoMappingsOnly.length()+
+        extra.maybeNoMappingsAndCompositions.length()+
+        extra.maybeYesCompositions.length();
+    int32_t minMaybeNo=Normalizer2Impl::MIN_NORMAL_MAYBE_YES-maybeDataLength*2;
+    // Adjust minMaybeNo down to 8-align it,
     // so that NO_NO_DELTA bits 2..1 can be used without subtracting the center.
-    while(extra.maybeYesCompositions.length()&3) {
-        extra.maybeYesCompositions.append((UChar)0);
-    }
-    extraData.insert(0, extra.maybeYesCompositions);
-    indexes[Normalizer2Impl::IX_MIN_MAYBE_YES]=
-        Normalizer2Impl::MIN_NORMAL_MAYBE_YES-
-        extra.maybeYesCompositions.length()*2;
+    minMaybeNo&=~7;
 
-    // Pad to even length for 4-byte alignment of following data.
+    int32_t index=minMaybeNo;
+    indexes[Normalizer2Impl::IX_MIN_MAYBE_NO]=index;
+    extraData.append(extra.maybeNoMappingsOnly);
+    index+=extra.maybeNoMappingsOnly.length()*2;
+    indexes[Normalizer2Impl::IX_MIN_MAYBE_NO_COMBINES_FWD]=index;
+    extraData.append(extra.maybeNoMappingsAndCompositions);
+    index+=extra.maybeNoMappingsAndCompositions.length()*2;
+    indexes[Normalizer2Impl::IX_MIN_MAYBE_YES]=index;
+    extraData.append(extra.maybeYesCompositions);
+
+    // Pad the extraData to even length for 4-byte alignment of following data.
     if(extraData.length()&1) {
-        extraData.append((UChar)0);
+        extraData.append(static_cast<char16_t>(0));
     }
 
     int32_t minNoNoDelta=getMinNoNoDelta();
@@ -655,7 +689,7 @@ LocalUCPTriePointer Normalizer2DataBuilder::processData() {
     if (value != Normalizer2Impl::INERT || end < 0xdfff) {
         fprintf(stderr,
                 "gennorm2 error: not all surrogate code points are inert: U+d800..U+%04x=%lx\n",
-                (int)end, (long)value);
+                static_cast<int>(end), static_cast<long>(value));
         exit(U_INTERNAL_PROGRAM_ERROR);
     }
     uint32_t maxNorm16 = 0;
@@ -682,11 +716,11 @@ LocalUCPTriePointer Normalizer2DataBuilder::processData() {
         UChar32 leadEnd = start | 0x3ff;
         if (leadEnd <= end) {
             // End of the supplementary block for a lead surrogate.
-            if (maxNorm16 >= (uint32_t)indexes[Normalizer2Impl::IX_LIMIT_NO_NO]) {
+            if (maxNorm16 >= static_cast<uint32_t>(indexes[Normalizer2Impl::IX_LIMIT_NO_NO])) {
                 // Set noNo ("worst" value) if it got into "less-bad" maybeYes or ccc!=0.
                 // Otherwise it might end up at something like JAMO_VT which stays in
                 // the inner decomposition quick check loop.
-                maxNorm16 = (uint32_t)indexes[Normalizer2Impl::IX_LIMIT_NO_NO];
+                maxNorm16 = static_cast<uint32_t>(indexes[Normalizer2Impl::IX_LIMIT_NO_NO]);
             }
             maxNorm16 =
                 (maxNorm16 & ~Normalizer2Impl::HAS_COMP_BOUNDARY_AFTER)|
@@ -736,7 +770,7 @@ LocalUCPTriePointer Normalizer2DataBuilder::processData() {
     ucptrie_toBinary(builtTrie.getAlias(), norm16TrieBytes, norm16TrieLength, errorCode);
     errorCode.assertSuccess();
 
-    int32_t offset=(int32_t)sizeof(indexes);
+    int32_t offset = static_cast<int32_t>(sizeof(indexes));
     indexes[Normalizer2Impl::IX_NORM_TRIE_OFFSET]=offset;
     offset+=norm16TrieLength;
     indexes[Normalizer2Impl::IX_EXTRA_DATA_OFFSET]=offset;
@@ -749,22 +783,38 @@ LocalUCPTriePointer Normalizer2DataBuilder::processData() {
     }
 
     if(beVerbose) {
-        printf("size of normalization trie:         %5ld bytes\n", (long)norm16TrieLength);
-        printf("size of 16-bit extra data:          %5ld uint16_t\n", (long)extraData.length());
-        printf("size of small-FCD data:             %5ld bytes\n", (long)sizeof(smallFCD));
-        printf("size of binary data file contents:  %5ld bytes\n", (long)totalSize);
-        printf("minDecompNoCodePoint:              U+%04lX\n", (long)indexes[Normalizer2Impl::IX_MIN_DECOMP_NO_CP]);
-        printf("minCompNoMaybeCodePoint:           U+%04lX\n", (long)indexes[Normalizer2Impl::IX_MIN_COMP_NO_MAYBE_CP]);
-        printf("minLcccCodePoint:                  U+%04lX\n", (long)indexes[Normalizer2Impl::IX_MIN_LCCC_CP]);
-        printf("minYesNo: (with compositions)      0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_YES_NO]);
-        printf("minYesNoMappingsOnly:              0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_YES_NO_MAPPINGS_ONLY]);
-        printf("minNoNo: (comp-normalized)         0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_NO_NO]);
-        printf("minNoNoCompBoundaryBefore:         0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_NO_NO_COMP_BOUNDARY_BEFORE]);
-        printf("minNoNoCompNoMaybeCC:              0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_NO_NO_COMP_NO_MAYBE_CC]);
-        printf("minNoNoEmpty:                      0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_NO_NO_EMPTY]);
-        printf("limitNoNo:                         0x%04x\n", (int)indexes[Normalizer2Impl::IX_LIMIT_NO_NO]);
-        printf("minNoNoDelta:                      0x%04x\n", (int)minNoNoDelta);
-        printf("minMaybeYes:                       0x%04x\n", (int)indexes[Normalizer2Impl::IX_MIN_MAYBE_YES]);
+        printf("size of normalization trie:         %5ld bytes\n", static_cast<long>(norm16TrieLength));
+        printf("size of 16-bit extra data:          %5ld uint16_t\n", static_cast<long>(extraData.length()));
+        printf("size of small-FCD data:             %5ld bytes\n", static_cast<long>(sizeof(smallFCD)));
+        printf("size of binary data file contents:  %5ld bytes\n", static_cast<long>(totalSize));
+        printf("minDecompNoCodePoint:              U+%04lX\n",
+               static_cast<long>(indexes[Normalizer2Impl::IX_MIN_DECOMP_NO_CP]));
+        printf("minCompNoMaybeCodePoint:           U+%04lX\n",
+               static_cast<long>(indexes[Normalizer2Impl::IX_MIN_COMP_NO_MAYBE_CP]));
+        printf("minLcccCodePoint:                  U+%04lX\n",
+               static_cast<long>(indexes[Normalizer2Impl::IX_MIN_LCCC_CP]));
+        printf("minYesNo: (with compositions)      0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_YES_NO]));
+        printf("minYesNoMappingsOnly:              0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_YES_NO_MAPPINGS_ONLY]));
+        printf("minNoNo: (comp-normalized)         0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_NO_NO]));
+        printf("minNoNoCompBoundaryBefore:         0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_NO_NO_COMP_BOUNDARY_BEFORE]));
+        printf("minNoNoCompNoMaybeCC:              0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_NO_NO_COMP_NO_MAYBE_CC]));
+        printf("minNoNoEmpty:                      0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_NO_NO_EMPTY]));
+        printf("limitNoNo:                         0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_LIMIT_NO_NO]));
+        printf("minNoNoDelta:                      0x%04x\n",
+               static_cast<int>(minNoNoDelta));
+        printf("minMaybeNo:                        0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_MAYBE_NO]));
+        printf("minMaybeNoCombinesFwd:             0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_MAYBE_NO_COMBINES_FWD]));
+        printf("minMaybeYes:                       0x%04x\n",
+               static_cast<int>(indexes[Normalizer2Impl::IX_MIN_MAYBE_YES]));
     }
 
     UVersionInfo nullVersion={ 0, 0, 0, 0 };
@@ -780,8 +830,8 @@ void Normalizer2DataBuilder::writeBinaryFile(const char *filename) {
 
     IcuToolErrorCode errorCode("gennorm2/writeBinaryFile()");
     UNewDataMemory *pData=
-        udata_create(NULL, NULL, filename, &dataInfo,
-                     haveCopyright ? U_COPYRIGHT_STRING : NULL, errorCode);
+        udata_create(nullptr, nullptr, filename, &dataInfo,
+                     haveCopyright ? U_COPYRIGHT_STRING : nullptr, errorCode);
     if(errorCode.isFailure()) {
         fprintf(stderr, "gennorm2 error: unable to create the output file %s - %s\n",
                 filename, errorCode.errorName());
@@ -799,7 +849,7 @@ void Normalizer2DataBuilder::writeBinaryFile(const char *filename) {
     int32_t totalSize=indexes[Normalizer2Impl::IX_TOTAL_SIZE];
     if(writtenSize!=totalSize) {
         fprintf(stderr, "gennorm2 error: written size %ld != calculated size %ld\n",
-            (long)writtenSize, (long)totalSize);
+            static_cast<long>(writtenSize), static_cast<long>(totalSize));
         exit(U_INTERNAL_PROGRAM_ERROR);
     }
 }
@@ -810,17 +860,17 @@ Normalizer2DataBuilder::writeCSourceFile(const char *filename) {
 
     IcuToolErrorCode errorCode("gennorm2/writeCSourceFile()");
     const char *basename=findBasename(filename);
-    CharString path(filename, (int32_t)(basename-filename), errorCode);
+    CharString path(filename, static_cast<int32_t>(basename - filename), errorCode);
     CharString dataName(basename, errorCode);
     const char *extension=strrchr(basename, '.');
-    if(extension!=NULL) {
-        dataName.truncate((int32_t)(extension-basename));
+    if(extension!=nullptr) {
+        dataName.truncate(static_cast<int32_t>(extension - basename));
     }
     const char *name=dataName.data();
     errorCode.assertSuccess();
 
     FILE *f=usrc_create(path.data(), basename, 2016, "icu/source/tools/gennorm2/n2builder.cpp");
-    if(f==NULL) {
+    if(f==nullptr) {
         fprintf(stderr, "gennorm2/writeCSourceFile() error: unable to create the output file %s\n",
                 filename);
         exit(U_FILE_ACCESS_ERROR);
@@ -828,18 +878,18 @@ Normalizer2DataBuilder::writeCSourceFile(const char *filename) {
     fputs("#ifdef INCLUDED_FROM_NORMALIZER2_CPP\n\n", f);
 
     char line[100];
-    sprintf(line, "static const UVersionInfo %s_formatVersion={", name);
+    snprintf(line, sizeof(line), "static const UVersionInfo %s_formatVersion={", name);
     usrc_writeArray(f, line, dataInfo.formatVersion, 8, 4, "", "};\n");
-    sprintf(line, "static const UVersionInfo %s_dataVersion={", name);
+    snprintf(line, sizeof(line), "static const UVersionInfo %s_dataVersion={", name);
     usrc_writeArray(f, line, dataInfo.dataVersion, 8, 4, "", "};\n\n");
-    sprintf(line, "static const int32_t %s_indexes[Normalizer2Impl::IX_COUNT]={\n", name);
+    snprintf(line, sizeof(line), "static const int32_t %s_indexes[Normalizer2Impl::IX_COUNT]={\n", name);
     usrc_writeArray(f, line, indexes, 32, Normalizer2Impl::IX_COUNT, "", "\n};\n\n");
 
     usrc_writeUCPTrie(f, name, norm16Trie.getAlias(), UPRV_TARGET_SYNTAX_CCODE);
 
-    sprintf(line, "static const uint16_t %s_extraData[%%ld]={\n", name);
+    snprintf(line, sizeof(line), "static const uint16_t %s_extraData[%%ld]={\n", name);
     usrc_writeArray(f, line, extraData.getBuffer(), 16, extraData.length(), "", "\n};\n\n");
-    sprintf(line, "static const uint8_t %s_smallFCD[%%ld]={\n", name);
+    snprintf(line, sizeof(line), "static const uint8_t %s_smallFCD[%%ld]={\n", name);
     usrc_writeArray(f, line, smallFCD, 8, sizeof(smallFCD), "", "\n};\n\n");
 
     fputs("#endif  // INCLUDED_FROM_NORMALIZER2_CPP\n", f);
@@ -864,10 +914,10 @@ void writeMapping(FILE *f, const UnicodeString *m) {
     if(m != nullptr && !m->isEmpty()) {
         int32_t i = 0;
         UChar32 c = m->char32At(i);
-        fprintf(f, "%04lX", (long)c);
+        fprintf(f, "%04lX", static_cast<long>(c));
         while((i += U16_LENGTH(c)) < m->length()) {
             c = m->char32At(i);
-            fprintf(f, " %04lX", (long)c);
+            fprintf(f, " %04lX", static_cast<long>(c));
         }
     }
     fputs("\n", f);
@@ -915,9 +965,9 @@ Normalizer2DataBuilder::writeDataFile(const char *filename, bool writeRemoved) c
         } else {
             if(prevCC != 0) {
                 if(start == end) {
-                    fprintf(f, "%04lX:%d\n", (long)start, (int)prevCC);
+                    fprintf(f, "%04lX:%d\n", static_cast<long>(start), static_cast<int>(prevCC));
                 } else {
-                    fprintf(f, "%04lX..%04lX:%d\n", (long)start, (long)end, (int)prevCC);
+                    fprintf(f, "%04lX..%04lX:%d\n", static_cast<long>(start), static_cast<long>(end), static_cast<int>(prevCC));
                 }
                 didWrite = true;
             }
@@ -964,9 +1014,9 @@ Normalizer2DataBuilder::writeDataFile(const char *filename, bool writeRemoved) c
         } else {
             if(writeRemoved ? prevType != Norm::NONE : prevType > Norm::REMOVED) {
                 if(start == end) {
-                    fprintf(f, "%04lX%c", (long)start, typeChars[prevType]);
+                    fprintf(f, "%04lX%c", static_cast<long>(start), typeChars[prevType]);
                 } else {
-                    fprintf(f, "%04lX..%04lX%c", (long)start, (long)end, typeChars[prevType]);
+                    fprintf(f, "%04lX..%04lX%c", static_cast<long>(start), static_cast<long>(end), typeChars[prevType]);
                 }
                 writeMapping(f, prevMapping);
             }
