@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
 *
@@ -16,6 +18,11 @@
 *******************************************************************************
 */
 
+#include <fstream>
+#include <iostream>
+#include <list>
+#include <string>
+
 #include <assert.h>
 #include "genrb.h"
 #include "unicode/localpointer.h"
@@ -23,13 +30,16 @@
 #include "unicode/utf16.h"
 #include "charstr.h"
 #include "cmemory.h"
+#include "filterrb.h"
 #include "reslist.h"
 #include "ucmndata.h"  /* TODO: for reading the pool bundle */
+#include "collationroot.h"
 
 U_NAMESPACE_USE
 
 /* Protos */
-void  processFile(const char *filename, const char* cp, const char *inputDir, const char *outputDir,
+void  processFile(const char *filename, const char* cp,
+                  const char *inputDir, const char *outputDir, const char *filterDir,
                   const char *packageName,
                   SRBRoot *newPoolBundle, UBool omitBinaryCollation, UErrorCode &status);
 static char *make_res_filename(const char *filename, const char *outputDir,
@@ -39,16 +49,16 @@ static char *make_res_filename(const char *filename, const char *outputDir,
 #define RES_SUFFIX ".res"
 #define COL_SUFFIX ".col"
 
-const char *gCurrentFileName = NULL;
+const char *gCurrentFileName = nullptr;
 #ifdef XP_MAC_CONSOLE
 #include <console.h>
 #endif
 
 void ResFile::close() {
     delete[] fBytes;
-    fBytes = NULL;
+    fBytes = nullptr;
     delete fStrings;
-    fStrings = NULL;
+    fStrings = nullptr;
 }
 
 enum
@@ -74,7 +84,10 @@ enum
     FORMAT_VERSION,
     WRITE_POOL_BUNDLE,
     USE_POOL_BUNDLE,
-    INCLUDE_UNIHAN_COLL
+    INCLUDE_UNIHAN_COLL,
+    FILTERDIR,
+    ICU4X_MODE,
+    UCADATA
 };
 
 UOption options[]={
@@ -97,31 +110,35 @@ UOption options[]={
                       UOPTION_DEF("language",  'l', UOPT_REQUIRES_ARG), /* 16 */
                       UOPTION_DEF("omitCollationRules", 'R', UOPT_NO_ARG),/* 17 */
                       UOPTION_DEF("formatVersion", '\x01', UOPT_REQUIRES_ARG),/* 18 */
-                      UOPTION_DEF("writePoolBundle", '\x01', UOPT_NO_ARG),/* 19 */
+                      UOPTION_DEF("writePoolBundle", '\x01', UOPT_OPTIONAL_ARG),/* 19 */
                       UOPTION_DEF("usePoolBundle", '\x01', UOPT_OPTIONAL_ARG),/* 20 */
                       UOPTION_DEF("includeUnihanColl", '\x01', UOPT_NO_ARG),/* 21 */ /* temporary, don't display in usage info */
+                      UOPTION_DEF("filterDir", '\x01', UOPT_OPTIONAL_ARG), /* 22 */
+                      UOPTION_DEF("icu4xMode", 'X', UOPT_NO_ARG),/* 23 */
+                      UOPTION_DEF("ucadata", '\x01', UOPT_REQUIRES_ARG),/* 24 */
                   };
 
-static     UBool       write_java = FALSE;
-static     UBool       write_xliff = FALSE;
+static     UBool       write_java = false;
+static     UBool       write_xliff = false;
 static     const char* outputEnc ="";
 
 static ResFile poolBundle;
 
 /*added by Jing*/
-static     const char* language = NULL;
-static     const char* xliffOutputFileName = NULL;
+static     const char* language = nullptr;
+static     const char* xliffOutputFileName = nullptr;
 int
 main(int argc,
      char* argv[])
 {
     UErrorCode  status    = U_ZERO_ERROR;
-    const char *arg       = NULL;
-    const char *outputDir = NULL; /* NULL = no output directory, use current */
-    const char *inputDir  = NULL;
+    const char *arg       = nullptr;
+    const char *outputDir = nullptr; /* nullptr = no output directory, use current */
+    const char *inputDir  = nullptr;
+    const char *filterDir = nullptr;
     const char *encoding  = "";
     int         i;
-    UBool illegalArg = FALSE;
+    UBool illegalArg = false;
 
     U_MAIN_INIT_ARGS(argc, argv);
 
@@ -132,24 +149,28 @@ main(int argc,
     /* error handling, printing usage message */
     if(argc<0) {
         fprintf(stderr, "%s: error in command line argument \"%s\"\n", argv[0], argv[-argc]);
-        illegalArg = TRUE;
+        illegalArg = true;
     } else if(argc<2) {
-        illegalArg = TRUE;
+        illegalArg = true;
     }
     if(options[WRITE_POOL_BUNDLE].doesOccur && options[USE_POOL_BUNDLE].doesOccur) {
         fprintf(stderr, "%s: cannot combine --writePoolBundle and --usePoolBundle\n", argv[0]);
-        illegalArg = TRUE;
+        illegalArg = true;
+    }
+    if (options[ICU4X_MODE].doesOccur && !options[UCADATA].doesOccur) {
+        fprintf(stderr, "%s: --icu4xMode requires --ucadata\n", argv[0]);
+        illegalArg = true;
     }
     if(options[FORMAT_VERSION].doesOccur) {
         const char *s = options[FORMAT_VERSION].value;
-        if(uprv_strlen(s) != 1 || (s[0] < '1' && '3' < s[0])) {
+        if(uprv_strlen(s) != 1 || (s[0] < '1' || '3' < s[0])) {
             fprintf(stderr, "%s: unsupported --formatVersion %s\n", argv[0], s);
-            illegalArg = TRUE;
+            illegalArg = true;
         } else if(s[0] == '1' &&
                   (options[WRITE_POOL_BUNDLE].doesOccur || options[USE_POOL_BUNDLE].doesOccur)
         ) {
             fprintf(stderr, "%s: cannot combine --formatVersion 1 with --writePoolBundle or --usePoolBundle\n", argv[0]);
-            illegalArg = TRUE;
+            illegalArg = true;
         } else {
             setFormatVersion(s[0] - '0');
         }
@@ -161,7 +182,7 @@ main(int argc,
                 "%s error: command line argument --java-package or --bundle-name "
                 "without --write-java\n",
                 argv[0]);
-        illegalArg = TRUE;
+        illegalArg = true;
     }
 
     if(options[VERSION].doesOccur) {
@@ -193,10 +214,10 @@ main(int argc,
                 "\t-c or --copyright        include copyright notice\n");
         fprintf(stderr,
                 "\t-e or --encoding         encoding of source files\n"
-                "\t-d of --destdir          destination directory, followed by the path, defaults to %s\n"
-                "\t-s or --sourcedir        source directory for files followed by path, defaults to %s\n"
+                "\t-d or --destdir          destination directory, followed by the path, defaults to '%s'\n"
+                "\t-s or --sourcedir        source directory for files followed by path, defaults to '%s'\n"
                 "\t-i or --icudatadir       directory for locating any needed intermediate data files,\n"
-                "\t                         followed by path, defaults to %s\n",
+                "\t                         followed by path, defaults to '%s'\n",
                 u_getDataDirectory(), u_getDataDirectory(), u_getDataDirectory());
         fprintf(stderr,
                 "\t-j or --write-java       write a Java ListResourceBundle for ICU4J, followed by optional encoding\n"
@@ -222,26 +243,29 @@ main(int argc,
                 "\t      --formatVersion      write a .res file compatible with the requested formatVersion (single digit);\n"
                 "\t                           for example, --formatVersion 1\n");
         fprintf(stderr,
-                "\t      --writePoolBundle    write a pool.res file with all of the keys of all input bundles\n"
-                "\t      --usePoolBundle [path-to-pool.res]  point to keys from the pool.res keys pool bundle if they are available there;\n"
+                "\t      --writePoolBundle [directory]  write a pool.res file with all of the keys of all input bundles\n"
+                "\t      --usePoolBundle [directory]  point to keys from the pool.res keys pool bundle if they are available there;\n"
                 "\t                           makes .res files smaller but dependent on the pool bundle\n"
                 "\t                           (--writePoolBundle and --usePoolBundle cannot be combined)\n");
+        fprintf(stderr,
+                "\t      --filterDir          Input directory where filter files are available.\n"
+                "\t                           For more on filter files, see ICU Data Build Tool.\n");
 
         return illegalArg ? U_ILLEGAL_ARGUMENT_ERROR : U_ZERO_ERROR;
     }
 
     if(options[VERBOSE].doesOccur) {
-        setVerbose(TRUE);
+        setVerbose(true);
     }
 
     if(options[QUIET].doesOccur) {
-        setShowWarning(FALSE);
+        setShowWarning(false);
     }
     if(options[STRICT].doesOccur) {
-        setStrict(TRUE);
+        setStrict(true);
     }
     if(options[COPYRIGHT].doesOccur){
-        setIncludeCopyright(TRUE);
+        setIncludeCopyright(true);
     }
 
     if(options[SOURCEDIR].doesOccur) {
@@ -252,6 +276,10 @@ main(int argc,
         outputDir = options[DESTDIR].value;
     }
 
+    if (options[FILTERDIR].doesOccur) {
+        filterDir = options[FILTERDIR].value;
+    }
+
     if(options[ENCODING].doesOccur) {
         encoding = options[ENCODING].value;
     }
@@ -272,17 +300,26 @@ main(int argc,
     }
     status = U_ZERO_ERROR;
     if(options[WRITE_JAVA].doesOccur) {
-        write_java = TRUE;
+        write_java = true;
         outputEnc = options[WRITE_JAVA].value;
     }
 
     if(options[WRITE_XLIFF].doesOccur) {
-        write_xliff = TRUE;
-        if(options[WRITE_XLIFF].value != NULL){
+        write_xliff = true;
+        if(options[WRITE_XLIFF].value != nullptr){
             xliffOutputFileName = options[WRITE_XLIFF].value;
         }
     }
 
+    if (options[UCADATA].doesOccur) {
+#if !UCONFIG_NO_COLLATION
+        CollationRoot::forceLoadFromFile(options[UCADATA].value, status);
+#else
+        fprintf(stderr, "--ucadata was used with UCONFIG_NO_COLLATION\n");
+        return status;
+#endif
+    }
+
     initParser();
 
     /*added by Jing*/
@@ -292,14 +329,14 @@ main(int argc,
 
     LocalPointer<SRBRoot> newPoolBundle;
     if(options[WRITE_POOL_BUNDLE].doesOccur) {
-        newPoolBundle.adoptInsteadAndCheckErrorCode(new SRBRoot(NULL, TRUE, status), status);
+        newPoolBundle.adoptInsteadAndCheckErrorCode(new SRBRoot(nullptr, true, status), status);
         if(U_FAILURE(status)) {
             fprintf(stderr, "unable to create an empty bundle for the pool keys: %s\n", u_errorName(status));
             return status;
         } else {
             const char *poolResName = "pool.res";
             char *nameWithoutSuffix = static_cast<char *>(uprv_malloc(uprv_strlen(poolResName) + 1));
-            if (nameWithoutSuffix == NULL) {
+            if (nameWithoutSuffix == nullptr) {
                 fprintf(stderr, "out of memory error\n");
                 return U_MEMORY_ALLOCATION_ERROR;
             }
@@ -323,7 +360,7 @@ main(int argc,
          * Also, make_res_filename() seems to be unused. Review and remove.
          */
         CharString poolFileName;
-        if (options[USE_POOL_BUNDLE].value!=NULL) {
+        if (options[USE_POOL_BUNDLE].value!=nullptr) {
             poolFileName.append(options[USE_POOL_BUNDLE].value, status);
         } else if (inputDir) {
             poolFileName.append(inputDir, status);
@@ -333,7 +370,7 @@ main(int argc,
             return status;
         }
         poolFile = T_FileStream_open(poolFileName.data(), "rb");
-        if (poolFile == NULL) {
+        if (poolFile == nullptr) {
             fprintf(stderr, "unable to open pool bundle file %s\n", poolFileName.data());
             return 1;
         }
@@ -343,7 +380,7 @@ main(int argc,
             return 1;
         }
         poolBundle.fBytes = new uint8_t[(poolFileSize + 15) & ~15];
-        if (poolFileSize > 0 && poolBundle.fBytes == NULL) {
+        if (poolFileSize > 0 && poolBundle.fBytes == nullptr) {
             fprintf(stderr, "unable to allocate memory for the pool bundle file %s\n", poolFileName.data());
             return U_MEMORY_ALLOCATION_ERROR;
         }
@@ -374,13 +411,13 @@ main(int argc,
                     poolFileName.data(), u_errorName(status));
             return status;
         }
-        header = (const DataHeader *)poolBundle.fBytes;
+        header = reinterpret_cast<const DataHeader*>(poolBundle.fBytes);
         if (header->info.formatVersion[0] < 2) {
             fprintf(stderr, "invalid format of pool bundle file %s\n", poolFileName.data());
             return U_INVALID_FORMAT_ERROR;
         }
-        const int32_t *pRoot = (const int32_t *)(
-                (const char *)header + header->dataHeader.headerSize);
+        const int32_t* pRoot = reinterpret_cast<const int32_t*>(
+                reinterpret_cast<const char*>(header) + header->dataHeader.headerSize);
         poolBundle.fIndexes = pRoot + 1;
         indexLength = poolBundle.fIndexes[URES_INDEX_LENGTH] & 0xff;
         if (indexLength <= URES_INDEX_POOL_CHECKSUM) {
@@ -389,7 +426,7 @@ main(int argc,
         }
         int32_t keysBottom = 1 + indexLength;
         int32_t keysTop = poolBundle.fIndexes[URES_INDEX_KEYS_TOP];
-        poolBundle.fKeys = (const char *)(pRoot + keysBottom);
+        poolBundle.fKeys = reinterpret_cast<const char*>(pRoot + keysBottom);
         poolBundle.fKeysLength = (keysTop - keysBottom) * 4;
         poolBundle.fChecksum = poolBundle.fIndexes[URES_INDEX_POOL_CHECKSUM];
 
@@ -404,15 +441,15 @@ main(int argc,
         // an explicit string length that exceeds the number of remaining 16-bit units.
         int32_t stringUnitsLength = (poolBundle.fIndexes[URES_INDEX_16BIT_TOP] - keysTop) * 2;
         if (stringUnitsLength >= 2 && getFormatVersion() >= 3) {
-            poolBundle.fStrings = new PseudoListResource(NULL, status);
-            if (poolBundle.fStrings == NULL) {
+            poolBundle.fStrings = new PseudoListResource(nullptr, status);
+            if (poolBundle.fStrings == nullptr) {
                 fprintf(stderr, "unable to allocate memory for the pool bundle strings %s\n",
                         poolFileName.data());
                 return U_MEMORY_ALLOCATION_ERROR;
             }
             // The PseudoListResource constructor call did not allocate further memory.
             assert(U_SUCCESS(status));
-            const UChar *p = (const UChar *)(pRoot + keysTop);
+            const char16_t* p = reinterpret_cast<const char16_t*>(pRoot + keysTop);
             int32_t remaining = stringUnitsLength;
             do {
                 int32_t first = *p;
@@ -432,7 +469,7 @@ main(int argc,
                     length = ((first - 0xdfef) << 16) | p[1];
                 } else if (first == 0xdfff && remaining >= 3) {
                     numCharsForLength = 3;
-                    length = ((int32_t)p[1] << 16) | p[2];
+                    length = (static_cast<int32_t>(p[1]) << 16) | p[2];
                 } else {
                     break;  // overrun
                 }
@@ -455,7 +492,7 @@ main(int argc,
                     StringResource *sr =
                             new StringResource(poolStringIndex, numCharsForLength,
                                                p, length, status);
-                    if (sr == NULL) {
+                    if (sr == nullptr) {
                         fprintf(stderr, "unable to allocate memory for a pool bundle string %s\n",
                                 poolFileName.data());
                         return U_MEMORY_ALLOCATION_ERROR;
@@ -470,22 +507,22 @@ main(int argc,
             } while (remaining > 0);
             if (poolBundle.fStrings->fCount == 0) {
                 delete poolBundle.fStrings;
-                poolBundle.fStrings = NULL;
+                poolBundle.fStrings = nullptr;
             }
         }
 
         T_FileStream_close(poolFile);
-        setUsePoolBundle(TRUE);
-        if (isVerbose() && poolBundle.fStrings != NULL) {
-            printf("number of shared strings: %d\n", (int)poolBundle.fStrings->fCount);
+        setUsePoolBundle(true);
+        if (isVerbose() && poolBundle.fStrings != nullptr) {
+            printf("number of shared strings: %d\n", static_cast<int>(poolBundle.fStrings->fCount));
             int32_t length = poolBundle.fStringIndexLimit + 1;  // incl. last NUL
             printf("16-bit units for strings: %6d = %6d bytes\n",
-                   (int)length, (int)length * 2);
+                   static_cast<int>(length), static_cast<int>(length) * 2);
         }
     }
 
     if(!options[FORMAT_VERSION].doesOccur && getFormatVersion() == 3 &&
-            poolBundle.fStrings == NULL &&
+            poolBundle.fStrings == nullptr &&
             !options[WRITE_POOL_BUNDLE].doesOccur) {
         // If we just default to formatVersion 3
         // but there are no pool bundle strings to share
@@ -522,7 +559,7 @@ main(int argc,
         if (isVerbose()) {
             printf("Processing file \"%s\"\n", theCurrentFileName.data());
         }
-        processFile(arg, encoding, inputDir, outputDir, NULL,
+        processFile(arg, encoding, inputDir, outputDir, filterDir, nullptr,
                     newPoolBundle.getAlias(),
                     options[NO_BINARY_COLLATION].doesOccur, status);
     }
@@ -530,8 +567,14 @@ main(int argc,
     poolBundle.close();
 
     if(U_SUCCESS(status) && options[WRITE_POOL_BUNDLE].doesOccur) {
+        const char* writePoolDir;
+        if (options[WRITE_POOL_BUNDLE].value!=nullptr) {
+            writePoolDir = options[WRITE_POOL_BUNDLE].value;
+        } else {
+            writePoolDir = outputDir;
+        }
         char outputFileName[256];
-        newPoolBundle->write(outputDir, NULL, outputFileName, sizeof(outputFileName), status);
+        newPoolBundle->write(writePoolDir, nullptr, outputFileName, sizeof(outputFileName), status);
         if(U_FAILURE(status)) {
             fprintf(stderr, "unable to write the pool bundle: %s\n", u_errorName(status));
         }
@@ -539,7 +582,7 @@ main(int argc,
 
     u_cleanup();
 
-    /* Dont return warnings as a failure */
+    /* Don't return warnings as a failure */
     if (U_SUCCESS(status)) {
         return 0;
     }
@@ -550,70 +593,48 @@ main(int argc,
 /* Process a file */
 void
 processFile(const char *filename, const char *cp,
-            const char *inputDir, const char *outputDir, const char *packageName,
+            const char *inputDir, const char *outputDir, const char *filterDir,
+            const char *packageName,
             SRBRoot *newPoolBundle,
             UBool omitBinaryCollation, UErrorCode &status) {
     LocalPointer<SRBRoot> data;
-    UCHARBUF       *ucbuf        = NULL;
-    char           *rbname       = NULL;
-    char           *openFileName = NULL;
-    char           *inputDirBuf  = NULL;
-
-    char           outputFileName[256];
+    LocalUCHARBUFPointer ucbuf;
+    CharString openFileName;
+    CharString inputDirBuf;
 
+    char outputFileName[256];
     int32_t dirlen  = 0;
-    int32_t filelen = 0;
 
     if (U_FAILURE(status)) {
         return;
     }
-    if(filename==NULL){
+    if(filename==nullptr){
         status=U_ILLEGAL_ARGUMENT_ERROR;
         return;
-    }else{
-        filelen = (int32_t)uprv_strlen(filename);
     }
 
-    if(inputDir == NULL) {
+    if(inputDir == nullptr) {
         const char *filenameBegin = uprv_strrchr(filename, U_FILE_SEP_CHAR);
-        openFileName = (char *) uprv_malloc(dirlen + filelen + 2);
-        openFileName[0] = '\0';
-        if (filenameBegin != NULL) {
+        if (filenameBegin != nullptr) {
             /*
              * When a filename ../../../data/root.txt is specified,
              * we presume that the input directory is ../../../data
              * This is very important when the resource file includes
              * another file, like UCARules.txt or thaidict.brk.
              */
-            int32_t filenameSize = (int32_t)(filenameBegin - filename + 1);
-            inputDirBuf = uprv_strncpy((char *)uprv_malloc(filenameSize), filename, filenameSize);
+            int32_t filenameSize = static_cast<int32_t>(filenameBegin - filename + 1);
+            inputDirBuf.append(filename, filenameSize, status);
 
-            /* test for NULL */
-            if(inputDirBuf == NULL) {
-                status = U_MEMORY_ALLOCATION_ERROR;
-                goto finish;
-            }
-
-            inputDirBuf[filenameSize - 1] = 0;
-            inputDir = inputDirBuf;
-            dirlen  = (int32_t)uprv_strlen(inputDir);
+            inputDir = inputDirBuf.data();
+            dirlen  = inputDirBuf.length();
         }
     }else{
-        dirlen  = (int32_t)uprv_strlen(inputDir);
+        dirlen = static_cast<int32_t>(uprv_strlen(inputDir));
 
         if(inputDir[dirlen-1] != U_FILE_SEP_CHAR) {
-            openFileName = (char *) uprv_malloc(dirlen + filelen + 2);
-
-            /* test for NULL */
-            if(openFileName == NULL) {
-                status = U_MEMORY_ALLOCATION_ERROR;
-                goto finish;
-            }
-
-            openFileName[0] = '\0';
             /*
              * append the input dir to openFileName if the first char in
-             * filename is not file seperation char and the last char input directory is  not '.'.
+             * filename is not file separation char and the last char input directory is  not '.'.
              * This is to support :
              * genrb -s. /home/icu/data
              * genrb -s. icu/data
@@ -623,49 +644,80 @@ processFile(const char *filename, const char *cp,
              * genrb -s. icu/data  --- start from CWD and look in icu/data dir
              */
             if( (filename[0] != U_FILE_SEP_CHAR) && (inputDir[dirlen-1] !='.')){
-                uprv_strcpy(openFileName, inputDir);
-                openFileName[dirlen]     = U_FILE_SEP_CHAR;
+                openFileName.append(inputDir, status);
             }
-            openFileName[dirlen + 1] = '\0';
         } else {
-            openFileName = (char *) uprv_malloc(dirlen + filelen + 1);
-
-            /* test for NULL */
-            if(openFileName == NULL) {
-                status = U_MEMORY_ALLOCATION_ERROR;
-                goto finish;
-            }
-
-            uprv_strcpy(openFileName, inputDir);
-
+            openFileName.append(inputDir, status);
         }
     }
+    openFileName.appendPathPart(filename, status);
 
-    uprv_strcat(openFileName, filename);
+    // Test for CharString failure
+    if (U_FAILURE(status)) {
+        return;
+    }
 
-    ucbuf = ucbuf_open(openFileName, &cp,getShowWarning(),TRUE, &status);
+    ucbuf.adoptInstead(ucbuf_open(openFileName.data(), &cp,getShowWarning(),true, &status));
     if(status == U_FILE_ACCESS_ERROR) {
 
-        fprintf(stderr, "couldn't open file %s\n", openFileName == NULL ? filename : openFileName);
-        goto finish;
+        fprintf(stderr, "couldn't open file %s\n", openFileName.data());
+        return;
     }
-    if (ucbuf == NULL || U_FAILURE(status)) {
-        fprintf(stderr, "An error occured processing file %s. Error: %s\n",
-                openFileName == NULL ? filename : openFileName, u_errorName(status));
-        goto finish;
+    if (ucbuf.isNull() || U_FAILURE(status)) {
+        fprintf(stderr, "An error occurred processing file %s. Error: %s\n",
+                openFileName.data(), u_errorName(status));
+        return;
     }
     /* auto detected popular encodings? */
-    if (cp!=NULL && isVerbose()) {
+    if (cp!=nullptr && isVerbose()) {
         printf("autodetected encoding %s\n", cp);
     }
     /* Parse the data into an SRBRoot */
-    data.adoptInstead(parse(ucbuf, inputDir, outputDir, filename,
-            !omitBinaryCollation, options[NO_COLLATION_RULES].doesOccur, &status));
+    data.adoptInstead(parse(ucbuf.getAlias(), inputDir, outputDir, filename,
+            !omitBinaryCollation, options[NO_COLLATION_RULES].doesOccur, options[ICU4X_MODE].doesOccur, &status));
 
     if (data.isNull() || U_FAILURE(status)) {
         fprintf(stderr, "couldn't parse the file %s. Error:%s\n", filename, u_errorName(status));
-        goto finish;
+        return;
     }
+
+    // Run filtering before writing pool bundle
+    if (filterDir != nullptr) {
+        CharString filterFileName(filterDir, status);
+        filterFileName.appendPathPart(filename, status);
+        if (U_FAILURE(status)) {
+            return;
+        }
+
+        // Open the file and read it into filter
+        SimpleRuleBasedPathFilter filter;
+        std::ifstream f(filterFileName.data());
+        if (f.fail()) {
+            std::cerr << "genrb error: unable to open " << filterFileName.data() << std::endl;
+            status = U_FILE_ACCESS_ERROR;
+            return;
+        }
+        std::string currentLine;
+        while (std::getline(f, currentLine)) {
+            // Ignore # comments and empty lines
+            if (currentLine.empty() || currentLine[0] == '#') {
+                continue;
+            }
+            filter.addRule(currentLine, status);
+            if (U_FAILURE(status)) {
+                return;
+            }
+        }
+
+        if (isVerbose()) {
+            filter.print(std::cout);
+        }
+
+        // Apply the filter to the data
+        ResKeyPath path;
+        data->fRoot->applyFilter(filter, path, data.getAlias());
+    }
+
     if(options[WRITE_POOL_BUNDLE].doesOccur) {
         data->fWritePoolBundle = newPoolBundle;
         data->compactKeys(status);
@@ -675,7 +727,7 @@ processFile(const char *filename, const char *cp,
         if(U_FAILURE(status)) {
             fprintf(stderr, "bundle_compactKeys(%s) or bundle_getKeyBytes() failed: %s\n",
                     filename, u_errorName(status));
-            goto finish;
+            return;
         }
         /* count the number of just-added key strings */
         for(const char *newKeysLimit = newKeys + newKeysLength; newKeys < newKeysLimit; ++newKeys) {
@@ -690,17 +742,17 @@ processFile(const char *filename, const char *cp,
     }
 
     /* Determine the target rb filename */
-    rbname = make_res_filename(filename, outputDir, packageName, status);
+    uprv_free(make_res_filename(filename, outputDir, packageName, status));
     if(U_FAILURE(status)) {
         fprintf(stderr, "couldn't make the res fileName for  bundle %s. Error:%s\n",
                 filename, u_errorName(status));
-        goto finish;
+        return;
     }
-    if(write_java== TRUE){
+    if(write_java== true){
         bundle_write_java(data.getAlias(), outputDir, outputEnc,
                           outputFileName, sizeof(outputFileName),
                           options[JAVA_PACKAGE].value, options[BUNDLE_NAME].value, &status);
-    }else if(write_xliff ==TRUE){
+    }else if(write_xliff ==true){
         bundle_write_xml(data.getAlias(), outputDir, outputEnc,
                          filename, outputFileName, sizeof(outputFileName),
                          language, xliffOutputFileName, &status);
@@ -711,24 +763,6 @@ processFile(const char *filename, const char *cp,
     if (U_FAILURE(status)) {
         fprintf(stderr, "couldn't write bundle %s. Error:%s\n", outputFileName, u_errorName(status));
     }
-
-finish:
-
-    if (inputDirBuf != NULL) {
-        uprv_free(inputDirBuf);
-    }
-
-    if (openFileName != NULL) {
-        uprv_free(openFileName);
-    }
-
-    if(ucbuf) {
-        ucbuf_close(ucbuf);
-    }
-
-    if (rbname) {
-        uprv_free(rbname);
-    }
 }
 
 /* Generate the target .res file name from the input file name */
@@ -745,48 +779,48 @@ make_res_filename(const char *filename,
 
 
     if (U_FAILURE(status)) {
-        return 0;
+        return nullptr;
     }
 
-    if(packageName != NULL)
+    if(packageName != nullptr)
     {
-        pkgLen = (int32_t)(1 + uprv_strlen(packageName));
+        pkgLen = static_cast<int32_t>(1 + uprv_strlen(packageName));
     }
 
     /* setup */
-    basename = dirname = resName = 0;
+    basename = dirname = resName = nullptr;
 
     /* determine basename, and compiled file names */
-    basename = (char*) uprv_malloc(sizeof(char) * (uprv_strlen(filename) + 1));
-    if(basename == 0) {
+    basename = static_cast<char*>(uprv_malloc(sizeof(char) * (uprv_strlen(filename) + 1)));
+    if (basename == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         goto finish;
     }
 
     get_basename(basename, filename);
 
-    dirname = (char*) uprv_malloc(sizeof(char) * (uprv_strlen(filename) + 1));
-    if(dirname == 0) {
+    dirname = static_cast<char*>(uprv_malloc(sizeof(char) * (uprv_strlen(filename) + 1)));
+    if (dirname == nullptr) {
         status = U_MEMORY_ALLOCATION_ERROR;
         goto finish;
     }
 
     get_dirname(dirname, filename);
 
-    if (outputDir == NULL) {
+    if (outputDir == nullptr) {
         /* output in same dir as .txt */
-        resName = (char*) uprv_malloc(sizeof(char) * (uprv_strlen(dirname)
+        resName = static_cast<char*>(uprv_malloc(sizeof(char) * (uprv_strlen(dirname)
                                       + pkgLen
                                       + uprv_strlen(basename)
-                                      + uprv_strlen(RES_SUFFIX) + 8));
-        if(resName == 0) {
+                                      + uprv_strlen(RES_SUFFIX) + 8)));
+        if (resName == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             goto finish;
         }
 
         uprv_strcpy(resName, dirname);
 
-        if(packageName != NULL)
+        if(packageName != nullptr)
         {
             uprv_strcat(resName, packageName);
             uprv_strcat(resName, "_");
@@ -795,12 +829,12 @@ make_res_filename(const char *filename,
         uprv_strcat(resName, basename);
 
     } else {
-        int32_t dirlen      = (int32_t)uprv_strlen(outputDir);
-        int32_t basenamelen = (int32_t)uprv_strlen(basename);
+        int32_t dirlen = static_cast<int32_t>(uprv_strlen(outputDir));
+        int32_t basenamelen = static_cast<int32_t>(uprv_strlen(basename));
 
-        resName = (char*) uprv_malloc(sizeof(char) * (dirlen + pkgLen + basenamelen + 8));
+        resName = static_cast<char*>(uprv_malloc(sizeof(char) * (dirlen + pkgLen + basenamelen + 8)));
 
-        if (resName == NULL) {
+        if (resName == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             goto finish;
         }
@@ -812,7 +846,7 @@ make_res_filename(const char *filename,
             resName[dirlen + 1] = '\0';
         }
 
-        if(packageName != NULL)
+        if(packageName != nullptr)
         {
             uprv_strcat(resName, packageName);
             uprv_strcat(resName, "_");
