@@ -16,6 +16,9 @@
 #   define NOMCX
 #include <windows.h>
 #include <time.h>
+#   if defined(__clang__)
+#       include <exception>
+#   endif
 #   ifdef __GNUC__
 #       define WINDOWS_WITH_GNUC
 #   endif
@@ -158,6 +161,22 @@ static const struct AssemblyType {
 
         ".long ","",HEX_0X
     },
+    /* macOS PPC should use `.p2align 4` instead `.balign 16` because is
+     * unknown pseudo ops for such legacy system*/
+    {"gcc-darwin-ppc",
+        /*"\t.section __TEXT,__text,regular,pure_instructions\n"
+        "\t.section __TEXT,__picsymbolstub1,symbol_stubs,pure_instructions,32\n"*/
+        ".globl _%s\n"
+        "#ifdef U_HIDE_DATA_SYMBOL\n"
+        "\t.private_extern _%s\n"
+        "#endif\n"
+        "\t.data\n"
+        "\t.const\n"
+        "\t.p2align 4\n"
+        "_%s:\n\n",
+
+        ".long ","",HEX_0X
+    },
     {"gcc-cygwin",
         ".globl _%s\n"
         "\t.section .rodata\n"
@@ -228,6 +247,16 @@ static const struct AssemblyType {
     },
 /* align 16 bytes */
 /*  http://msdn.microsoft.com/en-us/library/dwa9fwef.aspx */
+    {"nasm",
+        "global %s\n"
+#if defined(_WIN32)
+        "section .rdata align=16\n"
+#else
+        "section .rodata align=16\n"
+#endif
+        "%s:\n",
+        "  dd ","",HEX_0X
+    },
     { "masm",
       "\tTITLE %s\n"
       "; generated by genccode\n"
@@ -238,6 +267,15 @@ static const struct AssemblyType {
       "\tALIGN 16\n"
       "_%s\tLABEL DWORD\n",
       "\tDWORD ","\nICUDATA_%s\tENDS\n\tEND\n",HEX_0H
+    },
+    { "masm64",
+      "\tTITLE %s\n"
+      "; generated by genccode\n"
+      "\tPUBLIC _%s\n"
+      "ICUDATA_%s\tSEGMENT READONLY 'DATA'\n"
+      "\tALIGN 16\n"
+      "_%s\tLABEL DWORD\n",
+      "\tDWORD ","\nICUDATA_%s\tENDS\n\tEND\n",HEX_0H
     }
 };
 
@@ -252,16 +290,22 @@ checkAssemblyHeaderName(const char* optAssembly) {
         if (uprv_strcmp(optAssembly, assemblyHeader[idx].name) == 0) {
             assemblyHeaderIndex = idx;
             hexType = assemblyHeader[idx].hexType; /* set the hex type */
-            return TRUE;
+            return true;
         }
     }
 
-    return FALSE;
+    return false;
+}
+
+U_CAPI UBool U_EXPORT2
+checkCpuArchitecture(const char* optCpuArch) {
+    return strcmp(optCpuArch, "x64") == 0 || strcmp(optCpuArch, "x86") == 0 || strcmp(optCpuArch, "arm64") == 0 || strcmp(optCpuArch, "arm") == 0
+        || strcmp(optCpuArch, "X64") == 0 || strcmp(optCpuArch, "X86") == 0 || strcmp(optCpuArch, "ARM64") == 0 || strcmp(optCpuArch, "ARM") == 0;
 }
 
 
 U_CAPI void U_EXPORT2
-printAssemblyHeadersToStdErr(void) {
+printAssemblyHeadersToStdErr() {
     int32_t idx;
     fprintf(stderr, "%s", assemblyHeader[0].name);
     for (idx = 1; idx < UPRV_LENGTHOF(assemblyHeader); idx++) {
@@ -289,11 +333,22 @@ writeAssemblyCode(
     size_t i, length, count;
 
     in=T_FileStream_open(filename, "rb");
-    if(in==NULL) {
+    if(in==nullptr) {
         fprintf(stderr, "genccode: unable to open input file %s\n", filename);
         exit(U_FILE_ACCESS_ERROR);
     }
 
+    const char* newSuffix = nullptr;
+
+    if (uprv_strcmp(assemblyHeader[assemblyHeaderIndex].name, "masm") == 0) {
+        newSuffix = ".masm";
+    }
+    else if (uprv_strcmp(assemblyHeader[assemblyHeaderIndex].name, "nasm") == 0) {
+        newSuffix = ".asm";
+    } else {
+        newSuffix = ".S";
+    }
+
     getOutFilename(
         filename,
         destdir,
@@ -301,28 +356,27 @@ writeAssemblyCode(
         sizeof(buffer.chars),
         entry,
         sizeof(entry),
-        ".S",
+        newSuffix,
         optFilename);
     out=T_FileStream_open(buffer.chars, "w");
-    if(out==NULL) {
+    if(out==nullptr) {
         fprintf(stderr, "genccode: unable to open output file %s\n", buffer.chars);
         exit(U_FILE_ACCESS_ERROR);
     }
 
-    if (outFilePath != NULL) {
+    if (outFilePath != nullptr) {
         if (uprv_strlen(buffer.chars) >= outFilePathCapacity) {
             fprintf(stderr, "genccode: filename too long\n");
             exit(U_ILLEGAL_ARGUMENT_ERROR);
         }
         uprv_strcpy(outFilePath, buffer.chars);
+#if defined (WINDOWS_WITH_GNUC) && U_PLATFORM != U_PF_CYGWIN
+        /* Need to fix the file separator character when using MinGW. */
+        swapFileSepChar(outFilePath, U_FILE_SEP_CHAR, '/');
+#endif
     }
 
-#if defined (WINDOWS_WITH_GNUC) && U_PLATFORM != U_PF_CYGWIN
-    /* Need to fix the file separator character when using MinGW. */
-    swapFileSepChar(outFilePath, U_FILE_SEP_CHAR, '/');
-#endif
-
-    if(optEntryPoint != NULL) {
+    if(optEntryPoint != nullptr) {
         uprv_strcpy(entry, optEntryPoint);
         uprv_strcat(entry, "_dat");
     }
@@ -390,6 +444,7 @@ U_CAPI void U_EXPORT2
 writeCCode(
         const char *filename,
         const char *destdir,
+        const char *optEntryPoint,
         const char *optName,
         const char *optFilename,
         char *outFilePath,
@@ -400,12 +455,12 @@ writeCCode(
     size_t i, length, count;
 
     in=T_FileStream_open(filename, "rb");
-    if(in==NULL) {
+    if(in==nullptr) {
         fprintf(stderr, "genccode: unable to open input file %s\n", filename);
         exit(U_FILE_ACCESS_ERROR);
     }
 
-    if(optName != NULL) { /* prepend  'icudt28_' */
+    if(optName != nullptr) { /* prepend  'icudt28_' */
         // +2 includes the _ and the NUL
         if (uprv_strlen(optName) + 2 > sizeof(entry)) {
             fprintf(stderr, "genccode: entry name too long (long filename?)\n");
@@ -427,20 +482,29 @@ writeCCode(
         ".c",
         optFilename);
 
-    if (outFilePath != NULL) {
+    if (outFilePath != nullptr) {
         if (uprv_strlen(buffer) >= outFilePathCapacity) {
             fprintf(stderr, "genccode: filename too long\n");
             exit(U_ILLEGAL_ARGUMENT_ERROR);
         }
         uprv_strcpy(outFilePath, buffer);
+#if defined (WINDOWS_WITH_GNUC) && U_PLATFORM != U_PF_CYGWIN
+        /* Need to fix the file separator character when using MinGW. */
+        swapFileSepChar(outFilePath, U_FILE_SEP_CHAR, '/');
+#endif
     }
 
     out=T_FileStream_open(buffer, "w");
-    if(out==NULL) {
+    if(out==nullptr) {
         fprintf(stderr, "genccode: unable to open output file %s\n", buffer);
         exit(U_FILE_ACCESS_ERROR);
     }
 
+    if(optEntryPoint != nullptr) {
+        uprv_strcpy(entry, optEntryPoint);
+        uprv_strcat(entry, "_dat");
+    }
+
     /* turn dashes or dots in the entry name into underscores */
     length=uprv_strlen(entry);
     for(i=0; i<length; ++i) {
@@ -543,7 +607,7 @@ write32(FileStream *out, uint32_t bitField, uint32_t column) {
     int32_t i;
     char bitFieldStr[64]; /* This is more bits than needed for a 32-bit number */
     char *s = bitFieldStr;
-    uint8_t *ptrIdx = (uint8_t *)&bitField;
+    uint8_t* ptrIdx = reinterpret_cast<uint8_t*>(&bitField);
     static const char hexToStr[16] = {
         '0','1','2','3',
         '4','5','6','7',
@@ -610,14 +674,14 @@ write8(FileStream *out, uint8_t byte, uint32_t column) {
 
     /* convert the byte value to a string */
     if(byte>=100) {
-        s[i++]=(char)('0'+byte/100);
+        s[i++] = static_cast<char>('0' + byte / 100);
         byte%=100;
     }
     if(i>0 || byte>=10) {
-        s[i++]=(char)('0'+byte/10);
+        s[i++] = static_cast<char>('0' + byte / 10);
         byte%=10;
     }
-    s[i++]=(char)('0'+byte);
+    s[i++] = static_cast<char>('0' + byte);
     s[i]=0;
 
     /* write the value, possibly with comma and newline */
@@ -641,9 +705,9 @@ write8str(FileStream *out, uint8_t byte, uint32_t column) {
     char s[8];
 
     if (byte > 7)
-        sprintf(s, "\\x%X", byte);
+        snprintf(s, sizeof(s), "\\x%X", byte);
     else
-        sprintf(s, "\\%X", byte);
+        snprintf(s, sizeof(s), "\\%X", byte);
 
     /* write the value, possibly with comma and newline */
     if(column==MAX_COLUMN) {
@@ -678,7 +742,7 @@ getOutFilename(
     icu::ErrorCode status;
 
     /* copy path */
-    if(destdir!=NULL && *destdir!=0) {
+    if(destdir!=nullptr && *destdir!=0) {
         outFilenameBuilder.append(destdir, status);
         outFilenameBuilder.ensureEndsWithFileSeparator(status);
     } else {
@@ -686,10 +750,10 @@ getOutFilename(
     }
     inFilename=basename;
 
-    if(suffix==NULL) {
+    if(suffix==nullptr) {
         /* the filename does not have a suffix */
         entryNameBuilder.append(inFilename, status);
-        if(optFilename != NULL) {
+        if(optFilename != nullptr) {
             outFilenameBuilder.append(optFilename, status);
         } else {
             outFilenameBuilder.append(inFilename, status);
@@ -715,7 +779,7 @@ getOutFilename(
         outFilenameBuilder.append(inFilename, status);
         entryNameBuilder.append(inFilename, status);
 
-        if(optFilename != NULL) {
+        if(optFilename != nullptr) {
             outFilenameBuilder.truncate(saveOutFilenameLength);
             outFilenameBuilder.append(optFilename, status);
         }
@@ -744,7 +808,12 @@ getOutFilename(
 
 #ifdef CAN_GENERATE_OBJECTS
 static void
-getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char *optMatchArch) {
+getArchitecture(
+    uint16_t *pCPU,
+    uint16_t *pBits,
+    UBool *pIsBigEndian,
+    const char *optMatchArch,
+    [[maybe_unused]] const char *optCpuArch) {
     union {
         char        bytes[2048];
 #ifdef U_ELF
@@ -767,7 +836,7 @@ getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char
 #   error "Unknown platform for CAN_GENERATE_OBJECTS."
 #endif
 
-    if(optMatchArch != NULL) {
+    if(optMatchArch != nullptr) {
         filename=optMatchArch;
     } else {
         /* set defaults */
@@ -775,10 +844,10 @@ getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char
         /* set EM_386 because elf.h does not provide better defaults */
         *pCPU=EM_386;
         *pBits=32;
-        *pIsBigEndian=(UBool)(U_IS_BIG_ENDIAN ? ELFDATA2MSB : ELFDATA2LSB);
+        *pIsBigEndian = static_cast<UBool>(U_IS_BIG_ENDIAN ? ELFDATA2MSB : ELFDATA2LSB);
 #elif U_PLATFORM_HAS_WIN32_API
         // Windows always runs in little-endian mode.
-        *pIsBigEndian = FALSE;
+        *pIsBigEndian = false;
 
         // Note: The various _M_<arch> macros are predefined by the MSVC compiler based
         // on the target compilation architecture.
@@ -792,7 +861,30 @@ getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char
 #   if defined(_M_IX86)
         *pCPU = IMAGE_FILE_MACHINE_I386;
 #   else
-        *pCPU = IMAGE_FILE_MACHINE_UNKNOWN;
+        // Linker for ClangCL doesn't handle IMAGE_FILE_MACHINE_UNKNOWN the same as
+        // linker for MSVC. Because of this optCpuArch is used to define the CPU
+        // architecture in that case. While _M_AMD64 and _M_ARM64 could be used,
+        // this would potentially be problematic when cross-compiling as this code
+        // would most likely be ran on host machine to generate the .obj file for
+        // the target architecture.
+#       if defined(__clang__)
+            if (!optCpuArch) {
+                fprintf(stderr, "genccode: CPU architecture must be set for Clang-CL\n");
+                exit(U_ILLEGAL_ARGUMENT_ERROR);
+            } if (strcmp(optCpuArch, "x64") == 0 || strcmp(optCpuArch, "X64") == 0) {
+                *pCPU = IMAGE_FILE_MACHINE_AMD64;
+            } else if (strcmp(optCpuArch, "x86") == 0 || strcmp(optCpuArch, "X86") == 0) {
+                *pCPU = IMAGE_FILE_MACHINE_I386;
+            } else if (strcmp(optCpuArch, "arm64") == 0 || strcmp(optCpuArch, "ARM64") == 0) {
+                *pCPU = IMAGE_FILE_MACHINE_ARM64;
+            } else if (strcmp(optCpuArch, "arm") == 0 || strcmp(optCpuArch, "ARM") == 0) {
+                *pCPU = IMAGE_FILE_MACHINE_ARM;
+            } else {
+                std::terminate(); // Unreachable.
+            }
+#       else
+            *pCPU = IMAGE_FILE_MACHINE_UNKNOWN;
+#       endif
 #   endif
 #   if defined(_M_IA64) || defined(_M_AMD64) || defined (_M_ARM64)
         *pBits = 64; // Doesn't seem to be used for anything interesting though?
@@ -808,14 +900,14 @@ getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char
     }
 
     in=T_FileStream_open(filename, "rb");
-    if(in==NULL) {
+    if(in==nullptr) {
         fprintf(stderr, "genccode: unable to open match-arch file %s\n", filename);
         exit(U_FILE_ACCESS_ERROR);
     }
     length=T_FileStream_read(in, buffer.bytes, sizeof(buffer.bytes));
 
 #ifdef U_ELF
-    if(length<(int32_t)sizeof(Elf32_Ehdr)) {
+    if (length < static_cast<int32_t>(sizeof(Elf32_Ehdr))) {
         fprintf(stderr, "genccode: match-arch file %s is too short\n", filename);
         exit(U_UNSUPPORTED_ERROR);
     }
@@ -843,7 +935,7 @@ getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char
     }
 #endif
 
-    *pIsBigEndian=(UBool)(buffer.header32.e_ident[EI_DATA]==ELFDATA2MSB);
+    *pIsBigEndian = static_cast<UBool>(buffer.header32.e_ident[EI_DATA] == ELFDATA2MSB);
     if(*pIsBigEndian!=U_IS_BIG_ENDIAN) {
         fprintf(stderr, "genccode: currently only same-endianness ELF formats are supported\n");
         exit(U_UNSUPPORTED_ERROR);
@@ -865,7 +957,7 @@ getArchitecture(uint16_t *pCPU, uint16_t *pBits, UBool *pIsBigEndian, const char
      */
     *pBits= *pCPU==IMAGE_FILE_MACHINE_I386 ? 32 : 64;
     /* Windows always runs on little-endian CPUs. */
-    *pIsBigEndian=FALSE;
+    *pIsBigEndian=false;
 #else
 #   error "Unknown platform for CAN_GENERATE_OBJECTS."
 #endif
@@ -879,6 +971,7 @@ writeObjectCode(
         const char *destdir,
         const char *optEntryPoint,
         const char *optMatchArch,
+        const char *optCpuArch,
         const char *optFilename,
         char *outFilePath,
         size_t outFilePathCapacity,
@@ -1146,7 +1239,7 @@ writeObjectCode(
 #endif
 
     /* deal with options, files and the entry point name */
-    getArchitecture(&cpu, &bits, &makeBigEndian, optMatchArch);
+    getArchitecture(&cpu, &bits, &makeBigEndian, optMatchArch, optCpuArch);
     if (optMatchArch)
     {
         printf("genccode: --match-arch cpu=%hu bits=%hu big-endian=%d\n", cpu, bits, makeBigEndian);
@@ -1162,7 +1255,7 @@ writeObjectCode(
 #endif
 
     in=T_FileStream_open(filename, "rb");
-    if(in==NULL) {
+    if(in==nullptr) {
         fprintf(stderr, "genccode: unable to open input file %s\n", filename);
         exit(U_FILE_ACCESS_ERROR);
     }
@@ -1178,7 +1271,7 @@ writeObjectCode(
         newSuffix,
         optFilename);
 
-    if (outFilePath != NULL) {
+    if (outFilePath != nullptr) {
         if (uprv_strlen(buffer) >= outFilePathCapacity) {
             fprintf(stderr, "genccode: filename too long\n");
             exit(U_ILLEGAL_ARGUMENT_ERROR);
@@ -1186,7 +1279,7 @@ writeObjectCode(
         uprv_strcpy(outFilePath, buffer);
     }
 
-    if(optEntryPoint != NULL) {
+    if(optEntryPoint != nullptr) {
         uprv_strcpy(entry+entryOffset, optEntryPoint);
         uprv_strcat(entry+entryOffset, "_dat");
     }
@@ -1200,7 +1293,7 @@ writeObjectCode(
 
     /* open the output file */
     out=T_FileStream_open(buffer, "wb");
-    if(out==NULL) {
+    if(out==nullptr) {
         fprintf(stderr, "genccode: unable to open output file %s\n", buffer);
         exit(U_FILE_ACCESS_ERROR);
     }
@@ -1275,7 +1368,7 @@ writeObjectCode(
     /* set the file header */
     objHeader.fileHeader.Machine=cpu;
     objHeader.fileHeader.NumberOfSections=2;
-    objHeader.fileHeader.TimeDateStamp=(DWORD)time(NULL);
+    objHeader.fileHeader.TimeDateStamp=(DWORD)time(nullptr);
     objHeader.fileHeader.PointerToSymbolTable=IMAGE_SIZEOF_FILE_HEADER+2*IMAGE_SIZEOF_SECTION_HEADER+length+size; /* start of symbol table */
     objHeader.fileHeader.NumberOfSymbols=1;
 
@@ -1316,7 +1409,7 @@ writeObjectCode(
         if(length==0) {
             break;
         }
-        T_FileStream_write(out, buffer, (int32_t)length);
+        T_FileStream_write(out, buffer, length);
     }
 
 #if U_PLATFORM_HAS_WIN32_API
