@@ -60,8 +60,8 @@ string TZ_RESOURCE_NAME = ICU_TZ_RESOURCE;
 
 const int64_t SECS_PER_YEAR      = 31536000; // 365 days
 const int64_t SECS_PER_LEAP_YEAR = 31622400; // 366 days
-const int64_t LOWEST_TIME32    = (int64_t)((int32_t)0x80000000);
-const int64_t HIGHEST_TIME32    = (int64_t)((int32_t)0x7fffffff);
+const int64_t LOWEST_TIME32 = static_cast<int64_t>(static_cast<int32_t>(0x80000000));
+const int64_t HIGHEST_TIME32 = static_cast<int64_t>(static_cast<int32_t>(0x7fffffff));
 
 bool isLeap(int32_t y) {
     return (y%4 == 0) && ((y%100 != 0) || (y%400 == 0)); // Gregorian
@@ -218,7 +218,7 @@ void ZoneInfo::addAlias(int32_t index) {
 
 void ZoneInfo::setAliasTo(int32_t index) {
     assert(index >= 0);
-    assert(aliases.size() == 0);
+    assert(aliases.empty());
     aliasTo = index;
 }
 
@@ -242,7 +242,7 @@ int64_t readcoded(ifstream& file, int64_t minv=numeric_limits<int64_t>::min(),
                                int64_t maxv=numeric_limits<int64_t>::max()) {
     unsigned char buf[4]; // must be UNSIGNED
     int64_t val=0;
-    file.read((char*)buf, 4);
+    file.read(reinterpret_cast<char*>(buf), 4);
     for(int32_t i=0,shift=24;i<4;++i,shift-=8) {
         val |= buf[i] << shift;
     }
@@ -260,9 +260,9 @@ int64_t readcoded64(ifstream& file, int64_t minv=numeric_limits<int64_t>::min(),
                                int64_t maxv=numeric_limits<int64_t>::max()) {
     unsigned char buf[8]; // must be UNSIGNED
     int64_t val=0;
-    file.read((char*)buf, 8);
+    file.read(reinterpret_cast<char*>(buf), 8);
     for(int32_t i=0,shift=56;i<8;++i,shift-=8) {
-        val |= (int64_t)buf[i] << shift;
+        val |= static_cast<int64_t>(buf[i]) << shift;
     }
     if (val < minv || val > maxv) {
         ostringstream os;
@@ -279,7 +279,7 @@ bool readbool(ifstream& file) {
     file.read(&c, 1);
     if (c!=0 && c!=1) {
         ostringstream os;
-        os << "boolean value out-of-range: " << (int32_t)c;
+        os << "boolean value out-of-range: " << static_cast<int32_t>(c);
         throw out_of_range(os.str());
     }
     return (c!=0);
@@ -313,7 +313,7 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
 
     // Read reserved bytes.  The first of these will be a version byte.
     file.read(buf, 15);
-    if (*(ICUZoneinfoVersion*)&buf != TZ_ICU_VERSION) {
+    if (*reinterpret_cast<ICUZoneinfoVersion*>(&buf) != TZ_ICU_VERSION) {
         throw invalid_argument("File version mismatch");
     }
 
@@ -350,8 +350,8 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
     // Read transition types
     for (i=0; i<timecnt; ++i) {
         unsigned char c;
-        file.read((char*) &c, 1);
-        int32_t t = (int32_t) c;
+        file.read(reinterpret_cast<char*>(&c), 1);
+        int32_t t = static_cast<int32_t>(c);
         if (t < 0 || t >= typecnt) {
             ostringstream os;
             os << "illegal type: " << t << ", expected [0, " << (typecnt-1) << "]";
@@ -377,7 +377,7 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
                     // 32bit int by the ICU resource bundle.
                     break;
                 } else {
-                    info.transitions.push_back(Transition(transitionTimes[i], transitionTypes[i]));
+                    info.transitions.emplace_back(transitionTimes[i], transitionTypes[i]);
                 }
             }
     
@@ -393,7 +393,7 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
         }
     } else {
         for (i=0; i<timecnt; ++i) {
-            info.transitions.push_back(Transition(transitionTimes[i], transitionTypes[i]));
+            info.transitions.emplace_back(transitionTimes[i], transitionTypes[i]);
         }
     }
 
@@ -406,8 +406,8 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
         type.isdst = readbool(file);
 
         unsigned char c;
-        file.read((char*) &c, 1);
-        type.abbr = (int32_t) c;
+        file.read(reinterpret_cast<char*>(&c), 1);
+        type.abbr = static_cast<int32_t>(c);
 
         if (type.isdst != (type.dstoffset != 0)) {
             throw invalid_argument("isdst does not reflect dstoffset");
@@ -430,7 +430,7 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
             // first transition, but no DST is observed.
             int64_t rawoffset0 = (info.types.at(info.transitions.at(0).type)).rawoffset;    
             // Look for matching type
-            for (i=0; i<(int32_t)info.types.size(); ++i) {
+            for (i = 0; i < static_cast<int32_t>(info.types.size()); ++i) {
                 if (info.types.at(i).rawoffset == rawoffset0
                         && info.types.at(i).dstoffset == 0) {
                     initialTypeIdx = i;
@@ -461,7 +461,7 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
         for (char* p=str; p<limit; ++p) {
             char* start = p;
             while (*p != 0) ++p;
-            info.abbrs.push_back(string(start, p-start));
+            info.abbrs.emplace_back(start, p - start);
             abbroffset.push_back(start-str);
         }
 
@@ -504,7 +504,7 @@ void readzoneinfo(ifstream& file, ZoneInfo& info, bool is64bitData) {
             }
         }
 
-        for (int32_t ii=0;ii<(int32_t) abbrseen.size();++ii) {
+        for (int32_t ii = 0; ii < static_cast<int32_t>(abbrseen.size()); ++ii) {
             if (!abbrseen[ii]) {
                 cerr << "Warning: unused abbreviation: " << ii << endl;
             }
@@ -650,7 +650,7 @@ void scandir(string dir, string prefix="") {
     vector<string> subdirs;
     vector<string> subfiles;
 
-    if ((dp = opendir(dir.c_str())) == NULL) {
+    if ((dp = opendir(dir.c_str())) == nullptr) {
         cerr << "Error: Invalid directory: " << dir << endl;
         exit(1);
     }
@@ -659,7 +659,7 @@ void scandir(string dir, string prefix="") {
         exit(1);
     }
     chdir(dir.c_str());
-    while ((dir_entry = readdir(dp)) != NULL) {
+    while ((dir_entry = readdir(dp)) != nullptr) {
         string name = dir_entry->d_name;
         string path = dir + "/" + name;
         lstat(dir_entry->d_name,&stat_info);
@@ -685,7 +685,7 @@ void scandir(string dir, string prefix="") {
     closedir(dp);
     chdir(pwd);
 
-    for(int32_t i=0;i<(int32_t)subfiles.size();i+=2) {
+    for (int32_t i = 0; i < static_cast<int32_t>(subfiles.size()); i += 2) {
         try {
             handleFile(subfiles[i], subfiles[i+1]);
         } catch (const exception& e) {
@@ -694,7 +694,7 @@ void scandir(string dir, string prefix="") {
             exit(1);
         }
     }
-    for(int32_t i=0;i<(int32_t)subdirs.size();i+=2) {
+    for (int32_t i = 0; i < static_cast<int32_t>(subdirs.size()); i += 2) {
         scandir(subdirs[i], subdirs[i+1]);
     }
 }
@@ -774,16 +774,16 @@ struct FinalRulePart {
     // wall time, local standard time, and GMT standard time.
     // Here is how the isstd & isgmt flags are set by zic:
     //| case 's':       /* Standard */
-    //|         rp->r_todisstd = TRUE;
-    //|         rp->r_todisgmt = FALSE;
+    //|         rp->r_todisstd = true;
+    //|         rp->r_todisgmt = false;
     //| case 'w':       /* Wall */
-    //|         rp->r_todisstd = FALSE;
-    //|         rp->r_todisgmt = FALSE;
+    //|         rp->r_todisstd = false;
+    //|         rp->r_todisgmt = false;
     //| case 'g':       /* Greenwich */
     //| case 'u':       /* Universal */
     //| case 'z':       /* Zulu */
-    //|         rp->r_todisstd = TRUE;
-    //|         rp->r_todisgmt = TRUE;
+    //|         rp->r_todisstd = true;
+    //|         rp->r_todisgmt = true;
     bool isstd;
     bool isgmt;
 
@@ -999,7 +999,7 @@ void readFinalZonesAndRules(istream& in) {
     ruleIDset.clear();
     for_each(finalRules.begin(), finalRules.end(), insertRuleID);
     for_each(finalZones.begin(), finalZones.end(), eraseRuleID);
-    if (ruleIDset.size() != 0) {
+    if (!ruleIDset.empty()) {
         throw invalid_argument("Unused rules");
     }
 }
@@ -1015,7 +1015,7 @@ void ZoneInfo::print(ostream& os, const string& id) const {
   os << "  /* " << id << " */ ";
 
     if (aliasTo >= 0) {
-        assert(aliases.size() == 0);
+        assert(aliases.empty());
         os << ":int { " << aliasTo << " } "; // No endl - save room for comment.
         return;
     }
@@ -1042,7 +1042,7 @@ void ZoneInfo::print(ostream& os, const string& id) const {
                     os<< ", ";
                 }
                 first = false;
-                os << (int32_t)(trn->time >> 32) << ", " << (int32_t)(trn->time & 0x00000000ffffffff);
+                os << static_cast<int32_t>(trn->time >> 32) << ", " << static_cast<int32_t>(trn->time & 0x00000000ffffffff);
             }
             os << " }" << endl;
         }
@@ -1068,7 +1068,7 @@ void ZoneInfo::print(ostream& os, const string& id) const {
                     os<< ", ";
                 }
                 first = false;
-                os << (int32_t)(trn->time >> 32) << ", " << (int32_t)(trn->time & 0x00000000ffffffff);
+                os << static_cast<int32_t>(trn->time >> 32) << ", " << static_cast<int32_t>(trn->time & 0x00000000ffffffff);
             }
             os << " }" << endl;
         }
@@ -1097,7 +1097,7 @@ void ZoneInfo::print(ostream& os, const string& id) const {
     os << " }" << endl;
 
     if (ICU44PLUS) {
-        if (transitions.size() != 0) {
+        if (!transitions.empty()) {
             os << "    typeMap:bin { \"" << hex << setfill('0');
             for (trn = transitions.begin(); trn != transitions.end(); ++trn) {
                 os << setw(2) << trn->type;
@@ -1126,7 +1126,7 @@ void ZoneInfo::print(ostream& os, const string& id) const {
     }
 
     // Alias list, if any
-    if (aliases.size() != 0) {
+    if (!aliases.empty()) {
         first = true;
         if (ICU44PLUS) {
             os << "    links:intvector { ";
@@ -1190,11 +1190,6 @@ ostream& printStringList( ostream& os, const ZoneMap& zoneinfo) {
 // main
 //--------------------------------------------------------------------
 
-// Unary predicate for finding transitions after a given time
-bool isAfter(const Transition t, int64_t thresh) {
-    return t.time >= thresh;
-}
-
 /**
  * A zone type that contains only the raw and dst offset.  Used by the
  * optimizeTypeList() method.
@@ -1245,7 +1240,7 @@ void ZoneInfo::optimizeTypeList() {
         // by inserting the dummy transition indirectly.
 
         // If there are zero transitions and one type, then leave that as-is.
-        if (transitions.size() == 0) {
+        if (transitions.empty()) {
             if (types.size() != 1) {
                 cerr << "Error: transition count = 0, type count = " << types.size() << endl;
             }
@@ -1327,10 +1322,10 @@ void ZoneInfo::optimizeTypeList() {
 
             // Replace type list
             types.clear();
-            types.push_back(initialSimplifiedType);
+            types.emplace_back(initialSimplifiedType);
             for (set<SimplifiedZoneType>::const_iterator i=simpleset.begin(); i!=simpleset.end(); ++i) {
                 if (*i < initialSimplifiedType || initialSimplifiedType < *i) {
-                    types.push_back(*i);
+                    types.emplace_back(*i);
                 }
             }
 
@@ -1371,7 +1366,7 @@ void ZoneInfo::mergeFinalData(const FinalZone& fz) {
 
     vector<Transition>::iterator it =
         find_if(transitions.begin(), transitions.end(),
-                bind2nd(ptr_fun(isAfter), seconds));
+                [seconds](const Transition& t) { return t.time >= seconds; });
     transitions.erase(it, transitions.end());
 
     if (finalYear != -1) {
@@ -1672,7 +1667,7 @@ int main(int argc, char *argv[]) {
                 string zone, country;
                 istringstream is(line);
                 is >> zone >> country;
-                if (zone.size() == 0) continue;
+                if (zone.empty()) continue;
                 if (country.size() < 2) {
                     cerr << "Error: Can't parse " << line << " in " << ICU_REGIONS << endl;
                     return 1;
@@ -1703,8 +1698,8 @@ int main(int argc, char *argv[]) {
             string country, coord, zone;
             istringstream is(line);
             is >> country >> coord >> zone;
-            if (country.size() == 0) continue;
-            if (country.size() != 2 || zone.size() < 1) {
+            if (country.empty()) continue;
+            if (country.size() != 2 || zone.empty()) {
                 cerr << "Error: Can't parse " << line << " in " << zonetab << endl;
                 return 1;
             }
